@@ -97,40 +97,22 @@ void               action_message          (GimpDisplay          *display,
   if (! drawables) \
     return
 
-#define return_if_no_layer(image,layer,data) \
-  return_if_no_image (image,data); \
-  layer = gimp_image_get_active_layer (image); \
-  if (! layer) \
-    return
-
 #define return_if_no_layers(image,layers,data) \
   return_if_no_image (image,data); \
   layers = gimp_image_get_selected_layers (image); \
   if (! layers) \
     return
 
-#define return_if_no_channel(image,channel,data) \
-  return_if_no_image (image,data); \
-  channel = gimp_image_get_active_channel (image); \
-  if (! channel) \
-    return
-
 #define return_if_no_channels(image,channels,data) \
   return_if_no_image (image,data); \
   channels = gimp_image_get_selected_channels (image); \
   if (! channels) \
     return
 
-#define return_if_no_vectors(image,vectors,data) \
+#define return_if_no_paths(image,paths,data) \
   return_if_no_image (image,data); \
-  vectors = gimp_image_get_active_vectors (image); \
-  if (! vectors) \
-    return
-
-#define return_if_no_vectors_list(image,list,data) \
-  return_if_no_image (image,data); \
-  list = gimp_image_get_selected_vectors (image); \
-  if (! list) \
+  paths = gimp_image_get_selected_paths (image); \
+  if (! paths) \
     return
 
 
