@@ -38,18 +38,14 @@
 
 static const GimpActionEntry buffers_actions[] =
 {
-  { "buffers-popup", GIMP_ICON_BUFFER,
-    NC_("buffers-action", "Buffers Menu"), NULL, NULL, NULL,
-    GIMP_HELP_BUFFER_DIALOG },
-
   { "buffers-paste-as-new-image", GIMP_ICON_EDIT_PASTE_AS_NEW,
-    NC_("buffers-action", "Paste Buffer as _New Image"), NULL,
+    NC_("buffers-action", "Paste Buffer as _New Image"), NULL, { NULL },
     NC_("buffers-action", "Paste the selected buffer as a new image"),
     buffers_paste_as_new_image_cmd_callback,
     GIMP_HELP_BUFFER_PASTE_AS_NEW_IMAGE },
 
   { "buffers-delete", GIMP_ICON_EDIT_DELETE,
-    NC_("buffers-action", "_Delete Buffer"), NULL,
+    NC_("buffers-action", "_Delete Buffer"), NULL, { NULL },
     NC_("buffers-action", "Delete the selected buffer"),
     buffers_delete_cmd_callback,
     GIMP_HELP_BUFFER_DELETE }
@@ -58,38 +54,38 @@ static const GimpActionEntry buffers_actions[] =
 static const GimpEnumActionEntry buffers_paste_actions[] =
 {
   { "buffers-paste", GIMP_ICON_EDIT_PASTE,
-    NC_("buffers-action", "_Paste Buffer"), NULL,
+    NC_("buffers-action", "_Paste Buffer"), NULL, { NULL },
     NC_("buffers-action", "Paste the selected buffer"),
     GIMP_PASTE_TYPE_FLOATING, FALSE,
     GIMP_HELP_BUFFER_PASTE },
 
   { "buffers-paste-in-place", GIMP_ICON_EDIT_PASTE,
-    NC_("buffers-action", "Paste Buffer In Pl_ace"), NULL,
+    NC_("buffers-action", "Paste Buffer In Pl_ace"), NULL, { NULL },
     NC_("buffers-action", "Paste the selected buffer at its original position"),
     GIMP_PASTE_TYPE_FLOATING_IN_PLACE, FALSE,
     GIMP_HELP_BUFFER_PASTE_IN_PLACE },
 
   { "buffers-paste-into", GIMP_ICON_EDIT_PASTE_INTO,
-    NC_("buffers-action", "Paste Buffer _Into The Selection"), NULL,
+    NC_("buffers-action", "Paste Buffer _Into The Selection"), NULL, { NULL },
     NC_("buffers-action", "Paste the selected buffer into the selection"),
     GIMP_PASTE_TYPE_FLOATING_INTO, FALSE,
     GIMP_HELP_BUFFER_PASTE_INTO },
 
   { "buffers-paste-into-in-place", GIMP_ICON_EDIT_PASTE_INTO,
-    NC_("buffers-action", "Paste Buffer Into The Selection In Place"), NULL,
+    NC_("buffers-action", "Paste Buffer Into The Selection In Place"), NULL, { NULL },
     NC_("buffers-action",
         "Paste the selected buffer into the selection at its original position"),
     GIMP_PASTE_TYPE_FLOATING_INTO_IN_PLACE, FALSE,
     GIMP_HELP_BUFFER_PASTE_INTO_IN_PLACE },
 
   { "buffers-paste-as-new-layer", GIMP_ICON_EDIT_PASTE_AS_NEW,
-    NC_("buffers-action", "Paste Buffer as New _Layer"), NULL,
+    NC_("buffers-action", "Paste Buffer as New _Layer"), NULL, { NULL },
     NC_("buffers-action", "Paste the selected buffer as a new layer"),
     GIMP_PASTE_TYPE_NEW_LAYER, FALSE,
     GIMP_HELP_BUFFER_PASTE_AS_NEW_LAYER },
 
   { "buffers-paste-as-new-layer-in-place", GIMP_ICON_EDIT_PASTE_AS_NEW,
-    NC_("buffers-action", "Paste Buffer as New Layer in Place"), NULL,
+    NC_("buffers-action", "Paste Buffer as New Layer in Place"), NULL, { NULL },
     NC_("buffers-action",
         "Paste the selected buffer as a new layer at its original position"),
     GIMP_PASTE_TYPE_NEW_LAYER_IN_PLACE, FALSE,
