@@ -20,6 +20,7 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "actions-types.h"
@@ -90,9 +91,9 @@ buffers_paste_cmd_callback (GimpAction *action,
         {
           GList *drawables = gimp_image_get_selected_drawables (image);
 
-          g_list_free (gimp_edit_paste (image,
-                                        g_list_length (drawables) == 1 ? drawables->data : NULL,
+          g_list_free (gimp_edit_paste (image, drawables,
                                         GIMP_OBJECT (buffer), paste_type,
+                                        context, FALSE,
                                         x, y, width, height));
 
           gimp_image_flush (image);
@@ -122,9 +123,10 @@ buffers_paste_as_new_image_cmd_callback (GimpAction *action,
       GimpImage *new_image;
 
       new_image = gimp_edit_paste_as_new_image (context->gimp,
-                                                GIMP_OBJECT (buffer));
+                                                GIMP_OBJECT (buffer),
+                                                context);
       gimp_create_display (context->gimp, new_image,
-                           GIMP_UNIT_PIXEL, 1.0,
+                           gimp_unit_pixel (), 1.0,
                            G_OBJECT (gimp_widget_get_monitor (widget)));
       g_object_unref (new_image);
     }
