@@ -28,8 +28,10 @@
 #include "core/gimpdrawable.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-colormap.h"
+#include "core/gimppalette.h"
 
 #include "widgets/gimpactiongroup.h"
+#include "widgets/gimpcolormapeditor.h"
 #include "widgets/gimphelp-ids.h"
 
 #include "actions.h"
@@ -45,6 +47,12 @@ static const GimpActionEntry colormap_actions[] =
     NC_("colormap-action", "_Edit Color..."), NULL, { NULL },
     NC_("colormap-action", "Edit this color"),
     colormap_edit_color_cmd_callback,
+    GIMP_HELP_INDEXED_PALETTE_EDIT },
+
+  { "colormap-delete-color", GIMP_ICON_EDIT_DELETE,
+    NC_("colormap-action", "_Delete Color..."), NULL, { NULL },
+    NC_("colormap-action", "Delete this color"),
+    colormap_delete_color_cmd_callback,
     GIMP_HELP_INDEXED_PALETTE_EDIT }
 };
 
@@ -112,13 +120,12 @@ void
 colormap_actions_update (GimpActionGroup *group,
                          gpointer         data)
 {
-  GimpImage   *image            = action_data_get_image (data);
-  GimpContext *context          = action_data_get_context (data);
-  gboolean     indexed          = FALSE;
-  gboolean     drawable_indexed = FALSE;
-  gint         num_colors       = 0;
-  GimpRGB      fg;
-  GimpRGB      bg;
+  GimpColormapEditor *editor           = GIMP_COLORMAP_EDITOR (data);
+  GimpImage          *image            = action_data_get_image (data);
+  GimpContext        *context          = action_data_get_context (data);
+  gboolean            indexed          = FALSE;
+  gboolean            drawable_indexed = FALSE;
+  gint                num_colors       = 0;
 
   if (image)
     {
@@ -128,7 +135,7 @@ colormap_actions_update (GimpActionGroup *group,
         {
           GList *drawables = gimp_image_get_selected_drawables (image);
 
-          num_colors = gimp_image_get_colormap_size (image);
+          num_colors = gimp_palette_get_n_colors (gimp_image_get_colormap_palette ((image)));
 
           if (g_list_length (drawables) == 1)
             drawable_indexed = gimp_drawable_is_indexed (drawables->data);
@@ -137,12 +144,6 @@ colormap_actions_update (GimpActionGroup *group,
         }
     }
 
-  if (context)
-    {
-      gimp_context_get_foreground (context, &fg);
-      gimp_context_get_background (context, &bg);
-    }
-
 #define SET_SENSITIVE(action,condition) \
         gimp_action_group_set_action_sensitive (group, action, (condition) != 0, NULL)
 #define SET_COLOR(action,color) \
@@ -150,14 +151,17 @@ colormap_actions_update (GimpActionGroup *group,
 
   SET_SENSITIVE ("colormap-edit-color",
                  indexed && num_colors > 0);
+  SET_SENSITIVE ("colormap-delete-color",
+                 indexed && num_colors > 0 &&
+                 gimp_colormap_editor_is_color_deletable (editor));
 
   SET_SENSITIVE ("colormap-add-color-from-fg",
                  indexed && num_colors < 256);
   SET_SENSITIVE ("colormap-add-color-from-bg",
                  indexed && num_colors < 256);
 
-  SET_COLOR ("colormap-add-color-from-fg", context ? &fg : NULL);
-  SET_COLOR ("colormap-add-color-from-bg", context ? &bg : NULL);
+  SET_COLOR ("colormap-add-color-from-fg", context ? gimp_context_get_foreground (context) : NULL);
+  SET_COLOR ("colormap-add-color-from-bg", context ? gimp_context_get_background (context) : NULL);
 
   SET_SENSITIVE ("colormap-selection-replace",
                  drawable_indexed && num_colors > 0);
