@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __DIALOGS_ACTIONS_H__
@@ -21,16 +21,18 @@
 
 /*  this check is needed for the extern declaration below to be correct  */
 #ifndef __GIMP_ACTION_GROUP_H__
-#error "widgets/gimpactiongroup.h must be included prior to dialog-actions.h"
+#error "widgets/gimpactiongroup.h must be included prior to dialogs-actions.h"
 #endif
 
 extern const GimpStringActionEntry dialogs_dockable_actions[];
 extern gint                        n_dialogs_dockable_actions;
 
 
-void   dialogs_actions_setup  (GimpActionGroup *group);
-void   dialogs_actions_update (GimpActionGroup *group,
-                               gpointer         data);
+void       dialogs_actions_setup          (GimpActionGroup *group);
+void       dialogs_actions_update         (GimpActionGroup *group,
+                                           gpointer         data);
+
+gboolean   dialogs_actions_toolbox_exists (Gimp            *gimp);
 
 
 #endif /* __DIALOGS_ACTIONS_H__ */
