@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -30,15 +30,16 @@
 #include "core/gimp.h"
 #include "core/gimpcontainer.h"
 #include "core/gimpimage.h"
+#include "core/gimpimagefile.h"
 #include "core/gimpviewable.h"
 
-#include "file/file-utils.h"
 #include "file/gimp-file.h"
 
 #include "plug-in/gimppluginmanager-file.h"
 
 #include "widgets/gimpaction.h"
 #include "widgets/gimpactiongroup.h"
+#include "widgets/gimpactionimpl.h"
 #include "widgets/gimphelp-ids.h"
 
 #include "display/gimpdisplay.h"
@@ -63,102 +64,111 @@ static void    file_actions_close_all_update    (GimpContainer   *images,
                                                  GimpObject      *unused,
                                                  GimpActionGroup *group);
 static gchar * file_actions_create_label        (const gchar     *format,
-                                                 const gchar     *uri);
+                                                 GFile           *file);
 
 
 static const GimpActionEntry file_actions[] =
 {
-  { "file-menu",             NULL, NC_("file-action", "_File")        },
-  { "file-create-menu",      NULL, NC_("file-action", "Crea_te")      },
-  { "file-open-recent-menu", NULL, NC_("file-action", "Open _Recent") },
-
-  { "file-open", GTK_STOCK_OPEN,
-    NC_("file-action", "_Open..."), NULL,
+  { "file-open", GIMP_ICON_IMAGE_OPEN,
+    NC_("file-action", "_Open..."), NULL, { "<primary>O", NULL },
     NC_("file-action", "Open an image file"),
-    G_CALLBACK (file_open_cmd_callback),
+    file_open_cmd_callback,
     GIMP_HELP_FILE_OPEN },
 
-  { "file-open-as-layers", GIMP_STOCK_LAYER,
-    NC_("file-action", "Op_en as Layers..."), "<primary><alt>O",
+  { "file-open-as-layers", GIMP_ICON_LAYER,
+    NC_("file-action", "Op_en as Layers..."), NULL, { "<primary><alt>O", NULL },
     NC_("file-action", "Open an image file as layers"),
-    G_CALLBACK (file_open_as_layers_cmd_callback),
+    file_open_as_layers_cmd_callback,
     GIMP_HELP_FILE_OPEN_AS_LAYER },
 
-  { "file-open-location", GIMP_STOCK_WEB,
-    NC_("file-action", "Open _Location..."), NULL,
+  { "file-open-location", GIMP_ICON_WEB,
+    NC_("file-action", "Open _Location..."), NULL, { NULL },
     NC_("file-action", "Open an image file from a specified location"),
-    G_CALLBACK (file_open_location_cmd_callback),
+    file_open_location_cmd_callback,
     GIMP_HELP_FILE_OPEN_LOCATION },
 
   { "file-create-template", NULL,
-    NC_("file-action", "Create Template..."), NULL,
+    NC_("file-action", "Create _Template..."), NULL, { NULL },
     NC_("file-action", "Create a new template from this image"),
-    G_CALLBACK (file_create_template_cmd_callback),
+    file_create_template_cmd_callback,
     GIMP_HELP_FILE_CREATE_TEMPLATE },
 
-  { "file-revert", GTK_STOCK_REVERT_TO_SAVED,
-    NC_("file-action", "Re_vert"), NULL,
+  { "file-revert", GIMP_ICON_IMAGE_RELOAD,
+    NC_("file-action", "Re_vert"), NULL, { NULL },
     NC_("file-action", "Reload the image file from disk"),
-    G_CALLBACK (file_revert_cmd_callback),
+    file_revert_cmd_callback,
     GIMP_HELP_FILE_REVERT },
 
-  { "file-close-all", GTK_STOCK_CLOSE,
-    NC_("file-action", "Close all"), "<primary><shift>W",
+  { "file-close-all", GIMP_ICON_CLOSE_ALL,
+    NC_("file-action", "C_lose All"), NULL, { "<primary><shift>W", NULL },
     NC_("file-action", "Close all opened images"),
-    G_CALLBACK (file_close_all_cmd_callback),
+    file_close_all_cmd_callback,
     GIMP_HELP_FILE_CLOSE_ALL },
 
-  { "file-quit", GTK_STOCK_QUIT,
-    NC_("file-action", "_Quit"), "<primary>Q",
+  { "file-copy-location", GIMP_ICON_EDIT_COPY,
+    NC_("file-action", "Copy _Image Location"), NULL, { NULL },
+    NC_("file-action", "Copy image file location to clipboard"),
+    file_copy_location_cmd_callback,
+    GIMP_HELP_FILE_COPY_LOCATION },
+
+  { "file-show-in-file-manager", GIMP_ICON_FILE_MANAGER,
+    NC_("file-action", "Show in _File Manager"), NULL, { "<primary><alt>F", NULL },
+    NC_("file-action", "Show image file location in the file manager"),
+    file_show_in_file_manager_cmd_callback,
+    GIMP_HELP_FILE_SHOW_IN_FILE_MANAGER },
+
+  { "file-quit", GIMP_ICON_APPLICATION_EXIT,
+    NC_("file-action", "_Quit"), NULL, { "<primary>Q", NULL },
     NC_("file-action", "Quit the GNU Image Manipulation Program"),
-    G_CALLBACK (file_quit_cmd_callback),
+    file_quit_cmd_callback,
     GIMP_HELP_FILE_QUIT }
 };
 
 static const GimpEnumActionEntry file_save_actions[] =
 {
-  { "file-save", GTK_STOCK_SAVE,
-    NC_("file-action", "_Save"), "<primary>S",
+  { "file-save", GIMP_ICON_DOCUMENT_SAVE,
+    NC_("file-action", "_Save"), NULL, { "<primary>S", NULL },
     NC_("file-action", "Save this image"),
     GIMP_SAVE_MODE_SAVE, FALSE,
     GIMP_HELP_FILE_SAVE },
 
-  { "file-save-as", GTK_STOCK_SAVE_AS,
-    NC_("file-action", "Save _As..."), "<primary><shift>S",
+  { "file-save-as", GIMP_ICON_DOCUMENT_SAVE_AS,
+    NC_("file-action", "Save _As..."), NULL, { "<primary><shift>S", NULL },
     NC_("file-action", "Save this image with a different name"),
     GIMP_SAVE_MODE_SAVE_AS, FALSE,
     GIMP_HELP_FILE_SAVE_AS },
 
   { "file-save-a-copy", NULL,
-    NC_("file-action", "Save a Cop_y..."), NULL,
+    NC_("file-action", "Save a Cop_y..."), NULL, { NULL },
     NC_("file-action",
-        "Save a copy of this image, without affecting the source file (if any) or the current state of the image"),
+        "Save a copy of this image, without affecting the source file "
+        "(if any) or the current state of the image"),
     GIMP_SAVE_MODE_SAVE_A_COPY, FALSE,
     GIMP_HELP_FILE_SAVE_A_COPY },
 
   { "file-save-and-close", NULL,
-    NC_("file-action", "Save and Close..."), NULL,
+    NC_("file-action", "Save and Close..."), NULL, { NULL },
     NC_("file-action", "Save this image and close its window"),
     GIMP_SAVE_MODE_SAVE_AND_CLOSE, FALSE,
     GIMP_HELP_FILE_SAVE },
 
-  { "file-export-to", NULL,
-    NC_("file-action", "Export"), "<primary>E",
-    NC_("file-action", "Export the image again"),
-    GIMP_SAVE_MODE_EXPORT_TO, FALSE,
-    GIMP_HELP_FILE_EXPORT_TO },
+  { "file-export", NULL,
+    NC_("file-action", "E_xport..."), NULL, { "<primary>E", NULL },
+    NC_("file-action", "Export the image"),
+    GIMP_SAVE_MODE_EXPORT, FALSE,
+    GIMP_HELP_FILE_EXPORT },
 
   { "file-overwrite", NULL,
-    NC_("file-action", "Over_write"), "",
+    NC_("file-action", "Over_write"), NULL, { NULL },
     NC_("file-action", "Export the image back to the imported file in the import format"),
     GIMP_SAVE_MODE_OVERWRITE, FALSE,
     GIMP_HELP_FILE_OVERWRITE },
 
-  { "file-export", NULL,
-    NC_("file-action", "Export As..."), "<primary><shift>E",
+  { "file-export-as", NULL,
+    NC_("file-action", "E_xport As..."), NULL, { "<primary><shift>E", NULL },
     NC_("file-action", "Export the image to various file formats such as PNG or JPEG"),
-    GIMP_SAVE_MODE_EXPORT, FALSE,
-    GIMP_HELP_FILE_EXPORT }
+    GIMP_SAVE_MODE_EXPORT_AS, FALSE,
+    GIMP_HELP_FILE_EXPORT_AS }
 };
 
 void
@@ -175,7 +185,7 @@ file_actions_setup (GimpActionGroup *group)
   gimp_action_group_add_enum_actions (group, "file-action",
                                       file_save_actions,
                                       G_N_ELEMENTS (file_save_actions),
-                                      G_CALLBACK (file_save_cmd_callback));
+                                      file_save_cmd_callback);
 
   n_entries = GIMP_GUI_CONFIG (group->gimp->config)->last_opened_size;
 
@@ -185,40 +195,30 @@ file_actions_setup (GimpActionGroup *group)
     {
       entries[i].name           = g_strdup_printf ("file-open-recent-%02d",
                                                    i + 1);
-      entries[i].stock_id       = GTK_STOCK_OPEN;
+      entries[i].icon_name      = GIMP_ICON_DOCUMENT_OPEN,
       entries[i].label          = entries[i].name;
       entries[i].tooltip        = NULL;
       entries[i].value          = i;
       entries[i].value_variable = FALSE;
 
       if (i < 9)
-        entries[i].accelerator = g_strdup_printf ("<primary>%d", i + 1);
+        entries[i].accelerator[0] = g_strdup_printf ("<primary>%d", i + 1);
       else if (i == 9)
-        entries[i].accelerator = "<primary>0";
+        entries[i].accelerator[0] = g_strdup ("<primary>0");
       else
-        entries[i].accelerator = "";
+        entries[i].accelerator[0] = NULL;
     }
 
   gimp_action_group_add_enum_actions (group, NULL, entries, n_entries,
-                                      G_CALLBACK (file_open_recent_cmd_callback));
+                                      file_open_recent_cmd_callback);
 
   for (i = 0; i < n_entries; i++)
     {
-      GtkAction *action;
-
       gimp_action_group_set_action_visible (group, entries[i].name, FALSE);
-      gimp_action_group_set_action_always_show_image (group, entries[i].name,
-                                                      TRUE);
-
-      action = gtk_action_group_get_action (GTK_ACTION_GROUP (group),
-                                            entries[i].name);
-      g_object_set (action,
-                    "context", gimp_get_user_context (group->gimp),
-                    NULL);
 
       g_free ((gchar *) entries[i].name);
-      if (i < 9)
-        g_free ((gchar *) entries[i].accelerator);
+      if (entries[i].accelerator[0])
+        g_free ((gchar *) entries[i].accelerator[0]);
     }
 
   g_free (entries);
@@ -254,62 +254,104 @@ file_actions_update (GimpActionGroup *group,
 {
   Gimp         *gimp           = action_data_get_gimp (data);
   GimpImage    *image          = action_data_get_image (data);
-  GimpDrawable *drawable       = NULL;
-  const gchar  *source         = NULL;
-  const gchar  *export         = NULL;
+  GList        *drawables      = NULL;
+  GFile        *file           = NULL;
+  GFile        *source         = NULL;
+  GFile        *export         = NULL;
   gboolean      show_overwrite = FALSE;
 
   if (image)
     {
-      drawable = gimp_image_get_active_drawable (image);
-      source   = gimp_image_get_imported_uri (image);
-      export   = gimp_image_get_exported_uri (image);
+      drawables = gimp_image_get_selected_drawables (image);
+
+      file   = gimp_image_get_file (image);
+      source = gimp_image_get_imported_file (image);
+      export = gimp_image_get_exported_file (image);
     }
 
   show_overwrite =
     (source &&
-     gimp_plug_in_manager_uri_has_exporter (gimp->plug_in_manager,
-                                            source));
+     gimp_plug_in_manager_file_procedure_find (gimp->plug_in_manager,
+                                               GIMP_FILE_PROCEDURE_GROUP_EXPORT,
+                                               source, NULL));
 
 #define SET_VISIBLE(action,condition) \
         gimp_action_group_set_action_visible (group, action, (condition) != 0)
 #define SET_SENSITIVE(action,condition) \
-        gimp_action_group_set_action_sensitive (group, action, (condition) != 0)
+        gimp_action_group_set_action_sensitive (group, action, (condition) != 0, NULL)
 
-  SET_SENSITIVE ("file-save",            drawable);
-  SET_SENSITIVE ("file-save-as",         drawable);
-  SET_SENSITIVE ("file-save-a-copy",     drawable);
-  SET_SENSITIVE ("file-revert",          image && (gimp_image_get_uri (image) || source));
-  SET_SENSITIVE ("file-export-to",       drawable);
-  SET_VISIBLE   ("file-export-to",       ! show_overwrite);
-  SET_SENSITIVE ("file-overwrite",       show_overwrite);
-  SET_VISIBLE   ("file-overwrite",       show_overwrite);
-  SET_SENSITIVE ("file-export",          drawable);
-  SET_SENSITIVE ("file-create-template", image);
+  SET_SENSITIVE ("file-save",                 drawables);
+  SET_SENSITIVE ("file-save-as",              drawables);
+  SET_SENSITIVE ("file-save-a-copy",          drawables);
+  SET_SENSITIVE ("file-save-and-close",       drawables);
+  SET_SENSITIVE ("file-revert",               file || source);
+  SET_SENSITIVE ("file-export",               drawables);
+  SET_VISIBLE   ("file-export",               ! show_overwrite);
+  SET_SENSITIVE ("file-overwrite",            show_overwrite);
+  SET_VISIBLE   ("file-overwrite",            show_overwrite);
+  SET_SENSITIVE ("file-export-as",            drawables);
+  SET_SENSITIVE ("file-create-template",      image);
+  SET_SENSITIVE ("file-copy-location",        file || source || export);
+  SET_SENSITIVE ("file-show-in-file-manager", file || source || export);
 
-  if (export)
+  if (file)
     {
-      gchar *label = file_actions_create_label (_("Export to %s"), export);
-      gimp_action_group_set_action_label (group, "file-export-to", label);
-      g_free (label);
-    }
-  else if (show_overwrite)
-    {
-      gchar *label = file_actions_create_label (_("Over_write %s"), source);
-      gimp_action_group_set_action_label (group, "file-overwrite", label);
-      g_free (label);
-
+      gimp_action_group_set_action_label (group,
+                                          "file-save",
+                                          C_("file-action", "_Save"));
     }
   else
     {
       gimp_action_group_set_action_label (group,
-                                          "file-export-to", _("Export"));
+                                          "file-save",
+                                          C_("file-action", "_Save..."));
+    }
+
+  if (export)
+    {
+      GimpAction *action;
+      gchar      *label = file_actions_create_label (_("Export to %s"),
+                                                     export);
+
+      gimp_action_group_set_action_label (group, "file-export", label);
+
+      action = gimp_action_group_get_action (group, "file-export");
+      g_object_set (action,
+                    "ellipsize",       PANGO_ELLIPSIZE_MIDDLE,
+                    "max-width-chars", 40,
+                    NULL);
+
+      g_free (label);
+    }
+  else if (show_overwrite)
+    {
+      GimpAction *action;
+      gchar      *label = file_actions_create_label (_("Over_write %s"),
+                                                     source);
+
+      gimp_action_group_set_action_label (group, "file-overwrite", label);
+
+      action = gimp_action_group_get_action (group, "file-overwrite");
+      g_object_set (action,
+                    "ellipsize",       PANGO_ELLIPSIZE_MIDDLE,
+                    "max-width-chars", 40,
+                    NULL);
+
+      g_free (label);
+    }
+  else
+    {
+      gimp_action_group_set_action_label (group,
+                                          "file-export",
+                                          C_("file-action", "E_xport..."));
     }
 
   /*  needed for the empty display  */
   SET_SENSITIVE ("file-close-all", image);
 
 #undef SET_SENSITIVE
+
+  g_list_free (drawables);
 }
 
 
@@ -328,50 +370,59 @@ file_actions_last_opened_update (GimpContainer   *container,
 
   for (i = 0; i < n; i++)
     {
-      GtkAction *action;
-      gchar     *name = g_strdup_printf ("file-open-recent-%02d", i + 1);
+      GimpAction *action;
+      gchar      *name = g_strdup_printf ("file-open-recent-%02d", i + 1);
 
-      action = gtk_action_group_get_action (GTK_ACTION_GROUP (group), name);
+      action = gimp_action_group_get_action (group, name);
 
       if (i < num_documents)
         {
           GimpImagefile *imagefile = (GimpImagefile *)
             gimp_container_get_child_by_index (container, i);
 
-          if (GIMP_ACTION (action)->viewable != (GimpViewable *) imagefile)
+          if (gimp_action_get_viewable (action) != (GimpViewable *) imagefile)
             {
-              const gchar *uri;
-              gchar       *filename;
+              GFile       *file;
+              const gchar *name;
               gchar       *basename;
               gchar       *escaped;
+              gchar       *label;
 
-              uri = gimp_object_get_name (imagefile);
+              file = gimp_imagefile_get_file (imagefile);
 
-              filename = file_utils_uri_display_name (uri);
-              basename = file_utils_uri_display_basename (uri);
+              name     = gimp_file_get_utf8_name (file);
+              basename = g_path_get_basename (name);
 
               escaped = gimp_escape_uline (basename);
-
               g_free (basename);
+              /* TRANSLATORS: the %s will be replaced by the file name of a
+               * recently opened image.
+               */
+              label   = g_strdup_printf (_("Open \"%s\""), escaped);
 
               g_object_set (action,
-                            "label",    escaped,
-                            "tooltip",  filename,
-                            "visible",  TRUE,
-                            "viewable", imagefile,
+                            "label",           label,
+                            "short-label",     escaped,
+                            "tooltip",         name,
+                            "visible",         TRUE,
+                            "viewable",        imagefile,
+                            "ellipsize",       PANGO_ELLIPSIZE_MIDDLE,
+                            "max-width-chars", 40,
                             NULL);
 
-              g_free (filename);
+              g_free (label);
               g_free (escaped);
             }
         }
       else
         {
           g_object_set (action,
-                        "label",    name,
-                        "tooltip",  NULL,
-                        "visible",  FALSE,
-                        "viewable", NULL,
+                        "label",           name,
+                        "tooltip",         NULL,
+                        "visible",         FALSE,
+                        "viewable",        NULL,
+                        "ellipsize",       PANGO_ELLIPSIZE_MIDDLE,
+                        "max-width-chars", 40,
                         NULL);
         }
 
@@ -407,14 +458,14 @@ file_actions_close_all_update (GimpContainer   *images,
         sensitive = FALSE;
     }
 
-  gimp_action_group_set_action_sensitive (group, "file-close-all", sensitive);
+  gimp_action_group_set_action_sensitive (group, "file-close-all", sensitive, NULL);
 }
 
 static gchar *
 file_actions_create_label (const gchar *format,
-                           const gchar *uri)
+                           GFile       *file)
 {
-  gchar *basename         = file_utils_uri_display_basename (uri);
+  gchar *basename         = g_path_get_basename (gimp_file_get_utf8_name (file));
   gchar *escaped_basename = gimp_escape_uline (basename);
   gchar *label            = g_strdup_printf (format, escaped_basename);
 
