@@ -51,12 +51,6 @@ static gchar * filters_parse_operation (Gimp          *gimp,
                                         const gchar   *icon_name,
                                         GimpObject   **settings);
 
-static void    filters_run_procedure   (Gimp          *gimp,
-                                        GimpDisplay   *display,
-                                        GimpProcedure *procedure,
-                                        GimpRunMode    run_mode);
-
-
 /*  public functions  */
 
 void
@@ -86,6 +80,7 @@ filters_apply_cmd_callback (GimpAction *action,
                                        &settings);
 
   procedure = gimp_gegl_procedure_new (image->gimp,
+                                       NULL,
                                        GIMP_RUN_NONINTERACTIVE, settings,
                                        operation,
                                        gimp_action_get_name (action),
@@ -130,6 +125,7 @@ filters_apply_interactive_cmd_callback (GimpAction *action,
     }
 
   procedure = gimp_gegl_procedure_new (image->gimp,
+                                       NULL,
                                        GIMP_RUN_INTERACTIVE, NULL,
                                        g_variant_get_string (value, NULL),
                                        gimp_action_get_name (action),
@@ -240,7 +236,7 @@ filters_parse_operation (Gimp         *gimp,
   return g_strdup (operation_str);
 }
 
-static void
+void
 filters_run_procedure (Gimp          *gimp,
                        GimpDisplay   *display,
                        GimpProcedure *procedure,
@@ -281,7 +277,17 @@ filters_run_procedure (Gimp          *gimp,
                                                     display);
         }
 
-      if (success)
+      if (success &&
+          (! GIMP_IS_GEGL_PROCEDURE (procedure) ||
+           /* XXX GimpGeglProcedure made for filter-editing are
+            * short-lived and should not be added to history. I'm not
+            * sure it makes sense UX-wise anyway ("used" filters are for
+            * first calls, not edits).
+            * If ever we decide to change this UX logic, some code logic
+            * changes are needed too. Cf. crash report #12576 and my
+            * (Jehan's) technical comment in there..
+            */
+           ! gimp_gegl_procedure_is_editing_filter (GIMP_GEGL_PROCEDURE (procedure))))
         gimp_filter_history_add (gimp, procedure);
 
       gimp_value_array_unref (args);
