@@ -303,19 +303,19 @@ static const GimpEnumActionEntry tools_opacity_actions[] =
     GIMP_ACTION_SELECT_LAST, FALSE,
     NULL },
   { "tools-opacity-decrease", GIMP_ICON_DIALOG_TOOL_OPTIONS,
-    NC_("tools-action", "Tool's Opacity: Decrease by 1"), NULL, { "less" }, NULL,
+    NC_("tools-action", "Tool's Opacity: Decrease by 1"), NULL, { "less", NULL }, NULL,
     GIMP_ACTION_SELECT_PREVIOUS, FALSE,
     NULL },
   { "tools-opacity-increase", GIMP_ICON_DIALOG_TOOL_OPTIONS,
-    NC_("tools-action", "Tool's Opacity: Increase by 1"), NULL, { "greater" }, NULL,
+    NC_("tools-action", "Tool's Opacity: Increase by 1"), NULL, { "greater", NULL }, NULL,
     GIMP_ACTION_SELECT_NEXT, FALSE,
     NULL },
   { "tools-opacity-decrease-skip", GIMP_ICON_DIALOG_TOOL_OPTIONS,
-    NC_("tools-action", "Tool's Opacity: Decrease by 10"), NULL, { "<primary>less" }, NULL,
+    NC_("tools-action", "Tool's Opacity: Decrease by 10"), NULL, { "<primary>less", NULL }, NULL,
     GIMP_ACTION_SELECT_SKIP_PREVIOUS, FALSE,
     NULL },
   { "tools-opacity-increase-skip", GIMP_ICON_DIALOG_TOOL_OPTIONS,
-    NC_("tools-action", "Tool's Opacity: Increase by 10"), NULL, { "<primary>greater" }, NULL,
+    NC_("tools-action", "Tool's Opacity: Increase by 10"), NULL, { "<primary>greater", NULL }, NULL,
     GIMP_ACTION_SELECT_SKIP_NEXT, FALSE,
     NULL },
   { "tools-opacity-decrease-percent", GIMP_ICON_DIALOG_TOOL_OPTIONS,
