@@ -44,6 +44,7 @@
 #undef GIMP_DISABLE_DEPRECATED /* for compat enums */
 #include "libgimpbase/gimpbase.h"
 #define GIMP_DISABLE_DEPRECATED
+#include "libgimpbase/gimpbase-private.h"
 #include "libgimpconfig/gimpconfig.h"
 
 #include "core/core-types.h"
@@ -87,23 +88,22 @@ static void       app_init_update_noop       (const gchar        *text1,
                                               gdouble             percentage);
 static void       app_activate_callback      (GimpCoreApp        *app,
                                               gpointer            user_data);
-static void       app_restore_after_callback (Gimp               *gimp,
-                                              GimpInitStatusFunc  status_callback);
 static gboolean   app_exit_after_callback    (Gimp               *gimp,
                                               gboolean            kill_it,
                                               GApplication       *app);
 
+#ifdef G_OS_WIN32
+static BOOL       app_quit_on_ctrl_c         (DWORD               ctrl_type);
+#else
+static void       app_quit_on_ctrl_c         (gint                sig_num);
+#endif
+
 #if 0
 /*  left here as documentation how to do compat enums  */
 GType gimp_convert_dither_type_compat_get_type (void); /* compat cruft */
 #endif
 
 
-/*  local variables  */
-
-static GObject *initial_monitor = NULL;
-
-
 /*  public functions  */
 
 void
@@ -299,14 +299,6 @@ app_run (const gchar         *full_prog_name,
   /*  initialize lowlevel stuff  */
   gimp_gegl_init (gimp);
 
-  /*  Connect our restore_after callback before gui_init() connects
-   *  theirs, so ours runs first and can grab the initial monitor
-   *  before the GUI's restore_after callback resets it.
-   */
-  g_signal_connect_after (gimp, "restore",
-                          G_CALLBACK (app_restore_after_callback),
-                          NULL);
-
   g_signal_connect_after (gimp, "exit",
                           G_CALLBACK (app_exit_after_callback),
                           app);
@@ -359,6 +351,7 @@ app_activate_callback (GimpCoreApp *app,
   GimpInitStatusFunc  update_status_func = NULL;
   const gchar       **filenames;
   const gchar        *current_language;
+  const gchar        *system_lang_l10n   = NULL;
   gchar              *prev_language      = NULL;
   GError             *font_error         = NULL;
   gint                batch_retval;
@@ -369,9 +362,15 @@ app_activate_callback (GimpCoreApp *app,
 
   gimp_core_app_set_exit_status (app, EXIT_SUCCESS);
 
+  /* Language was already initialized. I call this again only to get the
+   * actual language information and the "System Language" string
+   * localized in the actual system language.
+   */
+  current_language = language_init (NULL, &system_lang_l10n);
 #ifndef GIMP_CONSOLE_COMPILATION
   if (! gimp->no_interface)
-    update_status_func = gui_init (gimp, gimp_app_get_no_splash (GIMP_APP (app)), GIMP_APP (app), NULL);
+    update_status_func = gui_init (gimp, gimp_app_get_no_splash (GIMP_APP (app)),
+                                   GIMP_APP (app), NULL, system_lang_l10n);
 #endif
 
   if (! update_status_func)
@@ -385,10 +384,7 @@ app_activate_callback (GimpCoreApp *app,
   g_object_get (gimp->edit_config,
                 "prev-language", &prev_language,
                 NULL);
-  /* Language was already initialized. I call this again only to get the
-   * actual language information.
-   */
-  current_language = language_init (NULL);
+
   gimp->query_all = (prev_language == NULL ||
                      g_strcmp0 (prev_language, current_language) != 0);
   g_free (prev_language);
@@ -401,19 +397,6 @@ app_activate_callback (GimpCoreApp *app,
    */
   gimp_rc_set_autosave (GIMP_RC (gimp->edit_config), TRUE);
 
-  /*  check for updates *after* enabling config autosave, so that the timestamp
-   *  is saved
-   */
-  gimp_update_auto_check (gimp->edit_config, gimp);
-
-  /* Setting properties to be used for the next run.  */
-  g_object_set (gimp->edit_config,
-                /* Set this after gimp_update_auto_check(). */
-                "config-version", GIMP_VERSION,
-                /* Set this after gimp_restore(). */
-                "prev-language",  current_language,
-                NULL);
-
 #ifndef GIMP_CONSOLE_COMPILATION
   if (! gimp->no_interface)
     {
@@ -441,7 +424,7 @@ app_activate_callback (GimpCoreApp *app,
                                               NULL,
                                               file,
                                               gimp_core_app_get_as_new (app),
-                                              initial_monitor,
+                                              NULL,
                                               &status, &error);
               if (image)
                 {
@@ -471,6 +454,19 @@ app_activate_callback (GimpCoreApp *app,
     }
 #endif
 
+  /*  check for updates *after* enabling config autosave, so that the timestamp
+   *  is saved
+   */
+  gimp_update_auto_check (gimp->edit_config, gimp);
+
+  /* Setting properties to be used for the next run.  */
+  g_object_set (gimp->edit_config,
+                /* Set this after gimp_update_auto_check(). */
+                "config-version", GIMP_VERSION,
+                /* Set this after gimp_restore(). */
+                "prev-language",  current_language,
+                NULL);
+
   /*  Load the images given on the command-line. */
   filenames = gimp_core_app_get_filenames (app);
   if (filenames != NULL)
@@ -483,7 +479,7 @@ app_activate_callback (GimpCoreApp *app,
 
           file_open_from_command_line (gimp, file,
                                        gimp_core_app_get_as_new (app),
-                                       initial_monitor);
+                                       NULL);
 
           g_object_unref (file);
         }
@@ -518,20 +514,29 @@ app_activate_callback (GimpCoreApp *app,
       */
       gimp_exit (gimp, TRUE);
     }
-}
-
-static void
-app_restore_after_callback (Gimp               *gimp,
-                            GimpInitStatusFunc  status_callback)
-{
-  gint dummy;
-
-  /*  Getting the display name for a -1 display returns the initial
-   *  monitor during startup. Need to call this from a restore_after
-   *  callback, because before restore(), the GUI can't return anything,
-   *  after after restore() the initial monitor gets reset.
-   */
-  g_free (gimp_get_display_name (gimp, -1, &initial_monitor, &dummy));
+  else
+#ifndef GIMP_CONSOLE_COMPILATION
+    if (gimp->no_interface)
+#endif
+    {
+      /* In console version or GUI version with no interface, we keep
+       * running when --quit was not set. For instance, there could be
+       * an always-ON plug-in (GIMP_PDB_PROC_TYPE_PERSISTENT) which is
+       * set up to receive commands for GIMP.
+       */
+#ifdef G_OS_WIN32
+      SetConsoleCtrlHandler ((PHANDLER_ROUTINE) app_quit_on_ctrl_c, TRUE);
+#else
+      gimp_signal_private (SIGINT, app_quit_on_ctrl_c, 0);
+#endif
+      g_printf ("\n== %s ==\n%s\n\n%s\n",
+                /* TODO: localize when string freeze is over. */
+                "INFO",
+                "GIMP is now running as a background process. "
+                "You can quit anytime with Ctrl-C (SIGINT).",
+                "If you wanted to quit immediately instead, call GIMP with --quit.");
+      g_application_hold (G_APPLICATION (app));
+    }
 }
 
 static gboolean
@@ -542,28 +547,40 @@ app_exit_after_callback (Gimp         *gimp,
   if (gimp->be_verbose)
     g_print ("EXIT: %s\n", G_STRFUNC);
 
-  /*
-   *  In stable releases, we simply call exit() here. This speeds up
-   *  the process of quitting GIMP and also works around the problem
-   *  that plug-ins might still be running.
-   *
-   *  In unstable releases, we shut down GIMP properly in an attempt
-   *  to catch possible problems in our finalizers.
-   */
-
-#ifdef GIMP_UNSTABLE
-
   g_application_quit (G_APPLICATION (app));
 
-#else
-
-  gimp_gegl_exit (gimp);
-
-  gegl_exit ();
-
-  exit (gimp_core_app_get_exit_status (GIMP_CORE_APP (app)));
-
-#endif
-
   return FALSE;
 }
+
+#ifdef G_OS_WIN32
+static BOOL
+app_quit_on_ctrl_c (DWORD ctrl_type)
+{
+  gboolean handled = FALSE;
+
+  if (ctrl_type == CTRL_C_EVENT)
+    {
+      GApplication *app = g_application_get_default ();
+      Gimp         *gimp;
+
+      g_application_release (app);
+      gimp = gimp_core_app_get_gimp (GIMP_CORE_APP (app));
+      gimp_exit (gimp, TRUE);
+
+      handled = TRUE;
+    }
+
+  return handled;
+}
+#else
+static void
+app_quit_on_ctrl_c (gint sig_num)
+{
+  GApplication *app = g_application_get_default ();
+  Gimp         *gimp;
+
+  g_application_release (app);
+  gimp = gimp_core_app_get_gimp (GIMP_CORE_APP (app));
+  gimp_exit (gimp, TRUE);
+}
+#endif
