@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gio/gio.h>
 
 #include "libgimpbase/gimpbase.h"
@@ -50,7 +51,6 @@ enum
   PROP_MOVE_TOOL_CHANGES_ACTIVE,
   PROP_FILTER_TOOL_MAX_RECENT,
   PROP_FILTER_TOOL_USE_LAST_SETTINGS,
-  PROP_FILTER_TOOL_SHOW_COLOR_OPTIONS,
   PROP_TRUST_DIRTY_FLAG,
   PROP_SAVE_DEVICE_STATUS,
   PROP_DEVICES_SHARE_TOOL,
@@ -70,12 +70,13 @@ enum
   PROP_TOOLBOX_GROUPS,
   PROP_THEME_PATH,
   PROP_THEME,
-  PROP_PREFER_DARK_THEME,
+  PROP_THEME_SCHEME,
   PROP_OVERRIDE_THEME_ICON_SIZE,
   PROP_CUSTOM_ICON_SIZE,
   PROP_ICON_THEME_PATH,
   PROP_ICON_THEME,
   PROP_PREFER_SYMBOLIC_ICONS,
+  PROP_FONT_RELATIVE_SIZE,
   PROP_USE_HELP,
   PROP_SHOW_HELP_BUTTON,
   PROP_HELP_LOCALES,
@@ -96,6 +97,7 @@ enum
   PROP_SHOW_TABS,
   PROP_TABS_POSITION,
   PROP_LAST_TIP_SHOWN,
+  PROP_SHOW_WELCOME_DIALOG,
 
   /* ignored, only for backward compatibility: */
   PROP_CURSOR_FORMAT,
@@ -162,13 +164,6 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_FILTER_TOOL_SHOW_COLOR_OPTIONS,
-                            "filter-tool-show-color-options",
-                            "Show advanced color options in filters",
-                            FILTER_TOOL_SHOW_COLOR_OPTIONS_BLURB,
-                            FALSE,
-                            GIMP_PARAM_STATIC_STRINGS);
-
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TRUST_DIRTY_FLAG,
                             "trust-dirty-flag",
                             "Trust dirty flag",
@@ -243,7 +238,7 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_RESTORE_ACCELS,
                             "restore-accels",
-                            "Restore acclerator",
+                            "Restore accelerator",
                             RESTORE_ACCELS_BLURB,
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
@@ -314,11 +309,12 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                             THEME_BLURB,
                             GIMP_CONFIG_DEFAULT_THEME,
                             GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_PREFER_DARK_THEME,
-                            "prefer-dark-theme",
-                            "Prefer Dark Theme",
-                            THEME_BLURB,
-                            TRUE,
+  GIMP_CONFIG_PROP_ENUM    (object_class, PROP_THEME_SCHEME,
+                            "theme-color-scheme",
+                            "Theme's Color Scheme",
+                            THEME_SCHEME_BLURB,
+                            GIMP_TYPE_THEME_SCHEME,
+                            GIMP_THEME_DARK,
                             GIMP_PARAM_STATIC_STRINGS);
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_OVERRIDE_THEME_ICON_SIZE,
                             "override-theme-icon-size",
@@ -358,6 +354,13 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_FONT_RELATIVE_SIZE,
+                           "font-relative-size",
+                           "Tweak font-size from the theme",
+                           FONT_SIZE_BLURB,
+                           0.5, 2.0, 1.0,
+                           GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_HELP,
                             "use-help",
                             "Use help",
@@ -502,6 +505,14 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                                                      G_PARAM_CONSTRUCT |
                                                      GIMP_PARAM_STATIC_STRINGS));
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SHOW_WELCOME_DIALOG,
+                            "show-welcome-dialog",
+                            "Show Welcome Dialog each time",
+                            NULL, TRUE,
+                            G_PARAM_READWRITE |
+                            G_PARAM_CONSTRUCT |
+                            GIMP_PARAM_STATIC_STRINGS);
+
   /*  only for backward compatibility:  */
   GIMP_CONFIG_PROP_ENUM (object_class, PROP_CURSOR_FORMAT,
                          "cursor-format",
@@ -613,9 +624,6 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_FILTER_TOOL_USE_LAST_SETTINGS:
       gui_config->filter_tool_use_last_settings = g_value_get_boolean (value);
       break;
-    case PROP_FILTER_TOOL_SHOW_COLOR_OPTIONS:
-      gui_config->filter_tool_show_color_options = g_value_get_boolean (value);
-      break;
     case PROP_TRUST_DIRTY_FLAG:
       gui_config->trust_dirty_flag = g_value_get_boolean (value);
       break;
@@ -675,8 +683,8 @@ gimp_gui_config_set_property (GObject      *object,
       g_free (gui_config->theme);
       gui_config->theme = g_value_dup_string (value);
       break;
-    case PROP_PREFER_DARK_THEME:
-      gui_config->prefer_dark_theme = g_value_get_boolean (value);
+    case PROP_THEME_SCHEME:
+      gui_config->theme_scheme = g_value_get_enum (value);
       break;
     case PROP_OVERRIDE_THEME_ICON_SIZE:
       gui_config->override_icon_size = g_value_get_boolean (value);
@@ -695,6 +703,9 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_PREFER_SYMBOLIC_ICONS:
       gui_config->prefer_symbolic_icons = g_value_get_boolean (value);
       break;
+    case PROP_FONT_RELATIVE_SIZE:
+      gui_config->font_relative_size = g_value_get_double (value);
+      break;
     case PROP_USE_HELP:
       gui_config->use_help = g_value_get_boolean (value);
       break;
@@ -753,6 +764,9 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_LAST_TIP_SHOWN:
       gui_config->last_tip_shown = g_value_get_int (value);
       break;
+    case PROP_SHOW_WELCOME_DIALOG:
+      gui_config->show_welcome_dialog = g_value_get_boolean (value);
+      break;
 
     case PROP_CURSOR_FORMAT:
     case PROP_INFO_WINDOW_PER_DISPLAY:
@@ -794,9 +808,6 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_FILTER_TOOL_USE_LAST_SETTINGS:
       g_value_set_boolean (value, gui_config->filter_tool_use_last_settings);
       break;
-    case PROP_FILTER_TOOL_SHOW_COLOR_OPTIONS:
-      g_value_set_boolean (value, gui_config->filter_tool_show_color_options);
-      break;
     case PROP_TRUST_DIRTY_FLAG:
       g_value_set_boolean (value, gui_config->trust_dirty_flag);
       break;
@@ -854,8 +865,8 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_THEME:
       g_value_set_string (value, gui_config->theme);
       break;
-    case PROP_PREFER_DARK_THEME:
-      g_value_set_boolean (value, gui_config->prefer_dark_theme);
+    case PROP_THEME_SCHEME:
+      g_value_set_enum (value, gui_config->theme_scheme);
       break;
     case PROP_OVERRIDE_THEME_ICON_SIZE:
       g_value_set_boolean (value, gui_config->override_icon_size);
@@ -872,6 +883,9 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_PREFER_SYMBOLIC_ICONS:
       g_value_set_boolean (value, gui_config->prefer_symbolic_icons);
       break;
+    case PROP_FONT_RELATIVE_SIZE:
+      g_value_set_double (value, gui_config->font_relative_size);
+      break;
     case PROP_USE_HELP:
       g_value_set_boolean (value, gui_config->use_help);
       break;
@@ -928,6 +942,9 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_LAST_TIP_SHOWN:
       g_value_set_int (value, gui_config->last_tip_shown);
       break;
+    case PROP_SHOW_WELCOME_DIALOG:
+      g_value_set_boolean (value, gui_config->show_welcome_dialog);
+      break;
 
     case PROP_CURSOR_FORMAT:
     case PROP_INFO_WINDOW_PER_DISPLAY:
