@@ -50,7 +50,7 @@ _("Timestamp of the last update check.")
 "Defines the color management behavior."
 
 #define COLOR_PROFILE_POLICY_BLURB \
-_("How to handle embedded color profiles when opening a file.")
+_("What to do when opening a file with an embedded ICC color profile.")
 
 #define COLOR_PROFILE_PATH_BLURB \
 _("Sets the default folder path for all color profile file dialogs.")
@@ -124,6 +124,12 @@ _("Snap to the canvas edges by default in new image windows.")
 #define SNAP_TO_PATH_BLURB \
 _("Snap to the active path by default in new image windows.")
 
+#define SNAP_TO_BBOX_BLURB \
+_("Snap to the layers bounding boxes by default in new image windows.")
+
+#define SNAP_TO_EQUIDISTANCE_BLURB \
+_("Snap to the equidistance between three bounding boxes by default in new image windows.")
+
 #define DEFAULT_THRESHOLD_BLURB \
 _("Tools such as fuzzy-select and bucket fill find regions based on a " \
   "seed-fill algorithm.  The seed fill starts at the initially selected " \
@@ -175,6 +181,9 @@ _("When enabled, the selected pattern will be used for all tools.")
 #define GLOBAL_PALETTE_BLURB \
 "When enabled, the selected palette will be used for all tools."
 
+#define GLOBAL_EXPAND_BLURB \
+_("When enabled, the selected auto expand layer settings will be used for all tools.")
+
 #define GRADIENT_PATH_BLURB \
 "Sets the gradient search path."
 
@@ -325,7 +334,7 @@ _("When enabled, non-visible layers can be edited as normal.")
 
 #define MOVE_TOOL_CHANGES_ACTIVE_BLURB \
 _("If enabled, the move tool sets the edited layer or path as active.  " \
-  "This used to be the default behaviour in older versions.")
+  "This used to be the default behavior in older versions.")
 
 #define MYPAINT_BRUSH_PATH_BLURB \
 "Sets the brush search path."
@@ -537,11 +546,14 @@ _("Sets the folder for temporary storage. Files will appear here " \
 #define THEME_BLURB \
 _("The name of the theme to use.")
 
+#define THEME_SCHEME_BLURB \
+_("Chooses the color scheme variant of the theme.")
+
 #define THEME_PATH_BLURB \
 "Sets the theme search path."
 
 #define ICON_THEME_BLURB \
-"The name of the icon theme to use."
+_("The name of the icon theme to use.")
 
 #define OVERRIDE_THEME_ICON_SIZE_BLURB \
 _("Override theme-set icon sizes.")
@@ -555,6 +567,9 @@ _("When enabled, symbolic icons will be preferred if available.")
 #define ICON_THEME_PATH_BLURB \
 "Sets the icon theme search path."
 
+#define FONT_SIZE_BLURB \
+_("Tweak font size of the graphical interface.")
+
 #define IMAGE_CONVERT_PROFILE_INTENT_BLURB \
 _("Sets the default rendering intent for the 'Convert to Color Profile' dialog.")
 
@@ -770,7 +785,7 @@ _("When enabled, uses OpenCL for some operations.")
 "'user-manual-online' is enabled."
 
 #define ZOOM_QUALITY_BLURB \
-"There's a tradeoff between speed and quality of the zoomed-out display."
+_("There's a tradeoff between speed and quality of the zoomed-out display.")
 
 #define DEFAULT_USE_EVENT_HISTORY_BLURB \
 "Bugs in event history buffer are frequent so in case of cursor " \
