@@ -71,10 +71,12 @@ struct _GimpUserInstall
 
   gint                    scale_factor;
 
-  const gchar            *migrate;
+  gboolean                migrate;
 
   GimpUserInstallLogFunc  log;
   gpointer                log_data;
+
+  GHashTable             *accels;
 };
 
 typedef enum
@@ -118,7 +120,8 @@ gimp_user_install_items[] =
 
 static gboolean  user_install_detect_old         (GimpUserInstall    *install,
                                                   const gchar        *gimp_dir);
-static gchar *   user_install_old_style_gimpdir  (void);
+static gchar   * user_install_old_style_gimpdir  (void);
+static gchar   * user_install_flatpak_gimpdir    (gint                minor);
 
 static void      user_install_log                (GimpUserInstall    *install,
                                                   const gchar        *format,
@@ -135,13 +138,15 @@ static gboolean  user_install_file_copy          (GimpUserInstall    *install,
                                                   const gchar        *source,
                                                   const gchar        *dest,
                                                   const gchar        *old_options_regexp,
-                                                  GRegexEvalCallback  update_callback);
+                                                  GRegexEvalCallback  update_callback,
+                                                  GimpCopyPostProcess post_process_callback);
 static gboolean  user_install_dir_copy           (GimpUserInstall    *install,
                                                   gint                level,
                                                   const gchar        *source,
                                                   const gchar        *base,
                                                   const gchar        *update_pattern,
-                                                  GRegexEvalCallback  update_callback);
+                                                  GRegexEvalCallback  update_callback,
+                                                  GimpCopyPostProcess post_process_callback);
 
 static gboolean  user_install_create_files       (GimpUserInstall    *install);
 static gboolean  user_install_migrate_files      (GimpUserInstall    *install);
@@ -157,6 +162,7 @@ gimp_user_install_new (GObject  *gimp,
 
   install->gimp    = gimp;
   install->verbose = verbose;
+  install->accels  = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
   user_install_detect_old (install, gimp_directory ());
 
@@ -214,16 +220,28 @@ gimp_user_install_run (GimpUserInstall *install,
   dirname = g_filename_display_name (gimp_directory ());
 
   if (install->migrate)
-    user_install_log (install,
-                      _("It seems you have used GIMP %s before.  "
-                        "GIMP will now migrate your user settings to '%s'."),
-                      install->migrate, dirname);
+    {
+      gchar *verstring;
+
+      /* TODO: these 2 strings should be merged into one, but it was not
+       * possible to do it at implementation time, in order not to break
+       * string freeze.
+       */
+      verstring = g_strdup_printf ("%d.%d", install->old_major, install->old_minor);
+      user_install_log (install,
+                        _("It seems you have used GIMP %s before.  "
+                          "GIMP will now migrate your user settings to '%s'."),
+                        verstring, dirname);
+      g_free (verstring);
+    }
   else
-    user_install_log (install,
-                      _("It appears that you are using GIMP for the "
-                        "first time.  GIMP will now create a folder "
-                        "named '%s' and copy some files to it."),
-                      dirname);
+    {
+      user_install_log (install,
+                        _("It appears that you are using GIMP for the "
+                          "first time.  GIMP will now create a folder "
+                          "named '%s' and copy some files to it."),
+                        dirname);
+    }
 
   g_free (dirname);
 
@@ -245,6 +263,7 @@ gimp_user_install_free (GimpUserInstall *install)
   g_return_if_fail (install != NULL);
 
   g_free (install->old_dir);
+  g_hash_table_destroy (install->accels);
 
   g_slice_free (GimpUserInstall, install);
 }
@@ -267,41 +286,111 @@ static gboolean
 user_install_detect_old (GimpUserInstall *install,
                          const gchar     *gimp_dir)
 {
-  gchar    *dir     = g_strdup (gimp_dir);
+  gchar    *dir     = g_strconcat (gimp_dir, "ZZZ", NULL);
   gchar    *version;
   gboolean  migrate = FALSE;
 
   version = strstr (dir, GIMP_APP_VERSION);
-  g_snprintf (version, 5, "%d.XY", 2);
 
   if (version)
     {
-      gint i;
+      gint major;
+      gint minor;
 
-      for (i = (GIMP_MINOR_VERSION & ~1); i >= 0; i -= 2)
+      for (major = 3; major >= 2; major--)
         {
-          /*  we assume that GIMP_APP_VERSION is in the form '2.x'  */
-          g_snprintf (version + 2, 3, "%d", i);
+          gint max_minor;
 
-          migrate = g_file_test (dir, G_FILE_TEST_IS_DIR);
+          g_snprintf (version, 5, "%d.XY", major);
 
-          if (migrate)
+          switch (major)
             {
-#ifdef GIMP_UNSTABLE
-              g_printerr ("gimp-user-install: migrating from %s\n", dir);
-#endif
-              install->old_major = 2;
-              install->old_minor = i;
-
+            case 3:
+              max_minor = GIMP_MINOR_VERSION;
               break;
+            case 2:
+              max_minor = 10;
+              break;
+            default:
+              g_return_val_if_reached (FALSE);
+            }
+
+          for (minor = (max_minor & ~1); minor >= 0; minor -= 2)
+            {
+              /*  we assume that GIMP_APP_VERSION is in the form '2.x'  */
+              g_snprintf (version + 2, 3, "%d", minor);
+
+              migrate = g_file_test (dir, G_FILE_TEST_IS_DIR);
+
+              if (migrate)
+                {
+                  install->old_major = 2;
+                  install->old_minor = minor;
+
+                  break;
+                }
+
+#ifdef G_OS_UNIX
+              if (minor == 10)
+                {
+                  /* This is special-casing for GIMP 2.10 as flatpak where
+                   * we had this weird inconsistency: depending on whether a
+                   * previous $XDG_CONFIG_HOME/GIMP/ folder was found or
+                   * not, the config folder would be either inside, or would
+                   * be in $HOME/.var/<etc> as other flatpak (see #5331).
+                   * For GIMP 3, even the flatpak will always be in
+                   * $XDG_CONFIG_HOME. But then we want a migration to still
+                   * find the previous config folder.
+                   * If one was found in $XDG_CONFIG_HOME, it is used in
+                   * priority. Then ~/.var/ is used a fallback, if found.
+                   */
+                  gchar *flatpak_dir = user_install_flatpak_gimpdir (minor);
+
+                  if (flatpak_dir)
+                    /* This first test is for finding a 2.10 flatpak config
+                     * dir from a non-flatpak GIMP 3+.
+                     */
+                    migrate = g_file_test (flatpak_dir, G_FILE_TEST_IS_DIR);
+
+                  if (! migrate && g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS))
+                    {
+                      /* Now we check /var/config/GIMP because this is where
+                       * local ~/.var/ is mounted inside the sandbox.
+                       * So this second test is for finding a 2.10 flatpak
+                       * config dir from a flatpak GIMP 3+.
+                       */
+                      g_free (flatpak_dir);
+                      flatpak_dir = g_build_filename ("/var/config/GIMP/", version, NULL);
+
+                      migrate = g_file_test (flatpak_dir, G_FILE_TEST_IS_DIR);
+                    }
+
+                  if (migrate)
+                    {
+                      install->old_major = 2;
+                      install->old_minor = minor;
+
+                      g_free (dir);
+                      dir = flatpak_dir;
+                      break;
+                    }
+                  else
+                    {
+                      g_free (flatpak_dir);
+                    }
+                }
+#endif
             }
         }
     }
 
+  install->migrate = migrate;
   if (migrate)
     {
       install->old_dir = dir;
-      install->migrate = (const gchar *) version;
+#ifdef GIMP_UNSTABLE
+      g_printerr ("gimp-user-install: migrating from %s\n", dir);
+#endif
     }
   else
     {
@@ -357,6 +446,27 @@ user_install_old_style_gimpdir (void)
   return gimp_dir;
 }
 
+static gchar *
+user_install_flatpak_gimpdir (gint minor)
+{
+  const gchar *home_dir = g_get_home_dir ();
+  gchar       *version  = g_strdup_printf ("2.%d", minor);
+  gchar       *gimp_dir = NULL;
+
+  if (home_dir)
+    /* AFAIK (after researching flatpak docs, and searching the web),
+     * the ~/.var/ directory is hardcoded and cannot be modified by an
+     * environment variable.
+     */
+    gimp_dir = g_build_filename (home_dir,
+                                 ".var/app/org.gimp.GIMP/config/GIMP/",
+                                 version, NULL);
+
+  g_free (version);
+
+  return gimp_dir;
+}
+
 static void
 user_install_log (GimpUserInstall *install,
                   const gchar     *format,
@@ -415,7 +525,8 @@ user_install_file_copy (GimpUserInstall    *install,
                         const gchar        *source,
                         const gchar        *dest,
                         const gchar        *old_options_regexp,
-                        GRegexEvalCallback  update_callback)
+                        GRegexEvalCallback  update_callback,
+                        GimpCopyPostProcess post_process_callback)
 {
   GError   *error = NULL;
   gboolean  success;
@@ -424,7 +535,9 @@ user_install_file_copy (GimpUserInstall    *install,
                     gimp_filename_to_utf8 (dest),
                     gimp_filename_to_utf8 (source));
 
-  success = gimp_config_file_copy (source, dest, old_options_regexp, update_callback, install, &error);
+  success = gimp_config_file_copy (source, dest, old_options_regexp,
+                                   update_callback, post_process_callback,
+                                   install, &error);
 
   user_install_log_error (install, &error);
 
@@ -507,6 +620,7 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
   gchar           *accel_match     = g_match_info_fetch (matched_value, 3);
   gchar           *ignore_match    = g_match_info_fetch (matched_value, 4);
   gchar           *new_action_name = NULL;
+  gboolean         accel_variant   = FALSE;
 
   if (strlen (ignore_match) == 0)
     {
@@ -536,17 +650,29 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
        * since GIMP 2.4, changed for 2.10 in commit 0bdb747.
        */
       else if (g_str_has_prefix (action_match, "tools-value-1-"))
-        new_action_name = g_strdup ("tools-opacity-");
+        new_action_name = g_strdup_printf ("tools-opacity-%s", action_match + 14);
       else if (g_str_has_prefix (action_match, "tools-value-2-"))
         new_action_name = g_strdup_printf ("tools-size-%s", action_match + 14);
       else if (g_str_has_prefix (action_match, "tools-value-3-"))
         new_action_name = g_strdup_printf ("tools-aspect-%s", action_match + 14);
       else if (g_str_has_prefix (action_match, "tools-value-4-"))
         new_action_name = g_strdup_printf ("tools-angle-%s", action_match + 14);
-      else if (g_strcmp0 (action_match, "vectors-path-tool") == 0)
-        new_action_name = g_strdup ("vectors-edit");
       else if (g_strcmp0 (action_match, "tools-blend") == 0)
         new_action_name = g_strdup ("tools-gradient");
+      else if (g_strcmp0 (action_match, "vectors-path-tool") == 0)
+        new_action_name = g_strdup ("paths-edit");
+      /* Following the s/GimpVectors/GimpPath/ renaming to be more
+       * consistent with the GUI, we also rename all the action names.
+       * Since GIMP 3.0, commit XXXX.
+       */
+      else if (g_strcmp0 (action_match, "vectors-selection-from-vectors") == 0)
+        new_action_name = g_strdup ("paths-selection-from-paths");
+      else if (g_str_has_prefix (action_match, "vectors-selection-to-vectors"))
+        new_action_name = g_strdup_printf ("paths-selection-to-path%s",
+                                           action_match + strlen ("vectors-selection-to-vectors"));
+      else if (g_str_has_prefix (action_match, "vectors-"))
+        new_action_name = g_strdup_printf ("paths-%s", action_match + 8);
+
       /* view-rotate-reset became view-reset and new view-rotate-reset and
        * view-flip-reset actions were created.  See commit 15fb4a7be0.
        */
@@ -556,19 +682,71 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
       /* select-float became select-cut-float in 3.0 (select-copy-float added). */
       else if (g_strcmp0 (action_match, "select-float") == 0)
         new_action_name = g_strdup ("select-cut-float");
+      /* edit-paste-as-new-layer* actions removed in 3.0.0 (commit
+       * 2c4f91f585) because the default "edit-paste" pastes as new layer.
+       *
+       * We used to migrate these to "edit-paste" and
+       * "edit-paste-in-place", but it meant that it would override the
+       * default shortcut (ctrl-v) for the basic paste, which may be
+       * counter-productive (cf. #13414). Furthermore, if you had a
+       * shortcut to "edit-paste*" actions (since these existed in 2.10
+       * too), you'd have a shortcut conflict.
+       *
+       * Instead since GIMP 3.0.4, we migrate them to "edit-paste-merged*"
+       * (introduced in 143496af22) though it's not an exact equivalent
+       * (but neither were "edit-paste*"), in particular:
+       *
+       * - If a layer mask is selected, "edit-paste-merged*" would still
+       *   create a floating mask (unlike old "edit-paste-as-new-layer*"
+       *   which was always creating a new layer).
+       * - Even when multiple layers are copied, the paste is always a
+       *   single layer. But the latter is not a big issue since in GIMP
+       *   2.x, you could not copy multiple layers anyway, so in this
+       *   aspect, we may say it works the same.
+       */
+      else if (g_strcmp0 (action_match, "edit-paste-as-new-layer") == 0)
+        new_action_name = g_strdup ("edit-paste-merged");
+      else if (g_strcmp0 (action_match, "edit-paste-as-new-layer-in-place") == 0)
+        new_action_name = g_strdup ("edit-paste-merged-in-place");
+      /* These actions had an -accel variant which got removed in commit
+       * 71c8ff1f21. Since we cannot know if both variants were given a
+       * custom shortcut when processing per-line, we temporarily store
+       * them all and will do a second pass allowing us to store one or
+       * both shortcuts if needed.
+       */
+      else if (g_str_has_suffix (action_match, "-accel")  ||
+               g_strcmp0 (action_match, "view-zoom-out")  == 0 ||
+               g_strcmp0 (action_match, "view-zoom-in")   == 0 ||
+               g_strcmp0 (action_match, "view-zoom-16-1") == 0 ||
+               g_strcmp0 (action_match, "view-zoom-8-1")  == 0 ||
+               g_strcmp0 (action_match, "view-zoom-4-1")  == 0 ||
+               g_strcmp0 (action_match, "view-zoom-2-1")  == 0 ||
+               g_strcmp0 (action_match, "view-zoom-1-1")  == 0)
+        accel_variant = TRUE;
 
       if (new_action_name == NULL)
         new_action_name = g_strdup (action_match);
 
       if (g_strcmp0 (comment_match, ";") == 0)
-        g_string_append (new_value, "# ");
+        {
+          g_string_append (new_value, "# ");
+        }
+      else if (accel_variant)
+        {
+          g_hash_table_insert (install->accels, action_match, accel_match);
+          action_match = NULL;
+          accel_match  = NULL;
+        }
 
-      if (strlen (accel_match) > 0)
-        g_string_append_printf (new_value, "(action \"%s\" \"%s\")",
-                                new_action_name, accel_match);
-      else
-        g_string_append_printf (new_value, "(action \"%s\")",
-                                new_action_name);
+      if (action_match)
+        {
+          if (strlen (accel_match) > 0)
+            g_string_append_printf (new_value, "(action \"%s\" \"%s\")",
+                                    new_action_name, accel_match);
+          else
+            g_string_append_printf (new_value, "(action \"%s\")",
+                                    new_action_name);
+        }
     }
 
   g_free (comment_match);
@@ -580,6 +758,97 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
   return FALSE;
 }
 
+gchar *
+user_update_post_process_menurc_over20 (gpointer user_data)
+{
+  GString         *string  = g_string_new (NULL);
+  GimpUserInstall *install = (GimpUserInstall *) user_data;
+
+  static gchar    * gimp_2_accels[][3] =
+  {
+    { "view-zoom-out",  "minus", "KP_Subtract" },
+    { "view-zoom-in",   "plus",  "KP_Add" },
+    { "view-zoom-16-1", "5",     "KP_5" },
+    { "view-zoom-8-1",  "4",     "KP_4" },
+    { "view-zoom-4-1",  "3",     "KP_3" },
+    { "view-zoom-2-1",  "2",     "KP_2" },
+    { "view-zoom-1-1",  "1",     "KP_1" }
+  };
+
+  for (gint i = 0; i < G_N_ELEMENTS (gimp_2_accels); i++)
+    {
+      const gchar *action = gimp_2_accels[i][0];
+      gchar       *action_variant = g_strconcat (action, "-accel", NULL);
+      gchar       *accel;
+      gchar       *accel_variant;
+
+      accel         = g_hash_table_lookup (install->accels, action);
+      accel_variant = g_hash_table_lookup (install->accels, action_variant);
+      if (accel != NULL && strlen (accel) > 0 &&
+          accel_variant != NULL && strlen (accel_variant) > 0)
+        {
+          g_string_append_printf (string, "\n(action \"%s\" \"%s\" \"%s\")",
+                                  action, accel, accel_variant);
+        }
+      else if (accel != NULL)
+        {
+          if (strlen (accel) > 0)
+            {
+              if (accel_variant == NULL)
+                g_string_append_printf (string, "\n(action \"%s\" \"%s\" \"%s\")", action, accel, gimp_2_accels[i][2]);
+              else
+                g_string_append_printf (string, "\n(action \"%s\" \"%s\")", action, accel);
+            }
+          else if (accel_variant != NULL)
+            {
+              if (strlen (accel_variant) > 0)
+                g_string_append_printf (string, "\n(action \"%s\" \"%s\")", action, accel_variant);
+              else
+                g_string_append_printf (string, "\n(action \"%s\")", action);
+            }
+          else
+            {
+              g_string_append_printf (string, "\n(action \"%s\" \"%s\")", action, gimp_2_accels[i][2]);
+            }
+        }
+      else if (accel_variant != NULL)
+        {
+          if (strlen (accel_variant) > 0)
+            {
+              g_string_append_printf (string, "\n(action \"%s\" \"%s\" \"%s\")", action, accel_variant, gimp_2_accels[i][1]);
+            }
+          else
+            {
+              g_string_append_printf (string, "\n(action \"%s\" \"%s\")", action, gimp_2_accels[i][1]);
+            }
+        }
+
+      g_free (action_variant);
+    }
+
+  return g_string_free (string, FALSE);
+}
+
+#define TEMPLATERC_UPDATE_PATTERN \
+  "\\(precision (.*)-gamma\\)"
+
+static gboolean
+user_update_templaterc (const GMatchInfo *matched_value,
+                        GString          *new_value,
+                        gpointer          data)
+{
+  gchar *original = g_match_info_fetch (matched_value, 0);
+  gchar *match    = g_match_info_fetch (matched_value, 1);
+
+  /* GIMP_PRECISION_*_GAMMA removed in GIMP 3.0.0 (commit 2559138931). */
+  g_string_append_printf (new_value, "(precision %s-non-linear)", match);
+
+  g_free (original);
+  g_free (match);
+
+  return FALSE;
+}
+
 #define CONTROLLERRC_UPDATE_PATTERN \
   "\\(map \"(scroll|cursor)-[^\"]*\\bcontrol\\b[^\"]*\""
 
@@ -692,9 +961,12 @@ user_update_sessionrc (const GMatchInfo *matched_value,
 }
 
 #define GIMPRC_UPDATE_PATTERN \
+  "\\(show-tooltips [^)]*\\)"  "|" \
   "\\(theme [^)]*\\)"          "|" \
   "^ *\\(.*-path \".*\"\\) *$" "|" \
-  "\\(style solid\\)"
+  "\\(style solid\\)"          "|" \
+  "\\(precision (.*)-gamma\\)" "|" \
+  "\\(filter-tool-show-color-options [^)]*\\)"
 
 static gboolean
 user_update_gimprc (const GMatchInfo *matched_value,
@@ -710,9 +982,24 @@ user_update_gimprc (const GMatchInfo *matched_value,
        */
       g_string_append (new_value, "(style fg-color)");
     }
+  else if (g_str_has_prefix (match, "(precision "))
+    {
+      gchar *precision_match = g_match_info_fetch (matched_value, 1);
+
+      /* GIMP_PRECISION_*_GAMMA removed in GIMP 3.0.0 (commit 2559138931). */
+      g_string_append_printf (new_value, "(precision %s-non-linear)", precision_match);
+
+      g_free (precision_match);
+    }
   else
     {
+      /* Do not migrate show-tooltips GIMP < 3.0. Cf. #1965. */
+
       /* Do not migrate paths and themes from GIMP < 3.0. */
+
+      /* Do not migrate the advanced color options which was the gamma
+       * hack removed for GIMP 3.0. Cf. #12577.
+       */
     }
 
   g_free (match);
@@ -798,9 +1085,10 @@ user_update_tool_presets (const GMatchInfo *matched_value,
  * well as "toolrc" (but this one is skipped anyway).
  */
 #define CONTEXTRC_UPDATE_PATTERN \
-  "gimp-blend-tool"           "|" \
-  "dynamics \"Dynamics Off\"" "|" \
-  "\\(dynamics-expanded yes\\)"
+  "gimp-blend-tool"             "|" \
+  "dynamics \"Dynamics Off\""   "|" \
+  "\\(dynamics-expanded yes\\)" "|" \
+  "\\(color-options-expanded [^)]*\\)"
 
 static gboolean
 user_update_contextrc_over20 (const GMatchInfo *matched_value,
@@ -821,6 +1109,10 @@ user_update_contextrc_over20 (const GMatchInfo *matched_value,
     {
       /* This option just doesn't exist anymore. */
     }
+  else if (g_str_has_prefix (match, "(color-options-expanded "))
+    {
+      /* This option was removed with the gamma-hack. Cf. #12577. */
+    }
   else
     {
       g_message ("(WARNING) %s: invalid match \"%s\"", G_STRFUNC, match);
@@ -837,7 +1129,8 @@ user_install_dir_copy (GimpUserInstall    *install,
                        const gchar        *source,
                        const gchar        *base,
                        const gchar        *update_pattern,
-                       GRegexEvalCallback  update_callback)
+                       GRegexEvalCallback  update_callback,
+                       GimpCopyPostProcess post_process_callback)
 {
   GDir        *source_dir = NULL;
   GDir        *dest_dir   = NULL;
@@ -888,7 +1181,8 @@ user_install_dir_copy (GimpUserInstall    *install,
 
           success = user_install_file_copy (install, name, dest,
                                             update_pattern,
-                                            update_callback);
+                                            update_callback,
+                                            post_process_callback);
           if (! success)
             {
               g_free (name);
@@ -898,7 +1192,8 @@ user_install_dir_copy (GimpUserInstall    *install,
       else
         {
           user_install_dir_copy (install, level + 1, name, dirname,
-                                 update_pattern, update_callback);
+                                 update_pattern, update_callback,
+                                 post_process_callback);
         }
 
       g_free (name);
@@ -948,7 +1243,7 @@ user_install_create_files (GimpUserInstall *install)
                       gimp_sysconf_directory (), G_DIR_SEPARATOR,
                       gimp_user_install_items[i].name);
 
-          if (! user_install_file_copy (install, source, dest, NULL, NULL))
+          if (! user_install_file_copy (install, source, dest, NULL, NULL, NULL))
             return FALSE;
           break;
         }
@@ -994,8 +1289,9 @@ user_install_migrate_files (GimpUserInstall *install)
 
       if (g_file_test (source, G_FILE_TEST_IS_REGULAR))
         {
-          const gchar        *update_pattern = NULL;
-          GRegexEvalCallback  update_callback = NULL;
+          const gchar        *update_pattern        = NULL;
+          GRegexEvalCallback  update_callback       = NULL;
+          GimpCopyPostProcess post_process_callback = NULL;
 
           /*  skip these files for all old versions  */
           if (strcmp (basename, "documents") == 0      ||
@@ -1025,13 +1321,19 @@ user_install_migrate_files (GimpUserInstall *install)
                   goto next_file;
                   break;
                 default:
-                  update_pattern  = MENURC_OVER20_UPDATE_PATTERN;
-                  update_callback = user_update_menurc_over20;
+                  update_pattern        = MENURC_OVER20_UPDATE_PATTERN;
+                  update_callback       = user_update_menurc_over20;
+                  post_process_callback = user_update_post_process_menurc_over20;
                   /* menurc becomes shortcutsrc in 3.0. */
-                  new_dest        = "shortcutsrc";
+                  new_dest              = "shortcutsrc";
                   break;
                 }
             }
+          else if (strcmp (basename, "templaterc") == 0)
+            {
+              update_pattern  = TEMPLATERC_UPDATE_PATTERN;
+              update_callback = user_update_templaterc;
+            }
           else if (strcmp (basename, "controllerrc") == 0)
             {
               update_pattern  = CONTROLLERRC_UPDATE_PATTERN;
@@ -1054,7 +1356,8 @@ user_install_migrate_files (GimpUserInstall *install)
                       new_dest ? new_dest : basename);
 
           user_install_file_copy (install, source, dest,
-                                  update_pattern, update_callback);
+                                  update_pattern, update_callback,
+                                  post_process_callback);
         }
       else if (g_file_test (source, G_FILE_TEST_IS_DIR))
         {
@@ -1087,7 +1390,7 @@ user_install_migrate_files (GimpUserInstall *install)
               update_callback = user_update_tool_presets;
             }
           user_install_dir_copy (install, 0, source, gimp_directory (),
-                                 update_pattern, update_callback);
+                                 update_pattern, update_callback, NULL);
         }
 
     next_file:
