@@ -33,9 +33,7 @@
 #include <glib.h>
 
 #ifdef G_OS_WIN32
-#define _WIN32_WINNT 0x0500
 #include <windows.h>
-#include <process.h>
 #endif
 
 #include <cairo.h>
@@ -195,7 +193,7 @@ gimp_get_default_language (const gchar *category)
   return lang;
 }
 
-GimpUnit
+GimpUnit *
 gimp_get_default_unit (void)
 {
 #if defined (HAVE__NL_MEASUREMENT_MEASUREMENT)
@@ -204,10 +202,10 @@ gimp_get_default_unit (void)
   switch (*((guchar *) measurement))
     {
     case 1: /* metric   */
-      return GIMP_UNIT_MM;
+      return gimp_unit_mm ();
 
     case 2: /* imperial */
-      return GIMP_UNIT_INCH;
+      return gimp_unit_inch ();
     }
 
 #elif defined (G_OS_WIN32)
@@ -224,15 +222,15 @@ gimp_get_default_unit (void)
     switch ((guint) measurement)
       {
       case 0: /* metric */
-        return GIMP_UNIT_MM;
+        return gimp_unit_mm ();
 
       case 1: /* imperial */
-        return GIMP_UNIT_INCH;
+        return gimp_unit_inch ();
       }
     }
 #endif
 
-  return GIMP_UNIT_MM;
+  return gimp_unit_mm ();
 }
 
 gchar **
@@ -529,7 +527,7 @@ gimp_enum_get_value_name (GType enum_type,
 gboolean
 gimp_get_fill_params (GimpContext   *context,
                       GimpFillType   fill_type,
-                      GimpRGB       *color,
+                      GeglColor    **color,
                       GimpPattern  **pattern,
                       GError       **error)
 
@@ -539,47 +537,34 @@ gimp_get_fill_params (GimpContext   *context,
   g_return_val_if_fail (pattern != NULL, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
+  *color   = NULL;
   *pattern = NULL;
 
   switch (fill_type)
     {
     case GIMP_FILL_FOREGROUND:
-      gimp_context_get_foreground (context, color);
+      *color = gegl_color_duplicate (gimp_context_get_foreground (context));
       break;
 
     case GIMP_FILL_BACKGROUND:
-      gimp_context_get_background (context, color);
+      *color = gegl_color_duplicate (gimp_context_get_background (context));
       break;
 
     case GIMP_FILL_CIELAB_MIDDLE_GRAY:
       {
-        const float        cielab_pixel[3] = {50, 0, 0};
-        float              pixel[3]        = {0, 0, 0};
-        GimpImage         *image           = gimp_context_get_image (context);
-        GimpImageBaseType  base_type;
-        const Babl        *format;
+        const float cielab_pixel[3] = {50.f, 0.f, 0.f};
 
-        base_type = gimp_image_get_base_type (image);
-        if (base_type == GIMP_INDEXED)
-          base_type = GIMP_RGB;
-
-        format = gimp_image_get_format (image, base_type,
-                                        GIMP_PRECISION_FLOAT_NON_LINEAR, FALSE,
-                                        gimp_image_get_layer_space (image));
-
-        babl_process (babl_fish (babl_format ("CIE Lab float"), format),
-                      cielab_pixel, pixel, 1);
-
-        gimp_rgba_set (color, pixel[0], pixel[1], pixel[2], GIMP_OPACITY_OPAQUE);
+        *color = gegl_color_new (NULL);
+        gegl_color_set_pixel (*color, babl_format ("CIE Lab float"), cielab_pixel);
       }
       break;
 
     case GIMP_FILL_WHITE:
-      gimp_rgba_set (color, 1.0, 1.0, 1.0, GIMP_OPACITY_OPAQUE);
+      *color = gegl_color_new ("white");
       break;
 
     case GIMP_FILL_TRANSPARENT:
-      gimp_rgba_set (color, 0.0, 0.0, 0.0, GIMP_OPACITY_TRANSPARENT);
+      *color = gegl_color_new ("transparent");
       break;
 
     case GIMP_FILL_PATTERN:
@@ -591,9 +576,9 @@ gimp_get_fill_params (GimpContext   *context,
                                _("No patterns available for this operation."));
 
           /*  fall back to BG fill  */
-          gimp_context_get_background (context, color);
+          *color = gegl_color_duplicate (gimp_context_get_background (context));
 
-          return FALSE;
+          return TRUE;
         }
       break;
 
@@ -730,7 +715,7 @@ gimp_file_is_executable (GFile *file)
 
   if (info)
     {
-      GFileType    file_type = g_file_info_get_file_type (info);
+      GFileType    file_type = g_file_info_get_attribute_uint32 (info, G_FILE_ATTRIBUTE_STANDARD_TYPE);
       const gchar *filename  = g_file_info_get_name (info);
 
       if (file_type == G_FILE_TYPE_REGULAR &&
@@ -760,31 +745,54 @@ gimp_file_is_executable (GFile *file)
 gchar *
 gimp_file_get_extension (GFile *file)
 {
-  gchar *uri;
-  gint   uri_len;
-  gchar *ext = NULL;
-  gint   search_len;
+  GFileInfo *info;
+  gchar     *basename;
+  gint       basename_len;
+  gchar     *ext = NULL;
+  gint       search_len;
 
   g_return_val_if_fail (G_IS_FILE (file), NULL);
 
-  uri     = g_file_get_uri (file);
-  uri_len = strlen (uri);
+  /* Certain cloud providers return a blob name rather than the
+   * actual file with g_file_get_uri (). Since we don't check
+   * the magic numbers for remote files, we can't open it. The
+   * actual name is stored as "display-name" in all cases, so we
+   * use that instead. */
+  info = g_file_query_info (file,
+                            G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
+                            0, NULL, NULL);
 
-  if (g_str_has_suffix (uri, ".gz"))
-    search_len = uri_len - 3;
-  else if (g_str_has_suffix (uri, ".bz2"))
-    search_len = uri_len - 4;
-  else if (g_str_has_suffix (uri, ".xz"))
-    search_len = uri_len - 3;
+  if (info != NULL)
+    basename =
+      g_file_info_get_attribute_as_string (info,
+                                           G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME);
   else
-    search_len = uri_len;
+    basename = g_file_get_basename (file);
 
-  ext = g_strrstr_len (uri, search_len, ".");
+  g_clear_object (&info);
+
+  /* When making a temporary file for saving/exporting, we may not
+   * have the display-name yet, so let's fallback to the URI */
+  if (! basename)
+    basename = g_file_get_uri (file);
+
+  basename_len = strlen (basename);
+
+  if (g_str_has_suffix (basename, ".gz"))
+    search_len = basename_len - 3;
+  else if (g_str_has_suffix (basename, ".bz2"))
+    search_len = basename_len - 4;
+  else if (g_str_has_suffix (basename, ".xz"))
+    search_len = basename_len - 3;
+  else
+    search_len = basename_len;
+
+  ext = g_strrstr_len (basename, search_len, ".");
 
   if (ext)
     ext = g_strdup (ext);
 
-  g_free (uri);
+  g_free (basename);
 
   return ext;
 }
@@ -1296,7 +1304,7 @@ gimp_create_image_from_buffer (Gimp        *gimp,
                                            NULL /* same image */);
   gimp_image_add_layer (image, layer, NULL, -1, FALSE);
 
-  gimp_create_display (gimp, image, GIMP_UNIT_PIXEL, 1.0, NULL);
+  gimp_create_display (gimp, image, gimp_unit_pixel (), 1.0, NULL);
 
   /* unref the image unconditionally, even when no display was created */
   g_object_add_weak_pointer (G_OBJECT (image), (gpointer) &image);
