@@ -63,6 +63,7 @@
 #include "gimpgradient.h"
 #include "gimpidtable.h"
 #include "gimpimage.h"
+#include "gimpimage-metadata.h"
 #include "gimpimagefile.h"
 #include "gimplist.h"
 #include "gimpmarshal.h"
@@ -103,34 +104,36 @@ enum
 };
 
 
-static void      gimp_constructed          (GObject           *object);
-static void      gimp_set_property         (GObject           *object,
-                                            guint              property_id,
-                                            const GValue      *value,
-                                            GParamSpec        *pspec);
-static void      gimp_get_property         (GObject           *object,
-                                            guint              property_id,
-                                            GValue            *value,
-                                            GParamSpec        *pspec);
-static void      gimp_dispose              (GObject           *object);
-static void      gimp_finalize             (GObject           *object);
+static void      gimp_constructed                    (GObject           *object);
+static void      gimp_set_property                   (GObject           *object,
+                                                      guint              property_id,
+                                                      const GValue      *value,
+                                                      GParamSpec        *pspec);
+static void      gimp_get_property                   (GObject           *object,
+                                                      guint              property_id,
+                                                      GValue            *value,
+                                                      GParamSpec        *pspec);
+static void      gimp_dispose                        (GObject           *object);
+static void      gimp_finalize                       (GObject           *object);
 
-static gint64    gimp_get_memsize          (GimpObject        *object,
-                                            gint64            *gui_size);
+static gint64    gimp_get_memsize                    (GimpObject        *object,
+                                                      gint64            *gui_size);
 
-static void      gimp_real_initialize      (Gimp              *gimp,
-                                            GimpInitStatusFunc status_callback);
-static void      gimp_real_restore         (Gimp              *gimp,
-                                            GimpInitStatusFunc status_callback);
-static gboolean  gimp_real_exit            (Gimp              *gimp,
-                                            gboolean           force);
+static void      gimp_real_initialize                (Gimp              *gimp,
+                                                      GimpInitStatusFunc status_callback);
+static void      gimp_real_restore                   (Gimp              *gimp,
+                                                      GimpInitStatusFunc status_callback);
+static gboolean  gimp_real_exit                      (Gimp              *gimp,
+                                                      gboolean           force);
 
-static void      gimp_global_config_notify (GObject           *global_config,
-                                            GParamSpec        *param_spec,
-                                            GObject           *edit_config);
-static void      gimp_edit_config_notify   (GObject           *edit_config,
-                                            GParamSpec        *param_spec,
-                                            GObject           *global_config);
+static void      gimp_global_config_notify           (GObject           *global_config,
+                                                      GParamSpec        *param_spec,
+                                                      GObject           *edit_config);
+static void      gimp_edit_config_notify             (GObject           *edit_config,
+                                                      GParamSpec        *param_spec,
+                                                      GObject           *global_config);
+
+static gboolean  gimp_exit_idle_cleanup_stray_images (Gimp              *gimp);
 
 
 G_DEFINE_TYPE (Gimp, gimp, GIMP_TYPE_OBJECT)
@@ -244,10 +247,11 @@ gimp_init (Gimp *gimp)
   gimp->images = gimp_list_new_weak (GIMP_TYPE_IMAGE, FALSE);
   gimp_object_set_static_name (GIMP_OBJECT (gimp->images), "images");
 
-  gimp->next_guide_id        = 1;
-  gimp->next_sample_point_id = 1;
-  gimp->image_table          = gimp_id_table_new ();
-  gimp->item_table           = gimp_id_table_new ();
+  gimp->next_guide_id         = 1;
+  gimp->next_sample_point_id  = 1;
+  gimp->image_table           = gimp_id_table_new ();
+  gimp->item_table            = gimp_id_table_new ();
+  gimp->drawable_filter_table = gimp_id_table_new ();
 
   gimp->displays = g_object_new (GIMP_TYPE_LIST,
                                  "children-type", GIMP_TYPE_OBJECT,
@@ -370,8 +374,6 @@ gimp_dispose (GObject *object)
   g_clear_object (&gimp->edit_config);
   g_clear_object (&gimp->config);
 
-  gimp_contexts_exit (gimp);
-
   g_clear_object (&gimp->image_new_last_template);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -426,6 +428,7 @@ gimp_finalize (GObject *object)
   g_clear_object (&gimp->clipboard_image);
   g_clear_object (&gimp->displays);
   g_clear_object (&gimp->item_table);
+  g_clear_object (&gimp->drawable_filter_table);
   g_clear_object (&gimp->image_table);
   g_clear_object (&gimp->images);
   g_clear_object (&gimp->plug_in_manager);
@@ -449,7 +452,9 @@ gimp_finalize (GObject *object)
                  G_STRFUNC, g_list_length (gimp->context_list));
 
       for (list = gimp->context_list; list; list = g_list_next (list))
-        g_printerr ("stale context: %s\n", gimp_object_get_name (list->data));
+        g_printerr ("stale context: %s (of type %s)\n",
+                    gimp_object_get_name (list->data),
+                    g_type_name (G_TYPE_FROM_INSTANCE (list->data)));
 
       g_list_free (gimp->context_list);
       gimp->context_list = NULL;
@@ -489,6 +494,7 @@ gimp_get_memsize (GimpObject *object,
 
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->image_table), 0);
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->item_table),  0);
+  memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->drawable_filter_table),  0);
 
   memsize += gimp_object_get_memsize (GIMP_OBJECT (gimp->displays), gui_size);
 
@@ -891,8 +897,7 @@ void
 gimp_exit (Gimp     *gimp,
            gboolean  force)
 {
-  gboolean  handled;
-  GList    *image_iter;
+  gboolean handled;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
@@ -906,17 +911,11 @@ gimp_exit (Gimp     *gimp,
   if (handled)
     return;
 
-  /* Get rid of images without display. We do this *after* handling the
-   * usual exit callbacks, because the things that are torn down there
-   * might have references to these images (for instance GimpActions
-   * in the UI manager).
-   */
-  while ((image_iter = gimp_get_image_iter (gimp)))
-    {
-      GimpImage *image = image_iter->data;
+  gimp_contexts_exit (gimp);
 
-      g_object_unref (image);
-    }
+  g_idle_add_full (G_PRIORITY_LOW,
+                   (GSourceFunc) gimp_exit_idle_cleanup_stray_images,
+                   gimp, NULL);
 }
 
 GList *
@@ -1038,12 +1037,17 @@ gimp_create_image (Gimp              *gimp,
                    GimpPrecision      precision,
                    gboolean           attach_comment)
 {
-  GimpImage *image;
+  GimpImage    *image;
+  GimpMetadata *metadata;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
 
   image = gimp_image_new (gimp, width, height, type, precision);
 
+  metadata = gimp_metadata_new ();
+  gimp_image_set_metadata (image, metadata, FALSE);
+  g_object_unref (metadata);
+
   if (attach_comment)
     {
       const gchar *comment;
@@ -1237,3 +1241,31 @@ gimp_get_temp_file (Gimp        *gimp,
 
   return file;
 }
+
+static gboolean
+gimp_exit_idle_cleanup_stray_images (Gimp *gimp)
+{
+  GList *image_iter;
+
+  while (g_main_context_pending (NULL))
+    g_main_context_iteration (NULL, TRUE);
+
+  /* Get rid of images without display. We do this *after* handling the
+   * usual exit callbacks and any other pending event, because the
+   * things that are torn down there might have references to these
+   * images, for instance GimpActions in the UI manager, or some plug-in
+   * which was still running and had to get killed in gimp_exit() (cf. #11922).
+   */
+  while ((image_iter = gimp_get_image_iter (gimp)))
+    {
+      GimpImage *image = image_iter->data;
+
+      /* TODO: localize after string freeze. */
+      g_printerr ("INFO: a stray image seems to have been left around by a plug-in: \"%s\"",
+                  gimp_image_get_display_name (image));
+
+      g_object_unref (image);
+    }
+
+  return G_SOURCE_REMOVE ;
+}
