@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_GRADIENT_SAVE_H__
@@ -20,12 +20,13 @@
 
 
 /*  don't call this function directly, use gimp_data_save() instead  */
-gboolean   gimp_gradient_save     (GimpData      *data,
-                                   GError       **error);
+gboolean   gimp_gradient_save     (GimpData       *data,
+                                   GOutputStream  *output,
+                                   GError        **error);
 
-gboolean   gimp_gradient_save_pov (GimpGradient  *gradient,
-                                   const gchar   *filename,
-                                   GError       **error);
+gboolean   gimp_gradient_save_pov (GimpGradient   *gradient,
+                                   GFile          *file,
+                                   GError        **error);
 
 
 #endif /* __GIMP_GRADIENT_SAVE_H__ */
