@@ -110,25 +110,12 @@ void                 gimp_image_import_color_profile   (GimpImage           *ima
 
 GimpColorTransform * gimp_image_get_color_transform_to_srgb_u8
                                                        (GimpImage           *image);
-GimpColorTransform * gimp_image_get_color_transform_from_srgb_u8
-                                                       (GimpImage           *image);
 
 GimpColorTransform * gimp_image_get_color_transform_to_srgb_double
                                                        (GimpImage           *image);
 GimpColorTransform * gimp_image_get_color_transform_from_srgb_double
                                                        (GimpImage           *image);
 
-void                 gimp_image_color_profile_pixel_to_rgb
-                                                       (GimpImage           *image,
-                                                        const Babl          *pixel_format,
-                                                        gpointer             pixel,
-                                                        GimpRGB             *color);
-void                 gimp_image_color_profile_rgb_to_pixel
-                                                       (GimpImage           *image,
-                                                        const GimpRGB       *color,
-                                                        const Babl          *pixel_format,
-                                                        gpointer             pixel);
-
 
 /*  internal API, to be called only from gimpimage.c  */
 
