@@ -44,7 +44,7 @@ struct _GimpImagePrivate
   gint               height;                /*  height in pixels             */
   gdouble            xresolution;           /*  image x-res, in dpi          */
   gdouble            yresolution;           /*  image y-res, in dpi          */
-  GimpUnit           resolution_unit;       /*  resolution unit              */
+  GimpUnit          *resolution_unit;       /*  resolution unit              */
   gboolean           resolution_set;        /*  resolution explicitly set    */
   GimpImageBaseType  base_type;             /*  base gimp_image type         */
   GimpPrecision      precision;             /*  image's precision            */
@@ -74,7 +74,6 @@ struct _GimpImagePrivate
   /*  Cached color transforms: from layer to sRGB u8 and double, and back    */
   gboolean            color_transforms_created;
   GimpColorTransform *transform_to_srgb_u8;
-  GimpColorTransform *transform_from_srgb_u8;
   GimpColorTransform *transform_to_srgb_double;
   GimpColorTransform *transform_from_srgb_double;
 
@@ -139,7 +138,7 @@ struct _GimpImagePrivate
 
   gboolean           quick_mask_state;      /*  TRUE if quick mask is on       */
   gboolean           quick_mask_inverted;   /*  TRUE if quick mask is inverted */
-  GimpRGB            quick_mask_color;      /*  rgba triplet of the color      */
+  GeglColor         *quick_mask_color;      /*  rgba triplet of the color      */
 
   /*  Undo apparatus  */
   GimpUndoStack     *undo_stack;            /*  stack for undo operations    */
