@@ -27,6 +27,8 @@
 #include "gimp.h"
 #include "gimpchannelpropundo.h"
 #include "gimpchannelundo.h"
+#include "gimpdrawablefilter.h"
+#include "gimpdrawablefilterundo.h"
 #include "gimpdrawablemodundo.h"
 #include "gimpdrawablepropundo.h"
 #include "gimpdrawableundo.h"
@@ -54,10 +56,10 @@
 #include "text/gimptextlayer.h"
 #include "text/gimptextundo.h"
 
-#include "vectors/gimpvectors.h"
-#include "vectors/gimpvectorsmodundo.h"
-#include "vectors/gimpvectorspropundo.h"
-#include "vectors/gimpvectorsundo.h"
+#include "vectors/gimppath.h"
+#include "vectors/gimppathmodundo.h"
+#include "vectors/gimppathpropundo.h"
+#include "vectors/gimppathundo.h"
 
 #include "gimp-intl.h"
 
@@ -306,6 +308,74 @@ gimp_image_undo_push_drawable_format (GimpImage    *image,
 }
 
 
+/***************************/
+/*  Drawable Filter Undos  */
+/***************************/
+GimpUndo *
+gimp_image_undo_push_filter_add (GimpImage          *image,
+                                 const gchar        *undo_desc,
+                                 GimpDrawable       *drawable,
+                                 GimpDrawableFilter *filter)
+{
+  g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
+  g_return_val_if_fail (GIMP_IS_DRAWABLE_FILTER (filter), NULL);
+
+  return gimp_image_undo_push (image, GIMP_TYPE_DRAWABLE_FILTER_UNDO,
+                               GIMP_UNDO_FILTER_ADD, undo_desc,
+                               GIMP_DIRTY_DRAWABLE,
+                               "filter", filter,
+                               NULL);
+}
+
+GimpUndo *
+gimp_image_undo_push_filter_remove (GimpImage          *image,
+                                    const gchar        *undo_desc,
+                                    GimpDrawable       *drawable,
+                                    GimpDrawableFilter *filter)
+{
+  g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
+  g_return_val_if_fail (GIMP_IS_DRAWABLE_FILTER (filter), NULL);
+
+  return gimp_image_undo_push (image, GIMP_TYPE_DRAWABLE_FILTER_UNDO,
+                               GIMP_UNDO_FILTER_REMOVE, undo_desc,
+                               GIMP_DIRTY_DRAWABLE,
+                               "filter", filter,
+                               NULL);
+}
+
+GimpUndo *
+gimp_image_undo_push_filter_reorder (GimpImage          *image,
+                                     const gchar        *undo_desc,
+                                     GimpDrawable       *drawable,
+                                     GimpDrawableFilter *filter)
+{
+  g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
+  g_return_val_if_fail (GIMP_IS_DRAWABLE_FILTER (filter), NULL);
+
+  return gimp_image_undo_push (image, GIMP_TYPE_DRAWABLE_FILTER_UNDO,
+                               GIMP_UNDO_FILTER_REORDER, undo_desc,
+                               GIMP_DIRTY_DRAWABLE,
+                               "filter", filter,
+                               NULL);
+}
+
+GimpUndo *
+gimp_image_undo_push_filter_modified (GimpImage          *image,
+                                      const gchar        *undo_desc,
+                                      GimpDrawable       *drawable,
+                                      GimpDrawableFilter *filter)
+{
+  g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
+  g_return_val_if_fail (GIMP_IS_DRAWABLE_FILTER (filter), NULL);
+
+  return gimp_image_undo_push (image, GIMP_TYPE_DRAWABLE_FILTER_UNDO,
+                               GIMP_UNDO_FILTER_MODIFIED, undo_desc,
+                               GIMP_DIRTY_ITEM_META,
+                               "filter", filter,
+                               NULL);
+}
+
+
 /****************/
 /*  Mask Undos  */
 /****************/
@@ -397,7 +467,7 @@ gimp_image_undo_push_item_displace (GimpImage   *image,
                                GIMP_UNDO_ITEM_DISPLACE, undo_desc,
                                GIMP_IS_DRAWABLE (item) ?
                                GIMP_DIRTY_ITEM | GIMP_DIRTY_DRAWABLE :
-                               GIMP_DIRTY_ITEM | GIMP_DIRTY_VECTORS,
+                               GIMP_DIRTY_ITEM | GIMP_DIRTY_PATH,
                                "item", item,
                                NULL);
 }
@@ -951,74 +1021,74 @@ gimp_image_undo_push_channel_color (GimpImage   *image,
 
 
 /*******************/
-/*  Vectors Undos  */
+/*  Path Undos  */
 /*******************/
 
 GimpUndo *
-gimp_image_undo_push_vectors_add (GimpImage   *image,
-                                  const gchar *undo_desc,
-                                  GimpVectors *vectors,
-                                  GList       *prev_vectors)
+gimp_image_undo_push_path_add (GimpImage   *image,
+                               const gchar *undo_desc,
+                               GimpPath    *path,
+                               GList       *prev_paths)
 {
   GList *iter;
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
-  g_return_val_if_fail (GIMP_IS_VECTORS (vectors), NULL);
-  g_return_val_if_fail (! gimp_item_is_attached (GIMP_ITEM (vectors)), NULL);
+  g_return_val_if_fail (GIMP_IS_PATH (path), NULL);
+  g_return_val_if_fail (! gimp_item_is_attached (GIMP_ITEM (path)), NULL);
 
-  for (iter = prev_vectors; iter; iter = iter->next)
-    g_return_val_if_fail (GIMP_IS_VECTORS (iter->data), NULL);
+  for (iter = prev_paths; iter; iter = iter->next)
+    g_return_val_if_fail (GIMP_IS_PATH (iter->data), NULL);
 
-  return gimp_image_undo_push (image, GIMP_TYPE_VECTORS_UNDO,
-                               GIMP_UNDO_VECTORS_ADD, undo_desc,
+  return gimp_image_undo_push (image, GIMP_TYPE_PATH_UNDO,
+                               GIMP_UNDO_PATH_ADD, undo_desc,
                                GIMP_DIRTY_IMAGE_STRUCTURE,
-                               "item",         vectors,
-                               "prev-vectors", prev_vectors,
+                               "item",         path,
+                               "prev-paths", prev_paths,
                                NULL);
 }
 
 GimpUndo *
-gimp_image_undo_push_vectors_remove (GimpImage   *image,
-                                     const gchar *undo_desc,
-                                     GimpVectors *vectors,
-                                     GimpVectors *prev_parent,
-                                     gint         prev_position,
-                                     GList       *prev_vectors)
+gimp_image_undo_push_path_remove (GimpImage   *image,
+                                  const gchar *undo_desc,
+                                  GimpPath    *path,
+                                  GimpPath    *prev_parent,
+                                  gint         prev_position,
+                                  GList       *prev_paths)
 {
   GList *iter;
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
-  g_return_val_if_fail (GIMP_IS_VECTORS (vectors), NULL);
-  g_return_val_if_fail (gimp_item_is_attached (GIMP_ITEM (vectors)), NULL);
-  g_return_val_if_fail (prev_parent == NULL || GIMP_IS_VECTORS (prev_parent),
+  g_return_val_if_fail (GIMP_IS_PATH (path), NULL);
+  g_return_val_if_fail (gimp_item_is_attached (GIMP_ITEM (path)), NULL);
+  g_return_val_if_fail (prev_parent == NULL || GIMP_IS_PATH (prev_parent),
                         NULL);
 
-  for (iter = prev_vectors; iter; iter = iter->next)
-    g_return_val_if_fail (GIMP_IS_VECTORS (iter->data), NULL);
+  for (iter = prev_paths; iter; iter = iter->next)
+    g_return_val_if_fail (GIMP_IS_PATH (iter->data), NULL);
 
-  return gimp_image_undo_push (image, GIMP_TYPE_VECTORS_UNDO,
-                               GIMP_UNDO_VECTORS_REMOVE, undo_desc,
+  return gimp_image_undo_push (image, GIMP_TYPE_PATH_UNDO,
+                               GIMP_UNDO_PATH_REMOVE, undo_desc,
                                GIMP_DIRTY_IMAGE_STRUCTURE,
-                               "item",          vectors,
+                               "item",          path,
                                "prev-parent",   prev_parent,
                                "prev-position", prev_position,
-                               "prev-vectors",  prev_vectors,
+                               "prev-paths",    prev_paths,
                                NULL);
 }
 
 GimpUndo *
-gimp_image_undo_push_vectors_mod (GimpImage   *image,
-                                  const gchar *undo_desc,
-                                  GimpVectors *vectors)
+gimp_image_undo_push_path_mod (GimpImage   *image,
+                               const gchar *undo_desc,
+                               GimpPath    *path)
 {
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
-  g_return_val_if_fail (GIMP_IS_VECTORS (vectors), NULL);
-  g_return_val_if_fail (gimp_item_is_attached (GIMP_ITEM (vectors)), NULL);
+  g_return_val_if_fail (GIMP_IS_PATH (path), NULL);
+  g_return_val_if_fail (gimp_item_is_attached (GIMP_ITEM (path)), NULL);
 
-  return gimp_image_undo_push (image, GIMP_TYPE_VECTORS_MOD_UNDO,
-                               GIMP_UNDO_VECTORS_MOD, undo_desc,
-                               GIMP_DIRTY_ITEM | GIMP_DIRTY_VECTORS,
-                               "item", vectors,
+  return gimp_image_undo_push (image, GIMP_TYPE_PATH_MOD_UNDO,
+                               GIMP_UNDO_PATH_MOD, undo_desc,
+                               GIMP_DIRTY_ITEM | GIMP_DIRTY_PATH,
+                               "item", path,
                                NULL);
 }
 
