@@ -243,18 +243,18 @@ gimp_layer_mask_convert_type (GimpDrawable      *drawable,
 }
 
 GimpLayerMask *
-gimp_layer_mask_new (GimpImage     *image,
-                     gint           width,
-                     gint           height,
-                     const gchar   *name,
-                     const GimpRGB *color)
+gimp_layer_mask_new (GimpImage   *image,
+                     gint         width,
+                     gint         height,
+                     const gchar *name,
+                     GeglColor   *color)
 {
   GimpLayerMask *layer_mask;
 
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (width > 0, NULL);
   g_return_val_if_fail (height > 0, NULL);
-  g_return_val_if_fail (color != NULL, NULL);
+  g_return_val_if_fail (GEGL_IS_COLOR (color), NULL);
 
   layer_mask =
     GIMP_LAYER_MASK (gimp_drawable_new (GIMP_TYPE_LAYER_MASK,
