@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,31 +12,33 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <time.h>
 
-#include <glib-object.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
 
 #include "core-types.h"
 
-#include "base/temp-buf.h"
-
 #include "config/gimpcoreconfig.h"
 
 #include "gimp.h"
 #include "gimpcontext.h"
 #include "gimpimage.h"
+#include "gimpimage-undo.h"
 #include "gimpmarshal.h"
+#include "gimptempbuf.h"
 #include "gimpundo.h"
 #include "gimpundostack.h"
 
+#include "gimp-priorities.h"
+
 #include "gimp-intl.h"
 
 
@@ -57,42 +59,40 @@ enum
 };
 
 
-static GObject * gimp_undo_constructor         (GType                type,
-                                                guint                n_params,
-                                                GObjectConstructParam *params);
-static void      gimp_undo_finalize            (GObject             *object);
-static void      gimp_undo_set_property        (GObject             *object,
-                                                guint                property_id,
-                                                const GValue        *value,
-                                                GParamSpec          *pspec);
-static void      gimp_undo_get_property        (GObject             *object,
-                                                guint                property_id,
-                                                GValue              *value,
-                                                GParamSpec          *pspec);
+static void          gimp_undo_constructed         (GObject             *object);
+static void          gimp_undo_finalize            (GObject             *object);
+static void          gimp_undo_set_property        (GObject             *object,
+                                                    guint                property_id,
+                                                    const GValue        *value,
+                                                    GParamSpec          *pspec);
+static void          gimp_undo_get_property        (GObject             *object,
+                                                    guint                property_id,
+                                                    GValue              *value,
+                                                    GParamSpec          *pspec);
 
-static gint64    gimp_undo_get_memsize         (GimpObject          *object,
-                                                gint64              *gui_size);
+static gint64        gimp_undo_get_memsize         (GimpObject          *object,
+                                                    gint64              *gui_size);
 
-static gboolean  gimp_undo_get_popup_size      (GimpViewable        *viewable,
-                                                gint                 width,
-                                                gint                 height,
-                                                gboolean             dot_for_dot,
-                                                gint                *popup_width,
-                                                gint                *popup_height);
-static TempBuf * gimp_undo_get_new_preview     (GimpViewable        *viewable,
-                                                GimpContext         *context,
-                                                gint                 width,
-                                                gint                 height);
+static gboolean      gimp_undo_get_popup_size      (GimpViewable        *viewable,
+                                                    gint                 width,
+                                                    gint                 height,
+                                                    gboolean             dot_for_dot,
+                                                    gint                *popup_width,
+                                                    gint                *popup_height);
+static GimpTempBuf * gimp_undo_get_new_preview     (GimpViewable        *viewable,
+                                                    GimpContext         *context,
+                                                    gint                 width,
+                                                    gint                 height);
 
-static void      gimp_undo_real_pop            (GimpUndo            *undo,
-                                                GimpUndoMode         undo_mode,
-                                                GimpUndoAccumulator *accum);
-static void      gimp_undo_real_free           (GimpUndo            *undo,
-                                                GimpUndoMode         undo_mode);
+static void          gimp_undo_real_pop            (GimpUndo            *undo,
+                                                    GimpUndoMode         undo_mode,
+                                                    GimpUndoAccumulator *accum);
+static void          gimp_undo_real_free           (GimpUndo            *undo,
+                                                    GimpUndoMode         undo_mode);
 
-static gboolean  gimp_undo_create_preview_idle (gpointer             data);
-static void   gimp_undo_create_preview_private (GimpUndo            *undo,
-                                                GimpContext         *context);
+static gboolean      gimp_undo_create_preview_idle (gpointer             data);
+static void       gimp_undo_create_preview_private (GimpUndo            *undo,
+                                                    GimpContext         *context);
 
 
 G_DEFINE_TYPE (GimpUndo, gimp_undo, GIMP_TYPE_VIEWABLE)
@@ -125,24 +125,23 @@ gimp_undo_class_init (GimpUndoClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpUndoClass, free),
-                  NULL, NULL,
-                  gimp_marshal_VOID__ENUM,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   GIMP_TYPE_UNDO_MODE);
 
-  object_class->constructor        = gimp_undo_constructor;
-  object_class->finalize           = gimp_undo_finalize;
-  object_class->set_property       = gimp_undo_set_property;
-  object_class->get_property       = gimp_undo_get_property;
+  object_class->constructed         = gimp_undo_constructed;
+  object_class->finalize            = gimp_undo_finalize;
+  object_class->set_property        = gimp_undo_set_property;
+  object_class->get_property        = gimp_undo_get_property;
 
-  gimp_object_class->get_memsize   = gimp_undo_get_memsize;
+  gimp_object_class->get_memsize    = gimp_undo_get_memsize;
 
-  viewable_class->default_stock_id = "gtk-undo";
-  viewable_class->get_popup_size   = gimp_undo_get_popup_size;
-  viewable_class->get_new_preview  = gimp_undo_get_new_preview;
+  viewable_class->default_icon_name = "edit-undo";
+  viewable_class->get_popup_size    = gimp_undo_get_popup_size;
+  viewable_class->get_new_preview   = gimp_undo_get_new_preview;
 
-  klass->pop                       = gimp_undo_real_pop;
-  klass->free                      = gimp_undo_real_free;
+  klass->pop                        = gimp_undo_real_pop;
+  klass->free                       = gimp_undo_real_free;
 
   g_object_class_install_property (object_class, PROP_IMAGE,
                                    g_param_spec_object ("image", NULL, NULL,
@@ -177,21 +176,14 @@ gimp_undo_init (GimpUndo *undo)
   undo->time = time (NULL);
 }
 
-static GObject *
-gimp_undo_constructor (GType                  type,
-                       guint                  n_params,
-                       GObjectConstructParam *params)
+static void
+gimp_undo_constructed (GObject *object)
 {
-  GObject  *object;
-  GimpUndo *undo;
+  GimpUndo *undo = GIMP_UNDO (object);
 
-  object = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  undo = GIMP_UNDO (object);
-
-  g_assert (GIMP_IS_IMAGE (undo->image));
-
-  return object;
+  gimp_assert (GIMP_IS_IMAGE (undo->image));
 }
 
 static void
@@ -205,11 +197,7 @@ gimp_undo_finalize (GObject *object)
       undo->preview_idle_id = 0;
     }
 
-  if (undo->preview)
-    {
-      temp_buf_free (undo->preview);
-      undo->preview = NULL;
-    }
+  g_clear_pointer (&undo->preview, gimp_temp_buf_unref);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -280,7 +268,7 @@ gimp_undo_get_memsize (GimpObject *object,
   GimpUndo *undo    = GIMP_UNDO (object);
   gint64    memsize = 0;
 
-  *gui_size += temp_buf_get_memsize (undo->preview);
+  *gui_size += gimp_temp_buf_get_memsize (undo->preview);
 
   return memsize + GIMP_OBJECT_CLASS (parent_class)->get_memsize (object,
                                                                   gui_size);
@@ -297,10 +285,11 @@ gimp_undo_get_popup_size (GimpViewable *viewable,
   GimpUndo *undo = GIMP_UNDO (viewable);
 
   if (undo->preview &&
-      (undo->preview->width > width || undo->preview->height > height))
+      (gimp_temp_buf_get_width  (undo->preview) > width ||
+       gimp_temp_buf_get_height (undo->preview) > height))
     {
-      *popup_width  = undo->preview->width;
-      *popup_height = undo->preview->height;
+      *popup_width  = gimp_temp_buf_get_width  (undo->preview);
+      *popup_height = gimp_temp_buf_get_height (undo->preview);
 
       return TRUE;
     }
@@ -308,7 +297,7 @@ gimp_undo_get_popup_size (GimpViewable *viewable,
   return FALSE;
 }
 
-static TempBuf *
+static GimpTempBuf *
 gimp_undo_get_new_preview (GimpViewable *viewable,
                            GimpContext  *context,
                            gint          width,
@@ -321,8 +310,8 @@ gimp_undo_get_new_preview (GimpViewable *viewable,
       gint preview_width;
       gint preview_height;
 
-      gimp_viewable_calc_preview_size (undo->preview->width,
-                                       undo->preview->height,
+      gimp_viewable_calc_preview_size (gimp_temp_buf_get_width  (undo->preview),
+                                       gimp_temp_buf_get_height (undo->preview),
                                        width,
                                        height,
                                        TRUE, 1.0, 1.0,
@@ -330,13 +319,14 @@ gimp_undo_get_new_preview (GimpViewable *viewable,
                                        &preview_height,
                                        NULL);
 
-      if (preview_width  < undo->preview->width &&
-          preview_height < undo->preview->height)
+      if (preview_width  < gimp_temp_buf_get_width  (undo->preview) &&
+          preview_height < gimp_temp_buf_get_height (undo->preview))
         {
-          return temp_buf_scale (undo->preview, preview_width, preview_height);
+          return gimp_temp_buf_scale (undo->preview,
+                                      preview_width, preview_height);
         }
 
-      return temp_buf_copy (undo->preview, NULL);
+      return gimp_temp_buf_copy (undo->preview);
     }
 
   return NULL;
@@ -431,7 +421,7 @@ gimp_undo_create_preview (GimpUndo    *undo,
         idle->context = g_object_ref (context);
 
       undo->preview_idle_id =
-        g_idle_add_full (GIMP_VIEWABLE_PRIORITY_IDLE,
+        g_idle_add_full (GIMP_PRIORITY_VIEWABLE_IDLE,
                          gimp_undo_create_preview_idle, idle,
                          (GDestroyNotify) gimp_undo_idle_free);
     }
@@ -440,9 +430,10 @@ gimp_undo_create_preview (GimpUndo    *undo,
 static gboolean
 gimp_undo_create_preview_idle (gpointer data)
 {
-  GimpUndoIdle *idle = data;
+  GimpUndoIdle *idle   = data;
+  GimpUndoStack *stack = gimp_image_get_undo_stack (idle->undo->image);
 
-  if (idle->undo == gimp_undo_stack_peek (idle->undo->image->undo_stack))
+  if (idle->undo == gimp_undo_stack_peek (stack))
     {
       gimp_undo_create_preview_private (idle->undo, idle->context);
     }
@@ -517,8 +508,7 @@ gimp_undo_refresh_preview (GimpUndo    *undo,
 
   if (undo->preview)
     {
-      temp_buf_free (undo->preview);
-      undo->preview = NULL;
+      g_clear_pointer (&undo->preview, gimp_temp_buf_unref);
       gimp_undo_create_preview (undo, context, FALSE);
     }
 }
@@ -537,13 +527,9 @@ gimp_undo_type_to_name (GimpUndoType type)
 gboolean
 gimp_undo_is_weak (GimpUndo *undo)
 {
-  GimpUndoType type;
-
   if (! undo)
     return FALSE;
 
-  type = undo->undo_type;
-
   switch (undo->undo_type)
     {
     case GIMP_UNDO_GROUP_ITEM_VISIBILITY:
@@ -568,7 +554,7 @@ gimp_undo_is_weak (GimpUndo *undo)
  * gimp_undo_get_age:
  * @undo:
  *
- * Return value: the time in seconds since this undo item was created
+ * Returns: the time in seconds since this undo item was created
  */
 gint
 gimp_undo_get_age (GimpUndo *undo)
