@@ -94,7 +94,7 @@ gimp_display_shell_appearance_update (GimpDisplayShell *shell)
                                               options->show_sample_points);
   gimp_display_shell_set_padding             (shell,
                                               options->padding_mode,
-                                              &options->padding_color);
+                                              options->padding_color);
   gimp_display_shell_set_padding_in_show_all (shell,
                                               options->padding_in_show_all);
 }
@@ -465,21 +465,63 @@ gimp_display_shell_get_snap_to_vectors (GimpDisplayShell *shell)
   return appearance_get_options (shell)->snap_to_path;
 }
 
+void
+gimp_display_shell_set_snap_to_bbox (GimpDisplayShell *shell,
+                                     gboolean          snap)
+{
+  GimpDisplayOptions *options;
+
+  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+
+  options = appearance_get_options (shell);
+
+  g_object_set (options, "snap-to-bbox", snap, NULL);
+}
+
+gboolean
+gimp_display_shell_get_snap_to_bbox (GimpDisplayShell *shell)
+{
+  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
+
+  return appearance_get_options (shell)->snap_to_bbox;
+}
+
+void
+gimp_display_shell_set_snap_to_equidistance (GimpDisplayShell *shell,
+                                             gboolean          snap)
+{
+  GimpDisplayOptions *options;
+
+  g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
+
+  options = appearance_get_options (shell);
+
+  g_object_set (options, "snap-to-equidistance", snap, NULL);
+}
+
+gboolean
+gimp_display_shell_get_snap_to_equidistance (GimpDisplayShell *shell)
+{
+  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), FALSE);
+
+  return appearance_get_options (shell)->snap_to_equidistance;
+}
+
 void
 gimp_display_shell_set_padding (GimpDisplayShell      *shell,
                                 GimpCanvasPaddingMode  padding_mode,
-                                const GimpRGB         *padding_color)
+                                GeglColor             *padding_color)
 {
   GimpImageWindow    *window;
   GimpMenuModel      *model;
   GimpDisplayOptions *options;
-  GimpRGB             color;
+  GeglColor          *color;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
-  g_return_if_fail (padding_color != NULL);
+  g_return_if_fail (GEGL_IS_COLOR (padding_color));
 
   options = appearance_get_options (shell);
-  color   = *padding_color;
+  color   = padding_color;
 
   switch (padding_mode)
     {
@@ -487,11 +529,11 @@ gimp_display_shell_set_padding (GimpDisplayShell      *shell,
       break;
 
     case GIMP_CANVAS_PADDING_MODE_LIGHT_CHECK:
-      color = *gimp_render_check_color1 ();
+      color = GEGL_COLOR (gimp_render_check_color1 ());
       break;
 
     case GIMP_CANVAS_PADDING_MODE_DARK_CHECK:
-      color = *gimp_render_check_color2 ();
+      color = GEGL_COLOR (gimp_render_check_color2 ());
       break;
 
     case GIMP_CANVAS_PADDING_MODE_CUSTOM:
@@ -499,28 +541,30 @@ gimp_display_shell_set_padding (GimpDisplayShell      *shell,
       break;
     }
 
+  color = gegl_color_duplicate (color);
   g_object_set (options,
                 "padding-mode",  padding_mode,
-                "padding-color", &color,
+                "padding-color", color,
                 NULL);
 
   gimp_canvas_set_padding (GIMP_CANVAS (shell->canvas),
-                           padding_mode, &color);
+                           padding_mode, color);
 
   window = gimp_display_shell_get_window (shell);
   model  = gimp_image_window_get_menubar_model (window);
+
   if (padding_mode != GIMP_CANVAS_PADDING_MODE_DEFAULT)
-    gimp_menu_model_set_color (model, "/View/Padding color",
-                               &options->padding_color);
+    gimp_menu_model_set_color (model, "/View/Padding color", options->padding_color);
   else
-    gimp_menu_model_set_color (model, "/View/Padding color",
-                               NULL);
+    gimp_menu_model_set_color (model, "/View/Padding color", NULL);
+
+  g_object_unref (color);
 }
 
 void
-gimp_display_shell_get_padding (GimpDisplayShell      *shell,
-                                GimpCanvasPaddingMode *padding_mode,
-                                GimpRGB               *padding_color)
+gimp_display_shell_get_padding (GimpDisplayShell       *shell,
+                                GimpCanvasPaddingMode  *padding_mode,
+                                GeglColor             **padding_color)
 {
   GimpDisplayOptions *options;
 
@@ -532,7 +576,7 @@ gimp_display_shell_get_padding (GimpDisplayShell      *shell,
     *padding_mode = options->padding_mode;
 
   if (padding_color)
-    *padding_color = options->padding_color;
+    *padding_color = gegl_color_duplicate (options->padding_color);
 }
 
 void
