@@ -22,13 +22,19 @@
 
 #include <string.h>
 
+#include <cairo.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
 #include <gegl-plugin.h>
 
+#include "libgimpcolor/gimpcolor.h"
+
 #include "gimp-gegl-types.h"
 
+#include "core/gimppattern.h"
 #include "core/gimpprogress.h"
 
+#include "gimp-babl.h"
 #include "gimp-gegl-loops.h"
 #include "gimp-gegl-utils.h"
 
@@ -87,22 +93,6 @@ gimp_gegl_get_op_enum_type (const gchar *operation,
   return G_TYPE_FROM_CLASS (G_PARAM_SPEC_ENUM (pspec)->enum_class);
 }
 
-GeglColor *
-gimp_gegl_color_new (const GimpRGB *rgb,
-                     const Babl    *space)
-{
-  GeglColor *color;
-
-  g_return_val_if_fail (rgb != NULL, NULL);
-
-  color = gegl_color_new (NULL);
-  gegl_color_set_pixel (color,
-                        babl_format_with_space ("R'G'B'A double", space),
-                        rgb);
-
-  return color;
-}
-
 static void
 gimp_gegl_progress_callback (GObject      *object,
                              gdouble       value,
@@ -135,15 +125,27 @@ gimp_gegl_progress_connect (GeglNode     *node,
   g_return_if_fail (GIMP_IS_PROGRESS (progress));
   g_return_if_fail (text != NULL);
 
-  g_signal_connect (node, "progress",
-                    G_CALLBACK (gimp_gegl_progress_callback),
-                    progress);
+  g_signal_connect_object (node, "progress",
+                           G_CALLBACK (gimp_gegl_progress_callback),
+                           progress, 0);
 
   g_object_set_data_full (G_OBJECT (node),
                           "gimp-progress-text", g_strdup (text),
                           (GDestroyNotify) g_free);
 }
 
+void
+gimp_gegl_progress_disconnect (GeglNode     *node,
+                               GimpProgress *progress)
+{
+  g_return_if_fail (GEGL_IS_NODE (node));
+  g_return_if_fail (GIMP_IS_PROGRESS (progress));
+
+  g_signal_handlers_disconnect_by_func (node,
+                                        gimp_gegl_progress_callback,
+                                        progress);
+}
+
 gboolean
 gimp_gegl_node_is_source_operation (GeglNode *node)
 {
@@ -333,6 +335,96 @@ gimp_gegl_buffer_dup (GeglBuffer *buffer)
   return new_buffer;
 }
 
+GeglBuffer *
+gimp_gegl_buffer_resize (GeglBuffer   *buffer,
+                         gint          new_width,
+                         gint          new_height,
+                         gint          offset_x,
+                         gint          offset_y,
+                         GeglColor    *color,
+                         GimpPattern  *pattern,
+                         gint          pattern_offset_x,
+                         gint          pattern_offset_y)
+{
+  GeglBuffer          *new_buffer;
+  gboolean             intersect;
+  GeglRectangle        copy_rect;
+  const GeglRectangle *extent;
+  const Babl          *format;
+
+  g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
+
+  extent     = gegl_buffer_get_extent (buffer);
+  format     = gegl_buffer_get_format (buffer);
+  new_buffer = gegl_buffer_new (GEGL_RECTANGLE (0, 0, new_width, new_height),
+                                format);
+
+  intersect = gegl_rectangle_intersect (&copy_rect,
+                                        GEGL_RECTANGLE (0, 0, extent->width,
+                                                        extent->height),
+                                        GEGL_RECTANGLE (offset_x, offset_y,
+                                                        new_width, new_height));
+
+  if (! intersect                   ||
+      copy_rect.width  != new_width ||
+      copy_rect.height != new_height)
+    {
+      /*  Clear the new buffer if needed and color/pattern is given */
+      if (pattern)
+        {
+          GeglBuffer       *src_buffer;
+          GeglBuffer       *dest_buffer;
+          GimpColorProfile *src_profile;
+          GimpColorProfile *dest_profile;
+
+          src_buffer = gimp_pattern_create_buffer (pattern);
+
+          src_profile  = gimp_babl_format_get_color_profile (
+                           gegl_buffer_get_format (src_buffer));
+          dest_profile = gimp_babl_format_get_color_profile (
+                           gegl_buffer_get_format (new_buffer));
+
+          if (gimp_color_transform_can_gegl_copy (src_profile, dest_profile))
+            {
+              dest_buffer = g_object_ref (src_buffer);
+            }
+          else
+            {
+              dest_buffer = gegl_buffer_new (gegl_buffer_get_extent (src_buffer),
+                                             gegl_buffer_get_format (new_buffer));
+
+              gimp_gegl_convert_color_profile (src_buffer,  NULL, src_profile,
+                                               dest_buffer, NULL, dest_profile,
+                                               GIMP_COLOR_RENDERING_INTENT_PERCEPTUAL,
+                                               TRUE, NULL);
+            }
+
+          g_object_unref (src_profile);
+          g_object_unref (dest_profile);
+
+          gegl_buffer_set_pattern (new_buffer, NULL, dest_buffer,
+                                   pattern_offset_x, pattern_offset_y);
+
+          g_object_unref (src_buffer);
+          g_object_unref (dest_buffer);
+        }
+      else if (color)
+        {
+          gegl_buffer_set_color (new_buffer, NULL, color);
+        }
+    }
+
+  if (intersect && copy_rect.width && copy_rect.height)
+    {
+      /*  Copy the pixels in the intersection  */
+      gimp_gegl_buffer_copy (buffer, &copy_rect, GEGL_ABYSS_NONE, new_buffer,
+                             GEGL_RECTANGLE (copy_rect.x - offset_x,
+                                             copy_rect.y - offset_y, 0, 0));
+    }
+
+  return new_buffer;
+}
+
 gboolean
 gimp_gegl_buffer_set_extent (GeglBuffer          *buffer,
                              const GeglRectangle *extent)
@@ -443,6 +535,8 @@ gimp_gegl_op_blacklisted (const gchar *name,
     "gegl:high-pass",
     "gegl:hue-chroma",
     "gegl:illusion",
+    "gegl:json:dropshadow2",
+    "gegl:json:grey2",
     "gegl:image-gradient",
     "gegl:invert-linear",
     "gegl:invert-gamma",
@@ -522,7 +616,7 @@ gimp_gegl_op_blacklisted (const gchar *name,
     "gegl:ditto", /* pointless */
     "gegl:fill-path",
     "gegl:gray", /* we use gimp's op */
-    "gegl:hstack", /* pointless */
+    "gegl:hstack", /* deleted from GEGL and replaced by gegl:pack */
     "gegl:introspect", /* pointless */
     "gegl:layer", /* we use gimp's ops */
     "gegl:lcms-from-profile", /* not usable here */
@@ -532,6 +626,7 @@ gimp_gegl_op_blacklisted (const gchar *name,
     "gegl:matting-global", /* used in the foreground select tool */
     "gegl:matting-levin", /* used in the foreground select tool */
     "gegl:opacity", /* poinless */
+    "gegl:pack", /* pointless */
     "gegl:path",
     "gegl:posterize", /* we use gimp's op */
     "gegl:radial-gradient", /* we use the blend tool */
