@@ -35,8 +35,10 @@
 #include "libgimpbase/gimpbase.h"
 
 #include "core/core-types.h"
+#include "core/gimp.h"
 
 #include "config/gimpcoreconfig.h"
+#include "config/gimpguiconfig.h"
 
 #ifndef GIMP_CONSOLE_COMPILATION
 #include "dialogs/about-dialog.h"
@@ -68,7 +70,9 @@ static void          gimp_update_about_dialog        (GimpCoreConfig   *config,
 static gboolean      gimp_version_break              (const gchar      *v,
                                                       gint             *major,
                                                       gint             *minor,
-                                                      gint             *micro);
+                                                      gint             *micro,
+                                                      gint             *rc,
+                                                      gboolean         *is_git);
 static gint          gimp_version_cmp                (const gchar      *v1,
                                                       const gchar      *v2);
 
@@ -363,7 +367,7 @@ gimp_check_updates_process (const gchar    *source,
   gimp_update_get_highest (parser, &last_version, &release_timestamp,
                            &build_revision, &build_comment, FALSE);
 
-#ifdef GIMP_UNSTABLE
+#if defined(GIMP_UNSTABLE) || defined(GIMP_RC_VERSION)
     {
       gchar  *dev_version = NULL;
       gchar  *dev_comment      = NULL;
@@ -435,14 +439,18 @@ gimp_update_about_dialog (GimpCoreConfig   *config,
                           const GParamSpec *pspec,
                           gpointer          user_data)
 {
+#ifndef GIMP_CONSOLE_COMPILATION
+  Gimp *gimp = user_data;
+#endif
+
   g_signal_handlers_disconnect_by_func (config,
                                         (GCallback) gimp_update_about_dialog,
-                                        NULL);
+                                        user_data);
 
   if (config->last_known_release != NULL)
     {
 #ifndef GIMP_CONSOLE_COMPILATION
-      gtk_widget_show (about_dialog_create (config));
+      gtk_widget_show (about_dialog_create (gimp, config));
 #else
       g_printerr (_("A new version of GIMP (%s) was released.\n"
                     "It is recommended to update."),
@@ -455,13 +463,17 @@ static gboolean
 gimp_version_break (const gchar *v,
                     gint        *major,
                     gint        *minor,
-                    gint        *micro)
+                    gint        *micro,
+                    gint        *rc,
+                    gboolean    *is_git)
 {
   gchar **versions;
 
-  *major = 0;
-  *minor = 0;
-  *micro = 0;
+  *major  = 0;
+  *minor  = 0;
+  *micro  = 0;
+  *rc     = 0;
+  *is_git = FALSE;
 
   if (v == NULL)
     return FALSE;
@@ -475,10 +487,40 @@ gimp_version_break (const gchar *v,
           *minor = g_ascii_strtoll (versions[1], NULL, 10);
           if (versions[2] != NULL)
             {
+              gchar **micro_rc_git;
+
               *micro = g_ascii_strtoll (versions[2], NULL, 10);
+
+              micro_rc_git = g_strsplit_set (versions[2], "-", 2);
+
+              if (g_strv_length (micro_rc_git) > 1 &&
+                  strlen (micro_rc_git[1]) > 2     &&
+                  micro_rc_git[1][0] == 'R'           &&
+                  micro_rc_git[1][1] == 'C')
+                {
+                  gchar **rc_git;
+
+                  *rc = g_ascii_strtoll (micro_rc_git[1] + 2, NULL, 10);
+
+                  rc_git = g_strsplit_set (micro_rc_git[1], "+", 2);
+
+                  if (g_strv_length (rc_git) > 1 &&
+                      strlen (rc_git[1]) == 3    &&
+                      rc_git[1][0] == 'g'        &&
+                      rc_git[1][1] == 'i'        &&
+                      rc_git[1][2] == 't')
+                    {
+                      *is_git = TRUE;
+                    }
+
+                  g_strfreev (rc_git);
+                }
+
+              g_strfreev (micro_rc_git);
             }
         }
     }
+
   g_strfreev (versions);
 
   return (*major > 0 || *minor > 0 || *micro > 0);
@@ -502,13 +544,24 @@ gimp_version_cmp (const gchar *v1,
   gint     major1;
   gint     minor1;
   gint     micro1;
-  gint     major2 = GIMP_MAJOR_VERSION;
-  gint     minor2 = GIMP_MINOR_VERSION;
-  gint     micro2 = GIMP_MICRO_VERSION;
+  gint     rc1;
+  gboolean is_git1;
+  gint     major2  = GIMP_MAJOR_VERSION;
+  gint     minor2  = GIMP_MINOR_VERSION;
+  gint     micro2  = GIMP_MICRO_VERSION;
+  gint     rc2     = 0;
+  gboolean is_git2 = FALSE;
+
+#if defined(GIMP_RC_VERSION)
+  rc2 = GIMP_RC_VERSION;
+#if defined(GIMP_IS_RC_GIT)
+  is_git2 = TRUE;
+#endif
+#endif
 
   g_return_val_if_fail (v1 != NULL, -1);
 
-  if (! gimp_version_break (v1, &major1, &minor1, &micro1))
+  if (! gimp_version_break (v1, &major1, &minor1, &micro1, &rc1, &is_git1))
     {
       /* If version is not properly parsed, something is wrong with
        * upstream version number or parsing. This should not happen.
@@ -518,7 +571,7 @@ gimp_version_cmp (const gchar *v1,
 
       return -1;
     }
-  if (v2 && ! gimp_version_break (v2, &major2, &minor2, &micro2))
+  if (v2 && ! gimp_version_break (v2, &major2, &minor2, &micro2, &rc2, &is_git2))
     {
       g_printerr ("%s: version not properly formatted: %s\n",
                   G_STRFUNC, v2);
@@ -526,11 +579,16 @@ gimp_version_cmp (const gchar *v1,
       return 1;
     }
 
-  if (major1 == major2 && minor1 == minor2 && micro1 == micro2)
+  if (major1 == major2 && minor1 == minor2 && micro1 == micro2 &&
+      rc1 == rc2 && is_git1 == is_git2)
     return 0;
-  else if (major1 > major2 ||
-           (major1 == major2 && minor1 > minor2) ||
-           (major1 == major2 && minor1 == minor2 && micro1 > micro2))
+  else if (major1 > major2                                                                    ||
+           (major1 == major2 && minor1 > minor2)                                              ||
+           (major1 == major2 && minor1 == minor2 && micro1 > micro2)                          ||
+           /* RC 0 is the real release, so it's "higher" than any other. */
+           (major1 == major2 && minor1 == minor2 && micro1 == micro2 && rc1 == 0 && rc2 > 0)  ||
+           (major1 == major2 && minor1 == minor2 && micro1 == micro2 && rc1 > rc2 && rc2 > 0) ||
+           (major1 == major2 && minor1 == minor2 && micro1 == micro2 && rc1 == rc2 && is_git1))
     return 1;
   else
     return -1;
@@ -539,7 +597,7 @@ gimp_version_cmp (const gchar *v1,
 static const gchar *
 gimp_get_version_url (void)
 {
-#ifdef GIMP_RELEASE
+#if defined(GIMP_RELEASE) && ! defined(GIMP_RC_VERSION)
   return "https://www.gimp.org/gimp_versions.json";
 #else
   if (g_getenv ("GIMP_DEV_VERSIONS_JSON"))
@@ -589,20 +647,29 @@ gboolean
 gimp_update_auto_check (GimpCoreConfig *config,
                         Gimp           *gimp)
 {
-  gint64 prev_update_timestamp;
-  gint64 current_timestamp;
+  gboolean is_update;
+  gint64   prev_update_timestamp;
+  gint64   current_timestamp;
 
-  if (config->config_version == NULL ||
-      gimp_version_cmp (GIMP_VERSION,
-                        config->config_version) > 0)
+  is_update = (config->config_version == NULL ||
+               gimp_version_cmp (GIMP_VERSION,
+                                 config->config_version) > 0);
+
+  if (is_update || GIMP_GUI_CONFIG (config)->show_welcome_dialog)
     {
 #ifndef GIMP_CONSOLE_COMPILATION
-      /* GIMP was just updated and this is the first time the new
-       * version is run. Display a welcome dialog, and do not check for
-       * updates right now. */
-      gtk_widget_show (welcome_dialog_create (gimp));
+      /* If GIMP was just updated and this is the first time the new
+       * version is run, display a welcome dialog and do not check for
+       * updates right now. Otherwise, if the user has set the welcome
+       * dialog to always appear on load, show the Create page on start.
+       */
+      if (! gimp->no_interface)
+        gtk_widget_set_visible (welcome_dialog_create (gimp, is_update),
+                                TRUE);
 
-      return FALSE;
+      /* Do not check for new updates when GIMP was just updated. */
+      if (is_update)
+        return FALSE;
 #else
       g_log (G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
              "Welcome to GIMP %s!", GIMP_VERSION);
@@ -613,8 +680,8 @@ gimp_update_auto_check (GimpCoreConfig *config,
 #ifdef CHECK_UPDATE
   /* Allows to disable updates at package level with a build having the
    * version check code built-in.
-   * For instance, it would allow to use the same Windows installer for
-   * the Windows Store (with update check disabled because it comes with
+   * For instance, it allows to use the same bundling artifact for the
+   * Microsoft Store (with update check disabled because it comes with
    * its own update channel).
    */
   if (! gimp_version_check_update () ||
@@ -639,7 +706,7 @@ gimp_update_auto_check (GimpCoreConfig *config,
 
   g_signal_connect (config, "notify::last-known-release",
                     (GCallback) gimp_update_about_dialog,
-                    NULL);
+                    gimp);
 
   gimp_update_check (config);
 
