@@ -29,7 +29,8 @@ GFile  * themes_get_theme_dir  (Gimp        *gimp,
 GFile  * themes_get_theme_file (Gimp        *gimp,
                                 const gchar *first_component,
                                 ...) G_GNUC_NULL_TERMINATED;
-
+#ifdef G_OS_WIN32
 void     themes_set_title_bar  (Gimp        *gimp);
+#endif
 
 #endif /* __THEMES_H__ */
