@@ -544,7 +544,6 @@ main (int    argc,
   GOptionContext *context;
   GError         *error = NULL;
   const gchar    *abort_message;
-  gchar          *basename;
   GFile          *system_gimprc_file = NULL;
   GFile          *user_gimprc_file   = NULL;
   GOptionGroup   *gimp_group         = NULL;
@@ -595,6 +594,15 @@ main (int    argc,
   gimp_init_signal_handlers (&backtrace_file);
 
 #ifdef G_OS_WIN32
+  /* Make Inno aware of gimp process avoiding broken install/unninstall */
+  char    *utf8_name = g_strdup_printf ("GIMP-%s", GIMP_MUTEX_VERSION);
+  wchar_t *name      = g_utf8_to_utf16 (utf8_name, -1, NULL, NULL, NULL);
+  
+  CreateMutexW (NULL, FALSE, name);
+  
+  g_free (utf8_name);
+  g_free (name);
+  
   /* Enable Anti-Aliasing*/
   g_setenv ("PANGOCAIRO_BACKEND", "fc", TRUE);
 
@@ -665,9 +673,7 @@ main (int    argc,
   argv = g_strdupv (argv);
 #endif
 
-  basename = g_path_get_basename (argv[0]);
-  g_set_prgname (basename);
-  g_free (basename);
+  g_set_prgname (GIMP_DESKTOP_NAME);
 
   /* Check argv[] for "--verbose" first */
   for (i = 1; i < argc; i++)
