@@ -36,6 +36,7 @@
 
 #include "gegl/gimp-gegl-utils.h"
 
+#include "gimpoperationsettings.h"
 #include "gimp-operation-config.h"
 
 
@@ -56,73 +57,80 @@ static void    gimp_operation_config_add_sep       (GimpContainer    *container)
 static void    gimp_operation_config_remove_sep    (GimpContainer    *container);
 
 
-/*  public functions  */
-
-static GHashTable *
-gimp_operation_config_get_type_table (Gimp *gimp)
-{
-  static GHashTable *config_types = NULL;
-
-  if (! config_types)
-    config_types = g_hash_table_new_full (g_str_hash,
-                                          g_str_equal,
-                                          (GDestroyNotify) g_free,
-                                          NULL);
-
-  return config_types;
-}
-
-static GHashTable *
-gimp_operation_config_get_container_table (Gimp *gimp)
-{
-  static GHashTable *config_containers = NULL;
-
-  if (! config_containers)
-    config_containers = g_hash_table_new_full (g_direct_hash,
-                                               g_direct_equal,
-                                               NULL,
-                                               (GDestroyNotify) g_object_unref);
-
-  return config_containers;
-}
+static GHashTable *config_types      = NULL;
+static GHashTable *config_containers = NULL;
+static GList      *custom_config_ops = NULL;
+static gboolean    custom_init_done  = FALSE;
 
 
 /*  public functions  */
 
+void
+gimp_operation_config_init_start (Gimp *gimp)
+{
+  config_types      = g_hash_table_new_full (g_str_hash,
+                                             g_str_equal,
+                                             (GDestroyNotify) g_free,
+                                             NULL);
+  config_containers = g_hash_table_new_full (g_direct_hash,
+                                             g_direct_equal,
+                                             NULL,
+                                             (GDestroyNotify) g_object_unref);
+}
+
+void
+gimp_operation_config_init_end (Gimp *gimp)
+{
+  custom_init_done = TRUE;
+}
+
+void
+gimp_operation_config_exit (Gimp *gimp)
+{
+  g_hash_table_unref (config_types);
+  g_hash_table_unref (config_containers);
+  g_list_free (custom_config_ops);
+}
+
+
 void
 gimp_operation_config_register (Gimp        *gimp,
                                 const gchar *operation,
                                 GType        config_type)
 {
-  GHashTable *config_types;
-
   g_return_if_fail (GIMP_IS_GIMP (gimp));
   g_return_if_fail (operation != NULL);
   g_return_if_fail (g_type_is_a (config_type, GIMP_TYPE_OBJECT));
 
-  config_types = gimp_operation_config_get_type_table (gimp);
+  if (! custom_init_done)
+    /* Custom ops are registered with static string, not generated names. */
+    custom_config_ops = g_list_prepend (custom_config_ops, (gpointer) operation);
 
   g_hash_table_insert (config_types,
                        g_strdup (operation),
                        (gpointer) config_type);
 }
 
+gboolean
+gimp_operation_config_is_custom (Gimp        *gimp,
+                                 const gchar *operation)
+{
+  return (g_list_find_custom (custom_config_ops, operation, (GCompareFunc) g_strcmp0) != NULL);
+}
+
 GType
 gimp_operation_config_get_type (Gimp        *gimp,
                                 const gchar *operation,
                                 const gchar *icon_name,
                                 GType        parent_type)
 {
-  GHashTable *config_types;
-  GType       config_type;
+  GType config_type;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), G_TYPE_NONE);
   g_return_val_if_fail (operation != NULL, G_TYPE_NONE);
   g_return_val_if_fail (g_type_is_a (parent_type, GIMP_TYPE_OBJECT),
                         G_TYPE_NONE);
 
-  config_types = gimp_operation_config_get_type_table (gimp);
-
   config_type = (GType) g_hash_table_lookup (config_types, operation);
 
   if (! config_type)
@@ -143,8 +151,45 @@ gimp_operation_config_get_type (Gimp        *gimp,
               strcmp (pspec->name, "input")     &&
               strcmp (pspec->name, "output"))
             {
-              pspecs[j] = pspec;
-              j++;
+              GParamFlags flags;
+
+              flags = pspec->flags & ~(GEGL_PARAM_PAD_INPUT | GEGL_PARAM_PAD_OUTPUT);
+
+              if (GEGL_IS_PARAM_SPEC_COLOR (pspec))
+                {
+                  /* As special exception, let's transform GeglParamColor
+                   * into GimpParamColor in all core code. This way, we
+                   * have one less param type to handle.
+                   */
+                  gchar **prop_keys;
+                  guint   n_keys = 0;
+
+                  pspecs[j] = gimp_param_spec_color (pspec->name,
+                                                     g_param_spec_get_nick (pspec),
+                                                     g_param_spec_get_blurb (pspec),
+                                                     TRUE,
+                                                     gegl_param_spec_color_get_default (pspec),
+                                                     flags);
+                  prop_keys = gegl_operation_list_property_keys (operation, pspec->name, &n_keys);
+                  for (gint k = 0; k < n_keys; k++)
+                    {
+                      const gchar *key;
+
+                      key = gegl_param_spec_get_property_key (pspec, prop_keys[k]);
+                      gegl_param_spec_set_property_key (pspecs[j], prop_keys[k], key);
+                    }
+
+                  g_free (prop_keys);
+                }
+              else
+                {
+                  pspecs[j] = gimp_config_param_spec_duplicate (pspec);
+                  if (pspecs[j])
+                    pspecs[j]->flags = flags;
+                }
+
+              if (pspecs[j])
+                j++;
             }
         }
 
@@ -182,14 +227,11 @@ gimp_operation_config_get_container (Gimp         *gimp,
                                      GType         config_type,
                                      GCompareFunc  sort_func)
 {
-  GHashTable    *config_containers;
   GimpContainer *container;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
   g_return_val_if_fail (g_type_is_a (config_type, GIMP_TYPE_OBJECT), NULL);
 
-  config_containers = gimp_operation_config_get_container_table (gimp);
-
   container = g_hash_table_lookup (config_containers, (gpointer) config_type);
 
   if (! container)
@@ -365,20 +407,6 @@ gimp_operation_config_sync_node (GObject  *config,
           g_object_get_property (G_OBJECT (config), gimp_pspec->name,
                                  &value);
 
-          if (GEGL_IS_PARAM_SPEC_COLOR (gegl_pspec))
-            {
-              GimpRGB    gimp_color;
-              GeglColor *gegl_color;
-
-              gimp_value_get_rgb (&value, &gimp_color);
-              g_value_unset (&value);
-
-              gegl_color = gimp_gegl_color_new (&gimp_color, NULL);
-
-              g_value_init (&value, gegl_pspec->value_type);
-              g_value_take_object (&value, gegl_color);
-            }
-
           gegl_node_set_property (node, gegl_pspec->name,
                                   &value);
 
@@ -524,20 +552,6 @@ gimp_operation_config_config_notify (GObject          *config,
       g_value_init (&value, gimp_pspec->value_type);
       g_object_get_property (config, gimp_pspec->name, &value);
 
-      if (GEGL_IS_PARAM_SPEC_COLOR (gegl_pspec))
-        {
-          GimpRGB    gimp_color;
-          GeglColor *gegl_color;
-
-          gimp_value_get_rgb (&value, &gimp_color);
-          g_value_unset (&value);
-
-          gegl_color = gimp_gegl_color_new (&gimp_color, NULL);
-
-          g_value_init (&value, gegl_pspec->value_type);
-          g_value_take_object (&value, gegl_color);
-        }
-
       handler = g_signal_handler_find (node,
                                        G_SIGNAL_MATCH_DETAIL |
                                        G_SIGNAL_MATCH_FUNC   |
@@ -576,32 +590,6 @@ gimp_operation_config_node_notify (GeglNode         *node,
       g_value_init (&value, gegl_pspec->value_type);
       gegl_node_get_property (node, gegl_pspec->name, &value);
 
-      if (GEGL_IS_PARAM_SPEC_COLOR (gegl_pspec))
-        {
-          GeglColor *gegl_color;
-          GimpRGB    gimp_color;
-
-          gegl_color = g_value_dup_object (&value);
-          g_value_unset (&value);
-
-          if (gegl_color)
-            {
-              gegl_color_get_rgba (gegl_color,
-                                   &gimp_color.r,
-                                   &gimp_color.g,
-                                   &gimp_color.b,
-                                   &gimp_color.a);
-              g_object_unref (gegl_color);
-            }
-          else
-            {
-              gimp_rgba_set (&gimp_color, 0.0, 0.0, 0.0, 1.0);
-            }
-
-          g_value_init (&value, gimp_pspec->value_type);
-          gimp_value_set_rgb (&value, &gimp_color);
-        }
-
       handler = g_signal_handler_find (config,
                                        G_SIGNAL_MATCH_DETAIL |
                                        G_SIGNAL_MATCH_FUNC   |
