@@ -35,7 +35,7 @@
 #include "core/gimpparamspecs.h"
 #include "core/gimpprogress.h"
 
-#include "vectors/gimpvectors.h"
+#include "vectors/gimppath.h"
 
 #include "gimppdbcontext.h"
 #include "gimppdberror.h"
@@ -107,7 +107,8 @@ gimp_procedure_class_init (GimpProcedureClass *klass)
 static void
 gimp_procedure_init (GimpProcedure *procedure)
 {
-  procedure->proc_type = GIMP_PDB_PROC_TYPE_INTERNAL;
+  procedure->proc_type   = GIMP_PDB_PROC_TYPE_INTERNAL;
+  procedure->is_private  = FALSE;
 }
 
 static void
@@ -282,7 +283,8 @@ gimp_procedure_real_execute_async (GimpProcedure  *procedure,
 /*  public functions  */
 
 GimpProcedure  *
-gimp_procedure_new (GimpMarshalFunc marshal_func)
+gimp_procedure_new (GimpMarshalFunc marshal_func,
+                    gboolean        is_private)
 {
   GimpProcedure *procedure;
 
@@ -291,6 +293,7 @@ gimp_procedure_new (GimpMarshalFunc marshal_func)
   procedure = g_object_new (GIMP_TYPE_PROCEDURE, NULL);
 
   procedure->marshal_func = marshal_func;
+  procedure->is_private   = is_private;
 
   return procedure;
 }
@@ -763,7 +766,7 @@ gimp_procedure_create_override (GimpProcedure   *procedure,
   const gchar   *name          = NULL;
   int            i             = 0;
 
-  new_procedure = gimp_procedure_new (new_marshal_func);
+  new_procedure = gimp_procedure_new (new_marshal_func, procedure->is_private);
   name          = gimp_object_get_name (procedure);
 
   gimp_object_set_static_name (GIMP_OBJECT (new_procedure), name);
@@ -870,7 +873,8 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
         }
       else if (! (pspec->flags & GIMP_PARAM_NO_VALIDATE))
         {
-          GValue string_value = G_VALUE_INIT;
+          GObject *old_value    = NULL;
+          GValue   string_value = G_VALUE_INIT;
 
           g_value_init (&string_value, G_TYPE_STRING);
 
@@ -880,6 +884,9 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
             g_value_set_static_string (&string_value,
                                        "<not transformable to string>");
 
+          if (G_IS_PARAM_SPEC_OBJECT (pspec))
+            old_value = g_value_dup_object (arg);
+
           if (g_param_value_validate (pspec, arg))
             {
               if (GIMP_IS_PARAM_SPEC_DRAWABLE (pspec) &&
@@ -942,6 +949,57 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
                                    g_param_spec_get_name (pspec));
                     }
                 }
+              else if (GIMP_IS_PARAM_SPEC_UNIT (pspec) &&
+                       ((GIMP_UNIT (old_value) == gimp_unit_pixel () &&
+                         ! gimp_param_spec_unit_pixel_allowed (pspec)) ||
+                        (GIMP_UNIT (old_value) == gimp_unit_percent () &&
+                         ! gimp_param_spec_unit_percent_allowed (pspec))))
+                {
+                  if (return_vals)
+                    {
+                      if (GIMP_UNIT (old_value) == gimp_unit_pixel ())
+                        g_set_error (error,
+                                     GIMP_PDB_ERROR,
+                                     GIMP_PDB_ERROR_INVALID_RETURN_VALUE,
+                                     _("Procedure '%s' returned "
+                                       "`gimp_unit_pixel()` as GimpUnit return value #%d '%s'. "
+                                       "This return value does not allow pixel unit."),
+                                     gimp_object_get_name (procedure),
+                                     i + 1, g_param_spec_get_name (pspec));
+                      else
+                        g_set_error (error,
+                                     GIMP_PDB_ERROR,
+                                     GIMP_PDB_ERROR_INVALID_RETURN_VALUE,
+                                     _("Procedure '%s' returned "
+                                       "`gimp_unit_percent()` as GimpUnit return value #%d '%s'. "
+                                       "This return value does not allow percent unit."),
+                                     gimp_object_get_name (procedure),
+                                     i + 1, g_param_spec_get_name (pspec));
+                    }
+                  else
+                    {
+                      if (GIMP_UNIT (old_value) == gimp_unit_pixel ())
+                        g_set_error (error,
+                                     GIMP_PDB_ERROR,
+                                     GIMP_PDB_ERROR_INVALID_ARGUMENT,
+                                     _("Procedure '%s' has been called with "
+                                       "`gimp_unit_pixel()` for GimpUnit "
+                                       "argument #%d '%s'. "
+                                       "This argument does not allow pixel unit."),
+                                     gimp_object_get_name (procedure),
+                                     i + 1, g_param_spec_get_name (pspec));
+                      else
+                        g_set_error (error,
+                                     GIMP_PDB_ERROR,
+                                     GIMP_PDB_ERROR_INVALID_ARGUMENT,
+                                     _("Procedure '%s' has been called with "
+                                       "`gimp_unit_percent()` for GimpUnit "
+                                       "argument #%d '%s'. "
+                                       "This argument does not allow percent unit."),
+                                     gimp_object_get_name (procedure),
+                                     i + 1, g_param_spec_get_name (pspec));
+                    }
+                }
               else
                 {
                   const gchar *value = g_value_get_string (&string_value);
@@ -980,9 +1038,11 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
                 }
 
               g_value_unset (&string_value);
+              g_clear_object (&old_value);
 
               return FALSE;
             }
+          g_clear_object (&old_value);
 
           /*  UTT-8 validate all strings  */
           if (G_PARAM_SPEC_TYPE (pspec) == G_TYPE_PARAM_STRING ||
