@@ -292,8 +292,11 @@ GIMP_IS_PARAM_SPEC_RUN_MODE (GParamSpec *pspec)
 }
 
 static inline gboolean
-GIMP_IS_PARAM_SPEC_FILE (GParamSpec *pspec)
+gimp_plug_in_is_file_spec (GParamSpec *pspec)
 {
+  /* This will work both for GimpParamSpecFile specs and
+   * GParamSpecObject with a GFile value.
+   */
   return (G_IS_PARAM_SPEC_OBJECT (pspec) &&
           pspec->value_type == G_TYPE_FILE);
 }
@@ -334,7 +337,7 @@ gimp_plug_in_set_file_proc_load_handler (GimpPlugIn   *plug_in,
   if (((procedure->num_args   < 2)                        ||
        (procedure->num_values < 1)                        ||
        ! GIMP_IS_PARAM_SPEC_RUN_MODE (procedure->args[0]) ||
-       ! GIMP_IS_PARAM_SPEC_FILE     (procedure->args[1]) ||
+       ! gimp_plug_in_is_file_spec   (procedure->args[1]) ||
        (! proc->generic_file_proc &&
         ! GIMP_IS_PARAM_SPEC_IMAGE (procedure->values[0]))))
     {
@@ -390,19 +393,18 @@ gimp_plug_in_set_file_proc_save_handler (GimpPlugIn   *plug_in,
 
   procedure = GIMP_PROCEDURE (proc);
 
-  if ((procedure->num_args < 5)                              ||
-      ! GIMP_IS_PARAM_SPEC_RUN_MODE     (procedure->args[0]) ||
-      ! GIMP_IS_PARAM_SPEC_IMAGE        (procedure->args[1]) ||
-      ! G_IS_PARAM_SPEC_INT             (procedure->args[2]) ||
-      ! GIMP_IS_PARAM_SPEC_OBJECT_ARRAY (procedure->args[3]) ||
-      ! GIMP_IS_PARAM_SPEC_FILE         (procedure->args[4]))
+  if ((procedure->num_args < 4)                                ||
+      ! GIMP_IS_PARAM_SPEC_RUN_MODE       (procedure->args[0]) ||
+      ! GIMP_IS_PARAM_SPEC_IMAGE          (procedure->args[1]) ||
+      ! gimp_plug_in_is_file_spec         (procedure->args[2]) ||
+      ! GIMP_IS_PARAM_SPEC_EXPORT_OPTIONS (procedure->args[3]))
     {
       g_set_error (error, GIMP_PDB_ERROR, GIMP_PDB_ERROR_FAILED,
                    "Plug-in \"%s\"\n(%s)\n"
                    "attempted to register procedure \"%s\" "
                    "as save handler which does not take the standard "
                    "save procedure arguments:\n"
-                   "(GimpRunMode, GimpImage, int [array size], GimpDrawable Array, GFile)",
+                   "(GimpRunMode, GimpImage, GFile, GimpExportOptions)",
                    gimp_object_get_name (plug_in),
                    gimp_file_get_utf8_name (plug_in->file),
                    proc_name);
@@ -542,11 +544,95 @@ gimp_plug_in_set_file_proc_handles_raw (GimpPlugIn   *plug_in,
       return FALSE;
     }
 
+  if (proc->handles_vector)
+    {
+      g_set_error (error, GIMP_PDB_ERROR, GIMP_PDB_ERROR_FAILED,
+                   "Plug-in \"%s\"\n(%s)\n"
+                   "attempted to register 'handles raw' "
+                   "for procedure \"%s\" which already 'handles vector'.\n"
+                   "It cannot handle both.",
+                   gimp_object_get_name (plug_in),
+                   gimp_file_get_utf8_name (plug_in->file),
+                   proc_name);
+
+      return FALSE;
+    }
   gimp_plug_in_procedure_set_handles_raw (proc);
 
   return TRUE;
 }
 
+gboolean
+gimp_plug_in_set_file_proc_handles_vector (GimpPlugIn   *plug_in,
+                                           const gchar  *proc_name,
+                                           GError      **error)
+{
+  GimpPlugInProcedure *proc;
+  GimpProcedure       *procedure;
+
+  g_return_val_if_fail (GIMP_IS_PLUG_IN (plug_in), FALSE);
+  g_return_val_if_fail (proc_name != NULL, FALSE);
+
+  proc = gimp_plug_in_proc_find (plug_in, proc_name);
+
+  if (! proc)
+    {
+      g_set_error (error, GIMP_PDB_ERROR, GIMP_PDB_ERROR_PROCEDURE_NOT_FOUND,
+                   "Plug-in \"%s\"\n(%s)\n"
+                   "attempted to register 'handles vector' "
+                   "for procedure \"%s\".\n"
+                   "It has however not installed that procedure. "
+                   "This is not allowed.",
+                   gimp_object_get_name (plug_in),
+                   gimp_file_get_utf8_name (plug_in->file),
+                   proc_name);
+
+      return FALSE;
+    }
+
+  procedure = GIMP_PROCEDURE (proc);
+
+  if (procedure->num_args < 4                            ||
+      procedure->num_values < 1                          ||
+      ! GIMP_IS_PARAM_SPEC_RUN_MODE (procedure->args[0]) ||
+      ! gimp_plug_in_is_file_spec   (procedure->args[1]) ||
+      ! G_IS_PARAM_SPEC_INT         (procedure->args[2]) ||
+      ! G_IS_PARAM_SPEC_INT         (procedure->args[3]) ||
+      ! GIMP_IS_PARAM_SPEC_IMAGE (procedure->values[0]))
+    {
+
+      g_set_error (error, GIMP_PDB_ERROR, GIMP_PDB_ERROR_FAILED,
+                   "Plug-in \"%s\"\n(%s)\n"
+                   "attempted to register procedure \"%s\" "
+                   "as a vector load procedure which does not take the "
+                   "standard load procedure procedure arguments: "
+                   "(GimpRunMode, file, int, int) -> (image)",
+                   gimp_object_get_name (plug_in),
+                   gimp_file_get_utf8_name (plug_in->file),
+                   proc_name);
+
+      return FALSE;
+    }
+
+  if (proc->handles_raw)
+    {
+      g_set_error (error, GIMP_PDB_ERROR, GIMP_PDB_ERROR_FAILED,
+                   "Plug-in \"%s\"\n(%s)\n"
+                   "attempted to register 'handles vector' "
+                   "for procedure \"%s\" which already 'handles raw'.\n"
+                   "It cannot handle both.",
+                   gimp_object_get_name (plug_in),
+                   gimp_file_get_utf8_name (plug_in->file),
+                   proc_name);
+
+      return FALSE;
+    }
+
+  gimp_plug_in_procedure_set_handles_vector (proc);
+
+  return TRUE;
+}
+
 gboolean
 gimp_plug_in_set_file_proc_thumb_loader (GimpPlugIn   *plug_in,
                                          const gchar  *proc_name,
