@@ -40,7 +40,7 @@
 #include "gimp-intl.h"
 
 
-#define PLUG_IN_RC_FILE_VERSION 13
+#define PLUG_IN_RC_FILE_VERSION 15
 
 
 /*
@@ -92,6 +92,7 @@ enum
   MIME_TYPES,
   HANDLES_REMOTE,
   HANDLES_RAW,
+  HANDLES_VECTOR,
   THUMB_LOADER,
   BATCH_INTERPRETER,
 };
@@ -162,6 +163,8 @@ plug_in_rc_parse (Gimp    *gimp,
                               "handles-remote", GINT_TO_POINTER (HANDLES_REMOTE));
   g_scanner_scope_add_symbol (scanner, LOAD_PROC,
                               "handles-raw", GINT_TO_POINTER (HANDLES_RAW));
+  g_scanner_scope_add_symbol (scanner, LOAD_PROC,
+                              "handles-vector", GINT_TO_POINTER (HANDLES_VECTOR));
   g_scanner_scope_add_symbol (scanner, LOAD_PROC,
                               "thumb-loader", GINT_TO_POINTER (THUMB_LOADER));
 
@@ -402,7 +405,7 @@ plug_in_procedure_deserialize (GScanner             *scanner,
     }
 
   if (proc_type != GIMP_PDB_PROC_TYPE_PLUGIN &&
-      proc_type != GIMP_PDB_PROC_TYPE_EXTENSION)
+      proc_type != GIMP_PDB_PROC_TYPE_PERSISTENT)
     {
       g_free (str);
       g_scanner_error (scanner, "procedure type %d is out of range",
@@ -717,6 +720,10 @@ plug_in_file_or_batch_proc_deserialize (GScanner            *scanner,
               gimp_plug_in_procedure_set_handles_raw (proc);
               break;
 
+            case HANDLES_VECTOR:
+              gimp_plug_in_procedure_set_handles_vector (proc);
+              break;
+
             case THUMB_LOADER:
                 {
                   gchar *thumb_loader;
@@ -826,6 +833,56 @@ plug_in_proc_arg_deserialize (GScanner      *scanner,
         }
       break;
 
+    case GP_PARAM_DEF_TYPE_CHOICE:
+        {
+          GimpChoice *choice;
+          gint        n_choices;
+
+          if (! gimp_scanner_parse_string (scanner,
+                                           &param_def.meta.m_choice.default_val) ||
+              ! gimp_scanner_parse_int (scanner, &n_choices))
+            {
+              token = G_TOKEN_INT;
+              goto error;
+            }
+
+          choice = gimp_choice_new ();
+          param_def.meta.m_choice.choice = choice;
+
+          for (int i = 0; i < n_choices; i++)
+            {
+              gchar *nick  = NULL;
+              gchar *label = NULL;
+              gchar *help  = NULL;
+              gint   id;
+
+              if (! gimp_scanner_parse_string (scanner, &nick))
+                {
+                  token = G_TOKEN_STRING;
+                  goto error;
+                }
+              if (! gimp_scanner_parse_int (scanner, &id))
+                {
+                  token = G_TOKEN_INT;
+                  goto error;
+                }
+              if (! gimp_scanner_parse_string (scanner, &label) ||
+                  ! gimp_scanner_parse_string (scanner, &help))
+                {
+                  token = G_TOKEN_STRING;
+                  g_free (nick);
+                  g_free (label);
+                  g_free (help);
+                  goto error;
+                }
+              gimp_choice_add (choice, nick, id, label, help);
+              g_free (nick);
+              g_free (label);
+              g_free (help);
+            }
+        }
+      break;
+
     case GP_PARAM_DEF_TYPE_BOOLEAN:
       if (! gimp_scanner_parse_int (scanner,
                                     &param_def.meta.m_boolean.default_val))
@@ -835,13 +892,13 @@ plug_in_proc_arg_deserialize (GScanner      *scanner,
         }
       break;
 
-    case GP_PARAM_DEF_TYPE_FLOAT:
-      if (! gimp_scanner_parse_float (scanner,
-                                      &param_def.meta.m_float.min_val) ||
-          ! gimp_scanner_parse_float (scanner,
-                                      &param_def.meta.m_float.max_val) ||
-          ! gimp_scanner_parse_float (scanner,
-                                      &param_def.meta.m_float.default_val))
+    case GP_PARAM_DEF_TYPE_DOUBLE:
+      if (! gimp_scanner_parse_double (scanner,
+                                       &param_def.meta.m_double.min_val) ||
+          ! gimp_scanner_parse_double (scanner,
+                                       &param_def.meta.m_double.max_val) ||
+          ! gimp_scanner_parse_double (scanner,
+                                       &param_def.meta.m_double.default_val))
         {
           token = G_TOKEN_FLOAT;
           goto error;
@@ -857,24 +914,70 @@ plug_in_proc_arg_deserialize (GScanner      *scanner,
         }
       break;
 
-    case GP_PARAM_DEF_TYPE_COLOR:
-      if (! gimp_scanner_parse_int (scanner,
-                                    &param_def.meta.m_color.has_alpha))
+    case GP_PARAM_DEF_TYPE_GEGL_COLOR:
         {
-          token = G_TOKEN_INT;
-          goto error;
-        }
-      if (! gimp_scanner_parse_float (scanner,
-                                      &param_def.meta.m_color.default_val.r) ||
-          ! gimp_scanner_parse_float (scanner,
-                                      &param_def.meta.m_color.default_val.g) ||
-          ! gimp_scanner_parse_float (scanner,
-                                      &param_def.meta.m_color.default_val.b) ||
-          ! gimp_scanner_parse_float (scanner,
-                                      &param_def.meta.m_color.default_val.a))
-        {
-          token = G_TOKEN_FLOAT;
-          goto error;
+          GPParamColor *default_val   = NULL;
+          gint          bpp           = 0;
+          guint8       *data          = NULL;
+          gchar        *encoding      = NULL;
+          gint          profile_size  = 0;
+          guint8       *profile_data  = NULL;
+
+          if (! gimp_scanner_parse_int (scanner,
+                                        &param_def.meta.m_gegl_color.has_alpha))
+            {
+              token = G_TOKEN_INT;
+              goto error;
+            }
+
+          if (g_scanner_peek_next_token (scanner) == G_TOKEN_IDENTIFIER)
+            {
+              if (g_strcmp0 (scanner->next_value.v_identifier, "NULL") != 0)
+                {
+                  token = G_TOKEN_INT;
+                  goto error;
+                }
+              g_scanner_get_next_token (scanner);
+              /* NULL default color. Skip. */
+            }
+          else if (! gimp_scanner_parse_int (scanner, &bpp)                         ||
+                   bpp > 40                                                         ||
+                   ! gimp_scanner_parse_data (scanner, bpp, &data)                  ||
+                   ! gimp_scanner_parse_string (scanner, &encoding)                 ||
+                   ! gimp_scanner_parse_int (scanner, &profile_size))
+            {
+              g_free (data);
+              g_free (encoding);
+
+              token = G_TOKEN_INT;
+              goto error;
+            }
+          if (profile_size > 0 && ! gimp_scanner_parse_data (scanner, profile_size, &profile_data))
+            {
+              g_free (profile_data);
+
+              token = G_TOKEN_STRING;
+              goto error;
+            }
+
+          if (bpp > 0)
+            {
+              default_val = g_new0 (GPParamColor, 1);
+
+              memcpy (default_val->data, data, bpp);
+              default_val->size                = bpp;
+              default_val->format.encoding     = encoding;
+              default_val->format.profile_size = profile_size;
+              default_val->format.profile_data = profile_data;
+            }
+          else
+            {
+              g_free (encoding);
+              g_free (profile_data);
+            }
+          g_free (data);
+
+          param_def.meta.m_gegl_color.default_val = default_val;
         }
       break;
 
@@ -895,6 +998,39 @@ plug_in_proc_arg_deserialize (GScanner      *scanner,
           goto error;
         }
       break;
+
+    case GP_PARAM_DEF_TYPE_EXPORT_OPTIONS:
+      break;
+
+    case GP_PARAM_DEF_TYPE_RESOURCE:
+      if (! gimp_scanner_parse_int (scanner,
+                                    &param_def.meta.m_resource.none_ok) ||
+          ! gimp_scanner_parse_int (scanner,
+                                    &param_def.meta.m_resource.default_to_context) ||
+          ! gimp_scanner_parse_int (scanner,
+                                    &param_def.meta.m_resource.default_resource_id))
+        {
+          token = G_TOKEN_INT;
+          goto error;
+        }
+      break;
+
+    case GP_PARAM_DEF_TYPE_FILE:
+      if (! gimp_scanner_parse_int (scanner,
+                                    (gint *) &param_def.meta.m_file.action) ||
+          ! gimp_scanner_parse_int (scanner,
+                                    &param_def.meta.m_file.none_ok))
+        {
+          token = G_TOKEN_INT;
+          goto error;
+        }
+      if (! gimp_scanner_parse_string (scanner,
+                                       &param_def.meta.m_file.default_uri))
+        {
+          token = G_TOKEN_STRING;
+          goto error;
+        }
+      break;
     }
 
   if (! gimp_scanner_parse_token (scanner, G_TOKEN_RIGHT_PAREN))
@@ -929,20 +1065,43 @@ plug_in_proc_arg_deserialize (GScanner      *scanner,
       break;
 
     case GP_PARAM_DEF_TYPE_BOOLEAN:
-    case GP_PARAM_DEF_TYPE_FLOAT:
+    case GP_PARAM_DEF_TYPE_DOUBLE:
       break;
 
     case GP_PARAM_DEF_TYPE_STRING:
       g_free (param_def.meta.m_string.default_val);
       break;
 
-    case GP_PARAM_DEF_TYPE_COLOR:
+    case GP_PARAM_DEF_TYPE_GEGL_COLOR:
+      if (param_def.meta.m_gegl_color.default_val)
+        {
+          g_free (param_def.meta.m_gegl_color.default_val->format.encoding);
+          g_free (param_def.meta.m_gegl_color.default_val->format.profile_data);
+          g_free (param_def.meta.m_gegl_color.default_val);
+        }
+      break;
+
     case GP_PARAM_DEF_TYPE_ID:
       break;
 
+    case GP_PARAM_DEF_TYPE_CHOICE:
+      g_clear_object (&param_def.meta.m_choice.choice);
+      g_free (param_def.meta.m_choice.default_val);
+      break;
+
     case GP_PARAM_DEF_TYPE_ID_ARRAY:
       g_free (param_def.meta.m_id_array.type_name);
       break;
+
+    case GP_PARAM_DEF_TYPE_EXPORT_OPTIONS:
+      break;
+
+    case GP_PARAM_DEF_TYPE_RESOURCE:
+      break;
+
+    case GP_PARAM_DEF_TYPE_FILE:
+      g_free (param_def.meta.m_file.default_uri);
+      break;
     }
 
   return token;
@@ -1034,18 +1193,43 @@ plug_in_rc_write_proc_arg (GimpConfigWriter *writer,
                                  param_def.meta.m_enum.default_val);
       break;
 
+    case GP_PARAM_DEF_TYPE_CHOICE:
+        {
+          GList *choices;
+
+          choices = gimp_choice_list_nicks (param_def.meta.m_choice.choice);
+          gimp_config_writer_string (writer, param_def.meta.m_choice.default_val);
+          gimp_config_writer_printf (writer, "%d", g_list_length (choices));
+          for (GList *iter = choices; iter; iter = iter->next)
+            {
+              const gchar *nick = iter->data;
+              const gchar *label;
+              const gchar *help;
+              gint         id;
+
+              gimp_choice_get_documentation (param_def.meta.m_choice.choice,
+                                             nick, &label, &help);
+              id = gimp_choice_get_id (param_def.meta.m_choice.choice, nick);
+              gimp_config_writer_string (writer, nick);
+              gimp_config_writer_printf (writer, "%d", id);
+              gimp_config_writer_string (writer, label);
+              gimp_config_writer_string (writer, help);
+            }
+        }
+      break;
+
     case GP_PARAM_DEF_TYPE_BOOLEAN:
       gimp_config_writer_printf (writer, "%d",
                                  param_def.meta.m_boolean.default_val);
       break;
 
-    case GP_PARAM_DEF_TYPE_FLOAT:
+    case GP_PARAM_DEF_TYPE_DOUBLE:
       g_ascii_dtostr (buf[0], sizeof (buf[0]),
-                      param_def.meta.m_float.min_val);
+                      param_def.meta.m_double.min_val);
       g_ascii_dtostr (buf[1], sizeof (buf[1]),
-                      param_def.meta.m_float.max_val),
+                      param_def.meta.m_double.max_val),
       g_ascii_dtostr (buf[2], sizeof (buf[2]),
-                      param_def.meta.m_float.default_val);
+                      param_def.meta.m_double.default_val);
       gimp_config_writer_printf (writer, "%s %s %s",
                                  buf[0], buf[1], buf[2]);
       break;
@@ -1055,18 +1239,39 @@ plug_in_rc_write_proc_arg (GimpConfigWriter *writer,
                                  param_def.meta.m_string.default_val);
       break;
 
-    case GP_PARAM_DEF_TYPE_COLOR:
-      g_ascii_dtostr (buf[0], sizeof (buf[0]),
-                      param_def.meta.m_color.default_val.r);
-      g_ascii_dtostr (buf[1], sizeof (buf[1]),
-                      param_def.meta.m_color.default_val.g),
-      g_ascii_dtostr (buf[2], sizeof (buf[2]),
-                      param_def.meta.m_color.default_val.b);
-      g_ascii_dtostr (buf[3], sizeof (buf[3]),
-                      param_def.meta.m_color.default_val.a);
-      gimp_config_writer_printf (writer, "%d %s %s %s %s",
-                                 param_def.meta.m_color.has_alpha,
-                                 buf[0], buf[1], buf[2], buf[3]);
+    case GP_PARAM_DEF_TYPE_GEGL_COLOR:
+        {
+          gint     bpp           = 0;
+          guint8  *data          = NULL;
+          gchar   *encoding      = NULL;
+          guint32  profile_size  = 0;
+          guint8  *profile_data  = NULL;
+
+          gimp_config_writer_printf (writer, "%d",
+                                     param_def.meta.m_gegl_color.has_alpha);
+
+          if (param_def.meta.m_gegl_color.default_val &&
+              param_def.meta.m_gegl_color.default_val->size > 0)
+            {
+              bpp          = param_def.meta.m_gegl_color.default_val->size;
+              data         = param_def.meta.m_gegl_color.default_val->data;
+              encoding     = param_def.meta.m_gegl_color.default_val->format.encoding;
+              profile_size = param_def.meta.m_gegl_color.default_val->format.profile_size;
+              profile_data = param_def.meta.m_gegl_color.default_val->format.profile_data;
+
+              gimp_config_writer_printf (writer, "%d", bpp);
+              gimp_config_writer_data (writer, bpp, data);
+              gimp_config_writer_string (writer, encoding);
+
+              gimp_config_writer_printf (writer, "%d", profile_size);
+              if (profile_size > 0)
+                gimp_config_writer_data (writer, profile_size, profile_data);
+            }
+          else
+            {
+              gimp_config_writer_printf (writer, "%s", "NULL");
+            }
+        }
       break;
 
     case GP_PARAM_DEF_TYPE_ID:
@@ -1078,6 +1283,24 @@ plug_in_rc_write_proc_arg (GimpConfigWriter *writer,
       gimp_config_writer_string (writer,
                                  param_def.meta.m_id_array.type_name);
       break;
+
+    case GP_PARAM_DEF_TYPE_EXPORT_OPTIONS:
+      break;
+
+    case GP_PARAM_DEF_TYPE_RESOURCE:
+      gimp_config_writer_printf (writer, "%d %d %d",
+                                 param_def.meta.m_resource.none_ok,
+                                 param_def.meta.m_resource.default_to_context,
+                                 param_def.meta.m_resource.default_resource_id);
+      break;
+
+    case GP_PARAM_DEF_TYPE_FILE:
+      gimp_config_writer_printf (writer, "%d %d",
+                                 param_def.meta.m_file.action,
+                                 param_def.meta.m_file.none_ok);
+      gimp_config_writer_string (writer,
+                                 param_def.meta.m_file.default_uri);
+      break;
     }
 
   gimp_config_writer_close (writer);
@@ -1253,6 +1476,12 @@ plug_in_rc_write (GSList  *plug_in_defs,
                       gimp_config_writer_close (writer);
                     }
 
+                  if (proc->handles_vector)
+                    {
+                      gimp_config_writer_open (writer, "handles-vector");
+                      gimp_config_writer_close (writer);
+                    }
+
                   if (proc->thumb_loader)
                     {
                       gimp_config_writer_open (writer, "thumb-loader");
