@@ -20,6 +20,8 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
+
 #include "display/display-types.h"
 
 #include "display/gimpdisplay.h"
@@ -40,16 +42,6 @@
 
 #include "gimp-app-test-utils.h"
 
-#ifdef G_OS_WIN32
-/* SendInput() requirement is Windows 2000 pro or over.
- * We may need to set WINVER to make sure the compiler does not try to
- * compile for on older version of win32, thus breaking the build.
- * See
- * http://msdn.microsoft.com/en-us/library/aa383745%28v=vs.85%29.aspx#setting_winver_or__win32_winnt
- */
-#define WINVER 0x0500
-#include <windows.h>
-#endif /* G_OS_WIN32 */
 
 #ifdef GDK_WINDOWING_QUARTZ
 // only to get keycode definitions from HIToolbox/Events.h
@@ -206,7 +198,7 @@ gimp_test_utils_create_image (Gimp *gimp,
 
   gimp_create_display (gimp,
                        image,
-                       GIMP_UNIT_PIXEL,
+                       gimp_unit_pixel (),
                        1.0 /*scale*/,
                        NULL);
 }
@@ -292,16 +284,9 @@ else
   }
 
 #else /* ! GDK_WINDOWING_QUARTZ */
-  gdk_test_simulate_key (gtk_widget_get_window (widget),
-                         -1, -1, /*x, y*/
-                         keyval,
-                         0 /*modifiers*/,
-                         GDK_KEY_PRESS);
-  gdk_test_simulate_key (gtk_widget_get_window (widget),
-                         -1, -1, /*x, y*/
-                         keyval,
-                         0 /*modifiers*/,
-                         GDK_KEY_RELEASE);
+
+  gtk_test_widget_send_key(widget, keyval, 0);
+
 #endif /* ! GDK_WINDOWING_QUARTZ */
 }
 
