@@ -26,10 +26,12 @@ const gchar  * gimp_text_parasite_name          (void) G_GNUC_CONST;
 GimpParasite * gimp_text_to_parasite            (GimpText      *text);
 GimpText     * gimp_text_from_parasite          (const GimpParasite  *parasite,
                                                  Gimp                *gimp,
+                                                 gboolean            *before_xcf_v19,
                                                  GError             **error);
 
 const gchar  * gimp_text_gdyntext_parasite_name (void) G_GNUC_CONST;
-GimpText     * gimp_text_from_gdyntext_parasite (const GimpParasite  *parasite);
+GimpText     * gimp_text_from_gdyntext_parasite (Gimp                *gimp,
+                                                 const GimpParasite  *parasite);
 
 
 #endif /* __GIMP_TEXT_PARASITE_H__ */
