@@ -53,7 +53,6 @@
 
 #include "gimptext.h"
 #include "gimptextlayer.h"
-#include "gimptextlayer-transform.h"
 #include "gimptextlayout.h"
 #include "gimptextlayout-render.h"
 
@@ -191,9 +190,10 @@ gimp_text_layer_class_init (GimpTextLayerClass *klass)
 static void
 gimp_text_layer_init (GimpTextLayer *layer)
 {
-  layer->text          = NULL;
-  layer->text_parasite = NULL;
-  layer->private       = gimp_text_layer_get_instance_private (layer);
+  layer->text                 = NULL;
+  layer->text_parasite        = NULL;
+  layer->text_parasite_is_old = FALSE;
+  layer->private              = gimp_text_layer_get_instance_private (layer);
 }
 
 static void
@@ -300,7 +300,10 @@ gimp_text_layer_duplicate (GimpItem *item,
 
       /*  this is just the parasite name, not a pointer to the parasite  */
       if (layer->text_parasite)
-        new_layer->text_parasite = layer->text_parasite;
+        {
+          new_layer->text_parasite        = layer->text_parasite;
+          new_layer->text_parasite_is_old = layer->text_parasite_is_old;
+        }
 
       new_layer->private->base_dir = layer->private->base_dir;
     }
@@ -617,7 +620,8 @@ gimp_text_layer_text_changed (GimpTextLayer *layer)
        */
       gimp_item_parasite_detach (GIMP_ITEM (layer), layer->text_parasite,
                                  FALSE);
-      layer->text_parasite = NULL;
+      layer->text_parasite        = NULL;
+      layer->text_parasite_is_old = FALSE;
     }
 
   if (layer->text->box_mode == GIMP_TEXT_BOX_DYNAMIC)
@@ -720,7 +724,7 @@ gimp_text_layer_render (GimpTextLayer *layer)
 
   gimp_image_get_resolution (image, &xres, &yres);
 
-  layout = gimp_text_layout_new (layer->text, xres, yres, &error);
+  layout = gimp_text_layout_new (layer->text, image, xres, yres, &error);
   if (error)
     {
       gimp_message_literal (image->gimp, NULL, GIMP_MESSAGE_ERROR, error->message);
@@ -922,9 +926,8 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
 {
   GimpDrawable       *drawable = GIMP_DRAWABLE (layer);
   GimpItem           *item     = GIMP_ITEM (layer);
-  GimpImage          *image    = gimp_item_get_image (item);
+  const Babl         *format;
   GeglBuffer         *buffer;
-  GimpColorTransform *transform;
   cairo_t            *cr;
   cairo_surface_t    *surface;
   gint                width;
@@ -936,7 +939,11 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
   width  = gimp_item_get_width  (item);
   height = gimp_item_get_height (item);
 
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 17, 2)
+  surface = cairo_image_surface_create (CAIRO_FORMAT_RGBA128F, width, height);
+#else
   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
+#endif
   status = cairo_surface_status (surface);
 
   if (status != CAIRO_STATUS_SUCCESS)
@@ -963,7 +970,6 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
   if (layer->text->outline != GIMP_TEXT_OUTLINE_NONE)
     {
       GimpText *text = layer->text;
-      GimpRGB   col  = text->outline_foreground;
 
       cairo_save (cr);
 
@@ -993,7 +999,16 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
         }
       else
         {
-          cairo_set_source_rgba (cr, col.r, col.g, col.b, col.a);
+          GeglColor  *col = text->outline_foreground;
+          gdouble     color[3];
+
+          format = gimp_text_layout_get_format (layout, "double");
+          gegl_color_get_pixel (col, format, color);
+          /* Text layout can be either grayscale or RGB without alpha. */
+          if (! babl_space_is_gray (babl_format_get_space (format)))
+            cairo_set_source_rgba (cr, color[0], color[1], color[2], 1.0);
+          else
+            cairo_set_source_rgba (cr, color[0], color[0], color[0], 1.0);
         }
 
       cairo_set_line_width (cr, text->outline_width * 2);
@@ -1009,23 +1024,29 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
 
   cairo_surface_flush (surface);
 
-  buffer = gimp_cairo_surface_create_buffer (surface);
-
-  transform = gimp_image_get_color_transform_from_srgb_u8 (image);
-
-  if (transform)
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 17, 2)
+  /* The CAIRO_FORMAT_RGBA128F surface maps to the layout TRC and space. */
+  switch (gimp_text_layout_get_trc (layout))
     {
-      gimp_color_transform_process_buffer (transform,
-                                           buffer,
-                                           NULL,
-                                           gimp_drawable_get_buffer (drawable),
-                                           NULL);
-    }
-  else
-    {
-      gimp_gegl_buffer_copy (buffer, NULL, GEGL_ABYSS_NONE,
-                             gimp_drawable_get_buffer (drawable), NULL);
+    case GIMP_TRC_LINEAR:
+      format = babl_format_with_space ("RaGaBaA float", gimp_text_layout_get_space (layout));
+      break;
+    case GIMP_TRC_NON_LINEAR:
+      format = babl_format_with_space ("R'aG'aB'aA float", gimp_text_layout_get_space (layout));
+      break;
+    case GIMP_TRC_PERCEPTUAL:
+      format = babl_format_with_space ("R~aG~aB~aA float", gimp_text_layout_get_space (layout));
+      break;
+    default:
+      g_return_if_reached ();
     }
+#else
+  format = babl_format_with_space ("cairo-ARGB32", gimp_text_layout_get_space (layout));
+#endif
+  buffer = gimp_cairo_surface_create_buffer (surface, format);
+
+  gimp_gegl_buffer_copy (buffer, NULL, GEGL_ABYSS_NONE,
+                         gimp_drawable_get_buffer (drawable), NULL);
 
   g_object_unref (buffer);
   cairo_surface_destroy (surface);
