@@ -37,6 +37,8 @@
 #include "operations/gimpcageconfig.h"
 
 #include "core/gimp.h"
+#include "core/gimpcontainer.h"
+#include "core/gimpdrawable-filters.h"
 #include "core/gimpdrawablefilter.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
@@ -1029,7 +1031,8 @@ gimp_cage_tool_commit (GimpCageTool *ct)
 
       gimp_tool_control_push_preserve (tool->control, TRUE);
 
-      gimp_drawable_filter_commit (ct->filter, GIMP_PROGRESS (tool), FALSE);
+      gimp_drawable_filter_commit (ct->filter, FALSE,
+                                   GIMP_PROGRESS (tool), FALSE);
       g_clear_object (&ct->filter);
 
       gimp_tool_control_pop_preserve (tool->control);
@@ -1299,6 +1302,7 @@ gimp_cage_tool_create_filter (GimpCageTool *ct)
                                          ct->render_node,
                                          GIMP_ICON_TOOL_CAGE);
   gimp_drawable_filter_set_region (ct->filter, GIMP_FILTER_REGION_DRAWABLE);
+  g_object_set (ct->filter, "temporary", TRUE, NULL);
 
   g_signal_connect (ct->filter, "flush",
                     G_CALLBACK (gimp_cage_tool_filter_flush),
@@ -1317,5 +1321,23 @@ gimp_cage_tool_filter_flush (GimpDrawableFilter *filter,
 static void
 gimp_cage_tool_filter_update (GimpCageTool *ct)
 {
+  GimpContainer *filters;
+
+  /* Move this operation below any non-destructive filters that
+   * may be active, so that it's directly affect the raw pixels. */
+  filters =
+    gimp_drawable_get_filters (gimp_drawable_filter_get_drawable (ct->filter));
+
+  if (gimp_container_have (filters, GIMP_OBJECT (ct->filter)))
+  {
+    gint end_index = gimp_container_get_n_children (filters) - 1;
+    gint index     = gimp_container_get_child_index (filters,
+                                                     GIMP_OBJECT (ct->filter));
+
+    if (end_index > 0 && index != end_index)
+      gimp_container_reorder (filters, GIMP_OBJECT (ct->filter),
+                              end_index);
+  }
+
   gimp_drawable_filter_apply (ct->filter, NULL);
 }
