@@ -20,6 +20,9 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
+#include <glib.h>
+#include <glib/gprintf.h>
+
 #include <gegl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -452,6 +455,11 @@ gimp_edit_selection_tool_button_release (GimpTool              *tool,
     }
 
   g_object_unref (edit_select);
+
+  shell->equidistance_side_horizontal = GIMP_ARRANGE_HFILL;
+  shell->equidistance_side_vertical   = GIMP_ARRANGE_HFILL;
+  shell->snapped_side_horizontal      = GIMP_ARRANGE_HFILL;
+  shell->snapped_side_vertical        = GIMP_ARRANGE_HFILL;
 }
 
 static void
@@ -626,14 +634,19 @@ gimp_edit_selection_tool_active_modifier_key (GimpTool        *tool,
 static void
 gimp_edit_selection_tool_draw (GimpDrawTool *draw_tool)
 {
-  GimpEditSelectionTool *edit_select = GIMP_EDIT_SELECTION_TOOL (draw_tool);
-  GimpDisplay           *display     = GIMP_TOOL (draw_tool)->display;
-  GimpImage             *image       = gimp_display_get_image (display);
+  GimpEditSelectionTool *edit_select  = GIMP_EDIT_SELECTION_TOOL (draw_tool);
+  GimpDisplay           *display      = GIMP_TOOL (draw_tool)->display;
+  GimpImage             *image        = gimp_display_get_image (display);
+  GimpDisplayShell      *shell        = gimp_display_get_shell (display);
+  GimpUnit              *unit         = gimp_display_shell_get_unit (shell);
+  const gchar           *abbreviation = gimp_unit_get_abbreviation (unit);
   GList                 *selected_items;
   GList                 *iter;
-  gint                   off_x       = G_MAXINT;
-  gint                   off_y       = G_MAXINT;
+  gdouble                xresolution, yresolution;
+  gint                   off_x        = G_MAXINT;
+  gint                   off_y        = G_MAXINT;
 
+  gimp_image_get_resolution (image, &xresolution, &yresolution);
   selected_items = gimp_edit_selection_tool_get_selected_items (edit_select, image);
   g_return_if_fail (selected_items != NULL);
 
@@ -706,14 +719,194 @@ gimp_edit_selection_tool_draw (GimpDrawTool *draw_tool)
     case GIMP_TRANSLATE_MODE_LAYER:
     case GIMP_TRANSLATE_MODE_VECTORS:
         {
-          GList *translate_items;
-          gint   x, y, w, h;
+          GList             *translate_items;
+          GimpAlignmentType  snapped_side_horizontal      = shell->snapped_side_horizontal;
+          GimpAlignmentType  snapped_side_vertical        = shell->snapped_side_vertical;
+          GimpAlignmentType  equidistance_side_horizontal = shell->equidistance_side_horizontal;
+          GimpAlignmentType  equidistance_side_vertical   = shell->equidistance_side_vertical;
+          gint               x, y, w, h;
 
           translate_items = gimp_image_item_list_filter (g_list_copy (selected_items));
           gimp_image_item_list_bounds (image, translate_items, &x, &y, &w, &h);
           g_list_free (translate_items);
 
           gimp_draw_tool_add_rectangle (draw_tool, FALSE, x, y, w, h);
+
+          if (snapped_side_horizontal != GIMP_ARRANGE_HFILL)
+            {
+              GimpLayer *snapped_layer_horizontal = shell->snapped_layer_horizontal;
+              gint       gx, gy, gw, gh;
+
+              gimp_item_bounds (GIMP_ITEM (snapped_layer_horizontal), &gx, &gy, &gw, &gh);
+              gimp_item_get_offset (GIMP_ITEM (snapped_layer_horizontal), &gx, &gy);
+
+              switch (snapped_side_horizontal)
+                {
+                case GIMP_ALIGN_LEFT:
+                  gimp_draw_tool_add_line (draw_tool, x, y+h/2, x, gy+gh/2);
+                  break;
+                case GIMP_ALIGN_RIGHT:
+                  gimp_draw_tool_add_line (draw_tool, x+w, y+h/2, x+w, gy+gh/2);
+                  break;
+                case GIMP_ALIGN_VCENTER:
+                  gimp_draw_tool_add_line (draw_tool, x+w/2, y+h/2, x+w/2, gy+gh/2);
+                  break;
+
+                default:
+                  break;
+                }
+            }
+
+          if (snapped_side_vertical != GIMP_ARRANGE_HFILL)
+            {
+              GimpLayer *snapped_layer_vertical = shell->snapped_layer_vertical;
+              gint       gx, gy, gw, gh;
+
+              gimp_item_bounds (GIMP_ITEM (snapped_layer_vertical), &gx, &gy, &gw, &gh);
+              gimp_item_get_offset (GIMP_ITEM (snapped_layer_vertical), &gx, &gy);
+
+              switch (snapped_side_vertical)
+                {
+                case GIMP_ALIGN_TOP:
+                  gimp_draw_tool_add_line (draw_tool, x+w/2, y, gx+gw/2, y);
+                  break;
+                case GIMP_ALIGN_BOTTOM:
+                  gimp_draw_tool_add_line (draw_tool, x+w/2, y+h, gx+gw/2, y+h);
+                  break;
+                case GIMP_ALIGN_HCENTER:
+                  gimp_draw_tool_add_line (draw_tool, x+w/2, y+h/2, gx+gw/2, y+h/2);
+                  break;
+
+                default:
+                  break;
+                }
+            }
+
+          if (equidistance_side_horizontal != GIMP_ARRANGE_HFILL)
+            {
+              GimpLayer       *near_layer1 = shell->near_layer_horizontal1;
+              GimpLayer       *near_layer2 = shell->near_layer_horizontal2;
+              GtkStyleContext *style       = gtk_widget_get_style_context (GTK_WIDGET (shell));
+              gdouble          pixels_to_units;
+              gdouble          font_size;
+              gint             gx1 = 0;
+              gint             gy1 = 0;
+              gint             gw1 = 0;
+              gint             gh1 = 0;
+              gint             gx2 = 0;
+              gint             gy2 = 0;
+              gint             gw2 = 0;
+              gint             gh2 = 0;
+              gint             distance;
+              gchar            distance_string[32];
+
+              gtk_style_context_get (style, gtk_style_context_get_state (style),
+                                     "font-size", &font_size, NULL);
+
+              gimp_item_bounds (GIMP_ITEM (near_layer1), &gx1, &gy1, &gw1, &gh1);
+              gimp_item_get_offset (GIMP_ITEM (near_layer1), &gx1, &gy1);
+
+              gimp_item_bounds (GIMP_ITEM (near_layer2), &gx2, &gy2, &gw2, &gh2);
+              gimp_item_get_offset (GIMP_ITEM (near_layer2), &gx2, &gy2);
+
+              gimp_draw_tool_add_rectangle (draw_tool, FALSE, gx1, gy1, gw1, gh1);
+              gimp_draw_tool_add_rectangle (draw_tool, FALSE, gx2, gy2, gw2, gh2);
+
+              switch (equidistance_side_horizontal)
+                {
+                case GIMP_ALIGN_LEFT:
+                  distance = x - (gx1+gw1);
+                  pixels_to_units = gimp_pixels_to_units ((gdouble) distance, unit, xresolution);
+
+                  gimp_draw_tool_add_line (draw_tool, x, y+h/2, gx1+gw1, y+h/2);
+                  gimp_draw_tool_add_line (draw_tool, gx1, gy1+gh1/2, gx2+gw2, gy1+gh1/2);
+
+                  g_sprintf (distance_string, "%g %s", pixels_to_units, abbreviation);
+                  gimp_draw_tool_add_text (draw_tool, x - distance/2, y+h/2, font_size, distance_string);
+                  gimp_draw_tool_add_text (draw_tool, gx1 - distance/2, gy1+gh1/2, font_size, distance_string);
+
+                  break;
+
+                case GIMP_ALIGN_RIGHT:
+                  distance = gx1 - (x+w);
+                  pixels_to_units = gimp_pixels_to_units ((gdouble) distance, unit, xresolution);
+
+                  gimp_draw_tool_add_line (draw_tool, x+w, y+h/2, gx1, y+h/2);
+                  gimp_draw_tool_add_line (draw_tool, gx1+gw1, gy1+gh1/2, gx2, gy1+gh1/2);
+
+                  g_sprintf (distance_string, "%g %s", pixels_to_units, abbreviation);
+                  gimp_draw_tool_add_text (draw_tool, (x+w) + distance/2, y+h/2, font_size, distance_string);
+                  gimp_draw_tool_add_text (draw_tool, (gx1+gw1) + distance/2, gy1+gh1/2, font_size, distance_string);
+
+                  break;
+
+                default:
+                  break;
+                }
+            }
+
+          if (equidistance_side_vertical != GIMP_ARRANGE_HFILL)
+            {
+              GimpLayer       *near_layer1 = shell->near_layer_vertical1;
+              GimpLayer       *near_layer2 = shell->near_layer_vertical2;
+              GtkStyleContext *style       = gtk_widget_get_style_context (GTK_WIDGET (shell));
+              gdouble          pixels_to_units;
+              gdouble          font_size;
+              gint             gx1 = 0;
+              gint             gy1 = 0;
+              gint             gw1 = 0;
+              gint             gh1 = 0;
+              gint             gx2 = 0;
+              gint             gy2 = 0;
+              gint             gw2 = 0;
+              gint             gh2 = 0;
+              gint             distance;
+              gchar            distance_string[32];
+
+              gtk_style_context_get (style, gtk_style_context_get_state (style),
+                                     "font-size", &font_size, NULL);
+
+              gimp_item_bounds (GIMP_ITEM (near_layer1), &gx1, &gy1, &gw1, &gh1);
+              gimp_item_get_offset (GIMP_ITEM (near_layer1), &gx1, &gy1);
+
+              gimp_item_bounds (GIMP_ITEM (near_layer2), &gx2, &gy2, &gw2, &gh2);
+              gimp_item_get_offset (GIMP_ITEM (near_layer2), &gx2, &gy2);
+
+              gimp_draw_tool_add_rectangle (draw_tool, FALSE, gx1, gy1, gw1, gh1);
+              gimp_draw_tool_add_rectangle (draw_tool, FALSE, gx2, gy2, gw2, gh2);
+
+              switch (equidistance_side_vertical)
+                {
+                case GIMP_ALIGN_TOP:
+                  distance = y - (gy1+gh1);
+                  pixels_to_units = gimp_pixels_to_units ((gdouble) distance, unit, yresolution);
+
+                  gimp_draw_tool_add_line (draw_tool, x+w/2, y, x+w/2, gy1+gh1);
+                  gimp_draw_tool_add_line (draw_tool, gx1+gw1/2, gy1, gx1+gw1/2, gy2+gh2);
+
+                  g_sprintf (distance_string, "%g %s", pixels_to_units, abbreviation);
+                  gimp_draw_tool_add_text (draw_tool, x+w/2, y - distance/2, font_size, distance_string);
+                  gimp_draw_tool_add_text (draw_tool, gx1+gw1/2, gy1 - distance/2, font_size, distance_string);
+
+                  break;
+
+                case GIMP_ALIGN_BOTTOM:
+                  distance = gy1 - (y+h);
+                  pixels_to_units = gimp_pixels_to_units ((gdouble) distance, unit, yresolution);
+
+                  gimp_draw_tool_add_line (draw_tool, x+w/2, y+h, x+w/2, gy1);
+                  gimp_draw_tool_add_line (draw_tool, gx1+gw1/2, gy1+gh1, gx1+gw1/2, gy2);
+
+                  g_sprintf (distance_string, "%g %s", pixels_to_units, abbreviation);
+                  gimp_draw_tool_add_text (draw_tool, x+w/2, (y+h) + distance/2, font_size, distance_string);
+                  gimp_draw_tool_add_text (draw_tool, gx1+gw1/2, (gy1+gh1) + distance/2, font_size, distance_string);
+
+                  break;
+
+                default:
+                  break;
+                }
+            }
         }
       break;
 
@@ -753,7 +946,7 @@ gimp_edit_selection_tool_get_selected_items (GimpEditSelectionTool *edit_select,
   switch (edit_select->edit_mode)
     {
     case GIMP_TRANSLATE_MODE_VECTORS:
-      selected_items = g_list_copy (gimp_image_get_selected_vectors (image));
+      selected_items = g_list_copy (gimp_image_get_selected_paths (image));
       break;
 
     case GIMP_TRANSLATE_MODE_LAYER:
@@ -1071,7 +1264,7 @@ gimp_edit_selection_tool_translate (GimpTool          *tool,
       break;
 
     case GIMP_TRANSFORM_TYPE_PATH:
-      selected_items = gimp_image_get_selected_vectors (image);
+      selected_items = gimp_image_get_selected_paths (image);
       selected_items = g_list_copy (selected_items);
 
       edit_mode = GIMP_TRANSLATE_MODE_VECTORS;
