@@ -50,7 +50,7 @@
 #include "core/gimptoolinfo.h"
 #include "core/gimpviewable.h"
 
-#include "vectors/gimpvectors.h"
+#include "vectors/gimppath.h"
 #include "vectors/gimpstroke.h"
 
 #include "widgets/gimpwidgets-utils.h"
@@ -830,12 +830,12 @@ gimp_transform_grid_tool_draw (GimpDrawTool *draw_tool)
     {
       GList *iter;
 
-      for (iter = gimp_image_get_selected_vectors (image); iter; iter = iter->next)
+      for (iter = gimp_image_get_selected_paths (image); iter; iter = iter->next)
         {
-          GimpVectors *vectors = iter->data;
-          GimpStroke  *stroke  = NULL;
+          GimpPath   *path   = iter->data;
+          GimpStroke *stroke = NULL;
 
-          while ((stroke = gimp_vectors_stroke_get_next (vectors, stroke)))
+          while ((stroke = gimp_path_stroke_get_next (path, stroke)))
             {
               GArray   *coords;
               gboolean  closed;
@@ -2040,11 +2040,12 @@ filter_new (GimpTransformGridTool *tg_tool,
 
       gimp_gegl_node_set_underlying_operation (node, filter->transform_node);
 
-      filter->filter = gimp_drawable_filter_new (
-        drawable,
-        GIMP_TRANSFORM_TOOL_GET_CLASS (tg_tool)->undo_desc,
-        node,
-        gimp_tool_get_icon_name (GIMP_TOOL (tg_tool)));
+      filter->filter =
+        gimp_drawable_filter_new (drawable,
+                                  GIMP_TRANSFORM_TOOL_GET_CLASS (tg_tool)->undo_desc,
+                                  node,
+                                  gimp_tool_get_icon_name (GIMP_TOOL (tg_tool)));
+      g_object_set (filter->filter, "temporary", TRUE, NULL);
 
       gimp_drawable_filter_set_clip (filter->filter, FALSE);
       gimp_drawable_filter_set_override_constraints (filter->filter, TRUE);
