@@ -4,9 +4,9 @@
  * gimpbrusheditor.c
  * Copyright 1998 Jay Cox <jaycox@earthlink.net>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,14 +15,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpmath/gimpmath.h"
@@ -49,9 +49,7 @@
 
 static void   gimp_brush_editor_docked_iface_init (GimpDockedInterface *face);
 
-static GObject * gimp_brush_editor_constructor (GType              type,
-                                                guint              n_params,
-                                                GObjectConstructParam *params);
+static void   gimp_brush_editor_constructed    (GObject            *object);
 
 static void   gimp_brush_editor_set_data       (GimpDataEditor     *editor,
                                                 GimpData           *data);
@@ -84,7 +82,7 @@ gimp_brush_editor_class_init (GimpBrushEditorClass *klass)
   GObjectClass        *object_class = G_OBJECT_CLASS (klass);
   GimpDataEditorClass *editor_class = GIMP_DATA_EDITOR_CLASS (klass);
 
-  object_class->constructor = gimp_brush_editor_constructor;
+  object_class->constructed = gimp_brush_editor_constructed;
 
   editor_class->set_data    = gimp_brush_editor_set_data;
   editor_class->title       = _("Brush Editor");
@@ -106,8 +104,10 @@ gimp_brush_editor_init (GimpBrushEditor *editor)
 {
   GimpDataEditor *data_editor = GIMP_DATA_EDITOR (editor);
   GtkWidget      *frame;
+  GtkWidget      *hbox;
+  GtkWidget      *label;
   GtkWidget      *box;
-  gint            row = 0;
+  GtkWidget      *scale;
 
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
@@ -127,120 +127,101 @@ gimp_brush_editor_init (GimpBrushEditor *editor)
 
   editor->shape_group = NULL;
 
-  /* table for sliders/labels */
-  editor->options_table = gtk_table_new (4, 3, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (editor->options_table), 2);
-  gtk_table_set_col_spacings (GTK_TABLE (editor->options_table), 2);
-  gtk_box_pack_start (GTK_BOX (editor), editor->options_table, FALSE, FALSE, 0);
-  gtk_widget_show (editor->options_table);
+  editor->options_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+  gtk_box_pack_start (GTK_BOX (editor), editor->options_box, FALSE, FALSE, 0);
+  gtk_widget_show (editor->options_box);
 
   /* Stock Box for the brush shape */
-  box = gimp_enum_stock_box_new (GIMP_TYPE_BRUSH_GENERATED_SHAPE,
-                                 "gimp-shape",
-                                 GTK_ICON_SIZE_MENU,
-                                 G_CALLBACK (gimp_brush_editor_update_shape),
-                                 editor,
-                                 &editor->shape_group);
-  gimp_table_attach_aligned (GTK_TABLE (editor->options_table),
-                             0, row++,
-                             _("Shape:"), 0.0, 0.5,
-                             box, 2, TRUE);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+  gtk_box_pack_start (GTK_BOX (editor->options_box), hbox, FALSE, FALSE, 0);
+  gtk_widget_show (hbox);
+
+  label = gtk_label_new (_("Shape:"));
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_widget_show (label);
+
+  box = gimp_enum_icon_box_new (GIMP_TYPE_BRUSH_GENERATED_SHAPE,
+                                "gimp-shape",
+                                GTK_ICON_SIZE_MENU,
+                                G_CALLBACK (gimp_brush_editor_update_shape),
+                                editor, NULL,
+                                &editor->shape_group);
+  gtk_box_pack_start (GTK_BOX (hbox), box, FALSE, FALSE, 0);
   gtk_widget_show (box);
 
   /*  brush radius scale  */
-  editor->radius_data =
-    GTK_ADJUSTMENT (gimp_scale_entry_new (GTK_TABLE (editor->options_table),
-                                          0, row++,
-                                          _("Radius:"), -1, 5,
-                                          0.0, 0.1, 1000.0, 0.1, 1.0, 1,
-                                          TRUE, 0.0, 0.0,
-                                          NULL, NULL));
-
-  gimp_scale_entry_set_logarithmic (GTK_OBJECT (editor->radius_data), TRUE);
+  editor->radius_data = gtk_adjustment_new (0.0,
+                                            GIMP_BRUSH_GENERATED_MIN_RADIUS,
+                                            GIMP_BRUSH_GENERATED_MAX_RADIUS,
+                                            1.0, 10.0, 0.0);
+  scale = gimp_spin_scale_new (editor->radius_data, _("Radius"), 1);
+  gtk_box_pack_start (GTK_BOX (editor->options_box), scale, FALSE, FALSE, 0);
+  gtk_widget_show (scale);
 
   g_signal_connect (editor->radius_data, "value-changed",
                     G_CALLBACK (gimp_brush_editor_update_brush),
                     editor);
 
   /*  number of spikes  */
-  editor->spikes_data =
-    GTK_ADJUSTMENT (gimp_scale_entry_new (GTK_TABLE (editor->options_table),
-                                          0, row++,
-                                          _("Spikes:"), -1, 5,
-                                          2.0, 2.0, 20.0, 1.0, 1.0, 0,
-                                          TRUE, 0.0, 0.0,
-                                          NULL, NULL));
+  editor->spikes_data = gtk_adjustment_new (2.0, 2.0, 20.0, 1.0, 1.0, 0.0);
+  scale = gimp_spin_scale_new (editor->spikes_data, _("Spikes"), 0);
+  gtk_box_pack_start (GTK_BOX (editor->options_box), scale, FALSE, FALSE, 0);
+  gtk_widget_show (scale);
 
   g_signal_connect (editor->spikes_data, "value-changed",
                     G_CALLBACK (gimp_brush_editor_update_brush),
                     editor);
 
   /*  brush hardness scale  */
-  editor->hardness_data =
-    GTK_ADJUSTMENT (gimp_scale_entry_new (GTK_TABLE (editor->options_table),
-                                          0, row++,
-                                          _("Hardness:"), -1, 5,
-                                          0.0, 0.0, 1.0, 0.01, 0.1, 2,
-                                          TRUE, 0.0, 0.0,
-                                          NULL, NULL));
+  editor->hardness_data = gtk_adjustment_new (0.0, 0.0, 1.0, 0.01, 0.1, 0.0);
+  scale = gimp_spin_scale_new (editor->hardness_data, _("Hardness"), 2);
+  gtk_box_pack_start (GTK_BOX (editor->options_box), scale, FALSE, FALSE, 0);
+  gtk_widget_show (scale);
 
   g_signal_connect (editor->hardness_data, "value-changed",
                     G_CALLBACK (gimp_brush_editor_update_brush),
                     editor);
 
   /*  brush aspect ratio scale  */
-  editor->aspect_ratio_data =
-    GTK_ADJUSTMENT (gimp_scale_entry_new (GTK_TABLE (editor->options_table),
-                                          0, row++,
-                                          _("Aspect ratio:"), -1, 5,
-                                          0.0, 1.0, 20.0, 0.1, 1.0, 1,
-                                          TRUE, 0.0, 0.0,
-                                          NULL, NULL));
+  editor->aspect_ratio_data = gtk_adjustment_new (0.0, 1.0, 20.0, 0.1, 1.0, 0.0);
+  scale = gimp_spin_scale_new (editor->aspect_ratio_data, _("Aspect ratio"), 1);
+  gtk_box_pack_start (GTK_BOX (editor->options_box), scale, FALSE, FALSE, 0);
+  gtk_widget_show (scale);
 
   g_signal_connect (editor->aspect_ratio_data,"value-changed",
                     G_CALLBACK (gimp_brush_editor_update_brush),
                     editor);
 
   /*  brush angle scale  */
-  editor->angle_data =
-    GTK_ADJUSTMENT (gimp_scale_entry_new (GTK_TABLE (editor->options_table),
-                                          0, row++,
-                                          _("Angle:"), -1, 5,
-                                          0.0, 0.0, 180.0, 0.1, 1.0, 1,
-                                          TRUE, 0.0, 0.0,
-                                          NULL, NULL));
+  editor->angle_data = gtk_adjustment_new (0.0, 0.0, 180.0, 0.1, 1.0, 0.0);
+  scale = gimp_spin_scale_new (editor->angle_data, _("Angle"), 1);
+  gtk_box_pack_start (GTK_BOX (editor->options_box), scale, FALSE, FALSE, 0);
+  gtk_widget_show (scale);
 
   g_signal_connect (editor->angle_data, "value-changed",
                     G_CALLBACK (gimp_brush_editor_update_brush),
                     editor);
 
   /*  brush spacing  */
-  editor->spacing_data =
-    GTK_ADJUSTMENT (gimp_scale_entry_new (GTK_TABLE (editor->options_table),
-                                          0, row++,
-                                          _("Spacing:"), -1, 5,
-                                          0.0, 1.0, 200.0, 1.0, 10.0, 1,
-                                          FALSE, 1.0, 5000.0,
-                                          _("Percentage of width of brush"),
-                                          NULL));
+  editor->spacing_data = gtk_adjustment_new (0.0, 1.0, 5000.0, 1.0, 10.0, 0.0);
+  scale = gimp_spin_scale_new (editor->spacing_data, _("Spacing"), 1);
+  gimp_spin_scale_set_scale_limits (GIMP_SPIN_SCALE (scale), 1.0, 200.0);
+  gtk_box_pack_start (GTK_BOX (editor->options_box), scale, FALSE, FALSE, 0);
+  gtk_widget_show (scale);
+
+  gimp_help_set_help_data (scale, _("Percentage of width of brush"), NULL);
 
   g_signal_connect (editor->spacing_data, "value-changed",
                     G_CALLBACK (gimp_brush_editor_update_brush),
                     editor);
 }
 
-static GObject *
-gimp_brush_editor_constructor (GType                  type,
-                               guint                  n_params,
-                               GObjectConstructParam *params)
+static void
+gimp_brush_editor_constructed (GObject *object)
 {
-  GObject *object;
-
-  object = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
   gimp_docked_set_show_button_bar (GIMP_DOCKED (object), FALSE);
-
-  return object;
 }
 
 static void
@@ -270,24 +251,29 @@ gimp_brush_editor_set_data (GimpDataEditor *editor,
 
   gimp_view_set_viewable (GIMP_VIEW (editor->view), GIMP_VIEWABLE (data));
 
-  if (editor->data && GIMP_IS_BRUSH_GENERATED (editor->data))
+  if (editor->data)
     {
-      GimpBrushGenerated *brush = GIMP_BRUSH_GENERATED (editor->data);
+      spacing = gimp_brush_get_spacing (GIMP_BRUSH (editor->data));
 
-      shape    = gimp_brush_generated_get_shape        (brush);
-      radius   = gimp_brush_generated_get_radius       (brush);
-      spikes   = gimp_brush_generated_get_spikes       (brush);
-      hardness = gimp_brush_generated_get_hardness     (brush);
-      ratio    = gimp_brush_generated_get_aspect_ratio (brush);
-      angle    = gimp_brush_generated_get_angle        (brush);
-      spacing  = gimp_brush_get_spacing                (GIMP_BRUSH (brush));
+      if (GIMP_IS_BRUSH_GENERATED (editor->data))
+        {
+          GimpBrushGenerated *brush = GIMP_BRUSH_GENERATED (editor->data);
+
+          shape    = gimp_brush_generated_get_shape        (brush);
+          radius   = gimp_brush_generated_get_radius       (brush);
+          spikes   = gimp_brush_generated_get_spikes       (brush);
+          hardness = gimp_brush_generated_get_hardness     (brush);
+          ratio    = gimp_brush_generated_get_aspect_ratio (brush);
+          angle    = gimp_brush_generated_get_angle        (brush);
+        }
     }
 
-  gtk_widget_set_sensitive (brush_editor->options_table,
+  gtk_widget_set_sensitive (brush_editor->options_box,
                             editor->data_editable);
 
   gimp_int_radio_group_set_active (GTK_RADIO_BUTTON (brush_editor->shape_group),
                                    shape);
+
   gtk_adjustment_set_value (brush_editor->radius_data,       radius);
   gtk_adjustment_set_value (brush_editor->spikes_data,       spikes);
   gtk_adjustment_set_value (brush_editor->hardness_data,     hardness);
@@ -335,53 +321,53 @@ gimp_brush_editor_update_brush (GtkAdjustment   *adjustment,
                                 GimpBrushEditor *editor)
 {
   GimpBrushGenerated *brush;
-  gdouble             radius;
-  gint                spikes;
-  gdouble             hardness;
-  gdouble             ratio;
-  gdouble             angle;
-  gdouble             spacing;
+  gdouble             value;
 
   if (! GIMP_IS_BRUSH_GENERATED (GIMP_DATA_EDITOR (editor)->data))
     return;
 
   brush = GIMP_BRUSH_GENERATED (GIMP_DATA_EDITOR (editor)->data);
 
-  radius   = gtk_adjustment_get_value (editor->radius_data);
-  spikes   = ROUND (gtk_adjustment_get_value (editor->spikes_data));
-  hardness = gtk_adjustment_get_value (editor->hardness_data);
-  ratio    = gtk_adjustment_get_value (editor->aspect_ratio_data);
-  angle    = gtk_adjustment_get_value (editor->angle_data);
-  spacing  = gtk_adjustment_get_value (editor->spacing_data);
+  g_signal_handlers_block_by_func (brush,
+                                   gimp_brush_editor_notify_brush,
+                                   editor);
 
-  if (radius   != gimp_brush_generated_get_radius       (brush) ||
-      spikes   != gimp_brush_generated_get_spikes       (brush) ||
-      hardness != gimp_brush_generated_get_hardness     (brush) ||
-      ratio    != gimp_brush_generated_get_aspect_ratio (brush) ||
-      angle    != gimp_brush_generated_get_angle        (brush) ||
-      spacing  != gimp_brush_get_spacing                (GIMP_BRUSH (brush)))
+  value = gtk_adjustment_get_value (adjustment);
+
+  if (adjustment == editor->radius_data)
     {
-      g_signal_handlers_block_by_func (brush,
-                                       gimp_brush_editor_notify_brush,
-                                       editor);
-
-      gimp_data_freeze (GIMP_DATA (brush));
-      g_object_freeze_notify (G_OBJECT (brush));
-
-      gimp_brush_generated_set_radius       (brush, radius);
-      gimp_brush_generated_set_spikes       (brush, spikes);
-      gimp_brush_generated_set_hardness     (brush, hardness);
-      gimp_brush_generated_set_aspect_ratio (brush, ratio);
-      gimp_brush_generated_set_angle        (brush, angle);
-      gimp_brush_set_spacing                (GIMP_BRUSH (brush), spacing);
-
-      g_object_thaw_notify (G_OBJECT (brush));
-      gimp_data_thaw (GIMP_DATA (brush));
-
-      g_signal_handlers_unblock_by_func (brush,
-                                         gimp_brush_editor_notify_brush,
-                                         editor);
+      if (value != gimp_brush_generated_get_radius (brush))
+        gimp_brush_generated_set_radius (brush, value);
     }
+  else if (adjustment == editor->spikes_data)
+    {
+      if (ROUND (value) != gimp_brush_generated_get_spikes (brush))
+        gimp_brush_generated_set_spikes (brush, ROUND (value));
+    }
+  else if (adjustment == editor->hardness_data)
+    {
+      if (value != gimp_brush_generated_get_hardness (brush))
+        gimp_brush_generated_set_hardness (brush, value);
+    }
+  else if (adjustment == editor->aspect_ratio_data)
+    {
+      if (value != gimp_brush_generated_get_aspect_ratio (brush))
+        gimp_brush_generated_set_aspect_ratio (brush, value);
+    }
+  else if (adjustment == editor->angle_data)
+    {
+      if (value != gimp_brush_generated_get_angle (brush))
+        gimp_brush_generated_set_angle (brush, value);
+    }
+  else if (adjustment == editor->spacing_data)
+    {
+      if (value != gimp_brush_get_spacing (GIMP_BRUSH (brush)))
+        gimp_brush_set_spacing (GIMP_BRUSH (brush), value);
+    }
+
+  g_signal_handlers_unblock_by_func (brush,
+                                     gimp_brush_editor_notify_brush,
+                                     editor);
 }
 
 static void
@@ -408,9 +394,9 @@ gimp_brush_editor_update_shape (GtkWidget       *widget,
 }
 
 static void
-gimp_brush_editor_notify_brush (GimpBrushGenerated   *brush,
-                                GParamSpec           *pspec,
-                                GimpBrushEditor      *editor)
+gimp_brush_editor_notify_brush (GimpBrushGenerated *brush,
+                                GParamSpec         *pspec,
+                                GimpBrushEditor    *editor)
 {
   GtkAdjustment *adj   = NULL;
   gdouble        value = 0.0;
@@ -427,39 +413,36 @@ gimp_brush_editor_notify_brush (GimpBrushGenerated   *brush,
       g_signal_handlers_unblock_by_func (editor->shape_group,
                                          gimp_brush_editor_update_shape,
                                          editor);
-
-      adj   = editor->radius_data;
-      value = brush->radius;
     }
   else if (! strcmp (pspec->name, "radius"))
     {
       adj   = editor->radius_data;
-      value = brush->radius;
+      value = gimp_brush_generated_get_radius (brush);
     }
   else if (! strcmp (pspec->name, "spikes"))
     {
       adj   = editor->spikes_data;
-      value = brush->spikes;
+      value = gimp_brush_generated_get_spikes (brush);
     }
   else if (! strcmp (pspec->name, "hardness"))
     {
       adj   = editor->hardness_data;
-      value = brush->hardness;
+      value = gimp_brush_generated_get_hardness (brush);
     }
   else if (! strcmp (pspec->name, "angle"))
     {
       adj   = editor->angle_data;
-      value = brush->angle;
+      value = gimp_brush_generated_get_angle (brush);
     }
   else if (! strcmp (pspec->name, "aspect-ratio"))
     {
       adj   = editor->aspect_ratio_data;
-      value = brush->aspect_ratio;
+      value = gimp_brush_generated_get_aspect_ratio (brush);
     }
   else if (! strcmp (pspec->name, "spacing"))
     {
       adj   = editor->spacing_data;
-      value = GIMP_BRUSH (brush)->spacing;
+      value = gimp_brush_get_spacing (GIMP_BRUSH (brush));
     }
 
   if (adj)
