@@ -102,32 +102,36 @@ gimp_channel_tree_view_class_init (GimpChannelTreeViewClass *klass)
   view_class->drop_viewables = gimp_channel_tree_view_drop_viewables;
   view_class->drop_component = gimp_channel_tree_view_drop_component;
 
+  iv_class->move_cursor_up_action    = "channels-select-previous";
+  iv_class->move_cursor_down_action  = "channels-select-next";
+  iv_class->move_cursor_start_action = "channels-select-top";
+  iv_class->move_cursor_end_action   = "channels-select-bottom";
+
   iv_class->set_image        = gimp_channel_tree_view_set_image;
 
   iv_class->item_type        = GIMP_TYPE_CHANNEL;
   iv_class->signal_name      = "selected-channels-changed";
 
   iv_class->get_container    = gimp_image_get_channels;
-  iv_class->get_active_item  = (GimpGetItemFunc) gimp_image_get_active_channel;
-  iv_class->set_active_item  = (GimpSetItemFunc) gimp_image_set_active_channel;
   iv_class->get_selected_items = (GimpGetItemsFunc) gimp_image_get_selected_channels;
   iv_class->set_selected_items = (GimpSetItemsFunc) gimp_image_set_selected_channels;
   iv_class->add_item         = (GimpAddItemFunc) gimp_image_add_channel;
   iv_class->remove_item      = (GimpRemoveItemFunc) gimp_image_remove_channel;
   iv_class->new_item         = gimp_channel_tree_view_item_new;
 
-  iv_class->action_group          = "channels";
-  iv_class->activate_action       = "channels-edit-attributes";
-  iv_class->new_action            = "channels-new";
-  iv_class->new_default_action    = "channels-new-last-values";
-  iv_class->raise_action          = "channels-raise";
-  iv_class->raise_top_action      = "channels-raise-to-top";
-  iv_class->lower_action          = "channels-lower";
-  iv_class->lower_bottom_action   = "channels-lower-to-bottom";
-  iv_class->duplicate_action      = "channels-duplicate";
-  iv_class->delete_action         = "channels-delete";
-  iv_class->lock_content_help_id  = GIMP_HELP_CHANNEL_LOCK_PIXELS;
-  iv_class->lock_position_help_id = GIMP_HELP_CHANNEL_LOCK_POSITION;
+  iv_class->action_group            = "channels";
+  iv_class->activate_action         = "channels-edit-attributes";
+  iv_class->new_action              = "channels-new";
+  iv_class->new_default_action      = "channels-new-last-values";
+  iv_class->raise_action            = "channels-raise";
+  iv_class->raise_top_action        = "channels-raise-to-top";
+  iv_class->lower_action            = "channels-lower";
+  iv_class->lower_bottom_action     = "channels-lower-to-bottom";
+  iv_class->duplicate_action        = "channels-duplicate";
+  iv_class->delete_action           = "channels-delete";
+  iv_class->lock_content_help_id    = GIMP_HELP_CHANNEL_LOCK_PIXELS;
+  iv_class->lock_position_help_id   = GIMP_HELP_CHANNEL_LOCK_POSITION;
+  iv_class->lock_visibility_help_id = GIMP_HELP_CHANNEL_LOCK_VISIBILITY;
 }
 
 static void
@@ -322,17 +326,17 @@ static GimpItem *
 gimp_channel_tree_view_item_new (GimpImage *image)
 {
   GimpChannel *new_channel;
-  GimpRGB      color;
-
-  gimp_rgba_set (&color, 0.0, 0.0, 0.0, 0.5);
+  GeglColor   *color = gegl_color_new ("black");
 
   gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_PASTE,
                                _("New Channel"));
 
+  gimp_color_set_alpha (color, 0.5);
   new_channel = gimp_channel_new (image,
                                   gimp_image_get_width (image),
                                   gimp_image_get_height (image),
-                                  _("Channel"), &color);
+                                  _("Channel"), color);
+  g_object_unref (color);
 
   gimp_image_add_channel (image, new_channel,
                           GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
