@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -136,7 +136,7 @@ gimp_component_editor_init (GimpComponentEditor *editor)
   editor->eye_column = gtk_tree_view_column_new ();
   gtk_tree_view_append_column (editor->view, editor->eye_column);
 
-  editor->eye_cell = gimp_cell_renderer_toggle_new (GIMP_STOCK_VISIBLE);
+  editor->eye_cell = gimp_cell_renderer_toggle_new (GIMP_ICON_VISIBLE);
   gtk_tree_view_column_pack_start (editor->eye_column, editor->eye_cell,
                                    FALSE);
   gtk_tree_view_column_set_attributes (editor->eye_column, editor->eye_cell,
@@ -285,30 +285,29 @@ void
 gimp_component_editor_set_view_size (GimpComponentEditor *editor,
                                      gint                 view_size)
 {
-  GtkWidget   *tree_widget;
-  GtkStyle    *tree_style;
-  GtkIconSize  icon_size;
-  GtkTreeIter  iter;
-  gboolean     iter_valid;
+  GtkWidget       *tree_widget;
+  GtkStyleContext *tree_style;
+  GtkBorder        border;
+  GtkTreeIter      iter;
+  gboolean         iter_valid;
+  gint             icon_size;
 
   g_return_if_fail (GIMP_IS_COMPONENT_EDITOR (editor));
   g_return_if_fail (view_size >  0 &&
                     view_size <= GIMP_VIEWABLE_MAX_PREVIEW_SIZE);
 
   tree_widget = GTK_WIDGET (editor->view);
-  tree_style  = gtk_widget_get_style (tree_widget);
+  tree_style  = gtk_widget_get_style_context (tree_widget);
 
-  icon_size = gimp_get_icon_size (tree_widget,
-                                  GIMP_STOCK_VISIBLE,
-                                  GTK_ICON_SIZE_BUTTON,
-                                  view_size -
-                                  2 * tree_style->xthickness,
-                                  view_size -
-                                  2 * tree_style->ythickness);
+  gtk_style_context_save (tree_style);
+  gtk_style_context_add_class (tree_style, GTK_STYLE_CLASS_BUTTON);
+  gtk_style_context_get_border (tree_style, 0, &border);
+  gtk_style_context_restore (tree_style);
 
-  g_object_set (editor->eye_cell,
-                "stock-size", icon_size,
-                NULL);
+  g_object_get (editor->eye_cell, "icon-size", &icon_size, NULL);
+  icon_size = MIN (icon_size, MAX (view_size - (border.left + border.right),
+                                   view_size - (border.top + border.bottom)));
+  g_object_set (editor->eye_cell, "icon-size", icon_size, NULL);
 
   for (iter_valid = gtk_tree_model_get_iter_first (editor->model, &iter);
        iter_valid;
@@ -338,28 +337,28 @@ gimp_component_editor_create_components (GimpComponentEditor *editor)
   GEnumClass      *enum_class;
   gint             i;
 
-  switch (gimp_image_base_type (image))
+  switch (gimp_image_get_base_type (image))
     {
     case GIMP_RGB:
       n_components  = 3;
-      components[0] = GIMP_RED_CHANNEL;
-      components[1] = GIMP_GREEN_CHANNEL;
-      components[2] = GIMP_BLUE_CHANNEL;
+      components[0] = GIMP_CHANNEL_RED;
+      components[1] = GIMP_CHANNEL_GREEN;
+      components[2] = GIMP_CHANNEL_BLUE;
       break;
 
     case GIMP_GRAY:
       n_components  = 1;
-      components[0] = GIMP_GRAY_CHANNEL;
+      components[0] = GIMP_CHANNEL_GRAY;
       break;
 
     case GIMP_INDEXED:
       n_components  = 1;
-      components[0] = GIMP_INDEXED_CHANNEL;
+      components[0] = GIMP_CHANNEL_INDEXED;
       break;
     }
 
   if (gimp_image_has_alpha (image))
-    components[n_components++] = GIMP_ALPHA_CHANNEL;
+    components[n_components++] = GIMP_CHANNEL_ALPHA;
 
   enum_class = g_type_class_ref (GIMP_TYPE_CHANNEL_TYPE);
 
@@ -504,7 +503,7 @@ gimp_component_editor_button_press (GtkWidget           *widget,
 
       if (gdk_event_triggers_context_menu ((GdkEvent *) bevent))
         {
-          gimp_editor_popup_menu (GIMP_EDITOR (editor), NULL, NULL);
+          gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (editor), (GdkEvent *) bevent);
         }
       else if (bevent->type == GDK_BUTTON_PRESS && bevent->button == 1 &&
                column != editor->eye_column)
