@@ -4,9 +4,9 @@
  * gimpcontainercombobox.c
  * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,16 +15,18 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
+
 #include "widgets-types.h"
 
 #include "core/gimpcontainer.h"
@@ -33,6 +35,7 @@
 
 #include "gimpcellrendererviewable.h"
 #include "gimpcontainercombobox.h"
+#include "gimpcontainertreestore.h"
 #include "gimpcontainerview.h"
 #include "gimpviewrenderer.h"
 
@@ -59,6 +62,7 @@ static void     gimp_container_combo_box_set_context  (GimpContainerView      *v
                                                        GimpContext            *context);
 static gpointer gimp_container_combo_box_insert_item  (GimpContainerView      *view,
                                                        GimpViewable           *viewable,
+                                                       gpointer                parent_insert_data,
                                                        gint                    index);
 static void     gimp_container_combo_box_remove_item  (GimpContainerView      *view,
                                                        GimpViewable           *viewable,
@@ -70,16 +74,17 @@ static void     gimp_container_combo_box_reorder_item (GimpContainerView      *v
 static void     gimp_container_combo_box_rename_item  (GimpContainerView      *view,
                                                        GimpViewable           *viewable,
                                                        gpointer                insert_data);
-static gboolean  gimp_container_combo_box_select_item (GimpContainerView      *view,
-                                                       GimpViewable           *viewable,
-                                                       gpointer                insert_data);
+static gboolean  gimp_container_combo_box_select_items(GimpContainerView      *view,
+                                                       GList                  *viewables,
+                                                       GList                  *paths);
 static void     gimp_container_combo_box_clear_items  (GimpContainerView      *view);
 static void    gimp_container_combo_box_set_view_size (GimpContainerView      *view);
+static gint    gimp_container_combo_box_get_selected  (GimpContainerView      *view,
+                                                       GList                 **items,
+                                                       GList                 **items_data);
 
 static void     gimp_container_combo_box_changed      (GtkComboBox            *combo_box,
                                                        GimpContainerView      *view);
-static void gimp_container_combo_box_renderer_update  (GimpViewRenderer       *renderer,
-                                                       GimpContainerView      *view);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpContainerComboBox, gimp_container_combo_box,
@@ -105,9 +110,9 @@ gimp_container_combo_box_class_init (GimpContainerComboBoxClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_ELLIPSIZE,
                                    g_param_spec_enum ("ellipsize", NULL, NULL,
-						      PANGO_TYPE_ELLIPSIZE_MODE,
-						      PANGO_ELLIPSIZE_MIDDLE,
-						      GIMP_PARAM_READWRITE |
+                                                      PANGO_TYPE_ELLIPSIZE_MODE,
+                                                      PANGO_ELLIPSIZE_MIDDLE,
+                                                      GIMP_PARAM_READWRITE |
                                                       G_PARAM_CONSTRUCT));
 }
 
@@ -124,53 +129,64 @@ gimp_container_combo_box_view_iface_init (GimpContainerViewInterface *iface)
   iface->remove_item   = gimp_container_combo_box_remove_item;
   iface->reorder_item  = gimp_container_combo_box_reorder_item;
   iface->rename_item   = gimp_container_combo_box_rename_item;
-  iface->select_item   = gimp_container_combo_box_select_item;
+  iface->select_items  = gimp_container_combo_box_select_items;
   iface->clear_items   = gimp_container_combo_box_clear_items;
   iface->set_view_size = gimp_container_combo_box_set_view_size;
+  iface->get_selected  = gimp_container_combo_box_get_selected;
 
   iface->insert_data_free = (GDestroyNotify) gtk_tree_iter_free;
 }
 
 static void
-gimp_container_combo_box_init (GimpContainerComboBox *combo_box)
+gimp_container_combo_box_init (GimpContainerComboBox *combo)
 {
-  GtkListStore    *store;
+  GtkTreeModel    *model;
   GtkCellLayout   *layout;
   GtkCellRenderer *cell;
+  GType            types[GIMP_CONTAINER_TREE_STORE_N_COLUMNS];
+  gint             n_types = 0;
 
-  store = gtk_list_store_new (GIMP_CONTAINER_COMBO_BOX_N_COLUMNS,
-                              GIMP_TYPE_VIEW_RENDERER,
-                              G_TYPE_STRING);
+  gimp_container_tree_store_columns_init (types, &n_types);
 
-  gtk_combo_box_set_model (GTK_COMBO_BOX (combo_box), GTK_TREE_MODEL (store));
+  model = gimp_container_tree_store_new (GIMP_CONTAINER_VIEW (combo),
+                                         n_types, types);
 
-  g_object_unref (store);
+  gtk_combo_box_set_model (GTK_COMBO_BOX (combo), model);
 
-  layout = GTK_CELL_LAYOUT (combo_box);
+  g_object_unref (model);
+
+  layout = GTK_CELL_LAYOUT (combo);
 
   cell = gimp_cell_renderer_viewable_new ();
   gtk_cell_layout_pack_start (layout, cell, FALSE);
   gtk_cell_layout_set_attributes (layout, cell,
                                   "renderer",
-                                  GIMP_CONTAINER_COMBO_BOX_COLUMN_RENDERER,
+                                  GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER,
+                                  "sensitive",
+                                  GIMP_CONTAINER_TREE_STORE_COLUMN_NAME_SENSITIVE,
                                   NULL);
 
-  combo_box->viewable_renderer = cell;
+  gimp_container_tree_store_add_renderer_cell (GIMP_CONTAINER_TREE_STORE (model),
+                                               cell, -1);
+
+  combo->viewable_renderer = cell;
 
   cell = gtk_cell_renderer_text_new ();
   gtk_cell_layout_pack_start (layout, cell, TRUE);
   gtk_cell_layout_set_attributes (layout, cell,
                                   "text",
-                                  GIMP_CONTAINER_COMBO_BOX_COLUMN_NAME,
+                                  GIMP_CONTAINER_TREE_STORE_COLUMN_NAME,
+                                  "sensitive",
+                                  GIMP_CONTAINER_TREE_STORE_COLUMN_NAME_SENSITIVE,
                                   NULL);
 
-  combo_box->text_renderer = cell;
+  combo->text_renderer = cell;
 
-  g_signal_connect (combo_box, "changed",
+  g_signal_connect (combo, "changed",
                     G_CALLBACK (gimp_container_combo_box_changed),
-                    combo_box);
+                    combo);
 
-  gtk_widget_set_sensitive (GTK_WIDGET (combo_box), FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (combo), FALSE);
 }
 
 static void
@@ -243,44 +259,6 @@ gimp_container_combo_box_new (GimpContainer *container,
   return combo_box;
 }
 
-static void
-gimp_container_combo_box_set (GimpContainerComboBox *combo_box,
-                              GtkTreeIter           *iter,
-                              GimpViewable          *viewable)
-{
-  GimpContainerView *view = GIMP_CONTAINER_VIEW (combo_box);
-  GtkTreeModel      *model;
-  GimpViewRenderer  *renderer;
-  gchar             *name;
-  gint               view_size;
-  gint               border_width;
-
-  model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo_box));
-
-  view_size = gimp_container_view_get_view_size (view, &border_width);
-
-  name = gimp_viewable_get_description (viewable, NULL);
-
-  renderer = gimp_view_renderer_new (gimp_container_view_get_context (view),
-                                     G_TYPE_FROM_INSTANCE (viewable),
-                                     view_size, border_width,
-                                     FALSE);
-  gimp_view_renderer_set_viewable (renderer, viewable);
-  gimp_view_renderer_remove_idle (renderer);
-
-  g_signal_connect (renderer, "update",
-                    G_CALLBACK (gimp_container_combo_box_renderer_update),
-                    view);
-
-  gtk_list_store_set (GTK_LIST_STORE (model), iter,
-                      GIMP_CONTAINER_COMBO_BOX_COLUMN_RENDERER, renderer,
-                      GIMP_CONTAINER_COMBO_BOX_COLUMN_NAME,     name,
-                      -1);
-
-  g_object_unref (renderer);
-  g_free (name);
-}
-
 
 /*  GimpContainerView methods  */
 
@@ -293,51 +271,39 @@ gimp_container_combo_box_set_context (GimpContainerView *view,
   parent_view_iface->set_context (view, context);
 
   if (model)
-    {
-      GtkTreeIter iter;
-      gboolean    iter_valid;
-
-      for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
-           iter_valid;
-           iter_valid = gtk_tree_model_iter_next (model, &iter))
-        {
-          GimpViewRenderer *renderer;
-
-          gtk_tree_model_get (model, &iter,
-                              GIMP_CONTAINER_COMBO_BOX_COLUMN_RENDERER, &renderer,
-                              -1);
-
-          gimp_view_renderer_set_context (renderer, context);
-          g_object_unref (renderer);
-        }
-    }
+    gimp_container_tree_store_set_context (GIMP_CONTAINER_TREE_STORE (model),
+                                           context);
 }
 
 static gpointer
 gimp_container_combo_box_insert_item (GimpContainerView *view,
                                       GimpViewable      *viewable,
+                                      gpointer           parent_insert_data,
                                       gint               index)
 {
   GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
-  GtkTreeIter   iter;
 
-  if (index == -1)
-    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-  else
-    gtk_list_store_insert (GTK_LIST_STORE (model), &iter, index);
-
-  if (gtk_tree_model_iter_n_children (model, NULL) == 1)
+  if (model)
     {
-      /*  GimpContainerViews don't select items by default  */
-      gtk_combo_box_set_active (GTK_COMBO_BOX (view), -1);
+      GtkTreeIter *iter;
 
-      gtk_widget_set_sensitive (GTK_WIDGET (view), TRUE);
+      iter = gimp_container_tree_store_insert_item (GIMP_CONTAINER_TREE_STORE (model),
+                                                    viewable,
+                                                    parent_insert_data,
+                                                    index);
+
+      if (gtk_tree_model_iter_n_children (model, NULL) == 1)
+        {
+          /*  GimpContainerViews don't select items by default  */
+          gtk_combo_box_set_active (GTK_COMBO_BOX (view), -1);
+
+          gtk_widget_set_sensitive (GTK_WIDGET (view), TRUE);
+        }
+
+      return iter;
     }
 
-  gimp_container_combo_box_set (GIMP_CONTAINER_COMBO_BOX (view),
-                                &iter, viewable);
-
-  return gtk_tree_iter_copy (&iter);
+  return NULL;
 }
 
 static void
@@ -346,21 +312,17 @@ gimp_container_combo_box_remove_item (GimpContainerView *view,
                                       gpointer           insert_data)
 {
   GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
-  GtkTreeIter  *iter  = insert_data;
 
-  if (iter)
+  if (model)
     {
-      gtk_list_store_remove (GTK_LIST_STORE (model), iter);
+      GtkTreeIter *iter = insert_data;
 
-      /*  If the store is now empty, clear out renderers from all cells
-       *  so that they don't reference the viewables.  See bug #149906.
-       */
-      if (gtk_tree_model_iter_n_children (model, NULL) == 0)
+      gimp_container_tree_store_remove_item (GIMP_CONTAINER_TREE_STORE (model),
+                                             viewable,
+                                             iter);
+
+      if (iter && gtk_tree_model_iter_n_children (model, NULL) == 0)
         {
-          g_object_set (GIMP_CONTAINER_COMBO_BOX (view)->viewable_renderer,
-                        "renderer", NULL,
-                        NULL);
-
           gtk_widget_set_sensitive (GTK_WIDGET (view), FALSE);
         }
     }
@@ -372,44 +334,13 @@ gimp_container_combo_box_reorder_item (GimpContainerView *view,
                                        gint               new_index,
                                        gpointer           insert_data)
 {
-  GtkTreeModel  *model     = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
-  GimpContainer *container = gimp_container_view_get_container (view);
-  GtkTreeIter   *iter      = insert_data;
+  GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
 
-  if (!iter)
-    return;
-
-  if (new_index == -1 || new_index == container->num_children - 1)
-    {
-      gtk_list_store_move_before (GTK_LIST_STORE (model), iter, NULL);
-    }
-  else if (new_index == 0)
-    {
-      gtk_list_store_move_after (GTK_LIST_STORE (model), iter, NULL);
-    }
-  else
-    {
-      GtkTreePath *path;
-      gint         old_index;
-
-      path = gtk_tree_model_get_path (model, iter);
-      old_index = gtk_tree_path_get_indices (path)[0];
-      gtk_tree_path_free (path);
-
-      if (new_index != old_index)
-        {
-          GtkTreeIter  place;
-
-          path = gtk_tree_path_new_from_indices (new_index, -1);
-          gtk_tree_model_get_iter (model, &place, path);
-          gtk_tree_path_free (path);
-
-          if (new_index > old_index)
-            gtk_list_store_move_after (GTK_LIST_STORE (model), iter, &place);
-          else
-            gtk_list_store_move_before (GTK_LIST_STORE (model), iter, &place);
-        }
-    }
+  if (model)
+    gimp_container_tree_store_reorder_item (GIMP_CONTAINER_TREE_STORE (model),
+                                            viewable,
+                                            new_index,
+                                            insert_data);
 }
 
 static void
@@ -418,44 +349,65 @@ gimp_container_combo_box_rename_item (GimpContainerView *view,
                                       gpointer           insert_data)
 {
   GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
-  GtkTreeIter  *iter  = insert_data;
 
-  if (iter)
-    {
-      gchar *name = gimp_viewable_get_description (viewable, NULL);
-
-      gtk_list_store_set (GTK_LIST_STORE (model), iter,
-                          GIMP_CONTAINER_COMBO_BOX_COLUMN_NAME, name,
-                          -1);
-
-      g_free (name);
-    }
+  if (model)
+    gimp_container_tree_store_rename_item (GIMP_CONTAINER_TREE_STORE (model),
+                                           viewable,
+                                           insert_data);
 }
 
 static gboolean
-gimp_container_combo_box_select_item (GimpContainerView *view,
-                                      GimpViewable      *viewable,
-                                      gpointer           insert_data)
+gimp_container_combo_box_select_items (GimpContainerView *view,
+                                       GList             *viewables,
+                                       GList             *paths)
 {
   GtkComboBox *combo_box = GTK_COMBO_BOX (view);
-  GtkTreeIter *iter      = insert_data;
 
-  g_signal_handlers_block_by_func (combo_box,
-                                   gimp_container_combo_box_changed,
-                                   view);
+  g_return_val_if_fail (GIMP_IS_CONTAINER_VIEW (view), FALSE);
+  /* Only zero or one items may selected in a GimpContainerComboBox. */
+  g_return_val_if_fail (g_list_length (viewables) < 2, FALSE);
 
-  if (iter)
+  if (gtk_combo_box_get_model (GTK_COMBO_BOX (view)))
     {
-      gtk_combo_box_set_active_iter (combo_box, iter);
-    }
-  else
-    {
-      gtk_combo_box_set_active (combo_box, -1);
-    }
+      g_signal_handlers_block_by_func (combo_box,
+                                       gimp_container_combo_box_changed,
+                                       view);
 
-  g_signal_handlers_unblock_by_func (combo_box,
-                                     gimp_container_combo_box_changed,
-                                     view);
+      if (viewables)
+        {
+          GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
+          GtkTreeIter   iter;
+          gboolean      iter_valid;
+
+          for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
+               iter_valid;
+               iter_valid = gtk_tree_model_iter_next (model, &iter))
+            {
+              GimpViewRenderer *renderer;
+
+              gtk_tree_model_get (model, &iter,
+                                  GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
+                                  -1);
+
+              if (renderer->viewable == viewables->data)
+                {
+                  gtk_combo_box_set_active_iter (combo_box, &iter);
+                  g_object_unref (renderer);
+
+                  break;
+                }
+              g_object_unref (renderer);
+            }
+        }
+      else
+        {
+          gtk_combo_box_set_active (combo_box, -1);
+        }
+
+      g_signal_handlers_unblock_by_func (combo_box,
+                                         gimp_container_combo_box_changed,
+                                         view);
+    }
 
   return TRUE;
 }
@@ -465,7 +417,8 @@ gimp_container_combo_box_clear_items (GimpContainerView *view)
 {
   GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
 
-  gtk_list_store_clear (GTK_LIST_STORE (model));
+  if (model)
+    gimp_container_tree_store_clear_items (GIMP_CONTAINER_TREE_STORE (model));
 
   gtk_widget_set_sensitive (GTK_WIDGET (view), FALSE);
 
@@ -476,63 +429,58 @@ static void
 gimp_container_combo_box_set_view_size (GimpContainerView *view)
 {
   GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
-  GtkTreeIter   iter;
-  gboolean      iter_valid;
-  gint          view_size;
-  gint          border_width;
 
-  view_size = gimp_container_view_get_view_size (view, &border_width);
+  if (model)
+    gimp_container_tree_store_set_view_size (GIMP_CONTAINER_TREE_STORE (model));
+}
 
-  for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
-       iter_valid;
-       iter_valid = gtk_tree_model_iter_next (model, &iter))
+static gint
+gimp_container_combo_box_get_selected (GimpContainerView  *view,
+                                       GList             **items,
+                                       GList             **items_data)
+{
+  GtkComboBox      *combo_box = GTK_COMBO_BOX (view);
+  GimpViewRenderer *renderer  = NULL;
+  GtkTreeIter       iter;
+  gint              selected  = 0;
+
+  if (gtk_combo_box_get_active_iter (combo_box, &iter))
+    gtk_tree_model_get (gtk_combo_box_get_model (combo_box), &iter,
+                        GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
+                        -1);
+
+  if (items)
     {
-      GimpViewRenderer *renderer;
-
-      gtk_tree_model_get (model, &iter,
-                          GIMP_CONTAINER_COMBO_BOX_COLUMN_RENDERER, &renderer,
-                          -1);
-
-      gimp_view_renderer_set_size (renderer, view_size, border_width);
-      g_object_unref (renderer);
+      if (renderer != NULL && renderer->viewable != NULL)
+        {
+          *items   = g_list_prepend (NULL, renderer->viewable);
+          selected = 1;
+        }
+      else
+        {
+          *items = NULL;
+        }
     }
+  g_clear_object (&renderer);
+
+  return selected;
 }
 
 static void
-gimp_container_combo_box_changed (GtkComboBox       *combo_box,
+gimp_container_combo_box_changed (GtkComboBox       *combo,
                                   GimpContainerView *view)
 {
   GtkTreeIter iter;
 
-  if (GTK_COMBO_BOX_CLASS (parent_class)->changed)
-    GTK_COMBO_BOX_CLASS (parent_class)->changed (combo_box);
-
-  if (gtk_combo_box_get_active_iter (combo_box, &iter))
+  if (gtk_combo_box_get_active_iter (combo, &iter))
     {
       GimpViewRenderer *renderer;
 
-      gtk_tree_model_get (gtk_combo_box_get_model (combo_box), &iter,
-                          GIMP_CONTAINER_COMBO_BOX_COLUMN_RENDERER, &renderer,
+      gtk_tree_model_get (gtk_combo_box_get_model (combo), &iter,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                           -1);
 
       gimp_container_view_item_selected (view, renderer->viewable);
       g_object_unref (renderer);
     }
 }
-
-static void
-gimp_container_combo_box_renderer_update (GimpViewRenderer  *renderer,
-                                          GimpContainerView *view)
-{
-  GtkTreeIter *iter = gimp_container_view_lookup (view, renderer->viewable);
-
-  if (iter)
-    {
-      GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (view));
-      GtkTreePath  *path;
-
-      path = gtk_tree_model_get_path (model, iter);
-      gtk_tree_model_row_changed (model, path, iter);
-      gtk_tree_path_free (path);
-    }
-}
