@@ -15,15 +15,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #define GIMP_ENABLE_CONTROLLER_UNDER_CONSTRUCTION
@@ -33,6 +35,7 @@
 
 #include "core/gimpcontext.h"
 
+#include "gimpaction.h"
 #include "gimpactioneditor.h"
 #include "gimpactionview.h"
 #include "gimpcontrollereditor.h"
@@ -40,7 +43,10 @@
 #include "gimpdialogfactory.h"
 #include "gimphelp-ids.h"
 #include "gimpuimanager.h"
+#include "gimpview.h"
 #include "gimpviewabledialog.h"
+#include "gimpviewrenderer.h"
+#include "gimpwidgets-utils.h"
 
 #include "gimp-intl.h"
 
@@ -56,7 +62,7 @@ enum
 {
   COLUMN_EVENT,
   COLUMN_BLURB,
-  COLUMN_STOCK_ID,
+  COLUMN_ICON_NAME,
   COLUMN_ACTION,
   N_COLUMNS
 };
@@ -159,7 +165,7 @@ gimp_controller_editor_constructed (GObject *object)
   GtkListStore         *store;
   GtkWidget            *frame;
   GtkWidget            *vbox;
-  GtkWidget            *table;
+  GtkWidget            *grid;
   GtkWidget            *hbox;
   GtkWidget            *button;
   GtkWidget            *tv;
@@ -173,10 +179,9 @@ gimp_controller_editor_constructed (GObject *object)
   gint                  row;
   gint                  i;
 
-  if (G_OBJECT_CLASS (parent_class)->constructed)
-    G_OBJECT_CLASS (parent_class)->constructed (object);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  g_assert (GIMP_IS_CONTROLLER_INFO (editor->info));
+  gimp_assert (GIMP_IS_CONTROLLER_INFO (editor->info));
 
   info             = editor->info;
   controller       = info->controller;
@@ -192,17 +197,14 @@ gimp_controller_editor_constructed (GObject *object)
 
   entry = gimp_prop_entry_new (G_OBJECT (info), "name", -1);
   gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0);
-  gtk_widget_show (entry);
 
   button = gimp_prop_check_button_new (G_OBJECT (info), "debug-events",
                                        _("_Dump events from this controller"));
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
 
   button = gimp_prop_check_button_new (G_OBJECT (info), "enabled",
                                        _("_Enable this controller"));
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
 
   frame = gimp_frame_new (controller_class->name);
   gtk_box_pack_start (GTK_BOX (editor), frame, TRUE, TRUE, 0);
@@ -212,25 +214,25 @@ gimp_controller_editor_constructed (GObject *object)
   gtk_container_add (GTK_CONTAINER (frame), vbox);
   gtk_widget_show (vbox);
 
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_box_pack_start (GTK_BOX (vbox), grid, FALSE, FALSE, 0);
+  gtk_widget_show (grid);
 
   row = 0;
 
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                             _("Name:"), 0.0, 0.5,
-                             gimp_prop_label_new (G_OBJECT (controller),
-                                                  "name"),
-                             1, TRUE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                            _("Name:"), 0.0, 0.5,
+                            gimp_prop_label_new (G_OBJECT (controller),
+                                                 "name"),
+                            1);
 
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                             _("State:"), 0.0, 0.5,
-                             gimp_prop_label_new (G_OBJECT (controller),
-                                                  "state"),
-                             1, TRUE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                            _("State:"), 0.0, 0.5,
+                            gimp_prop_label_new (G_OBJECT (controller),
+                                                 "state"),
+                            1);
 
   property_specs =
     g_object_class_list_properties (G_OBJECT_CLASS (controller_class),
@@ -248,21 +250,22 @@ gimp_controller_editor_constructed (GObject *object)
         {
           widget = gimp_controller_string_view_new (controller, pspec);
 
-          gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                                     g_param_spec_get_nick (pspec),
-                                     0.0, 0.5,
-                                     widget,
-                                     1, FALSE);
+          gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                                    g_param_spec_get_nick (pspec),
+                                    0.0, 0.5,
+                                    widget,
+                                    1);
         }
       else if (G_IS_PARAM_SPEC_INT (pspec))
         {
           widget = gimp_controller_int_view_new (controller, pspec);
+          gtk_widget_set_halign (widget, GTK_ALIGN_START);
 
-          gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                                     g_param_spec_get_nick (pspec),
-                                     0.0, 0.5,
-                                     widget,
-                                     1, TRUE);
+          gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                                    g_param_spec_get_nick (pspec),
+                                    0.0, 0.5,
+                                    widget,
+                                    1);
         }
     }
 
@@ -279,6 +282,7 @@ gimp_controller_editor_constructed (GObject *object)
   sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
                                        GTK_SHADOW_IN);
+  gtk_widget_set_size_request (sw, 400, 300);
   gtk_container_add (GTK_CONTAINER (sw), tv);
   gtk_widget_show (tv);
 
@@ -305,7 +309,7 @@ gimp_controller_editor_constructed (GObject *object)
       const gchar *event_name;
       const gchar *event_blurb;
       const gchar *event_action;
-      const gchar *stock_id = NULL;
+      const gchar *icon_name = NULL;
 
       event_name  = gimp_controller_get_event_name  (controller, i);
       event_blurb = gimp_controller_get_event_blurb (controller, i);
@@ -314,20 +318,20 @@ gimp_controller_editor_constructed (GObject *object)
 
       if (event_action)
         {
-          GtkAction *action;
+          GimpAction *action;
 
           action = gimp_ui_manager_find_action (ui_manager, NULL, event_action);
 
           if (action)
-            stock_id = gtk_action_get_stock_id (action);
+            icon_name = gimp_action_get_icon_name (action);
         }
 
       gtk_list_store_append (store, &iter);
       gtk_list_store_set (store, &iter,
-                          COLUMN_EVENT,    event_name,
-                          COLUMN_BLURB,    event_blurb,
-                          COLUMN_STOCK_ID, stock_id,
-                          COLUMN_ACTION,   event_action,
+                          COLUMN_EVENT,     event_name,
+                          COLUMN_BLURB,     event_blurb,
+                          COLUMN_ICON_NAME, icon_name,
+                          COLUMN_ACTION,    event_action,
                           -1);
     }
 
@@ -344,7 +348,7 @@ gimp_controller_editor_constructed (GObject *object)
   cell = gtk_cell_renderer_pixbuf_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
   gtk_tree_view_column_set_attributes (column, cell,
-                                       "stock-id", COLUMN_STOCK_ID,
+                                       "icon-name", COLUMN_ICON_NAME,
                                        NULL);
 
   cell = gtk_cell_renderer_text_new ();
@@ -370,7 +374,7 @@ gimp_controller_editor_constructed (GObject *object)
                              "the controller"),
                            NULL);
 
-  editor->edit_button = gtk_button_new_from_stock (GTK_STOCK_EDIT);
+  editor->edit_button = gtk_button_new_with_mnemonic (_("_Edit event"));
   gtk_box_pack_start (GTK_BOX (hbox), editor->edit_button, TRUE, TRUE, 0);
   gtk_widget_show (editor->edit_button);
 
@@ -378,7 +382,7 @@ gimp_controller_editor_constructed (GObject *object)
                     G_CALLBACK (gimp_controller_editor_edit_clicked),
                     editor);
 
-  editor->delete_button = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
+  editor->delete_button = gtk_button_new_with_mnemonic (_("_Clear event"));
   gtk_box_pack_start (GTK_BOX (hbox), editor->delete_button, TRUE, TRUE, 0);
   gtk_widget_show (editor->delete_button);
 
@@ -399,15 +403,10 @@ gimp_controller_editor_finalize (GObject *object)
     {
       gimp_controller_info_set_event_snooper (editor->info, NULL, NULL);
 
-      g_object_unref (editor->info);
-      editor->info = NULL;
+      g_clear_object (&editor->info);
     }
 
-  if (editor->context)
-    {
-      g_object_unref (editor->context);
-      editor->context = NULL;
-    }
+  g_clear_object (&editor->context);
 
   if (editor->edit_dialog)
     gtk_widget_destroy (editor->edit_dialog);
@@ -645,66 +644,76 @@ gimp_controller_editor_edit_clicked (GtkWidget            *button,
 
   if (event_name)
     {
-      GtkWidget *view;
-      gchar     *title;
+      GtkWidget        *view;
+      gchar            *title;
+      GimpViewRenderer *renderer;
+      GeglColor        *color = gegl_color_new ("transparent");
 
       title = g_strdup_printf (_("Select Action for Event '%s'"),
                                event_blurb);
 
-      editor->edit_dialog =
-        gimp_viewable_dialog_new (GIMP_VIEWABLE (editor->info), editor->context,
-                                  _("Select Controller Event Action"),
-                                  "gimp-controller-action-dialog",
-                                  GTK_STOCK_EDIT,
-                                  title,
-                                  gtk_widget_get_toplevel (GTK_WIDGET (editor)),
-                                  gimp_standard_help_func,
-                                  GIMP_HELP_PREFS_INPUT_CONTROLLERS,
+      g_set_weak_pointer
+        (&editor->edit_dialog,
+         gimp_viewable_dialog_new (g_list_prepend (NULL, editor->info), editor->context,
+                                   _("Select Controller Event Action"),
+                                   "gimp-controller-action-dialog",
+                                   GIMP_ICON_EDIT,
+                                   title,
+                                   gtk_widget_get_toplevel (GTK_WIDGET (editor)),
+                                   gimp_standard_help_func,
+                                   GIMP_HELP_PREFS_INPUT_CONTROLLERS,
 
-                                  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                  GTK_STOCK_OK,     GTK_RESPONSE_OK,
+                                   _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                   _("_OK"),     GTK_RESPONSE_OK,
 
-                                  NULL);
+                                   NULL));
 
       g_free (title);
 
-      gtk_dialog_set_alternative_button_order (GTK_DIALOG (editor->edit_dialog),
-                                               GTK_RESPONSE_OK,
-                                               GTK_RESPONSE_CANCEL,
-                                               -1);
-
-      g_object_add_weak_pointer (G_OBJECT (editor->edit_dialog),
-                                 (gpointer) &editor->edit_dialog);
+      gimp_dialog_set_alternative_button_order (GTK_DIALOG (editor->edit_dialog),
+                                                GTK_RESPONSE_OK,
+                                                GTK_RESPONSE_CANCEL,
+                                                -1);
 
       gimp_dialog_factory_add_foreign (gimp_dialog_factory_get_singleton (),
                                        "gimp-controller-action-dialog",
-                                       editor->edit_dialog);
+                                       editor->edit_dialog,
+                                       gimp_widget_get_monitor (button));
 
       g_signal_connect (editor->edit_dialog, "response",
                         G_CALLBACK (gimp_controller_editor_edit_response),
                         editor);
 
-      view = gimp_action_editor_new (gimp_ui_managers_from_name ("<Image>")->data,
+      view = gimp_action_editor_new (editor->context->gimp,
                                      action_name, FALSE);
       gtk_container_set_border_width (GTK_CONTAINER (view), 12);
       gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (editor->edit_dialog))),
                           view, TRUE, TRUE, 0);
-      gtk_widget_show (view);
+      gtk_widget_set_visible (view, TRUE);
 
       g_signal_connect (GIMP_ACTION_EDITOR (view)->view, "row-activated",
                         G_CALLBACK (gimp_controller_editor_edit_activated),
                         editor);
 
-      editor->edit_sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (GIMP_ACTION_EDITOR (view)->view));
-
-      g_object_add_weak_pointer (G_OBJECT (editor->edit_sel),
-                                 (gpointer) &editor->edit_sel);
+      g_set_weak_pointer
+        (&editor->edit_sel,
+         gtk_tree_view_get_selection (GTK_TREE_VIEW (GIMP_ACTION_EDITOR (view)->view)));
 
       gtk_widget_set_sensitive (GTK_WIDGET (editor), FALSE);
-      gtk_widget_show (editor->edit_dialog);
+      gtk_widget_set_visible (editor->edit_dialog, TRUE);
+
+      /* GimpViewDialog's icon is a GimpView, so it always has a border.
+       * Since we show a standard icon instead however, we'll set the
+       * GimpView border to transparent */
+      renderer =
+        GIMP_VIEW (GIMP_VIEWABLE_DIALOG (editor->edit_dialog)->view)->renderer;
+      gimp_view_renderer_set_border_color (renderer, color);
 
       g_free (event_name);
+      g_free (event_blurb);
       g_free (action_name);
+
+      g_object_unref (color);
     }
 }
 
@@ -729,8 +738,8 @@ gimp_controller_editor_delete_clicked (GtkWidget            *button,
       g_free (event_name);
 
       gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-                          COLUMN_STOCK_ID, NULL,
-                          COLUMN_ACTION,   NULL,
+                          COLUMN_ICON_NAME, NULL,
+                          COLUMN_ACTION,    NULL,
                           -1);
     }
 }
@@ -756,13 +765,13 @@ gimp_controller_editor_edit_response (GtkWidget            *dialog,
       GtkTreeModel *model;
       GtkTreeIter   iter;
       gchar        *event_name  = NULL;
-      gchar        *stock_id    = NULL;
+      gchar        *icon_name   = NULL;
       gchar        *action_name = NULL;
 
       if (gtk_tree_selection_get_selected (editor->edit_sel, &model, &iter))
         gtk_tree_model_get (model, &iter,
-                            GIMP_ACTION_VIEW_COLUMN_STOCK_ID, &stock_id,
-                            GIMP_ACTION_VIEW_COLUMN_NAME,     &action_name,
+                            GIMP_ACTION_VIEW_COLUMN_ICON_NAME, &icon_name,
+                            GIMP_ACTION_VIEW_COLUMN_NAME,      &action_name,
                             -1);
 
       if (gtk_tree_selection_get_selected (editor->sel, &model, &iter))
@@ -777,13 +786,13 @@ gimp_controller_editor_edit_response (GtkWidget            *dialog,
                                g_strdup (action_name));
 
           gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-                              COLUMN_STOCK_ID, stock_id,
-                              COLUMN_ACTION,   action_name,
+                              COLUMN_ICON_NAME, icon_name,
+                              COLUMN_ACTION,    action_name,
                               -1);
         }
 
       g_free (event_name);
-      g_free (stock_id);
+      g_free (icon_name);
       g_free (action_name);
 
       gimp_controller_editor_sel_changed (editor->sel, editor);
