@@ -238,7 +238,7 @@ gimp_controller_list_init (GimpControllerList *list)
   gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
   gtk_widget_show (vbox);
 
-  list->add_button = gtk_button_new ();
+  g_set_weak_pointer (&list->add_button, gtk_button_new ());
   gtk_box_pack_start (GTK_BOX (vbox), list->add_button, TRUE, FALSE, 0);
   gtk_widget_set_sensitive (list->add_button, FALSE);
   gtk_widget_show (list->add_button);
@@ -252,10 +252,7 @@ gimp_controller_list_init (GimpControllerList *list)
                     G_CALLBACK (gimp_controller_list_add_clicked),
                     list);
 
-  g_object_add_weak_pointer (G_OBJECT (list->add_button),
-                             (gpointer) &list->add_button);
-
-  list->remove_button = gtk_button_new ();
+  g_set_weak_pointer (&list->remove_button, gtk_button_new ());
   gtk_box_pack_start (GTK_BOX (vbox), list->remove_button, TRUE, FALSE, 0);
   gtk_widget_set_sensitive (list->remove_button, FALSE);
   gtk_widget_show (list->remove_button);
@@ -269,9 +266,6 @@ gimp_controller_list_init (GimpControllerList *list)
                     G_CALLBACK (gimp_controller_list_remove_clicked),
                     list);
 
-  g_object_add_weak_pointer (G_OBJECT (list->remove_button),
-                             (gpointer) &list->remove_button);
-
   gtk_icon_size_lookup (icon_size, &icon_width, &icon_height);
   list->dest = gimp_container_tree_view_new (NULL, NULL, icon_height, 0);
   gimp_container_tree_view_set_main_column_title (GIMP_CONTAINER_TREE_VIEW (list->dest),
@@ -295,7 +289,7 @@ gimp_controller_list_init (GimpControllerList *list)
                             NULL,
                             G_CALLBACK (gimp_controller_list_edit_clicked),
                             NULL,
-                            list);
+                            G_OBJECT (list));
   list->up_button =
     gimp_editor_add_button (GIMP_EDITOR (list->dest),
                             GIMP_ICON_GO_UP,
@@ -303,7 +297,7 @@ gimp_controller_list_init (GimpControllerList *list)
                             NULL,
                             G_CALLBACK (gimp_controller_list_up_clicked),
                             NULL,
-                            list);
+                            G_OBJECT (list));
   list->down_button =
     gimp_editor_add_button (GIMP_EDITOR (list->dest),
                             GIMP_ICON_GO_DOWN,
@@ -311,7 +305,7 @@ gimp_controller_list_init (GimpControllerList *list)
                             NULL,
                             G_CALLBACK (gimp_controller_list_down_clicked),
                             NULL,
-                            list);
+                            G_OBJECT (list));
 
   gtk_widget_set_sensitive (list->edit_button, FALSE);
   gtk_widget_set_sensitive (list->up_button,   FALSE);
