@@ -12,11 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "widgets-types.h"
@@ -38,7 +39,7 @@ gimp_device_info_get_event_coords (GimpDeviceInfo *info,
 {
   gdouble x;
 
-  if (gdk_event_get_axis (event, GDK_AXIS_X, &x))
+  if (event && gdk_event_get_axis (event, GDK_AXIS_X, &x))
     {
       *coords = default_coords;
 
@@ -62,11 +63,13 @@ gimp_device_info_get_event_coords (GimpDeviceInfo *info,
               gint offset_x;
               gint offset_y;
 
-              gtk_widget_translate_coordinates (src_widget, dest_widget,
-                                                0, 0, &offset_x, &offset_y);
-
-              coords->x += offset_x;
-              coords->y += offset_y;
+              if (gtk_widget_translate_coordinates (src_widget, dest_widget,
+                                                    0, 0,
+                                                    &offset_x, &offset_y))
+                {
+                  coords->x += offset_x;
+                  coords->y += offset_y;
+                }
             }
         }
 
@@ -98,6 +101,27 @@ gimp_device_info_get_event_coords (GimpDeviceInfo *info,
                                                      coords->wheel);
         }
 
+      if (gdk_event_get_axis (event, GDK_AXIS_DISTANCE, &coords->distance))
+        {
+          coords->distance = gimp_device_info_map_axis (info,
+                                                        GDK_AXIS_DISTANCE,
+                                                        coords->distance);
+        }
+
+      if (gdk_event_get_axis (event, GDK_AXIS_ROTATION, &coords->rotation))
+        {
+          coords->rotation = gimp_device_info_map_axis (info,
+                                                        GDK_AXIS_ROTATION,
+                                                        coords->rotation);
+        }
+
+      if (gdk_event_get_axis (event, GDK_AXIS_SLIDER, &coords->slider))
+        {
+          coords->slider = gimp_device_info_map_axis (info,
+                                                      GDK_AXIS_SLIDER,
+                                                      coords->slider);
+        }
+
       return TRUE;
     }
 
@@ -111,16 +135,34 @@ gimp_device_info_get_device_coords (GimpDeviceInfo *info,
                                     GdkWindow      *window,
                                     GimpCoords     *coords)
 {
-  gdouble axes[GDK_AXIS_LAST] = { 0, };
+  GdkDevice *device = gimp_device_info_get_device (info, NULL);
+  gdouble    axes[GDK_AXIS_LAST] = { 0, };
+
+  if (gdk_device_get_device_type (device) == GDK_DEVICE_TYPE_SLAVE)
+    device = gdk_device_get_associated_device (device);
+
+  if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
+    device = gdk_device_get_associated_device (device);
 
   *coords = default_coords;
 
-  gdk_device_get_state (info->device, window, axes, NULL);
+  gdk_device_get_state (device, window, axes, NULL);
 
-  gdk_device_get_axis (info->device, axes, GDK_AXIS_X, &coords->x);
-  gdk_device_get_axis (info->device, axes, GDK_AXIS_Y, &coords->y);
+#if 0
+  gdk_device_get_axis (device, axes, GDK_AXIS_X, &coords->x);
+  gdk_device_get_axis (device, axes, GDK_AXIS_Y, &coords->y);
+#else
+  {
+    gint x, y;
 
-  if (gdk_device_get_axis (info->device,
+    gdk_window_get_device_position (window, device, &x, &y, NULL);
+
+    coords->x = x;
+    coords->y = y;
+  }
+#endif
+
+  if (gdk_device_get_axis (device,
                            axes, GDK_AXIS_PRESSURE, &coords->pressure))
     {
       coords->pressure = gimp_device_info_map_axis (info,
@@ -128,7 +170,7 @@ gimp_device_info_get_device_coords (GimpDeviceInfo *info,
                                                     coords->pressure);
     }
 
-  if (gdk_device_get_axis (info->device,
+  if (gdk_device_get_axis (device,
                            axes, GDK_AXIS_XTILT, &coords->xtilt))
     {
       coords->xtilt = gimp_device_info_map_axis (info,
@@ -136,7 +178,7 @@ gimp_device_info_get_device_coords (GimpDeviceInfo *info,
                                                  coords->xtilt);
     }
 
-  if (gdk_device_get_axis (info->device,
+  if (gdk_device_get_axis (device,
                            axes, GDK_AXIS_YTILT, &coords->ytilt))
     {
       coords->ytilt = gimp_device_info_map_axis (info,
@@ -144,13 +186,37 @@ gimp_device_info_get_device_coords (GimpDeviceInfo *info,
                                                  coords->ytilt);
     }
 
-  if (gdk_device_get_axis (info->device,
+  if (gdk_device_get_axis (device,
                            axes, GDK_AXIS_WHEEL, &coords->wheel))
     {
       coords->wheel = gimp_device_info_map_axis (info,
                                                  GDK_AXIS_WHEEL,
                                                  coords->wheel);
     }
+
+  if (gdk_device_get_axis (device,
+                           axes, GDK_AXIS_DISTANCE, &coords->distance))
+    {
+      coords->distance = gimp_device_info_map_axis (info,
+                                                    GDK_AXIS_DISTANCE,
+                                                    coords->distance);
+    }
+
+  if (gdk_device_get_axis (device,
+                           axes, GDK_AXIS_ROTATION, &coords->rotation))
+    {
+      coords->rotation = gimp_device_info_map_axis (info,
+                                                    GDK_AXIS_ROTATION,
+                                                    coords->rotation);
+    }
+
+  if (gdk_device_get_axis (device,
+                           axes, GDK_AXIS_SLIDER, &coords->slider))
+    {
+      coords->slider = gimp_device_info_map_axis (info,
+                                                  GDK_AXIS_SLIDER,
+                                                  coords->slider);
+    }
 }
 
 void
@@ -158,16 +224,18 @@ gimp_device_info_get_time_coords (GimpDeviceInfo *info,
                                   GdkTimeCoord   *event,
                                   GimpCoords     *coords)
 {
+  GdkDevice *device = gimp_device_info_get_device (info, NULL);
+
   *coords = default_coords;
 
-  gdk_device_get_axis (info->device, event->axes, GDK_AXIS_X, &coords->x);
-  gdk_device_get_axis (info->device, event->axes, GDK_AXIS_Y, &coords->y);
+  gdk_device_get_axis (device, event->axes, GDK_AXIS_X, &coords->x);
+  gdk_device_get_axis (device, event->axes, GDK_AXIS_Y, &coords->y);
 
   /*  CLAMP() the return value of each *_get_axis() call to be safe
    *  against buggy XInput drivers.
    */
 
-  if (gdk_device_get_axis (info->device,
+  if (gdk_device_get_axis (device,
                            event->axes, GDK_AXIS_PRESSURE, &coords->pressure))
     {
       coords->pressure = gimp_device_info_map_axis (info,
@@ -175,7 +243,7 @@ gimp_device_info_get_time_coords (GimpDeviceInfo *info,
                                                     coords->pressure);
     }
 
-  if (gdk_device_get_axis (info->device,
+  if (gdk_device_get_axis (device,
                            event->axes, GDK_AXIS_XTILT, &coords->xtilt))
     {
       coords->xtilt = gimp_device_info_map_axis (info,
@@ -183,7 +251,7 @@ gimp_device_info_get_time_coords (GimpDeviceInfo *info,
                                                  coords->xtilt);
     }
 
-  if (gdk_device_get_axis (info->device,
+  if (gdk_device_get_axis (device,
                            event->axes, GDK_AXIS_YTILT, &coords->ytilt))
     {
       coords->ytilt = gimp_device_info_map_axis (info,
@@ -191,13 +259,37 @@ gimp_device_info_get_time_coords (GimpDeviceInfo *info,
                                                  coords->ytilt);
     }
 
-  if (gdk_device_get_axis (info->device,
+  if (gdk_device_get_axis (device,
                            event->axes, GDK_AXIS_WHEEL, &coords->wheel))
     {
       coords->wheel = gimp_device_info_map_axis (info,
                                                  GDK_AXIS_WHEEL,
                                                  coords->wheel);
     }
+
+  if (gdk_device_get_axis (device,
+                           event->axes, GDK_AXIS_DISTANCE, &coords->distance))
+    {
+      coords->distance = gimp_device_info_map_axis (info,
+                                                    GDK_AXIS_DISTANCE,
+                                                    coords->distance);
+    }
+
+  if (gdk_device_get_axis (device,
+                           event->axes, GDK_AXIS_ROTATION, &coords->rotation))
+    {
+      coords->rotation = gimp_device_info_map_axis (info,
+                                                    GDK_AXIS_ROTATION,
+                                                    coords->rotation);
+    }
+
+  if (gdk_device_get_axis (device,
+                           event->axes, GDK_AXIS_SLIDER, &coords->slider))
+    {
+      coords->slider = gimp_device_info_map_axis (info,
+                                                  GDK_AXIS_SLIDER,
+                                                  coords->slider);
+    }
 }
 
 gboolean
@@ -219,5 +311,29 @@ gimp_device_info_get_device_state (GimpDeviceInfo  *info,
                                    GdkWindow       *window,
                                    GdkModifierType *state)
 {
-  gdk_device_get_state (info->device, window, NULL, state);
+  GdkDevice *device = gimp_device_info_get_device (info, NULL);
+
+  switch (gdk_device_get_device_type (device))
+    {
+    case GDK_DEVICE_TYPE_SLAVE:
+      device = gdk_device_get_associated_device (device);
+      break;
+
+    case GDK_DEVICE_TYPE_FLOATING:
+      {
+        GdkDisplay *display = gdk_device_get_display (device);
+        GdkSeat    *seat    = gdk_display_get_default_seat (display);
+
+        device = gdk_seat_get_pointer (seat);
+      }
+      break;
+
+    default:
+      break;
+    }
+
+  if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
+    device = gdk_device_get_associated_device (device);
+
+  gdk_device_get_state (device, window, NULL, state);
 }
