@@ -47,7 +47,7 @@
 
 #include "text/gimpfont.h"
 
-#include "vectors/gimpvectors.h"
+#include "vectors/gimppath.h"
 
 #include "gimpdnd.h"
 #include "gimpdnd-xds.h"
@@ -476,7 +476,7 @@ static const GimpDndDataDef dnd_data_defs[] =
   },
 
   {
-    GIMP_TARGET_VECTORS,
+    GIMP_TARGET_PATH,
 
     "gimp-dnd-get-vectors-func",
     "gimp-dnd-get-vectors-data",
@@ -658,7 +658,7 @@ static const GimpDndDataDef dnd_data_defs[] =
   },
 
   {
-    GIMP_TARGET_VECTORS_LIST,
+    GIMP_TARGET_PATH_LIST,
 
     "gimp-dnd-get-vectors-list-func",
     "gimp-dnd-get-vectors-list-data",
@@ -1387,17 +1387,17 @@ gimp_dnd_get_color_icon (GtkWidget      *widget,
                          gpointer        get_color_data)
 {
   GtkWidget *color_area;
-  GimpRGB    color;
+  GeglColor *color = NULL;
 
   (* (GimpDndDragColorFunc) get_color_func) (widget, &color, get_color_data);
 
   GIMP_LOG (DND, "called");
 
   g_object_set_data_full (G_OBJECT (context),
-                          "gimp-dnd-color", g_memdup2 (&color, sizeof (GimpRGB)),
-                          (GDestroyNotify) g_free);
+                          "gimp-dnd-color", color,
+                          (GDestroyNotify) g_object_unref);
 
-  color_area = gimp_color_area_new (&color, GIMP_COLOR_AREA_SMALL_CHECKS, 0);
+  color_area = gimp_color_area_new (color, GIMP_COLOR_AREA_SMALL_CHECKS, 0);
   gimp_color_area_set_color_config (GIMP_COLOR_AREA (color_area),
                                     the_dnd_gimp->config->color_management);
   gtk_widget_set_size_request (color_area,
@@ -1413,19 +1413,19 @@ gimp_dnd_get_color_data (GtkWidget        *widget,
                          gpointer          get_color_data,
                          GtkSelectionData *selection)
 {
-  GimpRGB *c;
-  GimpRGB  color;
+  GeglColor *color = NULL;
 
-  c = g_object_get_data (G_OBJECT (context), "gimp-dnd-color");
+  color = g_object_get_data (G_OBJECT (context), "gimp-dnd-color");
 
-  if (c)
-    color = *c;
+  if (color)
+    color = g_object_ref (color);
   else
     (* (GimpDndDragColorFunc) get_color_func) (widget, &color, get_color_data);
 
   GIMP_LOG (DND, "called");
 
-  gimp_selection_data_set_color (selection, &color);
+  gimp_selection_data_set_color (selection, color);
+  g_object_unref (color);
 }
 
 static gboolean
@@ -1436,14 +1436,14 @@ gimp_dnd_set_color_data (GtkWidget        *widget,
                          gpointer          set_color_data,
                          GtkSelectionData *selection)
 {
-  GimpRGB color;
+  GeglColor *color;
 
   GIMP_LOG (DND, "called");
 
-  if (! gimp_selection_data_get_color (selection, &color))
+  if (! (color = gimp_selection_data_get_color (selection)))
     return FALSE;
 
-  (* (GimpDndDropColorFunc) set_color_func) (widget, x, y, &color,
+  (* (GimpDndDropColorFunc) set_color_func) (widget, x, y, color,
                                              set_color_data);
 
   return TRUE;
@@ -1948,7 +1948,7 @@ gimp_dnd_data_type_get_by_g_type (GType    type,
     {
       dnd_type = list ? GIMP_DND_TYPE_CHANNEL_LIST : GIMP_DND_TYPE_CHANNEL;
     }
-  else if (g_type_is_a (type, GIMP_TYPE_VECTORS))
+  else if (g_type_is_a (type, GIMP_TYPE_PATH))
     {
       dnd_type = list ? GIMP_DND_TYPE_VECTORS_LIST : GIMP_DND_TYPE_VECTORS;
     }
@@ -2204,6 +2204,10 @@ gimp_dnd_get_viewable_list_icon (GtkWidget      *widget,
                                                                &gimp_context,
                                                                get_list_data);
 
+  g_object_set_data_full (G_OBJECT (widget),
+                          "gimp-dnd-viewables",  viewables,
+                          (GDestroyNotify) g_list_free);
+
   if (! viewables)
     return NULL;
 
@@ -2234,7 +2238,6 @@ gimp_dnd_get_viewable_list_icon (GtkWidget      *widget,
       desc = gimp_viewable_get_description (viewable, NULL);
       desc_width_chars =  MIN (strlen (desc), 10);
     }
-  g_list_free (viewables);
 
   if (desc)
     {
@@ -2362,11 +2365,8 @@ gimp_dnd_viewable_list_dest_remove (GtkWidget *widget,
 GList *
 gimp_dnd_get_drag_list (GtkWidget *widget)
 {
-  const GimpDndDataDef    *dnd_data;
-  GimpDndType              data_type;
-  GimpDndDragViewableListFunc  get_data_func = NULL;
-  gpointer                 get_data_data = NULL;
-  GimpContext             *context;
+  GimpDndType  data_type;
+  GList       *viewables;
 
   g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
@@ -2376,20 +2376,15 @@ gimp_dnd_get_drag_list (GtkWidget *widget)
   if (! data_type)
     return NULL;
 
-  dnd_data = dnd_data_defs + data_type;
+  /* Note: do not use the stored get_data_func_name at the end of a
+   * paste, only at drag-begin (handled inside the get_icon_func() call)
+   * for item lists, because the content of item container may change
+   * during the paste, in particular when moving from one image tab to
+   * another. Cf. #7497.
+   */
+  viewables = g_object_get_data (G_OBJECT (widget), "gimp-dnd-viewables");
 
-  if (dnd_data->get_data_func_name)
-    get_data_func = g_object_get_data (G_OBJECT (widget),
-                                       dnd_data->get_data_func_name);
-
-  if (dnd_data->get_data_data_name)
-    get_data_data = g_object_get_data (G_OBJECT (widget),
-                                       dnd_data->get_data_data_name);
-
-  if (! get_data_func)
-    return NULL;
-
-  return (GList *) (* get_data_func) (widget, &context, get_data_data);
+  return g_list_copy (viewables);
 }
 
 
@@ -2504,15 +2499,17 @@ gimp_dnd_get_item_list_data (GtkWidget        *widget,
                              gpointer          get_item_data,
                              GtkSelectionData *selection)
 {
-  GList       *items;
-  GimpContext *gimp_context;
+  GList *items;
 
-  items = (* (GimpDndDragViewableListFunc) get_item_func) (widget, &gimp_context,
-                                                           get_item_data);
+  /* Note: do not use get_item_func() during a paste, only at drag-begin
+   * (handled inside the get_icon_func() call) for item lists, because
+   * the content of item container may change during the paste, in
+   * particular when moving from one image tab to another. Cf. #7497.
+   */
+  items = g_object_get_data (G_OBJECT (widget), "gimp-dnd-viewables");
 
   if (items)
     gimp_selection_data_set_item_list (selection, items);
-  g_list_free (items);
 }
 
 static gboolean
