@@ -36,6 +36,8 @@
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
 
+#include "menus/menus.h"
+
 #include "gimpactiongroup.h"
 #include "gimpdialogfactory.h"
 #include "gimpdnd.h"
@@ -93,8 +95,6 @@ struct _GimpDockbookPrivate
 };
 
 
-static void         gimp_dockbook_finalize          (GObject        *object);
-
 static void         gimp_dockbook_style_updated     (GtkWidget       *widget);
 static void         gimp_dockbook_drag_begin        (GtkWidget      *widget,
                                                      GdkDragContext *context);
@@ -126,6 +126,9 @@ static void         gimp_dockbook_page_reordered    (GtkNotebook    *notebook,
                                                      GtkWidget      *child,
                                                      guint           page_num);
 
+static gboolean     gimp_dockbook_tab_scroll_cb     (GtkWidget      *widget,
+                                                     GdkEventScroll *event);
+
 static gboolean     gimp_dockbook_menu_button_press (GimpDockbook   *dockbook,
                                                      GdkEventButton *bevent,
                                                      GtkWidget      *button);
@@ -153,7 +156,6 @@ static GList *drag_callbacks = NULL;
 static void
 gimp_dockbook_class_init (GimpDockbookClass *klass)
 {
-  GObjectClass     *object_class   = G_OBJECT_CLASS (klass);
   GtkWidgetClass   *widget_class   = GTK_WIDGET_CLASS (klass);
   GtkNotebookClass *notebook_class = GTK_NOTEBOOK_CLASS (klass);
 
@@ -184,8 +186,6 @@ gimp_dockbook_class_init (GimpDockbookClass *klass)
                   G_TYPE_NONE, 1,
                   GIMP_TYPE_DOCKABLE);
 
-  object_class->finalize         = gimp_dockbook_finalize;
-
   widget_class->style_updated    = gimp_dockbook_style_updated;
   widget_class->drag_begin       = gimp_dockbook_drag_begin;
   widget_class->drag_end         = gimp_dockbook_drag_end;
@@ -225,6 +225,12 @@ gimp_dockbook_init (GimpDockbook *dockbook)
   gtk_notebook_set_show_tabs (notebook, TRUE);
   gtk_notebook_set_group_name (notebook, "gimp-dockbook");
 
+  gtk_widget_add_events (GTK_WIDGET (notebook),
+                         GDK_SCROLL_MASK);
+  g_signal_connect (GTK_WIDGET (notebook), "scroll-event",
+                    G_CALLBACK (gimp_dockbook_tab_scroll_cb),
+                    NULL);
+
   gtk_drag_dest_set (GTK_WIDGET (dockbook),
                      0,
                      dialog_target_table, G_N_ELEMENTS (dialog_target_table),
@@ -256,16 +262,6 @@ gimp_dockbook_init (GimpDockbook *dockbook)
                             dockbook);
 }
 
-static void
-gimp_dockbook_finalize (GObject *object)
-{
-  GimpDockbook *dockbook = GIMP_DOCKBOOK (object);
-
-  g_clear_object (&dockbook->p->ui_manager);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
 static void
 gimp_dockbook_style_updated (GtkWidget *widget)
 {
@@ -274,6 +270,8 @@ gimp_dockbook_style_updated (GtkWidget *widget)
   GtkWidget    *tab_widget;
   GList        *children;
   GList        *iter;
+  GtkIconSize   tab_size = DEFAULT_TAB_ICON_SIZE;
+  gint          icon_size = 12;
 
   GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
 
@@ -281,6 +279,7 @@ gimp_dockbook_style_updated (GtkWidget *widget)
       ! (context = gimp_dock_get_context (dockbook->p->dock)))
     return;
 
+  /* Update size of 'Configure this tab' icon */
   children = gtk_container_get_children (GTK_CONTAINER (dockbook));
   for (iter = children; iter; iter = g_list_next (iter))
     {
@@ -293,6 +292,14 @@ gimp_dockbook_style_updated (GtkWidget *widget)
     }
   g_list_free (children);
 
+  children = gtk_container_get_children (GTK_CONTAINER (dockbook->p->menu_button));
+  gtk_widget_style_get (GTK_WIDGET (dockbook),
+                        "tab-icon-size", &tab_size,
+                        NULL);
+  gtk_icon_size_lookup (tab_size, &icon_size, NULL);
+  gtk_image_set_pixel_size (GTK_IMAGE (children->data), icon_size * 0.75f);
+  g_list_free (children);
+
   gimp_dock_invalidate_geometry (GIMP_DOCK (dockbook->p->dock));
 }
 
@@ -425,7 +432,7 @@ gimp_dockbook_create_window (GtkNotebook *notebook,
   src_dock_window = gimp_dock_window_from_dock (src_dock);
 
   dialog_factory = gimp_dock_get_dialog_factory (src_dock);
-  menu_factory   = gimp_dialog_factory_get_menu_factory (dialog_factory);
+  menu_factory   = menus_get_global_menu_factory (gimp_dialog_factory_get_context (dialog_factory)->gimp);
 
   new_dock = gimp_dock_with_window_new (dialog_factory,
                                         gimp_widget_get_monitor (page),
@@ -524,6 +531,41 @@ gimp_dockbook_page_removed (GtkNotebook *notebook,
     }
 }
 
+/* Restore GTK2 behavior of mouse-scrolling to switch between
+ * notebook tabs. References Geany's notebook_tab_bar_click_cb ()
+ * at https://github.com/geany/geany/blob/master/src/notebook.c
+ */
+static gboolean
+gimp_dockbook_tab_scroll_cb (GtkWidget      *widget,
+                             GdkEventScroll *event)
+{
+  GtkNotebook *notebook = GTK_NOTEBOOK (widget);
+  GtkWidget   *page     = NULL;
+
+  page = gtk_notebook_get_nth_page (notebook,
+                                    gtk_notebook_get_current_page (notebook));
+  if (! page)
+    return FALSE;
+
+  switch (event->direction)
+    {
+    case GDK_SCROLL_RIGHT:
+    case GDK_SCROLL_DOWN:
+      gtk_notebook_next_page (notebook);
+      break;
+
+    case GDK_SCROLL_LEFT:
+    case GDK_SCROLL_UP:
+      gtk_notebook_prev_page (notebook);
+      break;
+
+    default:
+      break;
+    }
+
+  return TRUE;
+}
+
 static void
 gimp_dockbook_page_reordered (GtkNotebook *notebook,
                               GtkWidget   *child,
@@ -551,26 +593,14 @@ gimp_dockbook_show_menu (GimpDockbook *dockbook)
   GimpUIManager *dialog_ui_manager;
   const gchar   *dialog_ui_path;
   gpointer       dialog_popup_data;
-  GtkWidget     *parent_menu_widget;
-  GimpAction    *parent_menu_action;
   GimpDockable  *dockable;
   gint           page_num;
 
-  dockbook_ui_manager = gimp_dockbook_get_ui_manager (dockbook);
+  dockbook_ui_manager = dockbook->p->ui_manager;
 
   if (! dockbook_ui_manager)
     return FALSE;
 
-  parent_menu_widget =
-    gimp_ui_manager_get_widget (dockbook_ui_manager,
-                                "/dockable-popup/dockable-menu");
-  parent_menu_action =
-    gimp_ui_manager_get_action (dockbook_ui_manager,
-                                "/dockable-popup/dockable-menu");
-
-  if (! parent_menu_widget || ! parent_menu_action)
-    return FALSE;
-
   page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (dockbook));
   dockable = GIMP_DOCKABLE (gtk_notebook_get_nth_page (GTK_NOTEBOOK (dockbook),
                                                        page_num));
@@ -582,69 +612,6 @@ gimp_dockbook_show_menu (GimpDockbook *dockbook)
                                               &dialog_ui_path,
                                               &dialog_popup_data);
 
-  if (dialog_ui_manager && dialog_ui_path)
-    {
-      GtkWidget  *child_menu_widget;
-      GimpAction *child_menu_action;
-      gchar      *label;
-
-      child_menu_widget =
-        gimp_ui_manager_get_widget (dialog_ui_manager, dialog_ui_path);
-
-      if (! child_menu_widget)
-        {
-          g_warning ("%s: UI manager '%s' has no widget at path '%s'",
-                     G_STRFUNC, dialog_ui_manager->name, dialog_ui_path);
-          return FALSE;
-        }
-
-      child_menu_action =
-        gimp_ui_manager_get_action (dialog_ui_manager,
-                                    dialog_ui_path);
-
-      if (! child_menu_action)
-        {
-          g_warning ("%s: UI manager '%s' has no action at path '%s'",
-                     G_STRFUNC, dialog_ui_manager->name, dialog_ui_path);
-          return FALSE;
-        }
-
-      g_object_get (child_menu_action,
-                    "label", &label,
-                    NULL);
-
-      g_object_set (parent_menu_action,
-                    "label",     label,
-                    "icon-name", gimp_dockable_get_icon_name (dockable),
-                    "visible",   TRUE,
-                    NULL);
-
-      g_free (label);
-
-      if (! GTK_IS_MENU (child_menu_widget))
-        {
-          g_warning ("%s: child_menu_widget (%p) is not a GtkMenu",
-                     G_STRFUNC, child_menu_widget);
-          return FALSE;
-        }
-
-      {
-        GtkWidget *image = gimp_dockable_get_icon (dockable,
-                                                   GTK_ICON_SIZE_MENU);
-        gimp_menu_item_set_image (GTK_MENU_ITEM (parent_menu_widget), image);
-        gtk_widget_show (image);
-      }
-
-      gtk_menu_item_set_submenu (GTK_MENU_ITEM (parent_menu_widget),
-                                 child_menu_widget);
-
-      gimp_ui_manager_update (dialog_ui_manager, dialog_popup_data);
-    }
-  else
-    {
-      g_object_set (parent_menu_action, "visible", FALSE, NULL);
-    }
-
   /*  an action callback may destroy both dockable and dockbook, so
    *  reference them for gimp_dockbook_menu_end()
    */
@@ -653,10 +620,15 @@ gimp_dockbook_show_menu (GimpDockbook *dockbook)
                           g_object_ref (dockbook),
                           g_object_unref);
 
+  if (dialog_ui_manager)
+    gimp_ui_manager_update (dialog_ui_manager, dialog_popup_data);
+
   gimp_ui_manager_update (dockbook_ui_manager, dockable);
 
   gimp_ui_manager_ui_popup_at_widget (dockbook_ui_manager,
                                       "/dockable-popup",
+                                      dialog_ui_manager,
+                                      dialog_ui_path,
                                       dockbook->p->menu_button,
                                       GDK_GRAVITY_WEST,
                                       GDK_GRAVITY_NORTH_EAST,
@@ -670,23 +642,6 @@ gimp_dockbook_show_menu (GimpDockbook *dockbook)
 static void
 gimp_dockbook_menu_end (GimpDockable *dockable)
 {
-  GimpUIManager *dialog_ui_manager;
-  const gchar   *dialog_ui_path;
-  gpointer       dialog_popup_data;
-
-  dialog_ui_manager = gimp_dockable_get_menu (dockable,
-                                              &dialog_ui_path,
-                                              &dialog_popup_data);
-
-  if (dialog_ui_manager && dialog_ui_path)
-    {
-      GtkWidget *child_menu_widget =
-        gimp_ui_manager_get_widget (dialog_ui_manager, dialog_ui_path);
-
-      if (child_menu_widget)
-        gtk_menu_detach (GTK_MENU (child_menu_widget));
-    }
-
   /*  release gimp_dockbook_show_menu()'s references  */
   g_object_set_data (G_OBJECT (dockable), GIMP_DOCKABLE_DETACH_REF_KEY, NULL);
   g_object_unref (dockable);
@@ -704,11 +659,11 @@ gimp_dockbook_new (GimpMenuFactory *menu_factory)
 
   dockbook = g_object_new (GIMP_TYPE_DOCKBOOK, NULL);
 
-  dockbook->p->ui_manager = gimp_menu_factory_manager_new (menu_factory,
+  dockbook->p->ui_manager = gimp_menu_factory_get_manager (menu_factory,
                                                            "<Dockable>",
                                                            dockbook);
 
-  gimp_help_connect (GTK_WIDGET (dockbook), gimp_dockbook_help_func,
+  gimp_help_connect (GTK_WIDGET (dockbook), NULL, gimp_dockbook_help_func,
                      GIMP_HELP_DOCK, dockbook, NULL);
 
   return GTK_WIDGET (dockbook);
@@ -759,14 +714,6 @@ gimp_dockbook_set_dock (GimpDockbook *dockbook,
     }
 }
 
-GimpUIManager *
-gimp_dockbook_get_ui_manager (GimpDockbook *dockbook)
-{
-  g_return_val_if_fail (GIMP_IS_DOCKBOOK (dockbook), NULL);
-
-  return dockbook->p->ui_manager;
-}
-
 /**
  * gimp_dockbook_add_from_dialog_factory:
  * @dockbook:    The #DockBook
