@@ -127,7 +127,12 @@ static void            gimp_dock_window_get_property              (GObject
 static void            gimp_dock_window_style_updated             (GtkWidget                  *widget);
 static gboolean        gimp_dock_window_delete_event              (GtkWidget                  *widget,
                                                                    GdkEventAny                *event);
+#ifdef G_OS_WIN32
+static void            gimp_dock_window_realize                   (GimpDockWindow             *dock_window,
+                                                                   gpointer                    data);
+#endif
 static GList         * gimp_dock_window_get_docks                 (GimpDockContainer          *dock_container);
+static GList         * gimp_dock_window_get_docks_self            (GimpDockWindow             *self);
 static GimpDialogFactory * gimp_dock_window_get_dialog_factory    (GimpDockContainer          *dock_container);
 static GimpUIManager * gimp_dock_window_get_ui_manager            (GimpDockContainer          *dock_container);
 static void            gimp_dock_window_add_dock_from_session     (GimpDockContainer          *dock_container,
@@ -265,6 +270,12 @@ gimp_dock_window_init (GimpDockWindow *dock_window)
   gtk_window_set_resizable (GTK_WINDOW (dock_window), TRUE);
   gtk_window_set_focus_on_map (GTK_WINDOW (dock_window), FALSE);
   gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dock_window), FALSE);
+
+#ifdef G_OS_WIN32
+  g_signal_connect (dock_window, "realize",
+                    G_CALLBACK (gimp_dock_window_realize),
+                    NULL);
+#endif
 }
 
 static void
@@ -450,7 +461,7 @@ gimp_dock_window_constructed (GObject *object)
                 "context",   dock_window->p->context,
                 NULL);
 
-  gimp_help_connect (GTK_WIDGET (dock_window), gimp_standard_help_func,
+  gimp_help_connect (GTK_WIDGET (dock_window), NULL, gimp_standard_help_func,
                      GIMP_HELP_DOCK, NULL, NULL);
 
   if (dock_window->p->auto_follow_active)
@@ -486,6 +497,11 @@ gimp_dock_window_dispose (GObject *object)
 
   g_clear_object (&dock_window->p->dialog_factory);
 
+  /* Null our reference, that update_title callback dereferences.
+   * Gtk will dispose the widget but doesn't know of our reference.
+   */
+  dock_window->p->dock_columns = NULL;
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -691,12 +707,46 @@ gimp_dock_window_delete_event (GtkWidget   *widget,
   return FALSE;
 }
 
+#ifdef G_OS_WIN32
+static void
+gimp_dock_window_realize (GimpDockWindow *dock_window,
+                          gpointer        data)
+{
+  dock_window->p = gimp_dock_window_get_instance_private (dock_window);
+
+  if (dock_window->p->context && dock_window->p->context->gimp)
+    gimp_window_set_title_bar_theme (dock_window->p->context->gimp,
+                                     GTK_WIDGET (dock_window));
+}
+#endif
+
+/* Returns list of docks from self.
+ * Delegate to self's dock_columns, when it exists.
+ *
+ * Returns NULL when self has no dock_columns,
+ * which can happen during destroy.
+ *
+ * A returned non-null list is owned by self's dock_columns
+ * and need not be freed.
+ */
+static GList *
+gimp_dock_window_get_docks_self (GimpDockWindow *self)
+{
+  if (self->p->dock_columns)
+    return gimp_dock_columns_get_docks (self->p->dock_columns);
+  else
+    return NULL;
+}
+
+/* Implementation of GimpDockContainer interface method get_docks
+ * Transfers ownership of list to caller.
+ */
 static GList *
 gimp_dock_window_get_docks (GimpDockContainer *dock_container)
 {
   GimpDockWindow *dock_window = GIMP_DOCK_WINDOW (dock_container);
 
-  return g_list_copy (gimp_dock_columns_get_docks (dock_window->p->dock_columns));
+  return g_list_copy (gimp_dock_window_get_docks_self (dock_window));
 }
 
 static GimpDialogFactory *
@@ -819,7 +869,7 @@ gimp_dock_window_should_add_to_recent (GimpDockWindow *dock_window)
   GList    *docks;
   gboolean  should_add = TRUE;
 
-  docks = gimp_dock_container_get_docks (GIMP_DOCK_CONTAINER (dock_window));
+  docks = gimp_dock_window_get_docks_self (dock_window);
 
   if (! docks)
     {
@@ -841,8 +891,6 @@ gimp_dock_window_should_add_to_recent (GimpDockWindow *dock_window)
         }
     }
 
-  g_list_free (docks);
-
   return should_add;
 }
 
@@ -895,7 +943,7 @@ gimp_dock_window_get_description (GimpDockWindow *dock_window,
   GList   *docks         = NULL;
   GList   *iter          = NULL;
 
-  docks = gimp_dock_container_get_docks (GIMP_DOCK_CONTAINER (dock_window));
+  docks = gimp_dock_window_get_docks_self (dock_window);
 
   for (iter = docks;
        iter;
@@ -909,8 +957,6 @@ gimp_dock_window_get_description (GimpDockWindow *dock_window,
         g_string_append (complete_desc, GIMP_DOCK_COLUMN_SEPARATOR);
     }
 
-  g_list_free (docks);
-
   return g_string_free (complete_desc, FALSE /*free_segment*/);
 }
 
@@ -972,7 +1018,6 @@ gimp_dock_window_image_changed (GimpDockWindow *dock_window,
                                 GimpImage      *image,
                                 GimpContext    *context)
 {
-  GimpContainer *image_container   = dock_window->p->image_container;
   GimpContainer *display_container = dock_window->p->display_container;
 
   /*  make sure auto-follow-active works both ways  */
@@ -984,21 +1029,7 @@ gimp_dock_window_image_changed (GimpDockWindow *dock_window,
       gimp_context_set_image (factory_context, image);
     }
 
-  if (image == NULL && ! gimp_container_is_empty (image_container))
-    {
-      image = GIMP_IMAGE (gimp_container_get_first_child (image_container));
-
-      /*  this invokes this function recursively but we don't enter
-       *  the if() branch the second time
-       */
-      gimp_context_set_image (context, image);
-
-      /*  stop the emission of the original signal (the emission of
-       *  the recursive signal is finished)
-       */
-      g_signal_stop_emission_by_name (context, "image-changed");
-    }
-  else if (image != NULL && ! gimp_container_is_empty (display_container))
+  if (image != NULL && ! gimp_container_is_empty (display_container))
     {
       GimpDisplay *display;
       GimpImage   *display_image;
@@ -1221,7 +1252,7 @@ gimp_dock_window_has_toolbox (GimpDockWindow *dock_window)
 
   g_return_val_if_fail (GIMP_IS_DOCK_WINDOW (dock_window), FALSE);
 
-  for (iter = gimp_dock_columns_get_docks (dock_window->p->dock_columns);
+  for (iter = gimp_dock_window_get_docks_self (dock_window);
        iter;
        iter = g_list_next (iter))
     {
