@@ -698,7 +698,7 @@ gimp_editor_add_action_button (GimpEditor  *editor,
   gtk_button_set_relief (GTK_BUTTON (button), button_relief);
 
   icon_name = gimp_action_get_icon_name (action);
-  tooltip   = g_strdup (gimp_action_get_tooltip (action));
+  tooltip   = g_markup_escape_text (gimp_action_get_tooltip (action), -1);
   help_id   = g_object_get_qdata (G_OBJECT (action), GIMP_HELP_ID);
 
   old_child = gtk_bin_get_child (GTK_BIN (button));
@@ -738,7 +738,7 @@ gimp_editor_add_action_button (GimpEditor  *editor,
 
           if (tooltip)
             {
-              const gchar *ext_tooltip = gimp_action_get_tooltip (action);
+              gchar *ext_tooltip = g_markup_escape_text (gimp_action_get_tooltip (action), -1);
 
               if (ext_tooltip)
                 {
@@ -747,6 +747,8 @@ gimp_editor_add_action_button (GimpEditor  *editor,
                                             "</b>  ", ext_tooltip, NULL);
                   g_free (tooltip);
                   tooltip = tmp;
+
+                  g_free (ext_tooltip);
                 }
             }
         }
@@ -774,6 +776,31 @@ gimp_editor_add_action_button (GimpEditor  *editor,
   return button;
 }
 
+void
+gimp_editor_set_action_sensitive (GimpEditor  *editor,
+                                  const gchar *group_name,
+                                  const gchar *action_name,
+                                  gboolean     sensitive,
+                                  const gchar *reason)
+{
+  GimpActionGroup *group;
+  GimpAction      *action;
+
+  g_return_if_fail (GIMP_IS_EDITOR (editor));
+  g_return_if_fail (action_name != NULL);
+  g_return_if_fail (editor->priv->ui_manager != NULL);
+
+  group = gimp_ui_manager_get_action_group (editor->priv->ui_manager,
+                                            group_name);
+
+  g_return_if_fail (group != NULL);
+
+  action = gimp_action_group_get_action (group, action_name);
+  g_return_if_fail (action != NULL);
+
+  gimp_action_set_sensitive (action, sensitive, reason);
+}
+
 void
 gimp_editor_set_show_name (GimpEditor *editor,
                            gboolean    show)
@@ -789,8 +816,9 @@ gimp_editor_set_name (GimpEditor  *editor,
 {
   g_return_if_fail (GIMP_IS_EDITOR (editor));
 
-  gtk_label_set_text (GTK_LABEL (editor->priv->name_label),
-                      name ? name : _("(None)"));
+  if (editor->priv->name_label && GTK_IS_LABEL (editor->priv->name_label))
+    gtk_label_set_text (GTK_LABEL (editor->priv->name_label),
+                        name ? name : _("(None)"));
 }
 
 void
