@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -24,11 +24,11 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
 
-#include "core/gimpmarshal.h"
 #include "core/gimptoolinfo.h"
 
 #include "gimpoverlaydialog.h"
@@ -72,8 +72,24 @@ static void       gimp_overlay_dialog_get_property  (GObject           *object,
                                                      GValue            *value,
                                                      GParamSpec        *pspec);
 
-static void       gimp_overlay_dialog_size_request  (GtkWidget         *widget,
-                                                     GtkRequisition    *requisition);
+static void       gimp_overlay_dialog_get_preferred_width
+                                                    (GtkWidget         *widget,
+                                                     gint              *minimum_width,
+                                                     gint              *natural_width);
+static void       gimp_overlay_dialog_get_preferred_height
+                                                    (GtkWidget         *widget,
+                                                     gint              *minimum_height,
+                                                     gint              *natural_height);
+static void       gimp_overlay_dialog_get_preferred_width_for_height
+                                                    (GtkWidget         *widget,
+                                                     gint               height,
+                                                     gint              *minimum_width,
+                                                     gint              *natural_width);
+static void       gimp_overlay_dialog_get_preferred_height_for_width
+                                                    (GtkWidget         *widget,
+                                                     gint               width,
+                                                     gint              *minimum_height,
+                                                     gint              *natural_height);
 static void       gimp_overlay_dialog_size_allocate (GtkWidget         *widget,
                                                      GtkAllocation     *allocation);
 
@@ -88,8 +104,8 @@ static void       gimp_overlay_dialog_real_detach   (GimpOverlayDialog *dialog);
 static void       gimp_overlay_dialog_close         (GimpOverlayDialog *dialog);
 static void       gimp_overlay_dialog_real_close    (GimpOverlayDialog *dialog);
 
-static ResponseData * get_response_data             (GtkWidget         *widget,
-                                                     gboolean          create);
+static ResponseData * get_response_data                    (GtkWidget         *widget,
+                                                            gboolean          create);
 
 
 G_DEFINE_TYPE (GimpOverlayDialog, gimp_overlay_dialog,
@@ -113,10 +129,13 @@ gimp_overlay_dialog_class_init (GimpOverlayDialogClass *klass)
   object_class->get_property  = gimp_overlay_dialog_get_property;
   object_class->set_property  = gimp_overlay_dialog_set_property;
 
-  widget_class->size_request  = gimp_overlay_dialog_size_request;
-  widget_class->size_allocate = gimp_overlay_dialog_size_allocate;
+  widget_class->get_preferred_width  = gimp_overlay_dialog_get_preferred_width;
+  widget_class->get_preferred_height = gimp_overlay_dialog_get_preferred_height;
+  widget_class->get_preferred_width_for_height = gimp_overlay_dialog_get_preferred_width_for_height;
+  widget_class->get_preferred_height_for_width = gimp_overlay_dialog_get_preferred_height_for_width;
+  widget_class->size_allocate        = gimp_overlay_dialog_size_allocate;
 
-  container_class->forall     = gimp_overlay_dialog_forall;
+  container_class->forall            = gimp_overlay_dialog_forall;
 
   klass->detach               = gimp_overlay_dialog_real_detach;
   klass->close                = gimp_overlay_dialog_real_close;
@@ -140,8 +159,7 @@ gimp_overlay_dialog_class_init (GimpOverlayDialogClass *klass)
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GimpOverlayDialogClass, response),
-                  NULL, NULL,
-                  gimp_marshal_VOID__INT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   G_TYPE_INT);
 
@@ -150,8 +168,7 @@ gimp_overlay_dialog_class_init (GimpOverlayDialogClass *klass)
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                   G_STRUCT_OFFSET (GimpOverlayDialogClass, detach),
-                  NULL, NULL,
-                  gimp_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   signals[CLOSE] =
@@ -159,12 +176,13 @@ gimp_overlay_dialog_class_init (GimpOverlayDialogClass *klass)
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                   G_STRUCT_OFFSET (GimpOverlayDialogClass, close),
-                  NULL, NULL,
-                  gimp_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   gtk_binding_entry_add_signal (gtk_binding_set_by_class (klass),
                                 GDK_KEY_Escape, 0, "close", 0);
+
+  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (klass), "GimpOverlayDialog");
 }
 
 static void
@@ -265,17 +283,8 @@ gimp_overlay_dialog_finalize (GObject *object)
 {
   GimpOverlayDialog *dialog = GIMP_OVERLAY_DIALOG (object);
 
-  if (dialog->title)
-    {
-      g_free (dialog->title);
-      dialog->title = NULL;
-    }
-
-  if (dialog->icon_name)
-    {
-      g_free (dialog->icon_name);
-      dialog->icon_name = NULL;
-    }
+  g_clear_pointer (&dialog->title,     g_free);
+  g_clear_pointer (&dialog->icon_name, g_free);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -336,42 +345,111 @@ gimp_overlay_dialog_get_property (GObject    *object,
 }
 
 static void
-gimp_overlay_dialog_size_request (GtkWidget      *widget,
-                                  GtkRequisition *requisition)
+gimp_overlay_dialog_get_preferred_width (GtkWidget *widget,
+                                         gint      *minimum_width,
+                                         gint      *natural_width)
 {
-  GtkContainer      *container = GTK_CONTAINER (widget);
-  GimpOverlayDialog *dialog    = GIMP_OVERLAY_DIALOG (widget);
-  GtkWidget         *child     = gtk_bin_get_child (GTK_BIN (widget));
-  GtkRequisition     child_requisition;
-  GtkRequisition     header_requisition;
-  GtkRequisition     action_requisition;
+  GimpOverlayDialog *dialog = GIMP_OVERLAY_DIALOG (widget);
+  gint               header_minimum;
+  gint               header_natural;
+  gint               action_minimum;
+  gint               action_natural;
+
+  GTK_WIDGET_CLASS (parent_class)->get_preferred_width (widget,
+                                                        minimum_width,
+                                                        natural_width);
+
+  gtk_widget_get_preferred_width (dialog->header,
+                                  &header_minimum, &header_natural);
+  gtk_widget_get_preferred_width (dialog->action_area,
+                                  &action_minimum, &action_natural);
+
+  *minimum_width = MAX (MAX (*minimum_width, action_minimum), header_minimum);
+  *natural_width = MAX (MAX (*natural_width, action_natural), header_natural);
+}
+
+static void
+gimp_overlay_dialog_get_preferred_height (GtkWidget *widget,
+                                          gint      *minimum_height,
+                                          gint      *natural_height)
+{
+  GimpOverlayDialog *dialog = GIMP_OVERLAY_DIALOG (widget);
+  gint               header_minimum;
+  gint               header_natural;
+  gint               action_minimum;
+  gint               action_natural;
   gint               border_width;
 
-  border_width = gtk_container_get_border_width (container);
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
-  requisition->width  = border_width * 2;
-  requisition->height = border_width * 2;
+  GTK_WIDGET_CLASS (parent_class)->get_preferred_height (widget,
+                                                         minimum_height,
+                                                         natural_height);
 
-  if (child && gtk_widget_get_visible (child))
-    {
-      gtk_widget_size_request (child, &child_requisition);
-    }
-  else
-    {
-      child_requisition.width  = 0;
-      child_requisition.height = 0;
-    }
+  gtk_widget_get_preferred_height (dialog->header,
+                                   &header_minimum, &header_natural);
+  gtk_widget_get_preferred_height (dialog->action_area,
+                                   &action_minimum, &action_natural);
 
-  gtk_widget_size_request (dialog->header,      &header_requisition);
-  gtk_widget_size_request (dialog->action_area, &action_requisition);
+  *minimum_height += header_minimum + action_minimum + 2 * border_width;
+  *natural_height += header_natural + action_natural + 2 * border_width;
+}
 
-  requisition->width  += MAX (MAX (child_requisition.width,
-                                   action_requisition.width),
-                              header_requisition.width);
-  requisition->height += (child_requisition.height +
-                          2 * border_width +
-                          header_requisition.height +
-                          action_requisition.height);
+static void
+gimp_overlay_dialog_get_preferred_width_for_height (GtkWidget *widget,
+                                                    gint       height,
+                                                    gint      *minimum_width,
+                                                    gint      *natural_width)
+{
+  GimpOverlayDialog *dialog = GIMP_OVERLAY_DIALOG (widget);
+  gint               header_minimum;
+  gint               header_natural;
+  gint               action_minimum;
+  gint               action_natural;
+
+  GTK_WIDGET_CLASS (parent_class)->get_preferred_width_for_height (widget,
+                                                                   height,
+                                                                   minimum_width,
+                                                                   natural_width);
+
+  gtk_widget_get_preferred_width (dialog->header,
+                                  &header_minimum, &header_natural);
+  gtk_widget_get_preferred_width (dialog->action_area,
+                                  &action_minimum, &action_natural);
+
+  *minimum_width = MAX (MAX (*minimum_width, action_minimum), header_minimum);
+  *natural_width = MAX (MAX (*natural_width, action_natural), header_natural);
+}
+
+static void
+gimp_overlay_dialog_get_preferred_height_for_width (GtkWidget *widget,
+                                                    gint       width,
+                                                    gint      *minimum_height,
+                                                    gint      *natural_height)
+{
+  GimpOverlayDialog *dialog = GIMP_OVERLAY_DIALOG (widget);
+  gint               header_minimum;
+  gint               header_natural;
+  gint               action_minimum;
+  gint               action_natural;
+  gint               border_width;
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  GTK_WIDGET_CLASS (parent_class)->get_preferred_height_for_width (widget,
+                                                                   width,
+                                                                   minimum_height,
+                                                                   natural_height);
+
+  gtk_widget_get_preferred_height_for_width (dialog->header,
+                                             width - 2 * border_width,
+                                             &header_minimum, &header_natural);
+  gtk_widget_get_preferred_height_for_width (dialog->action_area,
+                                             width - 2 * border_width,
+                                             &action_minimum, &action_natural);
+
+  *minimum_height += header_minimum + action_minimum + 2 * border_width;
+  *natural_height += header_natural + action_natural + 2 * border_width;
 }
 
 static void
@@ -392,8 +470,8 @@ gimp_overlay_dialog_size_allocate (GtkWidget     *widget,
 
   border_width = gtk_container_get_border_width (container);
 
-  gtk_widget_size_request (dialog->header,      &header_requisition);
-  gtk_widget_size_request (dialog->action_area, &action_requisition);
+  gtk_widget_get_preferred_size (dialog->header,      &header_requisition, NULL);
+  gtk_widget_get_preferred_size (dialog->action_area, &action_requisition, NULL);
 
   if (child && gtk_widget_get_visible (child))
     {
@@ -484,7 +562,7 @@ gimp_overlay_dialog_new (GimpToolInfo *tool_info,
   icon_name = gimp_viewable_get_icon_name (GIMP_VIEWABLE (tool_info));
 
   dialog = g_object_new (GIMP_TYPE_OVERLAY_DIALOG,
-                         "title",     tool_info->blurb,
+                         "title",     tool_info->label,
                          "icon-name", icon_name,
                          NULL);
 
@@ -549,10 +627,8 @@ gimp_overlay_dialog_add_button (GimpOverlayDialog *dialog,
       response_id == GIMP_RESPONSE_DETACH)
     return NULL;
 
-  button = gtk_button_new_from_stock (button_text);
-
+  button = gtk_button_new_with_mnemonic (button_text);
   gtk_widget_set_can_default (button, TRUE);
-
   gtk_widget_show (button);
 
   ad = get_response_data (button, TRUE);
