@@ -93,7 +93,7 @@ static void   palette_editor_drop_palette          (GtkWidget         *widget,
 static void   palette_editor_drop_color            (GtkWidget         *widget,
                                                     gint               x,
                                                     gint               y,
-                                                    const GimpRGB     *color,
+                                                    GeglColor         *color,
                                                     gpointer           data);
 
 static void   palette_editor_entry_clicked         (GimpPaletteView   *view,
@@ -113,7 +113,7 @@ static gboolean palette_editor_popup_menu          (GtkWidget *widget,
                                                     gpointer   user_data);
 static void   palette_editor_color_dropped         (GimpPaletteView   *view,
                                                     GimpPaletteEntry  *entry,
-                                                    const GimpRGB     *color,
+                                                    GeglColor         *color,
                                                     GimpPaletteEditor *editor);
 
 static void   palette_editor_color_name_changed    (GtkWidget         *widget,
@@ -126,7 +126,7 @@ static void   palette_editor_resize                (GimpPaletteEditor *editor,
                                                     gdouble            zoom_factor);
 static void   palette_editor_scroll_top_left       (GimpPaletteEditor *editor);
 static void   palette_editor_edit_color_update     (GimpColorDialog   *dialog,
-                                                    const GimpRGB     *color,
+                                                    GeglColor         *color,
                                                     GimpColorDialogState state,
                                                     GimpPaletteEditor *editor);
 
@@ -493,7 +493,7 @@ gimp_palette_editor_edit_color (GimpPaletteEditor *editor)
                                GTK_WIDGET (editor),
                                gimp_dialog_factory_get_singleton (),
                                "gimp-palette-editor-color-dialog",
-                               &editor->color->color,
+                               editor->color->color,
                                FALSE, FALSE);
 
       g_signal_connect (editor->color_dialog, "destroy",
@@ -510,7 +510,7 @@ gimp_palette_editor_edit_color (GimpPaletteEditor *editor)
                                           g_list_prepend (NULL, palette),
                                           data_editor->context);
       gimp_color_dialog_set_color (GIMP_COLOR_DIALOG (editor->color_dialog),
-                                   &editor->color->color);
+                                   editor->color->color);
 
       if (! gtk_widget_get_visible (editor->color_dialog))
         gimp_dialog_factory_position_dialog (gimp_dialog_factory_get_singleton (),
@@ -524,11 +524,11 @@ gimp_palette_editor_edit_color (GimpPaletteEditor *editor)
 
 void
 gimp_palette_editor_pick_color (GimpPaletteEditor  *editor,
-                                const GimpRGB      *color,
+                                GeglColor          *color,
                                 GimpColorPickState  pick_state)
 {
   g_return_if_fail (GIMP_IS_PALETTE_EDITOR (editor));
-  g_return_if_fail (color != NULL);
+  g_return_if_fail (GEGL_IS_COLOR (color));
 
   if (GIMP_DATA_EDITOR (editor)->data_editable)
     {
@@ -556,7 +556,7 @@ gimp_palette_editor_pick_color (GimpPaletteEditor  *editor,
         case GIMP_COLOR_PICK_STATE_UPDATE:
         case GIMP_COLOR_PICK_STATE_END:
           gimp_palette_set_entry_color (GIMP_PALETTE (data),
-                                        index, color);
+                                        index, color, FALSE);
           break;
         }
     }
@@ -635,12 +635,12 @@ gimp_palette_editor_zoom (GimpPaletteEditor  *editor,
 
 gint
 gimp_palette_editor_get_index (GimpPaletteEditor *editor,
-                               const GimpRGB     *search)
+                               GeglColor         *search)
 {
   GimpPalette *palette;
 
   g_return_val_if_fail (GIMP_IS_PALETTE_EDITOR (editor), -1);
-  g_return_val_if_fail (search != NULL, -1);
+  g_return_val_if_fail (GEGL_IS_COLOR (search), -1);
 
   palette = GIMP_PALETTE (GIMP_DATA_EDITOR (editor)->data);
 
@@ -660,7 +660,7 @@ gimp_palette_editor_get_index (GimpPaletteEditor *editor,
 gboolean
 gimp_palette_editor_set_index (GimpPaletteEditor *editor,
                                gint               index,
-                               GimpRGB           *color)
+                               GeglColor         *color)
 {
   GimpPalette *palette;
 
@@ -680,7 +680,14 @@ gimp_palette_editor_set_index (GimpPaletteEditor *editor,
                                       entry);
 
       if (color)
-        *color = editor->color->color;
+        {
+          const Babl *format;
+          guchar      pixel[40];
+
+          format = gegl_color_get_format (editor->color->color);
+          gegl_color_get_pixel (editor->color->color, format, pixel);
+          gegl_color_set_pixel (color, format, pixel);
+        }
 
       return TRUE;
     }
@@ -742,11 +749,11 @@ palette_editor_drop_palette (GtkWidget    *widget,
 }
 
 static void
-palette_editor_drop_color (GtkWidget     *widget,
-                           gint           x,
-                           gint           y,
-                           const GimpRGB *color,
-                           gpointer       data)
+palette_editor_drop_color (GtkWidget *widget,
+                           gint       x,
+                           gint       y,
+                           GeglColor *color,
+                           gpointer   data)
 {
   GimpPaletteEditor *editor = data;
 
@@ -774,9 +781,9 @@ palette_editor_entry_clicked (GimpPaletteView   *view,
       GimpDataEditor *data_editor = GIMP_DATA_EDITOR (editor);
 
       if (state & gimp_get_toggle_behavior_mask ())
-        gimp_context_set_background (data_editor->context, &entry->color);
+        gimp_context_set_background (data_editor->context, entry->color);
       else
-        gimp_context_set_foreground (data_editor->context, &entry->color);
+        gimp_context_set_foreground (data_editor->context, entry->color);
     }
 }
 
@@ -878,7 +885,7 @@ palette_editor_popup_menu (GtkWidget *widget,
 static void
 palette_editor_color_dropped (GimpPaletteView   *view,
                               GimpPaletteEntry  *entry,
-                              const GimpRGB     *color,
+                              GeglColor         *color,
                               GimpPaletteEditor *editor)
 {
   if (GIMP_DATA_EDITOR (editor)->data_editable)
@@ -991,7 +998,7 @@ palette_editor_scroll_top_left (GimpPaletteEditor *palette_editor)
 
 static void
 palette_editor_edit_color_update (GimpColorDialog      *dialog,
-                                  const GimpRGB        *color,
+                                  GeglColor            *color,
                                   GimpColorDialogState  state,
                                   GimpPaletteEditor    *editor)
 {
@@ -1005,7 +1012,8 @@ palette_editor_edit_color_update (GimpColorDialog      *dialog,
     case GIMP_COLOR_DIALOG_OK:
       if (editor->color)
         {
-          editor->color->color = *color;
+          g_clear_object (&editor->color->color);
+          editor->color->color = gegl_color_duplicate (color);
           gimp_data_dirty (GIMP_DATA (palette));
         }
       /* Fallthrough */
