@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -30,10 +30,11 @@
 #include "widgets-types.h"
 
 #include "core/gimp.h"
+#include "core/gimpcontainer.h"
 #include "core/gimpcontext.h"
 
-#include "text/gimpfontlist.h"
 #include "text/gimptext.h"
+#include "text/gimpfont.h"
 
 #include "gimpcolorpanel.h"
 #include "gimpcontainerentry.h"
@@ -126,7 +127,8 @@ G_DEFINE_TYPE (GimpTextStyleEditor, gimp_text_style_editor,
 static void
 gimp_text_style_editor_class_init (GimpTextStyleEditorClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->constructed  = gimp_text_style_editor_constructed;
   object_class->dispose      = gimp_text_style_editor_dispose;
@@ -158,7 +160,7 @@ gimp_text_style_editor_class_init (GimpTextStyleEditorClass *klass)
   g_object_class_install_property (object_class, PROP_FONTS,
                                    g_param_spec_object ("fonts",
                                                         NULL, NULL,
-                                                        GIMP_TYPE_FONT_LIST,
+                                                        GIMP_TYPE_CONTAINER,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
 
@@ -179,13 +181,15 @@ gimp_text_style_editor_class_init (GimpTextStyleEditorClass *klass)
                                                         1.0,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT));
+
+  gtk_widget_class_set_css_name (widget_class, "GimpTextStyleEditor");
 }
 
 static void
 gimp_text_style_editor_init (GimpTextStyleEditor *editor)
 {
   GtkWidget *image;
-  GimpRGB    color;
+  GeglColor *color;
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (editor),
                                   GTK_ORIENTATION_VERTICAL);
@@ -207,9 +211,8 @@ gimp_text_style_editor_init (GimpTextStyleEditor *editor)
                            _("Change font of selected text"), NULL);
 
   editor->size_entry =
-    gimp_size_entry_new (1, 0, "%a", TRUE, FALSE, FALSE, 10,
+    gimp_size_entry_new (1, gimp_unit_pixel (), "%a", TRUE, FALSE, FALSE, 10,
                          GIMP_SIZE_ENTRY_UPDATE_SIZE);
-  gtk_table_set_col_spacing (GTK_TABLE (editor->size_entry), 1, 0);
   gtk_box_pack_start (GTK_BOX (editor->upper_hbox), editor->size_entry,
                       FALSE, FALSE, 0);
   gtk_widget_show (editor->size_entry);
@@ -244,11 +247,12 @@ gimp_text_style_editor_init (GimpTextStyleEditor *editor)
   gtk_container_add (GTK_CONTAINER (editor->clear_button), image);
   gtk_widget_show (image);
 
-  gimp_rgba_set (&color, 0.0, 0.0, 0.0, 1.0);
+  color = gegl_color_new ("black");
   editor->color_button = gimp_color_panel_new (_("Change color of selected text"),
-                                               &color,
+                                               color,
                                                GIMP_COLOR_AREA_FLAT, 20, 20);
   gimp_widget_set_fully_opaque (editor->color_button, TRUE);
+  g_object_unref (color);
 
   gtk_box_pack_end (GTK_BOX (editor->lower_hbox), editor->color_button,
                     FALSE, FALSE, 0);
@@ -261,10 +265,10 @@ gimp_text_style_editor_init (GimpTextStyleEditor *editor)
                     G_CALLBACK (gimp_text_style_editor_color_changed),
                     editor);
 
-  editor->kerning_adjustment =
-    GTK_ADJUSTMENT (gtk_adjustment_new (0.0, -1000.0, 1000.0, 1.0, 10.0, 0.0));
-  editor->kerning_spinbutton = gtk_spin_button_new (editor->kerning_adjustment,
-                                                    1.0, 1);
+  editor->kerning_adjustment = gtk_adjustment_new (0.0, -1000.0, 1000.0,
+                                                   1.0, 10.0, 0.0);
+  editor->kerning_spinbutton =
+    gimp_spin_button_new (editor->kerning_adjustment, 1.0, 1);
   gtk_entry_set_width_chars (GTK_ENTRY (editor->kerning_spinbutton), 5);
   gtk_box_pack_end (GTK_BOX (editor->lower_hbox), editor->kerning_spinbutton,
                     FALSE, FALSE, 0);
@@ -277,10 +281,10 @@ gimp_text_style_editor_init (GimpTextStyleEditor *editor)
                     G_CALLBACK (gimp_text_style_editor_kerning_changed),
                     editor);
 
-  editor->baseline_adjustment =
-    GTK_ADJUSTMENT (gtk_adjustment_new (0.0, -1000.0, 1000.0, 1.0, 10.0, 0.0));
-  editor->baseline_spinbutton = gtk_spin_button_new (editor->baseline_adjustment,
-                                                     1.0, 1);
+  editor->baseline_adjustment = gtk_adjustment_new (0.0, -1000.0, 1000.0,
+                                                    1.0, 10.0, 0.0);
+  editor->baseline_spinbutton =
+    gimp_spin_button_new (editor->baseline_adjustment, 1.0, 1);
   gtk_entry_set_width_chars (GTK_ENTRY (editor->baseline_spinbutton), 5);
   gtk_box_pack_end (GTK_BOX (editor->lower_hbox), editor->baseline_spinbutton,
                     FALSE, FALSE, 0);
@@ -301,10 +305,10 @@ gimp_text_style_editor_constructed (GObject *object)
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  g_assert (GIMP_IS_GIMP (editor->gimp));
-  g_assert (GIMP_IS_FONT_LIST (editor->fonts));
-  g_assert (GIMP_IS_TEXT (editor->text));
-  g_assert (GIMP_IS_TEXT_BUFFER (editor->buffer));
+  gimp_assert (GIMP_IS_GIMP (editor->gimp));
+  gimp_assert (GIMP_IS_CONTAINER (editor->fonts));
+  gimp_assert (GIMP_IS_TEXT (editor->text));
+  gimp_assert (GIMP_IS_TEXT_BUFFER (editor->buffer));
 
   editor->context = gimp_context_new (editor->gimp, "text style editor", NULL);
 
@@ -389,6 +393,13 @@ gimp_text_style_editor_dispose (GObject *object)
                                             editor);
     }
 
+  if (editor->context)
+    {
+      g_signal_handlers_disconnect_by_func (editor->context,
+                                            gimp_text_style_editor_font_changed,
+                                            editor);
+    }
+
   if (editor->update_idle_id)
     {
       g_source_remove (editor->update_idle_id);
@@ -403,29 +414,10 @@ gimp_text_style_editor_finalize (GObject *object)
 {
   GimpTextStyleEditor *editor = GIMP_TEXT_STYLE_EDITOR (object);
 
-  if (editor->context)
-    {
-      g_object_unref (editor->context);
-      editor->context = NULL;
-    }
-
-  if (editor->text)
-    {
-      g_object_unref (editor->text);
-      editor->text = NULL;
-    }
-
-  if (editor->buffer)
-    {
-      g_object_unref (editor->buffer);
-      editor->buffer = NULL;
-    }
-
-  if (editor->fonts)
-    {
-      g_object_unref (editor->fonts);
-      editor->fonts = NULL;
-    }
+  g_clear_object (&editor->context);
+  g_clear_object (&editor->text);
+  g_clear_object (&editor->buffer);
+  g_clear_object (&editor->fonts);
 
   if (editor->toggles)
     {
@@ -577,7 +569,7 @@ gimp_text_style_editor_list_tags (GimpTextStyleEditor  *editor,
         gdouble     points;
 
         points = gimp_units_to_points (pixels,
-                                       GIMP_UNIT_PIXEL,
+                                       gimp_unit_pixel (),
                                        editor->resolution_y);
         tag = gimp_text_buffer_get_size_tag (editor->buffer,
                                              PANGO_SCALE * points);
@@ -604,22 +596,23 @@ gimp_text_style_editor_list_tags (GimpTextStyleEditor  *editor,
   }
 
   {
-    GList   *list;
-    GimpRGB  color;
+    GList     *list;
+    GeglColor *color;
 
     for (list = editor->buffer->color_tags; list; list = g_list_next (list))
       *remove_tags = g_list_prepend (*remove_tags, list->data);
 
-    gimp_color_button_get_color (GIMP_COLOR_BUTTON (editor->color_button),
-                                 &color);
+    color = gimp_color_button_get_color (GIMP_COLOR_BUTTON (editor->color_button));
 
     if (TRUE) /* FIXME should have "inconsistent" state as for font and size */
       {
         GtkTextTag *tag;
 
-        tag = gimp_text_buffer_get_color_tag (editor->buffer, &color);
+        tag = gimp_text_buffer_get_color_tag (editor->buffer, color);
         tags = g_list_prepend (tags, tag);
       }
+
+    g_object_unref (color);
   }
 
   *remove_tags = g_list_reverse (*remove_tags);
@@ -732,7 +725,7 @@ gimp_text_style_editor_set_default_font (GimpTextStyleEditor *editor)
                                    gimp_text_style_editor_font_changed,
                                    editor);
 
-  gimp_context_set_font_name (editor->context, editor->text->font);
+  gimp_context_set_font (editor->context, editor->text->font);
 
   g_signal_handlers_unblock_by_func (editor->context,
                                      gimp_text_style_editor_font_changed,
@@ -749,13 +742,14 @@ gimp_text_style_editor_color_changed (GimpColorButton     *button,
 
   if (gtk_text_buffer_get_has_selection (buffer))
     {
-      GtkTextIter start, end;
-      GimpRGB     color;
+      GeglColor   *color;
+      GtkTextIter  start, end;
 
       gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
+      color = gimp_color_button_get_color (button);
+      gimp_text_buffer_set_color (editor->buffer, &start, &end, color);
 
-      gimp_color_button_get_color (button, &color);
-      gimp_text_buffer_set_color (editor->buffer, &start, &end, &color);
+      g_object_unref (color);
     }
 
   insert_tags = gimp_text_style_editor_list_tags (editor, &remove_tags);
@@ -766,19 +760,21 @@ static void
 gimp_text_style_editor_set_color (GimpTextStyleEditor *editor,
                                   GtkTextTag          *color_tag)
 {
-  GimpRGB color;
-
-  gimp_rgba_set (&color, 0.0, 0.0, 0.0, 1.0);
+  GeglColor *color = NULL;
 
   if (color_tag)
-    gimp_text_tag_get_color (color_tag, &color);
+    gimp_text_tag_get_fg_color (color_tag, &color);
+
+  if (! color)
+    color = gegl_color_new ("black");
 
   g_signal_handlers_block_by_func (editor->color_button,
                                    gimp_text_style_editor_color_changed,
                                    editor);
 
   gimp_color_button_set_color (GIMP_COLOR_BUTTON (editor->color_button),
-                               &color);
+                               color);
+  g_clear_object (&color);
 
   /* FIXME should have "inconsistent" state as for font and size */
 
@@ -794,8 +790,7 @@ gimp_text_style_editor_set_default_color (GimpTextStyleEditor *editor)
                                    gimp_text_style_editor_color_changed,
                                    editor);
 
-  gimp_color_button_set_color (GIMP_COLOR_BUTTON (editor->color_button),
-                               &editor->text->color);
+  gimp_color_button_set_color (GIMP_COLOR_BUTTON (editor->color_button), editor->text->color);
 
   g_signal_handlers_unblock_by_func (editor->color_button,
                                      gimp_text_style_editor_color_changed,
@@ -858,21 +853,24 @@ gimp_text_style_editor_size_changed (GimpSizeEntry       *entry,
   GtkTextBuffer *buffer = GTK_TEXT_BUFFER (editor->buffer);
   GList         *insert_tags;
   GList         *remove_tags;
+  GtkTextIter    start, end;
+  gdouble        points;
+
+  points = gimp_units_to_points (gimp_size_entry_get_refval (entry, 0),
+                                 gimp_unit_pixel (), editor->resolution_y);
 
   if (gtk_text_buffer_get_has_selection (buffer))
     {
-      GtkTextIter start, end;
-      gdouble     points;
-
       gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
-
-      points = gimp_units_to_points (gimp_size_entry_get_refval (entry, 0),
-                                     GIMP_UNIT_PIXEL,
-                                     editor->resolution_y);
-
-      gimp_text_buffer_set_size (editor->buffer, &start, &end,
-                                 PANGO_SCALE * points);
     }
+  else
+    {
+      gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
+      gtk_text_buffer_get_iter_at_offset (buffer, &end, -1);
+    }
+
+  gimp_text_buffer_set_size (editor->buffer, &start, &end,
+                             PANGO_SCALE * points);
 
   insert_tags = gimp_text_style_editor_list_tags (editor, &remove_tags);
   gimp_text_buffer_set_insert_tags (editor->buffer, insert_tags, remove_tags);
@@ -893,7 +891,7 @@ gimp_text_style_editor_set_size (GimpTextStyleEditor *editor,
                                    editor);
 
   pixels = gimp_units_to_pixels ((gdouble) size / PANGO_SCALE,
-                                 GIMP_UNIT_POINT,
+                                 gimp_unit_point (),
                                  editor->resolution_y);
   gimp_size_entry_set_refval (GIMP_SIZE_ENTRY (editor->size_entry), 0, pixels);
 
@@ -961,10 +959,17 @@ gimp_text_style_editor_set_baseline (GimpTextStyleEditor *editor,
                                    gimp_text_style_editor_baseline_changed,
                                    editor);
 
-  gtk_adjustment_set_value (editor->baseline_adjustment,
-                            (gdouble) baseline / PANGO_SCALE);
-  /* make sure the "" really gets replaced */
-  gtk_adjustment_value_changed (editor->baseline_adjustment);
+  if (gtk_adjustment_get_value (editor->baseline_adjustment) !=
+      (gdouble) baseline / PANGO_SCALE)
+    {
+      gtk_adjustment_set_value (editor->baseline_adjustment,
+                                (gdouble) baseline / PANGO_SCALE);
+    }
+  else
+    {
+      /* make sure the "" really gets replaced */
+      g_signal_emit_by_name (editor->baseline_adjustment, "value-changed");
+    }
 
   g_signal_handlers_unblock_by_func (editor->baseline_adjustment,
                                      gimp_text_style_editor_baseline_changed,
@@ -1004,10 +1009,17 @@ gimp_text_style_editor_set_kerning (GimpTextStyleEditor *editor,
                                    gimp_text_style_editor_kerning_changed,
                                    editor);
 
-  gtk_adjustment_set_value (editor->kerning_adjustment,
-                            (gdouble) kerning / PANGO_SCALE);
-  /* make sure the "" really gets replaced */
-  gtk_adjustment_value_changed (editor->kerning_adjustment);
+  if (gtk_adjustment_get_value (editor->baseline_adjustment) !=
+      (gdouble) kerning / PANGO_SCALE)
+    {
+      gtk_adjustment_set_value (editor->kerning_adjustment,
+                                (gdouble) kerning / PANGO_SCALE);
+    }
+  else
+    {
+      /* make sure the "" really gets replaced */
+      g_signal_emit_by_name (editor->kerning_adjustment, "value-changed");
+    }
 
   g_signal_handlers_unblock_by_func (editor->kerning_adjustment,
                                      gimp_text_style_editor_kerning_changed,
@@ -1301,7 +1313,7 @@ gimp_text_style_editor_update_idle (GimpTextStyleEditor *editor)
 
   if (editor->context->font_name &&
       g_strcmp0 (editor->context->font_name,
-                 gimp_object_get_name (gimp_context_get_font (editor->context))))
+                 gimp_font_get_lookup_name (gimp_context_get_font (editor->context))))
     {
       /* A font is set, but is unavailable; change the help text. */
       gchar *help_text;
