@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -59,14 +59,17 @@ static void        gimp_view_realize              (GtkWidget        *widget);
 static void        gimp_view_unrealize            (GtkWidget        *widget);
 static void        gimp_view_map                  (GtkWidget        *widget);
 static void        gimp_view_unmap                (GtkWidget        *widget);
-static void        gimp_view_size_request         (GtkWidget        *widget,
-                                                   GtkRequisition   *requisition);
+static void        gimp_view_get_preferred_width  (GtkWidget        *widget,
+                                                   gint             *minimum_width,
+                                                   gint             *natural_width);
+static void        gimp_view_get_preferred_height (GtkWidget        *widget,
+                                                   gint             *minimum_height,
+                                                   gint             *natural_height);
 static void        gimp_view_size_allocate        (GtkWidget        *widget,
                                                    GtkAllocation    *allocation);
-static void        gimp_view_style_set            (GtkWidget        *widget,
-                                                   GtkStyle         *prev_style);
-static gboolean    gimp_view_expose_event         (GtkWidget        *widget,
-                                                   GdkEventExpose   *event);
+static void        gimp_view_style_updated        (GtkWidget        *widget);
+static gboolean    gimp_view_draw                 (GtkWidget        *widget,
+                                                   cairo_t          *cr);
 static gboolean    gimp_view_button_press_event   (GtkWidget        *widget,
                                                    GdkEventButton   *bevent);
 static gboolean    gimp_view_button_release_event (GtkWidget        *widget,
@@ -120,8 +123,7 @@ gimp_view_class_init (GimpViewClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpViewClass, clicked),
-                  NULL, NULL,
-                  gimp_marshal_VOID__FLAGS,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   GDK_TYPE_MODIFIER_TYPE);
 
@@ -130,8 +132,7 @@ gimp_view_class_init (GimpViewClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpViewClass, double_clicked),
-                  NULL, NULL,
-                  gimp_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   view_signals[CONTEXT] =
@@ -139,8 +140,7 @@ gimp_view_class_init (GimpViewClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpViewClass, context),
-                  NULL, NULL,
-                  gimp_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   object_class->dispose              = gimp_view_dispose;
@@ -150,10 +150,11 @@ gimp_view_class_init (GimpViewClass *klass)
   widget_class->unrealize            = gimp_view_unrealize;
   widget_class->map                  = gimp_view_map;
   widget_class->unmap                = gimp_view_unmap;
-  widget_class->size_request         = gimp_view_size_request;
+  widget_class->get_preferred_width  = gimp_view_get_preferred_width;
+  widget_class->get_preferred_height = gimp_view_get_preferred_height;
   widget_class->size_allocate        = gimp_view_size_allocate;
-  widget_class->style_set            = gimp_view_style_set;
-  widget_class->expose_event         = gimp_view_expose_event;
+  widget_class->style_updated        = gimp_view_style_updated;
+  widget_class->draw                 = gimp_view_draw;
   widget_class->button_press_event   = gimp_view_button_press_event;
   widget_class->button_release_event = gimp_view_button_release_event;
   widget_class->enter_notify_event   = gimp_view_enter_notify_event;
@@ -186,7 +187,7 @@ gimp_view_init (GimpView *view)
 
   gimp_widget_track_monitor (GTK_WIDGET (view),
                              G_CALLBACK (gimp_view_monitor_changed),
-                             NULL);
+                             NULL, NULL);
 }
 
 static void
@@ -197,11 +198,7 @@ gimp_view_dispose (GObject *object)
   if (view->viewable)
     gimp_view_set_viewable (view, NULL);
 
-  if (view->renderer)
-    {
-      g_object_unref (view->renderer);
-      view->renderer = NULL;
-    }
+  g_clear_object (&view->renderer);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -277,22 +274,38 @@ gimp_view_unmap (GtkWidget *widget)
 }
 
 static void
-gimp_view_size_request (GtkWidget      *widget,
-                        GtkRequisition *requisition)
+gimp_view_get_preferred_width (GtkWidget *widget,
+                               gint      *minimum_width,
+                               gint      *natural_width)
 {
   GimpView *view = GIMP_VIEW (widget);
 
   if (view->expand)
     {
-      requisition->width  = 2 * view->renderer->border_width + 1;
-      requisition->height = 2 * view->renderer->border_width + 1;
+      *minimum_width = *natural_width = 2 * view->renderer->border_width + 1;
     }
   else
     {
-      requisition->width  = (view->renderer->width +
-                             2 * view->renderer->border_width);
-      requisition->height = (view->renderer->height +
-                             2 * view->renderer->border_width);
+      *minimum_width = *natural_width = (view->renderer->width +
+                                         2 * view->renderer->border_width);
+    }
+}
+
+static void
+gimp_view_get_preferred_height (GtkWidget *widget,
+                                gint      *minimum_height,
+                                gint      *natural_height)
+{
+  GimpView *view = GIMP_VIEW (widget);
+
+  if (view->expand)
+    {
+      *minimum_height = *natural_height = 2 * view->renderer->border_width + 1;
+    }
+  else
+    {
+      *minimum_height = *natural_height = (view->renderer->height +
+                                           2 * view->renderer->border_width);
     }
 }
 
@@ -387,40 +400,27 @@ gimp_view_size_allocate (GtkWidget     *widget,
 }
 
 static void
-gimp_view_style_set (GtkWidget *widget,
-                     GtkStyle  *prev_style)
+gimp_view_style_updated (GtkWidget *widget)
 {
   GimpView *view = GIMP_VIEW (widget);
 
-  GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+  GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
 
   gimp_view_renderer_invalidate (view->renderer);
 }
 
 static gboolean
-gimp_view_expose_event (GtkWidget      *widget,
-                        GdkEventExpose *event)
+gimp_view_draw (GtkWidget *widget,
+                cairo_t   *cr)
 {
-  if (gtk_widget_is_drawable (widget))
-    {
-      GtkAllocation  allocation;
-      cairo_t       *cr;
+  GtkAllocation allocation;
 
-      gtk_widget_get_allocation (widget, &allocation);
+  gtk_widget_get_allocation (widget, &allocation);
 
-      cr = gdk_cairo_create (event->window);
-      gdk_cairo_region (cr, event->region);
-      cairo_clip (cr);
-
-      cairo_translate (cr, allocation.x, allocation.y);
-
-      gimp_view_renderer_draw (GIMP_VIEW (widget)->renderer,
-                               widget, cr,
-                               allocation.width,
-                               allocation.height);
-
-      cairo_destroy (cr);
-    }
+  gimp_view_renderer_draw (GIMP_VIEW (widget)->renderer,
+                           widget, cr,
+                           allocation.width,
+                           allocation.height);
 
   return FALSE;
 }
@@ -585,9 +585,6 @@ gimp_view_real_set_viewable (GimpView     *view,
 
   if (view->viewable)
     {
-      g_object_remove_weak_pointer (G_OBJECT (view->viewable),
-                                    (gpointer) &view->viewable);
-
       if (! viewable && ! view->renderer->is_popup)
         {
           if (gimp_dnd_viewable_source_remove (GTK_WIDGET (view),
@@ -620,13 +617,8 @@ gimp_view_real_set_viewable (GimpView     *view,
     }
 
   gimp_view_renderer_set_viewable (view->renderer, viewable);
-  view->viewable = viewable;
 
-  if (view->viewable)
-    {
-      g_object_add_weak_pointer (G_OBJECT (view->viewable),
-                                 (gpointer) &view->viewable);
-    }
+  g_set_weak_pointer (&view->viewable, viewable);
 }
 
 /*  public functions  */
@@ -809,7 +801,7 @@ gimp_view_update_callback (GimpViewRenderer *renderer,
   gint            width;
   gint            height;
 
-  gtk_widget_get_requisition (widget, &requisition);
+  gtk_widget_get_preferred_size (widget, &requisition, NULL);
 
   width  = renderer->width  + 2 * renderer->border_width;
   height = renderer->height + 2 * renderer->border_width;
