@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_VIEW_RENDERER_H__
@@ -33,7 +33,8 @@
 #define GIMP_VIEW_RENDERER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GIMP_TYPE_VIEW_RENDERER, GimpViewRendererClass))
 
 
-typedef struct _GimpViewRendererClass  GimpViewRendererClass;
+typedef struct _GimpViewRendererPrivate GimpViewRendererPrivate;
+typedef struct _GimpViewRendererClass   GimpViewRendererClass;
 
 struct _GimpViewRenderer
 {
@@ -51,19 +52,15 @@ struct _GimpViewRenderer
   guint               is_popup    : 1;
 
   GimpViewBorderType  border_type;
-  GimpRGB             border_color;
+  GeglColor          *border_color;
 
   /*< protected >*/
   cairo_surface_t    *surface;
 
-  /*< private >*/
-  cairo_pattern_t    *pattern;
-  GdkPixbuf          *pixbuf;
-  gchar              *bg_stock_id;
-
   gint                size;
-  gboolean            needs_render;
-  guint               idle_id;
+
+  /*< private >*/
+  GimpViewRendererPrivate *priv;
 };
 
 struct _GimpViewRendererClass
@@ -123,9 +120,13 @@ void   gimp_view_renderer_set_dot_for_dot  (GimpViewRenderer   *renderer,
 void   gimp_view_renderer_set_border_type  (GimpViewRenderer   *renderer,
                                             GimpViewBorderType  border_type);
 void   gimp_view_renderer_set_border_color (GimpViewRenderer   *renderer,
-                                            const GimpRGB      *border_color);
+                                            GeglColor          *border_color);
 void   gimp_view_renderer_set_background   (GimpViewRenderer   *renderer,
-                                            const gchar        *stock_id);
+                                            const gchar        *icon_name);
+void   gimp_view_renderer_set_color_config (GimpViewRenderer   *renderer,
+                                            GimpColorConfig    *color_config);
+GimpColorConfig *
+       gimp_view_renderer_get_color_config (GimpViewRenderer   *renderer);
 
 void   gimp_view_renderer_invalidate       (GimpViewRenderer   *renderer);
 void   gimp_view_renderer_update           (GimpViewRenderer   *renderer);
@@ -141,18 +142,28 @@ void   gimp_view_renderer_draw             (GimpViewRenderer   *renderer,
 /*  protected  */
 
 void   gimp_view_renderer_render_temp_buf_simple (GimpViewRenderer *renderer,
-                                                  TempBuf          *temp_buf);
+                                                  GtkWidget        *widget,
+                                                  GimpTempBuf      *temp_buf);
 void   gimp_view_renderer_render_temp_buf        (GimpViewRenderer *renderer,
-                                                  TempBuf          *temp_buf,
+                                                  GtkWidget        *widget,
+                                                  GimpTempBuf      *temp_buf,
+                                                  gint              temp_buf_x,
+                                                  gint              temp_buf_y,
                                                   gint              channel,
                                                   GimpViewBG        inside_bg,
                                                   GimpViewBG        outside_bg);
 void   gimp_view_renderer_render_pixbuf          (GimpViewRenderer *renderer,
-                                                  GdkPixbuf        *pixbuf);
-void   gimp_view_renderer_render_stock           (GimpViewRenderer *renderer,
                                                   GtkWidget        *widget,
-                                                  const gchar      *stock_id);
-
+                                                  GdkPixbuf        *pixbuf);
+void   gimp_view_renderer_render_icon            (GimpViewRenderer *renderer,
+                                                  GtkWidget        *widget,
+                                                  const gchar      *icon_name);
+GimpColorTransform *
+       gimp_view_renderer_get_color_transform    (GimpViewRenderer *renderer,
+                                                  GtkWidget        *widget,
+                                                  const Babl       *src_format,
+                                                  const Babl       *dest_format);
+void   gimp_view_renderer_free_color_transform   (GimpViewRenderer *renderer);
 
 
 #endif /* __GIMP_VIEW_RENDERER_H__ */
