@@ -26,7 +26,12 @@
 #include <gtk/gtk.h>
 
 #ifdef GDK_WINDOWING_WIN32
+#include <dwmapi.h>
 #include <gdk/gdkwin32.h>
+
+#ifndef DWMWA_USE_IMMERSIVE_DARK_MODE
+#define DWMWA_USE_IMMERSIVE_DARK_MODE 20
+#endif
 #endif
 
 #ifdef GDK_WINDOWING_X11
@@ -51,7 +56,10 @@
 
 #include "gegl/gimp-babl.h"
 
+#include "config/gimpguiconfig.h"
+
 #include "core/gimp.h"
+#include "core/gimpprogress.h"
 #include "core/gimptoolinfo.h"
 
 #include "gimpaccellabel.h"
@@ -87,11 +95,18 @@ typedef struct
   gchar     *settings_value;
 } BlinkStep;
 
-static void         gimp_widget_blink_after        (GtkWidget   *widget,
-                                                    gint         ms_timeout);
-static void         gimp_search_widget_rec         (GtkWidget   *widget,
-                                                    BlinkSearch *data);
-static void         gimp_blink_free_script         (GList       *blink_scenario);
+static void         gimp_widget_blink_after             (GtkWidget    *widget,
+                                                         gint          ms_timeout);
+static void         gimp_search_widget_rec              (GtkWidget    *widget,
+                                                         BlinkSearch  *data);
+static void         gimp_blink_free_script              (GList        *blink_scenario);
+
+static gboolean     gimp_window_transient_on_mapped     (GtkWidget    *widget,
+                                                         GdkEventAny  *event,
+                                                         GimpProgress *progress);
+static void         gimp_window_set_transient_cb        (GtkWidget    *window,
+                                                         GdkEventAny  *event,
+                                                         GBytes       *handle);
 
 
 GtkWidget *
@@ -905,105 +920,51 @@ gimp_window_set_hint (GtkWindow      *window,
     }
 }
 
-/**
- * gimp_window_get_native_id:
- * @window: a #GtkWindow
- *
- * This function is used to pass a window handle to plug-ins so that
- * they can set their dialog windows transient to the parent window.
- *
- * Returns: a native window ID of the window's #GdkWindow or 0
- *               if the window isn't realized yet
- */
-guint32
-gimp_window_get_native_id (GtkWindow *window)
-{
-  GdkWindow *surface;
-
-  g_return_val_if_fail (GTK_IS_WINDOW (window), 0);
-
-  surface = gtk_widget_get_window (GTK_WIDGET (window));
-  if (!surface) /* aka window is not yet realized */
-    return 0;
-
-#ifdef GDK_WINDOWING_WIN32
-  if (GDK_IS_WIN32_WINDOW (surface))
-    return GDK_WINDOW_HWND (gtk_widget_get_window (GTK_WIDGET (window)));
-#endif
-
-#ifdef GDK_WINDOWING_X11
-  if (GDK_IS_X11_WINDOW (surface))
-    return GDK_WINDOW_XID (gtk_widget_get_window (GTK_WIDGET (window)));
-#endif
-
-#ifdef GDK_WINDOWING_WAYLAND
-  if (GDK_IS_WAYLAND_WINDOW (surface))
-    g_debug ("Getting window ID for progress not supported on Wayland yet");
-#endif
-
-  return 0;
-}
-
-#ifndef GDK_WINDOWING_WIN32
-static void
-gimp_window_transient_realized (GtkWidget *window,
-                                GdkWindow *parent)
-{
-  if (gtk_widget_get_realized (window))
-    gdk_window_set_transient_for (gtk_widget_get_window (window), parent);
-}
-
 /* similar to what we have in libgimp/gimpui.c */
 static GdkWindow *
-gimp_get_foreign_window (guint32 window)
+gimp_get_foreign_window (gpointer window)
 {
 #ifdef GDK_WINDOWING_X11
   if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
     return gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),
-                                                   window);
+                                                   (Window) window);
 #endif
 
 #ifdef GDK_WINDOWING_WIN32
   return gdk_win32_window_foreign_new_for_display (gdk_display_get_default (),
-                                                   window);
+                                                   (HWND) window);
 #endif
 
   return NULL;
 }
-#endif
 
 void
-gimp_window_set_transient_for (GtkWindow *window,
-                               guint32    parent_ID)
+gimp_window_set_transient_for (GtkWindow    *window,
+                               GimpProgress *parent)
 {
-  /* Cross-process transient-for is broken in gdk/win32 <= 2.10.6. It
-   * causes hangs, at least when used as by the gimp and script-fu
-   * processes. In some newer GTK+ version it will be fixed to be a
-   * no-op. If it eventually is fixed to actually work, change this to
-   * a run-time check of GTK+ version. Remember to change also the
-   * function with the same name in libgimp/gimpui.c
-   *
-   * Note: this hanging bug is still happening with GTK+3 as of 2019-10,
-   * with steps described in comment 4 in:
-   * https://bugzilla.gnome.org/show_bug.cgi?id=359538
-   */
-#ifndef GDK_WINDOWING_WIN32
-  GdkWindow *parent;
+  g_signal_connect_after (window, "map-event",
+                          G_CALLBACK (gimp_window_transient_on_mapped),
+                          parent);
 
-  parent = gimp_get_foreign_window (parent_ID);
-  if (! parent)
-    return;
+  if (gtk_widget_get_mapped (GTK_WIDGET (window)))
+    gimp_window_transient_on_mapped (GTK_WIDGET (window), NULL, parent);
+}
 
-  if (gtk_widget_get_realized (GTK_WIDGET (window)))
-    gdk_window_set_transient_for (gtk_widget_get_window (GTK_WIDGET (window)),
-                                  parent);
+void
+gimp_window_set_transient_for_handle (GtkWindow *window,
+                                      GBytes    *handle)
+{
+  g_return_if_fail (GTK_IS_WINDOW (window));
+  g_return_if_fail (handle != NULL);
 
-  g_signal_connect_object (window, "realize",
-                           G_CALLBACK (gimp_window_transient_realized),
-                           parent, 0);
+  g_signal_connect_data (window, "map-event",
+                         G_CALLBACK (gimp_window_set_transient_cb),
+                         g_bytes_ref (handle),
+                         (GClosureNotify) g_bytes_unref,
+                         G_CONNECT_AFTER);
 
-  g_object_unref (parent);
-#endif
+  if (gtk_widget_get_mapped (GTK_WIDGET (window)))
+    gimp_window_set_transient_cb (GTK_WIDGET (window), NULL, handle);
 }
 
 static void
@@ -1069,12 +1030,12 @@ gimp_widget_set_accel_help (GtkWidget  *widget,
       g_signal_handlers_disconnect_by_func (prev_action,
                                             gimp_widget_accels_changed,
                                             widget);
-      g_object_weak_unref (G_OBJECT (action),
+      g_object_weak_unref (G_OBJECT (prev_action),
                            gimp_accel_help_accel_group_weak_notify,
                            widget);
       g_object_weak_unref (G_OBJECT (widget),
                            gimp_accel_help_widget_weak_notify,
-                           action);
+                           prev_action);
       g_object_set_data (G_OBJECT (widget), "gimp-accel-help-action", NULL);
     }
 
@@ -1135,7 +1096,7 @@ gimp_get_message_icon_name (GimpMessageSeverity severity)
 
 gboolean
 gimp_get_color_tag_color (GimpColorTag  color_tag,
-                          GimpRGB      *color,
+                          GeglColor    *color,
                           gboolean      inherited)
 {
   static const struct
@@ -1162,16 +1123,25 @@ gimp_get_color_tag_color (GimpColorTag  color_tag,
 
   if (color_tag > GIMP_COLOR_TAG_NONE)
     {
-      gimp_rgba_set_uchar (color,
-                           colors[color_tag].r,
-                           colors[color_tag].g,
-                           colors[color_tag].b,
-                           255);
-
       if (inherited)
         {
-          gimp_rgb_composite (color, &(GimpRGB) {1.0, 1.0, 1.0, 0.2},
-                              GIMP_RGB_COMPOSITE_NORMAL);
+          /* Composite color on top of white (20% opacity) */
+          gdouble opacity = 0.2f;
+          gdouble factor  = 1.0f - opacity;
+
+          gegl_color_set_rgba (color,
+                               (colors[color_tag].r / 255.0f) * factor + opacity,
+                               (colors[color_tag].g / 255.0f) * factor + opacity,
+                               (colors[color_tag].b / 255.0f) * factor + opacity,
+                               1.0f);
+        }
+      else
+        {
+          gegl_color_set_rgba (color,
+                               colors[color_tag].r / 255.0f,
+                               colors[color_tag].g / 255.0f,
+                               colors[color_tag].b / 255.0f,
+                               1.0f);
         }
 
       return TRUE;
@@ -2392,6 +2362,24 @@ gimp_search_widget_rec (GtkWidget   *widget,
       gtk_container_foreach (GTK_CONTAINER (widget),
                              (GtkCallback) gimp_search_widget_rec,
                              (gpointer) data);
+
+      if (GTK_IS_TREE_VIEW (widget))
+        {
+          GList *columns;
+
+          columns = gtk_tree_view_get_columns (GTK_TREE_VIEW (widget));
+          for (GList *iter = columns; iter; iter = iter->next)
+            {
+              GtkWidget *column_widget;
+
+              column_widget = gtk_tree_view_column_get_widget (iter->data);
+
+              if (column_widget)
+                gimp_search_widget_rec (column_widget, data);
+            }
+
+          g_list_free (columns);
+        }
     }
 }
 
@@ -2424,17 +2412,43 @@ gimp_utils_make_canonical_menu_label (const gchar *path)
   return canon_path;
 }
 
+/**
+ * gimp_utils_break_menu_path:
+ * @path:
+ * @section_name:
+ *
+ * Break @path which is in the form "/_some/p_ath" into a GStrv containing "some"
+ * and "path", in a canonical way:
+ * - Multiple slashes are folded to one (e.g. "//some///path/" and "/some/path"
+ *   are equivalent).
+ * - End slash or none are equivalent.
+ * - Individual path components are canonicalized with
+ *   gimp_utils_make_canonical_menu_label(), in particular with mnemonic
+ *   underscore removed.
+ *
+ * Moreover if @section_name is not %NULL, then a path component of the form
+ * "[section]" at the end of the path will be removed and "section" will be
+ * stored inside @section_name. Furthermore, "[[label]]" or "[[label]" will both
+ * be transformed into a component path "[label]", without being removed, as a
+ * way to create menu paths containing square brackets.
+ */
 gchar **
-gimp_utils_break_menu_path (const gchar *path)
+gimp_utils_break_menu_path (const gchar  *path,
+                            gchar       **mnemonic_path1,
+                            gchar       **section_name)
 {
-  GRegex *path_regex;
-  gchar **paths;
-  gint    start = 0;
+  GRegex   *path_regex;
+  gchar   **paths;
+  GString  *mnemonic_string = NULL;
+  gint      start = 0;
 
   g_return_val_if_fail (path != NULL, NULL);
 
   path_regex = g_regex_new ("/+", 0, 0, NULL);
 
+  if (mnemonic_path1 != NULL)
+    mnemonic_string = g_string_new (NULL);
+
   /* Get rid of leading slashes. */
   while (path[start] == '/' && path[start] != '\0')
     start++;
@@ -2452,19 +2466,56 @@ gimp_utils_break_menu_path (const gchar *path)
     {
       gchar *canon_path;
 
+      if (section_name && paths[i + 1] == NULL)
+        {
+          gint path_len;
+
+          path_len = strlen (paths[i]);
+          if (path_len > 2 && paths[i][0] == '[' && paths[i][path_len - 1] == ']')
+            {
+              if (paths[i][1] == '[')
+                {
+                  if (paths[i][path_len - 2] == ']')
+                    paths[i][path_len - 1] = '\0';
+
+                  canon_path = g_strdup (paths[i] + 1);
+                  g_free (paths[i]);
+                  paths[i] = canon_path;
+                }
+              else
+                {
+                  paths[i][path_len - 1] = '\0';
+                  *section_name = g_strdup (paths[i] + 1);
+
+                  g_free (paths[i]);
+                  paths[i] = NULL;
+                  break;
+                }
+            }
+        }
+
+      if (mnemonic_string != NULL)
+        g_string_append_printf (mnemonic_string, "/%s", paths[i]);
+
       canon_path = gimp_utils_make_canonical_menu_label (paths[i]);
       g_free (paths[i]);
       paths[i] = canon_path;
     }
 
+  if (mnemonic_path1 != NULL)
+    *mnemonic_path1 = g_string_free (mnemonic_string, FALSE);
+
   g_regex_unref (path_regex);
 
   return paths;
 }
 
 gboolean
-gimp_utils_are_menu_path_identical (const gchar *path1,
-                                    const gchar *path2)
+gimp_utils_are_menu_path_identical (const gchar  *path1,
+                                    const gchar  *path2,
+                                    gchar       **canonical_path1,
+                                    gchar       **mnemonic_path1,
+                                    gchar       **path1_section_name)
 {
   gchar    **paths1;
   gchar    **paths2;
@@ -2475,8 +2526,8 @@ gimp_utils_are_menu_path_identical (const gchar *path1,
   if (path2 == NULL)
     path2 = "/";
 
-  paths1 = gimp_utils_break_menu_path (path1);
-  paths2 = gimp_utils_break_menu_path (path2);
+  paths1 = gimp_utils_break_menu_path (path1, mnemonic_path1, path1_section_name);
+  paths2 = gimp_utils_break_menu_path (path2, NULL, NULL);
   if (g_strv_length (paths1) != g_strv_length (paths2))
     {
       identical = FALSE;
@@ -2492,8 +2543,158 @@ gimp_utils_are_menu_path_identical (const gchar *path1,
             }
         }
     }
+
+  if (canonical_path1)
+    {
+      gchar *joined = g_strjoinv ("/", paths1);
+
+      *canonical_path1 = g_strdup_printf ("/%s", joined);
+
+      g_free (joined);
+    }
+
   g_strfreev (paths1);
   g_strfreev (paths2);
 
   return identical;
 }
+
+static gboolean
+gimp_window_transient_on_mapped (GtkWidget    *window,
+                                 GdkEventAny  *event G_GNUC_UNUSED,
+                                 GimpProgress *progress)
+{
+  GBytes *handle;
+
+  handle = gimp_progress_get_window_id (progress);
+
+  if (handle == NULL)
+    return FALSE;
+
+  gimp_window_set_transient_cb (window, NULL, handle);
+
+  g_bytes_unref (handle);
+
+  return FALSE;
+}
+
+static void
+gimp_window_set_transient_cb (GtkWidget   *window,
+                              GdkEventAny *event G_GNUC_UNUSED,
+                              GBytes      *handle)
+{
+  gboolean transient_set = FALSE;
+
+  g_return_if_fail (handle != NULL);
+
+#ifdef GDK_WINDOWING_WAYLAND
+  if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ()))
+    {
+      char *wayland_handle;
+
+      wayland_handle = (char *) g_bytes_get_data (handle, NULL);
+      gdk_wayland_window_set_transient_for_exported (gtk_widget_get_window (window),
+                                                     wayland_handle);
+      transient_set = TRUE;
+    }
+#endif
+
+#ifdef GDK_WINDOWING_X11
+  if (! transient_set && GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    {
+      GdkWindow *parent;
+      Window    *handle_data;
+      Window     parent_ID;
+      gsize      handle_size;
+
+      handle_data = (Window *) g_bytes_get_data (handle, &handle_size);
+      g_return_if_fail (handle_size == sizeof (Window));
+      parent_ID = *handle_data;
+
+      parent = gimp_get_foreign_window ((gpointer) parent_ID);
+
+      if (parent)
+        gdk_window_set_transient_for (gtk_widget_get_window (window), parent);
+
+      transient_set = TRUE;
+    }
+#endif
+  /* Cross-process transient-for is broken in gdk/win32. It causes hangs of the
+   * main process and we still don't know why.
+   * If it eventually is fixed to actually work, change this to a run-time check
+   * of GTK+ version. Remember to change also gimp_window_transient_on_mapped()
+   * in libgimp/gimpui.c
+   *
+   * Note: this hanging bug is still happening with GTK+3 as of mid-2023 with
+   * steps described in comment 4 in:
+   * https://bugzilla.gnome.org/show_bug.cgi?id=359538
+   */
+#if 0 && defined (GDK_WINDOWING_WIN32)
+  if (! transient_set)
+    {
+      GdkWindow *parent;
+      HANDLE    *handle_data;
+      HANDLE     parent_ID;
+      gsize      handle_size;
+
+      handle_data = (HANDLE *) g_bytes_get_data (handle, &handle_size);
+      g_return_if_fail (handle_size == sizeof (HANDLE));
+      parent_ID = *handle_data;
+
+      parent = gimp_get_foreign_window ((gpointer) parent_ID);
+
+      if (parent)
+        gdk_window_set_transient_for (gtk_widget_get_window (window), parent);
+
+      transient_set = TRUE;
+    }
+#endif
+}
+
+#ifdef G_OS_WIN32
+void
+gimp_window_set_title_bar_theme (Gimp      *gimp,
+                                 GtkWidget *dialog)
+{
+  HWND           hwnd;
+  GdkWindow     *window        = NULL;
+  gboolean       use_dark_mode = FALSE;
+
+  window = gtk_widget_get_window (GTK_WIDGET (dialog));
+  if (window)
+    {
+      if (gimp)
+        {
+          GimpGuiConfig *config;
+
+          config = GIMP_GUI_CONFIG (gimp->config);
+          use_dark_mode = (config->theme_scheme != GIMP_THEME_LIGHT);
+        }
+      else
+        {
+          GtkStyleContext *style;
+          GdkRGBA         *color = NULL;
+
+          /* Workaround if we don't have access to GimpGuiConfig.
+           * If the background color is below the threshold, then we're
+           * likely in dark mode.
+           */
+          style = gtk_widget_get_style_context (dialog);
+          gtk_style_context_get (style, gtk_style_context_get_state (style),
+                                 GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &color,
+                                 NULL);
+          if (color)
+            {
+              if (color->red < 0.5 && color->green < 0.5 && color->blue < 0.5)
+                use_dark_mode = TRUE;
+
+              gdk_rgba_free (color);
+            }
+        }
+
+        hwnd = (HWND) gdk_win32_window_get_handle (window);
+        DwmSetWindowAttribute (hwnd, DWMWA_USE_IMMERSIVE_DARK_MODE,
+                               &use_dark_mode, sizeof (use_dark_mode));
+    }
+}
+#endif
