@@ -71,15 +71,16 @@ gboolean          gimp_get_style_color             (GtkWidget            *widget
                                                     GdkRGBA              *color);
 void              gimp_window_set_hint             (GtkWindow            *window,
                                                     GimpWindowHint        hint);
-guint32           gimp_window_get_native_id        (GtkWindow            *window);
 void              gimp_window_set_transient_for    (GtkWindow            *window,
-                                                    guint32               parent_ID);
+                                                    GimpProgress         *progress);
+void          gimp_window_set_transient_for_handle (GtkWindow            *window,
+                                                    GBytes               *handle);
 void              gimp_widget_set_accel_help       (GtkWidget            *widget,
                                                     GimpAction           *action);
 
 const gchar     * gimp_get_message_icon_name       (GimpMessageSeverity   severity);
 gboolean          gimp_get_color_tag_color         (GimpColorTag          color_tag,
-                                                    GimpRGB              *color,
+                                                    GeglColor            *color,
                                                     gboolean              inherited);
 
 void              gimp_pango_layout_set_scale      (PangoLayout          *layout,
@@ -155,9 +156,17 @@ void              gimp_widget_flush_expose         (void);
 void              gimp_make_valid_action_name      (gchar                 *action_name);
 
 gchar           * gimp_utils_make_canonical_menu_label (const gchar       *path);
-gchar          ** gimp_utils_break_menu_path           (const gchar       *path);
+gchar          ** gimp_utils_break_menu_path           (const gchar       *path,
+                                                        gchar            **mnemonic_path1,
+                                                        gchar            **section_name);
 gboolean          gimp_utils_are_menu_path_identical   (const gchar       *path1,
-                                                        const gchar       *path2);
-
+                                                        const gchar       *path2,
+                                                        gchar            **canonical_path1,
+                                                        gchar            **mnemonic_path1,
+                                                        gchar            **path1_section_name);
+#ifdef G_OS_WIN32
+void              gimp_window_set_title_bar_theme      (Gimp              *gimp,
+                                                        GtkWidget         *dialog);
+#endif
 
 #endif /* __APP_GIMP_WIDGETS_UTILS_H__ */
