@@ -22,6 +22,8 @@
 
 #include "gimp.h"
 
+#include <gobject/gvaluecollector.h>
+
 #include "gimppixbuf.h"
 #include "gimptilebackendplugin.h"
 
@@ -378,8 +380,9 @@ gimp_drawable_get_format (GimpDrawable *drawable)
         {
           const Babl *palette;
           const Babl *palette_alpha;
+          const Babl *color_format;
           guchar     *colormap;
-          gint        colormap_len, n_colors;
+          gint        n_colors;
 
           babl_new_palette_with_space (format_str, space,
                                        &palette, &palette_alpha);
@@ -389,14 +392,12 @@ gimp_drawable_get_format (GimpDrawable *drawable)
           else
             format = palette;
 
-          colormap = gimp_image_get_colormap (image, &colormap_len, &n_colors);
+          color_format = babl_format_with_space ("R'G'B' u8", space);
+          colormap     = gimp_palette_get_colormap (gimp_image_get_palette (image), color_format, &n_colors, NULL);
 
           if (colormap)
             {
-              babl_palette_set_palette (format,
-                                        babl_format_with_space ("R'G'B' u8",
-                                                                space),
-                                        colormap, n_colors);
+              babl_palette_set_palette (format, color_format, colormap, n_colors);
               g_free (colormap);
             }
         }
@@ -434,3 +435,239 @@ gimp_drawable_get_thumbnail_format (GimpDrawable *drawable)
 
   return format;
 }
+
+/**
+ * gimp_drawable_append_filter:
+ * @drawable: The drawable.
+ * @filter: The drawable filter to append.
+ *
+ * This procedure appends the specified drawable effect at the top of the
+ * effect list of @drawable.
+ *
+ * The @drawable argument must be the same as the one used when you
+ * created the effect with [ctor@Gimp.DrawableFilter.new].
+ * Some effects may be slower than others to render. In order to
+ * minimize processing time, it is preferred to customize the
+ * operation's arguments as received with
+ * [method@Gimp.DrawableFilter.get_config] before adding the effect.
+ *
+ * Since: 3.0
+ **/
+void
+gimp_drawable_append_filter (GimpDrawable       *drawable,
+                             GimpDrawableFilter *filter)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
+  g_return_if_fail (GIMP_IS_DRAWABLE_FILTER (filter));
+
+  gimp_drawable_filter_update (filter);
+  _gimp_drawable_append_filter_private (drawable, filter);
+}
+
+/**
+ * gimp_drawable_merge_filter:
+ * @drawable: The drawable.
+ * @filter: The drawable filter to merge.
+ *
+ * This procedure applies the specified drawable effect on @drawable
+ * and merge it (therefore before any non-destructive effects are
+ * computed).
+ *
+ * The @drawable argument must be the same as the one used when you
+ * created the effect with [ctor@Gimp.DrawableFilter.new].
+ * Once this is run, @filter is not valid anymore and you should not
+ * try to do anything with it. In particular, you must customize the
+ * operation's arguments as received with
+ * [method@Gimp.DrawableFilter.get_config] or set the filter's opacity
+ * and blend mode before merging the effect.
+ *
+ * Since: 3.0
+ **/
+void
+gimp_drawable_merge_filter (GimpDrawable       *drawable,
+                            GimpDrawableFilter *filter)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
+  g_return_if_fail (GIMP_IS_DRAWABLE_FILTER (filter));
+
+  gimp_drawable_filter_update (filter);
+  _gimp_drawable_merge_filter_private (drawable, filter);
+}
+
+/**
+ * gimp_drawable_append_new_filter: (skip)
+ * @drawable:       The #GimpDrawable.
+ * @operation_name: The GEGL operation's name.
+ * @name:           The effect name.
+ * @mode:           The blend mode.
+ * @opacity:        The opacity from 0.0 (transparent) to 1.0 (opaque).
+ * @...:            a %NULL-terminated list of operation argument names
+ *                  and values.
+ *
+ * Utility function which combines [ctor@Gimp.DrawableFilter.new]
+ * followed by setting arguments for the
+ * [class@Gimp.DrawableFilterConfig] returned by
+ * [method@Gimp.DrawableFilter.get_config], and finally appending with
+ * [method@Gimp.Drawable.append_filter]
+ *
+ * The variable arguments are couples of an argument name followed by a
+ * value, NULL-terminated, such as:
+ *
+ * ```C
+ * filter = gimp_drawable_append_new_filter (drawable,
+ *                                           GIMP_LAYER_MODE_REPLACE, 1.0,
+ *                                           "gegl:gaussian-blur", "My Gaussian Blur",
+ *                                           "std-dev-x", 2.5,
+ *                                           "std-dev-y", 2.5,
+ *                                           "abyss-policy", "clamp",
+ *                                           NULL);
+ * ```
+ *
+ * Returns: (transfer none): The newly created filter.
+ */
+GimpDrawableFilter *
+gimp_drawable_append_new_filter (GimpDrawable  *drawable,
+                                 const gchar   *operation_name,
+                                 const gchar   *name,
+                                 GimpLayerMode  mode,
+                                 gdouble        opacity,
+                                 ...)
+{
+  GimpDrawableFilter       *filter;
+  GimpDrawableFilterConfig *config;
+  const gchar              *arg_name;
+  va_list                   va_args;
+
+  g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
+  g_return_val_if_fail (opacity >= 0.0 && opacity <= 1.0, NULL);
+
+  filter = gimp_drawable_filter_new (drawable, operation_name, name);
+
+  g_return_val_if_fail (filter != NULL, NULL);
+
+  config = gimp_drawable_filter_get_config (filter);
+
+  va_start (va_args, opacity);
+  while ((arg_name = va_arg (va_args, const gchar *)))
+    {
+      GParamSpec *pspec;
+      gchar      *error = NULL;
+      GValue      value = G_VALUE_INIT;
+
+      pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (config), arg_name);
+      if (pspec == NULL)
+        {
+          g_warning ("%s: %s has no property named '%s'",
+                     G_STRFUNC,
+                     g_type_name (G_TYPE_FROM_INSTANCE (config)),
+                     arg_name);
+          break;
+        }
+      g_value_init (&value, pspec->value_type);
+      G_VALUE_COLLECT (&value, va_args, G_VALUE_NOCOPY_CONTENTS, &error);
+
+      if (error)
+        {
+          g_warning ("%s: %s", G_STRFUNC, error);
+          g_free (error);
+          break;
+        }
+
+      g_object_set_property (G_OBJECT (config), arg_name, &value);
+      g_value_unset (&value);
+    }
+  va_end (va_args);
+
+  gimp_drawable_filter_set_blend_mode (filter, mode);
+  gimp_drawable_filter_set_opacity (filter, opacity);
+  gimp_drawable_append_filter (drawable, filter);
+
+  return filter;
+}
+
+/**
+ * gimp_drawable_merge_new_filter: (skip)
+ * @drawable:       The #GimpDrawable.
+ * @operation_name: The GEGL operation's name.
+ * @name:           The effect name which will show in undo step.
+ * @mode:           The blend mode.
+ * @opacity:        The opacity from 0.0 (transparent) to 1.0 (opaque).
+ * @...:            a %NULL-terminated list of operation argument names
+ *                  and values.
+ *
+ * Utility function which combines [ctor@Gimp.DrawableFilter.new]
+ * followed by setting arguments for the
+ * [class@Gimp.DrawableFilterConfig] returned by
+ * [method@Gimp.DrawableFilter.get_config], and finally applying the
+ * effect to @drawable with [method@Gimp.Drawable.merge_filter]
+ *
+ * The variable arguments are couples of an argument name followed by a
+ * value, NULL-terminated, such as:
+ *
+ * ```C
+ * filter = gimp_drawable_merge_new_filter (drawable,
+ *                                          GIMP_LAYER_MODE_REPLACE, 1.0,
+ *                                          "gegl:gaussian-blur", "My Gaussian Blur",
+ *                                          "std-dev-x", 2.5,
+ *                                          "std-dev-y", 2.5,
+ *                                          "abyss-policy", "clamp",
+ *                                          NULL);
+ * ```
+ */
+void
+gimp_drawable_merge_new_filter (GimpDrawable  *drawable,
+                                const gchar   *operation_name,
+                                const gchar   *name,
+                                GimpLayerMode  mode,
+                                gdouble        opacity,
+                                ...)
+{
+  GimpDrawableFilter       *filter;
+  GimpDrawableFilterConfig *config;
+  const gchar              *arg_name;
+  va_list                   va_args;
+
+  g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
+  g_return_if_fail (opacity >= 0.0 && opacity <= 1.0);
+
+  filter = gimp_drawable_filter_new (drawable, operation_name, name);
+
+  g_return_if_fail (filter != NULL);
+
+  config = gimp_drawable_filter_get_config (filter);
+
+  va_start (va_args, opacity);
+  while ((arg_name = va_arg (va_args, const gchar *)))
+    {
+      GParamSpec *pspec;
+      gchar      *error = NULL;
+      GValue      value = G_VALUE_INIT;
+
+      pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (config), arg_name);
+      if (pspec == NULL)
+        {
+          g_warning ("%s: %s has no property named '%s'",
+                     G_STRFUNC,
+                     g_type_name (G_TYPE_FROM_INSTANCE (config)),
+                     arg_name);
+          break;
+        }
+      g_value_init (&value, pspec->value_type);
+      G_VALUE_COLLECT (&value, va_args, G_VALUE_NOCOPY_CONTENTS, &error);
+
+      if (error)
+        {
+          g_warning ("%s: %s", G_STRFUNC, error);
+          g_free (error);
+          break;
+        }
+
+      g_object_set_property (G_OBJECT (config), arg_name, &value);
+      g_value_unset (&value);
+    }
+  va_end (va_args);
+
+  gimp_drawable_filter_set_blend_mode (filter, mode);
+  gimp_drawable_filter_set_opacity (filter, opacity);
+  gimp_drawable_merge_filter (drawable, filter);
+}
