@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #if !defined (__GIMP_BASE_H_INSIDE__) && !defined (GIMP_BASE_COMPILATION)
@@ -26,6 +26,23 @@
 G_BEGIN_DECLS
 
 
+/**
+ * GimpCpuAccelFlags:
+ * @GIMP_CPU_ACCEL_NONE:        None
+ * @GIMP_CPU_ACCEL_X86_MMX:     MMX
+ * @GIMP_CPU_ACCEL_X86_3DNOW:   3dNow
+ * @GIMP_CPU_ACCEL_X86_MMXEXT:  MMXEXT
+ * @GIMP_CPU_ACCEL_X86_SSE:     SSE
+ * @GIMP_CPU_ACCEL_X86_SSE2:    SSE2
+ * @GIMP_CPU_ACCEL_X86_SSE3:    SSE3
+ * @GIMP_CPU_ACCEL_X86_SSSE3:   SSSE3
+ * @GIMP_CPU_ACCEL_X86_SSE4_1:  SSE4_1
+ * @GIMP_CPU_ACCEL_X86_SSE4_2:  SSE4_2
+ * @GIMP_CPU_ACCEL_X86_AVX:     AVX
+ * @GIMP_CPU_ACCEL_PPC_ALTIVEC: Altivec
+ *
+ * Types of detectable CPU accelerations
+ **/
 typedef enum
 {
   GIMP_CPU_ACCEL_NONE        = 0x0,
@@ -37,6 +54,10 @@ typedef enum
   GIMP_CPU_ACCEL_X86_SSE     = 0x10000000,
   GIMP_CPU_ACCEL_X86_SSE2    = 0x08000000,
   GIMP_CPU_ACCEL_X86_SSE3    = 0x02000000,
+  GIMP_CPU_ACCEL_X86_SSSE3   = 0x01000000,
+  GIMP_CPU_ACCEL_X86_SSE4_1  = 0x00800000,
+  GIMP_CPU_ACCEL_X86_SSE4_2  = 0x00400000,
+  GIMP_CPU_ACCEL_X86_AVX     = 0x00200000,
 
   /* powerpc accelerations */
   GIMP_CPU_ACCEL_PPC_ALTIVEC = 0x04000000
@@ -46,10 +67,6 @@ typedef enum
 GimpCpuAccelFlags  gimp_cpu_accel_get_support (void);
 
 
-/* for internal use only */
-void               gimp_cpu_accel_set_use     (gboolean use);
-
-
 G_END_DECLS
 
 #endif  /* __GIMP_CPU_ACCEL_H__ */
