@@ -404,11 +404,19 @@ static void
 gimp_module_db_load_module (GimpModuleDB *db,
                             GFile        *file)
 {
+  gchar      *filename;
   GimpModule *module;
+  GModule    *gmodule;
   gboolean   load_inhibit;
 
-  if (! gimp_file_has_extension (file, "." G_MODULE_SUFFIX))
-    return;
+  filename = g_file_get_path (file);
+  if (! (gmodule = g_module_open (filename, G_MODULE_BIND_LAZY)))
+    {
+      g_free (filename);
+      return;
+    }
+  g_free (filename);
+  g_module_close (gmodule);
 
   /* don't load if we already know about it */
   if (gimp_module_db_module_find_by_file (db, file))
