@@ -69,14 +69,6 @@ enum
 };
 
 
-typedef struct _GimpLCH  GimpLCH;
-
-struct _GimpLCH
-{
-  gdouble l, c, h, a;
-};
-
-
 typedef struct _ColorScale ColorScale;
 
 struct _ColorScale
@@ -92,17 +84,12 @@ struct _ColorScale
 };
 
 
-#define GIMP_COLOR_SCALES_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GIMP_TYPE_COLOR_SCALES, GimpColorScalesClass))
-#define GIMP_IS_COLOR_SCALES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GIMP_TYPE_COLOR_SCALES))
-#define GIMP_COLOR_SCALES_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GIMP_TYPE_COLOR_SCALES, GimpColorScalesClass))
-
-
-typedef struct _GimpColorScalesClass GimpColorScalesClass;
-
 struct _GimpColorScales
 {
   GimpColorSelector  parent_instance;
 
+  const Babl        *format;
+
   gboolean           show_rgb_u8;
   GBinding          *show_rgb_u8_binding;
   GBinding          *show_hsv_binding;
@@ -117,11 +104,8 @@ struct _GimpColorScales
   GtkWidget         *dummy_u8_toggle;
   GtkWidget         *toggles[14];
   GtkWidget         *scales[14];
-};
 
-struct _GimpColorScalesClass
-{
-  GimpColorSelectorClass  parent_class;
+  GList             *profile_labels;
 };
 
 
@@ -143,8 +127,7 @@ static void   gimp_color_scales_togg_visible   (GimpColorSelector *selector,
 static void   gimp_color_scales_set_show_alpha (GimpColorSelector *selector,
                                                 gboolean           show_alpha);
 static void   gimp_color_scales_set_color      (GimpColorSelector *selector,
-                                                const GimpRGB     *rgb,
-                                                const GimpHSV     *hsv);
+                                                GeglColor         *color);
 static void   gimp_color_scales_set_channel    (GimpColorSelector *selector,
                                                 GimpColorSelectorChannel  channel);
 static void   gimp_color_scales_set_model_visible
@@ -153,6 +136,8 @@ static void   gimp_color_scales_set_model_visible
                                                 gboolean           visible);
 static void   gimp_color_scales_set_config     (GimpColorSelector *selector,
                                                 GimpColorConfig   *config);
+static void   gimp_color_scales_set_format     (GimpColorSelector *selector,
+                                                const Babl        *format);
 
 static void   gimp_color_scales_update_visible (GimpColorScales   *scales);
 static void   gimp_color_scales_update_scales  (GimpColorScales   *scales,
@@ -169,8 +154,6 @@ G_DEFINE_TYPE (GimpColorScales, gimp_color_scales, GIMP_TYPE_COLOR_SELECTOR)
 
 #define parent_class gimp_color_scales_parent_class
 
-static const Babl *fish_rgb_to_lch = NULL;
-static const Babl *fish_lch_to_rgb = NULL;
 
 static const ColorScale scale_defs[] =
 {
@@ -219,6 +202,7 @@ gimp_color_scales_class_init (GimpColorScalesClass *klass)
   selector_class->set_channel           = gimp_color_scales_set_channel;
   selector_class->set_model_visible     = gimp_color_scales_set_model_visible;
   selector_class->set_config            = gimp_color_scales_set_config;
+  selector_class->set_format            = gimp_color_scales_set_format;
 
   g_object_class_install_property (object_class, PROP_SHOW_RGB_U8,
                                    g_param_spec_boolean ("show-rgb-u8",
@@ -236,11 +220,6 @@ gimp_color_scales_class_init (GimpColorScalesClass *klass)
                                                          G_PARAM_CONSTRUCT));
 
   gtk_widget_class_set_css_name (widget_class, "GimpColorScales");
-
-  fish_rgb_to_lch = babl_fish (babl_format ("R'G'B'A double"),
-                               babl_format ("CIE LCH(ab) alpha double"));
-  fish_lch_to_rgb = babl_fish (babl_format ("CIE LCH(ab) alpha double"),
-                               babl_format ("R'G'B'A double"));
 }
 
 static GtkWidget *
@@ -255,6 +234,8 @@ create_group (GimpColorScales           *scales,
   GimpColorSelector *selector = GIMP_COLOR_SELECTOR (scales);
   GtkWidget         *grid;
   GEnumClass        *enum_class;
+  GtkWidget         *label     = NULL;
+  gboolean           add_label = FALSE;
   gint               row;
   gint               i;
 
@@ -270,6 +251,12 @@ create_group (GimpColorScales           *scales,
       gint                enum_value = i;
       gboolean            is_u8      = FALSE;
 
+      if ((enum_value >= GIMP_COLOR_SELECTOR_RED_U8 &&
+           enum_value <= GIMP_COLOR_SELECTOR_BLUE_U8) ||
+          (enum_value >= GIMP_COLOR_SELECTOR_HUE &&
+           enum_value <= GIMP_COLOR_SELECTOR_BLUE))
+        add_label = TRUE;
+
       if (enum_value >= GIMP_COLOR_SELECTOR_RED_U8 &&
           enum_value <= GIMP_COLOR_SELECTOR_ALPHA_U8)
         {
@@ -355,6 +342,27 @@ create_group (GimpColorScales           *scales,
                         scales);
     }
 
+  if (add_label)
+    {
+      GtkWidget *scrolled_window;
+
+      scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                      GTK_POLICY_EXTERNAL, GTK_POLICY_NEVER);
+      gtk_grid_attach (GTK_GRID (grid), scrolled_window, 1, row, 3, 1);
+      gtk_widget_set_visible (scrolled_window, TRUE);
+
+      label = gtk_label_new (NULL);
+      gtk_widget_set_halign (label, GTK_ALIGN_START);
+      gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+      gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+      gtk_label_set_text (GTK_LABEL (label), _("Profile: sRGB"));
+      gtk_container_add (GTK_CONTAINER (scrolled_window), label);
+      gtk_widget_set_visible (label, TRUE);
+
+      scales->profile_labels = g_list_prepend (scales->profile_labels, label);
+    }
+
   g_type_class_unref (enum_class);
 
   return grid;
@@ -380,7 +388,7 @@ gimp_color_scales_init (GimpColorScales *scales)
   scales->show_hsv_binding    = NULL;
 
   /*  don't need the toggles for our own operation  */
-  selector->toggles_visible = FALSE;
+  gimp_color_selector_set_toggles_visible (selector, FALSE);
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_box_pack_start (GTK_BOX (scales), hbox, 0, 0, FALSE);
@@ -389,6 +397,8 @@ gimp_color_scales_init (GimpColorScales *scales)
   main_group = NULL;
   u8_group   = NULL;
 
+  scales->profile_labels = NULL;
+
   scales->dummy_u8_toggle = gtk_radio_button_new (NULL);
   g_object_ref_sink (scales->dummy_u8_toggle);
   u8_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (scales->dummy_u8_toggle));
@@ -506,6 +516,7 @@ gimp_color_scales_dispose (GObject *object)
 
   g_clear_pointer (&scales->show_rgb_u8_binding, g_binding_unbind);
   g_clear_pointer (&scales->show_hsv_binding, g_binding_unbind);
+  g_clear_pointer (&scales->profile_labels, g_list_free);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -600,8 +611,7 @@ gimp_color_scales_set_show_alpha (GimpColorSelector *selector,
 
 static void
 gimp_color_scales_set_color (GimpColorSelector *selector,
-                             const GimpRGB     *rgb,
-                             const GimpHSV     *hsv)
+                             GeglColor         *color)
 {
   GimpColorScales *scales = GIMP_COLOR_SCALES (selector);
 
@@ -666,6 +676,60 @@ gimp_color_scales_set_config (GimpColorSelector *selector,
     }
 }
 
+static void
+gimp_color_scales_set_format (GimpColorSelector *selector,
+                              const Babl        *format)
+{
+  GimpColorScales *scales = GIMP_COLOR_SCALES (selector);
+
+  scales->format = format;
+
+  if (format == NULL || babl_format_get_space (format) == babl_space ("sRGB"))
+    {
+      for (GList *iter = scales->profile_labels; iter; iter = iter->next)
+        {
+          gtk_label_set_text (GTK_LABEL (iter->data), _("Profile: sRGB"));
+          gimp_help_set_help_data (iter->data, NULL, NULL);
+        }
+    }
+  else
+    {
+      GimpColorProfile *profile = NULL;
+      const gchar      *icc;
+      gint              icc_len;
+
+      icc = babl_space_get_icc (babl_format_get_space (format), &icc_len);
+      profile = gimp_color_profile_new_from_icc_profile ((const guint8 *) icc, icc_len, NULL);
+
+      if (profile != NULL)
+        {
+          gchar *text;
+
+          text = g_strdup_printf (_("Profile: %s"), gimp_color_profile_get_label (profile));
+          for (GList *iter = scales->profile_labels; iter; iter = iter->next)
+            {
+              gtk_label_set_text (GTK_LABEL (iter->data), text);
+              gimp_help_set_help_data (iter->data,
+                                       gimp_color_profile_get_summary (profile),
+                                       NULL);
+            }
+          g_free (text);
+        }
+      else
+        {
+          for (GList *iter = scales->profile_labels; iter; iter = iter->next)
+            {
+              gtk_label_set_markup (GTK_LABEL (iter->data), _("Profile: <i>unknown</i>"));
+              gimp_help_set_help_data (iter->data, NULL, NULL);
+            }
+        }
+
+      g_clear_object (&profile);
+    }
+
+  gimp_color_scales_update_scales (scales, -1);
+}
+
 
 /*  public functions  */
 
@@ -734,29 +798,32 @@ gimp_color_scales_update_scales (GimpColorScales *scales,
                                  gint             skip)
 {
   GimpColorSelector *selector = GIMP_COLOR_SELECTOR (scales);
-  GimpLCH            lch;
+  GeglColor         *color    = gimp_color_selector_get_color (selector);
+  gdouble            pixel[4];
+  gfloat             pixel_f[4];
   gdouble            values[G_N_ELEMENTS (scale_defs)];
   gint               i;
 
-  babl_process (fish_rgb_to_lch, &selector->rgb, &lch, 1);
+  gegl_color_get_pixel (color, babl_format_with_space ("HSV float", scales->format), pixel_f);
+  values[GIMP_COLOR_SELECTOR_HUE]           = pixel_f[0] * 360.0;
+  values[GIMP_COLOR_SELECTOR_SATURATION]    = pixel_f[1] * 100.0;
+  values[GIMP_COLOR_SELECTOR_VALUE]         = pixel_f[2] * 100.0;
 
-  values[GIMP_COLOR_SELECTOR_HUE]           = selector->hsv.h * 360.0;
-  values[GIMP_COLOR_SELECTOR_SATURATION]    = selector->hsv.s * 100.0;
-  values[GIMP_COLOR_SELECTOR_VALUE]         = selector->hsv.v * 100.0;
+  gegl_color_get_pixel (color, babl_format_with_space ("R'G'B'A double", scales->format), pixel);
+  values[GIMP_COLOR_SELECTOR_RED]           = pixel[0] * 100.0;
+  values[GIMP_COLOR_SELECTOR_GREEN]         = pixel[1] * 100.0;
+  values[GIMP_COLOR_SELECTOR_BLUE]          = pixel[2] * 100.0;
+  values[GIMP_COLOR_SELECTOR_ALPHA]         = pixel[3] * 100.0;
 
-  values[GIMP_COLOR_SELECTOR_RED]           = selector->rgb.r * 100.0;
-  values[GIMP_COLOR_SELECTOR_GREEN]         = selector->rgb.g * 100.0;
-  values[GIMP_COLOR_SELECTOR_BLUE]          = selector->rgb.b * 100.0;
-  values[GIMP_COLOR_SELECTOR_ALPHA]         = selector->rgb.a * 100.0;
+  values[GIMP_COLOR_SELECTOR_RED_U8]        = pixel[0] * 255.0;
+  values[GIMP_COLOR_SELECTOR_GREEN_U8]      = pixel[1] * 255.0;
+  values[GIMP_COLOR_SELECTOR_BLUE_U8]       = pixel[2] * 255.0;
+  values[GIMP_COLOR_SELECTOR_ALPHA_U8]      = pixel[3] * 255.0;
 
-  values[GIMP_COLOR_SELECTOR_LCH_LIGHTNESS] = lch.l;
-  values[GIMP_COLOR_SELECTOR_LCH_CHROMA]    = lch.c;
-  values[GIMP_COLOR_SELECTOR_LCH_HUE]       = lch.h;
-
-  values[GIMP_COLOR_SELECTOR_RED_U8]        = selector->rgb.r * 255.0;
-  values[GIMP_COLOR_SELECTOR_GREEN_U8]      = selector->rgb.g * 255.0;
-  values[GIMP_COLOR_SELECTOR_BLUE_U8]       = selector->rgb.b * 255.0;
-  values[GIMP_COLOR_SELECTOR_ALPHA_U8]      = selector->rgb.a * 255.0;
+  gegl_color_get_pixel (color, babl_format ("CIE LCH(ab) float"), pixel_f);
+  values[GIMP_COLOR_SELECTOR_LCH_LIGHTNESS] = pixel_f[0];
+  values[GIMP_COLOR_SELECTOR_LCH_CHROMA]    = pixel_f[1];
+  values[GIMP_COLOR_SELECTOR_LCH_HUE]       = pixel_f[2];
 
   for (i = 0; i < G_N_ELEMENTS (scale_defs); i++)
     {
@@ -773,9 +840,12 @@ gimp_color_scales_update_scales (GimpColorScales *scales,
                                              scales);
         }
 
-      gimp_color_scale_set_color (GIMP_COLOR_SCALE (gimp_scale_entry_get_range (GIMP_SCALE_ENTRY (scales->scales[i]))),
-                                  &selector->rgb, &selector->hsv);
+      gimp_color_scale_set_format (GIMP_COLOR_SCALE (gimp_scale_entry_get_range (GIMP_SCALE_ENTRY (scales->scales[i]))),
+                                   scales->format);
+      gimp_color_scale_set_color (GIMP_COLOR_SCALE (gimp_scale_entry_get_range (GIMP_SCALE_ENTRY (scales->scales[i]))), color);
     }
+
+  g_object_unref (color);
 }
 
 static void
@@ -812,101 +882,102 @@ gimp_color_scales_scale_changed (GtkWidget       *scale,
                                  GimpColorScales *scales)
 {
   GimpColorSelector *selector = GIMP_COLOR_SELECTOR (scales);
+  GeglColor         *color    = gimp_color_selector_get_color (selector);
   gdouble            value    = gimp_label_spin_get_value (GIMP_LABEL_SPIN (scale));
-  GimpLCH            lch;
+  gfloat             lch[4];
+  gfloat             hsv[4];
+  gdouble            rgb[4];
   gint               i;
 
   for (i = 0; i < G_N_ELEMENTS (scale_defs); i++)
     if (scales->scales[i] == scale)
       break;
 
+  gegl_color_get_pixel (color, babl_format_with_space ("R'G'B'A double", scales->format), rgb);
+  gegl_color_get_pixel (color, babl_format_with_space ("HSVA float", scales->format), hsv);
+  gegl_color_get_pixel (color, babl_format ("CIE LCH(ab) alpha float"), lch);
+
   switch (i)
     {
     case GIMP_COLOR_SELECTOR_HUE:
-      selector->hsv.h = value / 360.0;
+      hsv[0] = value / 360.0f;
       break;
 
     case GIMP_COLOR_SELECTOR_SATURATION:
-      selector->hsv.s = value / 100.0;
+      hsv[1] = value / 100.0f;
       break;
 
     case GIMP_COLOR_SELECTOR_VALUE:
-      selector->hsv.v = value / 100.0;
+      hsv[2] = value / 100.0f;
       break;
 
     case GIMP_COLOR_SELECTOR_RED:
-      selector->rgb.r = value / 100.0;
+      rgb[0] = value / 100.0;
       break;
 
     case GIMP_COLOR_SELECTOR_GREEN:
-      selector->rgb.g = value / 100.0;
+      rgb[1] = value / 100.0;
       break;
 
     case GIMP_COLOR_SELECTOR_BLUE:
-      selector->rgb.b = value / 100.0;
+      rgb[2] = value / 100.0;
       break;
 
     case GIMP_COLOR_SELECTOR_ALPHA:
-      selector->hsv.a = selector->rgb.a = value / 100.0;
+      gimp_color_set_alpha (color, value / 100.0);
       break;
 
     case GIMP_COLOR_SELECTOR_LCH_LIGHTNESS:
-      babl_process (fish_rgb_to_lch, &selector->rgb, &lch, 1);
-      lch.l = value;
+      lch[0] = (gfloat) value;
       break;
 
     case GIMP_COLOR_SELECTOR_LCH_CHROMA:
-      babl_process (fish_rgb_to_lch, &selector->rgb, &lch, 1);
-      lch.c = value;
+      lch[1] = (gfloat) value;
       break;
 
     case GIMP_COLOR_SELECTOR_LCH_HUE:
-      babl_process (fish_rgb_to_lch, &selector->rgb, &lch, 1);
-      lch.h = value;
+      lch[2] = (gfloat) value;
       break;
 
     case GIMP_COLOR_SELECTOR_RED_U8:
-      selector->rgb.r = value / 255.0;
+      rgb[0] = value / 255.0;
       break;
 
     case GIMP_COLOR_SELECTOR_GREEN_U8:
-      selector->rgb.g = value / 255.0;
+      rgb[1] = value / 255.0;
       break;
 
     case GIMP_COLOR_SELECTOR_BLUE_U8:
-      selector->rgb.b = value / 255.0;
+      rgb[2] = value / 255.0;
       break;
 
     case GIMP_COLOR_SELECTOR_ALPHA_U8:
-      selector->hsv.a = selector->rgb.a = value / 255.0;
+      gimp_color_set_alpha (color, value / 255.0);
       break;
     }
 
+
   if ((i >= GIMP_COLOR_SELECTOR_HUE) &&
       (i <= GIMP_COLOR_SELECTOR_VALUE))
     {
-      gimp_hsv_to_rgb (&selector->hsv, &selector->rgb);
+      gegl_color_set_pixel (color, babl_format_with_space ("HSVA float", scales->format), hsv);
     }
   else if ((i >= GIMP_COLOR_SELECTOR_LCH_LIGHTNESS) &&
            (i <= GIMP_COLOR_SELECTOR_LCH_HUE))
     {
-      babl_process (fish_lch_to_rgb, &lch, &selector->rgb, 1);
-      gimp_rgb_to_hsv (&selector->rgb, &selector->hsv);
+      gegl_color_set_pixel (color, babl_format ("CIE LCH(ab) alpha float"), lch);
     }
-  else if ((i >= GIMP_COLOR_SELECTOR_RED) &&
-           (i <= GIMP_COLOR_SELECTOR_BLUE))
+  else if (((i >= GIMP_COLOR_SELECTOR_RED) &&
+            (i <= GIMP_COLOR_SELECTOR_BLUE)) ||
+           ((i >= GIMP_COLOR_SELECTOR_RED_U8) &&
+            (i <= GIMP_COLOR_SELECTOR_BLUE_U8)))
     {
-      gimp_rgb_to_hsv (&selector->rgb, &selector->hsv);
-    }
-  else if ((i >= GIMP_COLOR_SELECTOR_RED_U8) &&
-           (i <= GIMP_COLOR_SELECTOR_BLUE_U8))
-    {
-      gimp_rgb_to_hsv (&selector->rgb, &selector->hsv);
+      gegl_color_set_pixel (color, babl_format_with_space ("R'G'B'A double", scales->format), rgb);
     }
 
-  gimp_color_scales_update_scales (scales, i);
+  gimp_color_selector_set_color (selector, color);
 
-  gimp_color_selector_emit_color_changed (selector);
+  g_object_unref (color);
 }
 
 static void
