@@ -16,21 +16,16 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
 
-#include "gimpwidgetstypes.h"
-
-#include "gimpdialog.h"
-#include "gimppixmap.h"
-#include "gimpquerybox.h"
-#include "gimpsizeentry.h"
 #include "gimpwidgets.h"
 
 #include "libgimp/libgimp-intl.h"
@@ -65,13 +60,14 @@ typedef struct _QueryBox QueryBox;
 
 struct _QueryBox
 {
-  GtkWidget *qbox;
-  GtkWidget *vbox;
-  GtkWidget *entry;
-  GObject   *object;
-  gulong     response_handler;
-  GCallback  callback;
-  gpointer   callback_data;
+  GtkWidget      *qbox;
+  GtkWidget      *vbox;
+  GtkWidget      *entry;
+  GObject        *object;
+  gulong          response_handler;
+  GCallback       callback;
+  gpointer        callback_data;
+  GDestroyNotify  callback_data_destroy;
 };
 
 
@@ -80,14 +76,15 @@ static QueryBox * create_query_box             (const gchar   *title,
                                                 GimpHelpFunc   help_func,
                                                 const gchar   *help_id,
                                                 GCallback      response_callback,
-                                                const gchar   *stock_id,
+                                                const gchar   *icon_name,
                                                 const gchar   *message,
                                                 const gchar   *ok_button,
                                                 const gchar   *cancel_button,
                                                 GObject       *object,
                                                 const gchar   *signal,
                                                 GCallback      callback,
-                                                gpointer       callback_data);
+                                                gpointer       callback_data,
+                                                GDestroyNotify callback_data_destroy);
 
 static void       query_box_disconnect         (QueryBox      *query_box);
 static void       query_box_destroy            (QueryBox      *query_box);
@@ -120,14 +117,15 @@ create_query_box (const gchar   *title,
                   GimpHelpFunc   help_func,
                   const gchar   *help_id,
                   GCallback      response_callback,
-                  const gchar   *stock_id,
+                  const gchar   *icon_name,
                   const gchar   *message,
                   const gchar   *ok_button,
                   const gchar   *cancel_button,
                   GObject       *object,
                   const gchar   *signal,
                   GCallback      callback,
-                  gpointer       callback_data)
+                  gpointer       callback_data,
+                  GDestroyNotify callback_data_destroy)
 {
   QueryBox  *query_box;
   GtkWidget *hbox = NULL;
@@ -150,9 +148,11 @@ create_query_box (const gchar   *title,
 
                                      NULL);
 
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (query_box->qbox),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
+  gtk_dialog_set_default_response (GTK_DIALOG (query_box->qbox),
+                                   GTK_RESPONSE_OK);
+  gimp_dialog_set_alternative_button_order (GTK_DIALOG (query_box->qbox),
+                                            GTK_RESPONSE_OK,
+                                            GTK_RESPONSE_CANCEL,
                                            -1);
 
   query_box->response_handler =
@@ -177,7 +177,7 @@ create_query_box (const gchar   *title,
       g_signal_connect_closure (object, signal, closure, FALSE);
     }
 
-  if (stock_id)
+  if (icon_name)
     {
       GtkWidget *content_area;
       GtkWidget *image;
@@ -189,8 +189,8 @@ create_query_box (const gchar   *title,
       gtk_box_pack_start (GTK_BOX (content_area), hbox, TRUE, TRUE, 0);
       gtk_widget_show (hbox);
 
-      image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
-      gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+      image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_DIALOG);
+      gtk_widget_set_valign (image, GTK_ALIGN_START);
       gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
       gtk_widget_show (image);
     }
@@ -220,36 +220,38 @@ create_query_box (const gchar   *title,
   if (message)
     {
       label = gtk_label_new (message);
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_label_set_xalign (GTK_LABEL (label), 0.0);
       gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
       gtk_box_pack_start (GTK_BOX (query_box->vbox), label, FALSE, FALSE, 0);
       gtk_widget_show (label);
     }
 
-  query_box->entry         = NULL;
-  query_box->object        = object;
-  query_box->callback      = callback;
-  query_box->callback_data = callback_data;
+  query_box->entry                 = NULL;
+  query_box->object                = object;
+  query_box->callback              = callback;
+  query_box->callback_data         = callback_data;
+  query_box->callback_data_destroy = callback_data_destroy;
 
   return query_box;
 }
 
 /**
  * gimp_query_string_box:
- * @title:     The query box dialog's title.
- * @parent:    The dialog's parent widget.
- * @help_func: The help function to show this dialog's help page.
- * @help_id:   A string identifying this dialog's help page.
- * @message:   A string which will be shown above the dialog's entry widget.
- * @initial:   The initial value.
- * @object:    The object this query box is associated with.
- * @signal:    The object's signal which will cause the query box to be closed.
- * @callback:  The function which will be called when the user selects "OK".
- * @data:      The callback's user data.
+ * @title:        The query box dialog's title.
+ * @parent:       The dialog's parent widget.
+ * @help_func:    (scope async): The help function to show this dialog's help page.
+ * @help_id:      A string identifying this dialog's help page.
+ * @message:      A string which will be shown above the dialog's entry widget.
+ * @initial:      The initial value.
+ * @object:       The object this query box is associated with.
+ * @signal:       The object's signal which will cause the query box to be closed.
+ * @callback:     The function which will be called when the user selects "OK".
+ * @data:         The callback's user data.
+ * @data_destroy: Destroy function for @data.
  *
  * Creates a new #GtkDialog that queries the user for a string value.
  *
- * Returns: A pointer to the new #GtkDialog.
+ * Returns: (transfer full): A pointer to the new #GtkDialog.
  **/
 GtkWidget *
 gimp_query_string_box (const gchar             *title,
@@ -261,18 +263,20 @@ gimp_query_string_box (const gchar             *title,
                        GObject                 *object,
                        const gchar             *signal,
                        GimpQueryStringCallback  callback,
-                       gpointer                 data)
+                       gpointer                 data,
+                       GDestroyNotify           data_destroy)
 {
   QueryBox  *query_box;
   GtkWidget *entry;
 
   query_box = create_query_box (title, parent, help_func, help_id,
                                 G_CALLBACK (string_query_box_response),
-                                GTK_STOCK_DIALOG_QUESTION,
+                                "dialog-question",
                                 message,
-                                GTK_STOCK_OK, GTK_STOCK_CANCEL,
+                                _("_OK"), _("_Cancel"),
                                 object, signal,
-                                G_CALLBACK (callback), data);
+                                G_CALLBACK (callback),
+                                data, data_destroy);
 
   if (! query_box)
     return NULL;
@@ -291,22 +295,23 @@ gimp_query_string_box (const gchar             *title,
 
 /**
  * gimp_query_int_box:
- * @title:     The query box dialog's title.
- * @parent:    The dialog's parent widget.
- * @help_func: The help function to show this dialog's help page.
- * @help_id:   A string identifying this dialog's help page.
- * @message:   A string which will be shown above the dialog's entry widget.
- * @initial:   The initial value.
- * @lower:     The lower boundary of the range of possible values.
- * @upper:     The upper boundray of the range of possible values.
- * @object:    The object this query box is associated with.
- * @signal:    The object's signal which will cause the query box to be closed.
- * @callback:  The function which will be called when the user selects "OK".
- * @data:      The callback's user data.
+ * @title:        The query box dialog's title.
+ * @parent:       The dialog's parent widget.
+ * @help_func:    (scope async): The help function to show this dialog's help page.
+ * @help_id:      A string identifying this dialog's help page.
+ * @message:      A string which will be shown above the dialog's entry widget.
+ * @initial:      The initial value.
+ * @lower:        The lower boundary of the range of possible values.
+ * @upper:        The upper boundray of the range of possible values.
+ * @object:       The object this query box is associated with.
+ * @signal:       The object's signal which will cause the query box to be closed.
+ * @callback:     The function which will be called when the user selects "OK".
+ * @data:         The callback's user data.
+ * @data_destroy: Destroy function for @data.
  *
  * Creates a new #GtkDialog that queries the user for an integer value.
  *
- * Returns: A pointer to the new #GtkDialog.
+ * Returns: (transfer full): A pointer to the new #GtkDialog.
  **/
 GtkWidget *
 gimp_query_int_box (const gchar          *title,
@@ -320,26 +325,28 @@ gimp_query_int_box (const gchar          *title,
                     GObject              *object,
                     const gchar          *signal,
                     GimpQueryIntCallback  callback,
-                    gpointer              data)
+                    gpointer              data,
+                    GDestroyNotify        data_destroy)
 {
-  QueryBox  *query_box;
-  GtkWidget *spinbutton;
-  GtkObject *adjustment;
+  QueryBox      *query_box;
+  GtkWidget     *spinbutton;
+  GtkAdjustment *adjustment;
 
   query_box = create_query_box (title, parent, help_func, help_id,
                                 G_CALLBACK (int_query_box_response),
-                                GTK_STOCK_DIALOG_QUESTION,
+                                "dialog-question",
                                 message,
-                                GTK_STOCK_OK, GTK_STOCK_CANCEL,
+                                _("_OK"), _("_Cancel"),
                                 object, signal,
-                                G_CALLBACK (callback), data);
+                                G_CALLBACK (callback),
+                                data, data_destroy);
 
   if (! query_box)
     return NULL;
 
-  spinbutton = gimp_spin_button_new (&adjustment,
-                                     initial, lower, upper, 1, 10, 0,
-                                     1, 0);
+  adjustment = gtk_adjustment_new (initial, lower, upper, 1, 10, 0);
+  spinbutton = gimp_spin_button_new (adjustment, 1.0, 0);
+  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY (spinbutton), TRUE);
   gtk_box_pack_start (GTK_BOX (query_box->vbox), spinbutton, FALSE, FALSE, 0);
   gtk_widget_grab_focus (spinbutton);
@@ -352,23 +359,24 @@ gimp_query_int_box (const gchar          *title,
 
 /**
  * gimp_query_double_box:
- * @title:     The query box dialog's title.
- * @parent:    The dialog's parent widget.
- * @help_func: The help function to show this dialog's help page.
- * @help_id:   A string identifying this dialog's help page.
- * @message:   A string which will be shown above the dialog's entry widget.
- * @initial:   The initial value.
- * @lower:     The lower boundary of the range of possible values.
- * @upper:     The upper boundray of the range of possible values.
- * @digits:    The number of decimal digits the #GtkSpinButton will provide.
- * @object:    The object this query box is associated with.
- * @signal:    The object's signal which will cause the query box to be closed.
- * @callback:  The function which will be called when the user selects "OK".
- * @data:      The callback's user data.
+ * @title:       The query box dialog's title.
+ * @parent:      The dialog's parent widget.
+ * @help_func:   (scope async): The help function to show this dialog's help page.
+ * @help_id:     A string identifying this dialog's help page.
+ * @message:     A string which will be shown above the dialog's entry widget.
+ * @initial:     The initial value.
+ * @lower:       The lower boundary of the range of possible values.
+ * @upper:       The upper boundray of the range of possible values.
+ * @digits:      The number of decimal digits the #GtkSpinButton will provide.
+ * @object:      The object this query box is associated with.
+ * @signal:      The object's signal which will cause the query box to be closed.
+ * @callback:    The function which will be called when the user selects "OK".
+ * @data:        The callback's user data.
+ * @data_destroy: Destroy function for @data.
  *
  * Creates a new #GtkDialog that queries the user for a double value.
  *
- * Returns: A pointer to the new #GtkDialog.
+ * Returns: (transfer full): A pointer to the new #GtkDialog.
  **/
 GtkWidget *
 gimp_query_double_box (const gchar             *title,
@@ -383,26 +391,28 @@ gimp_query_double_box (const gchar             *title,
                        GObject                 *object,
                        const gchar             *signal,
                        GimpQueryDoubleCallback  callback,
-                       gpointer                 data)
+                       gpointer                 data,
+                       GDestroyNotify           data_destroy)
 {
-  QueryBox  *query_box;
-  GtkWidget *spinbutton;
-  GtkObject *adjustment;
+  QueryBox      *query_box;
+  GtkWidget     *spinbutton;
+  GtkAdjustment *adjustment;
 
   query_box = create_query_box (title, parent, help_func, help_id,
                                 G_CALLBACK (double_query_box_response),
-                                GTK_STOCK_DIALOG_QUESTION,
+                                "dialog-question",
                                 message,
-                                GTK_STOCK_OK, GTK_STOCK_CANCEL,
+                                _("_OK"), _("_Cancel"),
                                 object, signal,
-                                G_CALLBACK (callback), data);
+                                G_CALLBACK (callback),
+                                data, data_destroy);
 
   if (! query_box)
     return NULL;
 
-  spinbutton = gimp_spin_button_new (&adjustment,
-                                     initial, lower, upper, 1, 10, 0,
-                                     1, digits);
+  adjustment = gtk_adjustment_new (initial, lower, upper, 1, 10, 0);
+  spinbutton = gimp_spin_button_new (adjustment, 1.0, 0);
+  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY (spinbutton), TRUE);
   gtk_box_pack_start (GTK_BOX (query_box->vbox), spinbutton, FALSE, FALSE, 0);
   gtk_widget_grab_focus (spinbutton);
@@ -415,30 +425,31 @@ gimp_query_double_box (const gchar             *title,
 
 /**
  * gimp_query_size_box:
- * @title:       The query box dialog's title.
- * @parent:      The dialog's parent widget.
- * @help_func:   The help function to show this dialog's help page.
- * @help_id:     A string identifying this dialog's help page.
- * @message:     A string which will be shown above the dialog's entry widget.
- * @initial:     The initial value.
- * @lower:       The lower boundary of the range of possible values.
- * @upper:       The upper boundray of the range of possible values.
- * @digits:      The number of decimal digits the #GimpSizeEntry provide in
- *               "pixel" mode.
- * @unit:        The unit initially shown by the #GimpUnitMenu.
- * @resolution:  The resolution (in dpi) which will be used for pixel/unit
- *               calculations.
- * @dot_for_dot: %TRUE if the #GimpUnitMenu's initial unit should be "pixels".
- * @object:      The object this query box is associated with.
- * @signal:      The object's signal which will cause the query box
- *               to be closed.
- * @callback:    The function which will be called when the user selects "OK".
- * @data:        The callback's user data.
+ * @title:        The query box dialog's title.
+ * @parent:       The dialog's parent widget.
+ * @help_func:    (scope async): The help function to show this dialog's help page.
+ * @help_id:      A string identifying this dialog's help page.
+ * @message:      A string which will be shown above the dialog's entry widget.
+ * @initial:      The initial value.
+ * @lower:        The lower boundary of the range of possible values.
+ * @upper:        The upper boundray of the range of possible values.
+ * @digits:       The number of decimal digits the #GimpSizeEntry provide in
+ *                "pixel" mode.
+ * @unit:         The unit initially shown by the #GimpUnitMenu.
+ * @resolution:   The resolution (in dpi) which will be used for pixel/unit
+ *                calculations.
+ * @dot_for_dot:  %TRUE if the #GimpUnitMenu's initial unit should be "pixels".
+ * @object:       The object this query box is associated with.
+ * @signal:       The object's signal which will cause the query box
+ *                to be closed.
+ * @callback:     The function which will be called when the user selects "OK".
+ * @data:         The callback's user data.
+ * @data_destroy: Destroy function for @data.
  *
  * Creates a new #GtkDialog that queries the user for a size using a
  * #GimpSizeEntry.
  *
- * Returns: A pointer to the new #GtkDialog.
+ * Returns: (transfer full): A pointer to the new #GtkDialog.
  **/
 GtkWidget *
 gimp_query_size_box (const gchar           *title,
@@ -450,13 +461,14 @@ gimp_query_size_box (const gchar           *title,
                      gdouble                lower,
                      gdouble                upper,
                      gint                   digits,
-                     GimpUnit               unit,
+                     GimpUnit              *unit,
                      gdouble                resolution,
                      gboolean               dot_for_dot,
                      GObject               *object,
                      const gchar           *signal,
                      GimpQuerySizeCallback  callback,
-                     gpointer               data)
+                     gpointer               data,
+                     GDestroyNotify         data_destroy)
 {
   QueryBox  *query_box;
   GtkWidget *sizeentry;
@@ -464,19 +476,20 @@ gimp_query_size_box (const gchar           *title,
 
   query_box = create_query_box (title, parent, help_func, help_id,
                                 G_CALLBACK (size_query_box_response),
-                                GTK_STOCK_DIALOG_QUESTION,
+                                "dialog-question",
                                 message,
-                                GTK_STOCK_OK, GTK_STOCK_CANCEL,
+                                _("_OK"), _("_Cancel"),
                                 object, signal,
-                                G_CALLBACK (callback), data);
+                                G_CALLBACK (callback),
+                                data, data_destroy);
 
   if (! query_box)
     return NULL;
 
-  sizeentry = gimp_size_entry_new (1, unit, "%p", TRUE, FALSE, FALSE, 12,
+  sizeentry = gimp_size_entry_new (1, unit, "%n", TRUE, FALSE, FALSE, 12,
                                    GIMP_SIZE_ENTRY_UPDATE_SIZE);
   if (dot_for_dot)
-    gimp_size_entry_set_unit (GIMP_SIZE_ENTRY (sizeentry), GIMP_UNIT_PIXEL);
+    gimp_size_entry_set_unit (GIMP_SIZE_ENTRY (sizeentry), gimp_unit_pixel ());
   gimp_size_entry_set_resolution (GIMP_SIZE_ENTRY (sizeentry), 0,
                                   resolution, FALSE);
   gimp_size_entry_set_refval_digits (GIMP_SIZE_ENTRY (sizeentry), 0, digits);
@@ -500,9 +513,9 @@ gimp_query_size_box (const gchar           *title,
  * gimp_query_boolean_box:
  * @title:        The query box dialog's title.
  * @parent:       The dialog's parent widget.
- * @help_func:    The help function to show this dialog's help page.
+ * @help_func:    (scope async): The help function to show this dialog's help page.
  * @help_id:      A string identifying this dialog's help page.
- * @stock_id:     A stock_id to specify an icon to appear on the left
+ * @icon_name:    An icon name to specify an icon to appear on the left
  *                on the dialog's message.
  * @message:      A string which will be shown in the query box.
  * @true_button:  The string to be shown in the dialog's left button.
@@ -513,34 +526,37 @@ gimp_query_size_box (const gchar           *title,
  * @callback:     The function which will be called when the user clicks one
  *                of the buttons.
  * @data:         The callback's user data.
+ * @data_destroy: Destroy function for @data.
  *
  * Creates a new #GtkDialog that asks the user to do a boolean decision.
  *
- * Returns: A pointer to the new #GtkDialog.
+ * Returns: (transfer full): A pointer to the new #GtkDialog.
  **/
 GtkWidget *
 gimp_query_boolean_box (const gchar              *title,
                         GtkWidget                *parent,
                         GimpHelpFunc              help_func,
                         const gchar              *help_id,
-                        const gchar              *stock_id,
+                        const gchar              *icon_name,
                         const gchar              *message,
                         const gchar              *true_button,
                         const gchar              *false_button,
                         GObject                  *object,
                         const gchar              *signal,
                         GimpQueryBooleanCallback  callback,
-                        gpointer                  data)
+                        gpointer                  data,
+                        GDestroyNotify            data_destroy)
 {
   QueryBox  *query_box;
 
   query_box = create_query_box (title, parent, help_func, help_id,
                                 G_CALLBACK (boolean_query_box_response),
-                                stock_id,
+                                icon_name,
                                 message,
                                 true_button, false_button,
                                 object, signal,
-                                G_CALLBACK (callback), data);
+                                G_CALLBACK (callback),
+                                data, data_destroy);
 
   if (! query_box)
     return NULL;
@@ -581,6 +597,9 @@ query_box_destroy (QueryBox *query_box)
   if (query_box->qbox)
     gtk_widget_destroy (query_box->qbox);
 
+  if (query_box->callback_data_destroy)
+    query_box->callback_data_destroy (query_box->callback_data);
+
   g_slice_free (QueryBox, query_box);
 }
 
@@ -652,8 +671,8 @@ size_query_box_response (GtkWidget *widget,
                          gint       response_id,
                          QueryBox  *query_box)
 {
-  gdouble  size;
-  GimpUnit unit;
+  gdouble   size;
+  GimpUnit *unit;
 
   query_box_disconnect (query_box);
 
