@@ -26,11 +26,17 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
-#ifdef G_OS_WIN32
-#ifdef _WIN32_WINNT
-#undef _WIN32_WINNT
+#ifdef GDK_WINDOWING_WIN32
+#include <gdk/gdkwin32.h>
 #endif
-#define _WIN32_WINNT 0x0600
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+#ifdef GDK_WINDOWING_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif
+
+#ifdef G_OS_WIN32
 #include <windows.h>
 #include <icm.h>
 #endif
@@ -40,6 +46,9 @@
 #include <ApplicationServices/ApplicationServices.h>
 #include <CoreServices/CoreServices.h>
 #endif
+#ifdef PLATFORM_OSX
+#import <Cocoa/Cocoa.h>
+#endif
 
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpconfig/gimpconfig.h"
@@ -61,6 +70,19 @@
  **/
 
 
+#ifdef GDK_WINDOWING_WAYLAND
+static void     gimp_widget_wayland_window_exported  (GdkWindow   *window,
+                                                      const char  *handle,
+                                                      GBytes     **phandle);
+#endif
+
+static void     gimp_widget_set_handle_on_realize    (GtkWidget    *widget,
+                                                      GBytes      **phandle);
+static gboolean gimp_widget_set_handle_on_mapped     (GtkWidget    *widget,
+                                                      GdkEventAny  *event,
+                                                      GBytes      **phandle);
+
+
 static GtkWidget *
 find_mnemonic_widget (GtkWidget *widget,
                       gint       level)
@@ -695,25 +717,28 @@ gimp_monitor_get_color_profile (GdkMonitor *monitor)
                   }
               }
 
-            filename = g_utf16_to_utf8 (filename_utf16, -1, NULL, NULL, NULL);
+            if (filename_utf16 != NULL)
+              {
+                filename = g_utf16_to_utf8 (filename_utf16, -1, NULL, NULL, NULL);
 
-            GetColorDirectoryW (NULL, NULL, &len);
-            dir_utf16 = g_malloc0 (len);
-            GetColorDirectoryW (NULL, dir_utf16, &len);
+                GetColorDirectoryW (NULL, NULL, &len);
+                dir_utf16 = g_malloc0 (len);
+                GetColorDirectoryW (NULL, dir_utf16, &len);
 
-            dir = g_utf16_to_utf8 (dir_utf16, -1, NULL, NULL, NULL);
+                dir = g_utf16_to_utf8 (dir_utf16, -1, NULL, NULL, NULL);
 
-            fullpath = g_build_filename (dir, filename, NULL);
-            file = g_file_new_for_path (fullpath);
+                fullpath = g_build_filename (dir, filename, NULL);
+                file = g_file_new_for_path (fullpath);
 
-            profile = gimp_color_profile_new_from_file (file, NULL);
-            g_object_unref (file);
+                profile = gimp_color_profile_new_from_file (file, NULL);
+                g_object_unref (file);
 
-            g_free (fullpath);
-            g_free (dir);
-            g_free (dir_utf16);
-            g_free (filename);
-            g_free (filename_utf16);
+                g_free (fullpath);
+                g_free (dir);
+                g_free (dir_utf16);
+                g_free (filename);
+                g_free (filename_utf16);
+              }
           }
       }
   }
@@ -986,20 +1011,19 @@ gimp_widget_get_color_transform (GtkWidget               *widget,
 
       if (gimp_color_config_get_simulation_gamut_check (config))
         {
-          GimpRGB         color;
-          cmsUInt16Number alarmCodes[cmsMAXCHANNELS] = { 0, };
-          guchar          r, g, b;
+          GeglColor       *color;
+          cmsUInt16Number  alarmCodes[cmsMAXCHANNELS] = { 0, };
 
           flags |= GIMP_COLOR_TRANSFORM_FLAGS_GAMUT_CHECK;
 
-          gimp_color_config_get_out_of_gamut_color (config, &color);
-          gimp_rgb_get_uchar (&color, &r, &g, &b);
-
-          alarmCodes[0] = (cmsUInt16Number) r * 256;
-          alarmCodes[1] = (cmsUInt16Number) g * 256;
-          alarmCodes[2] = (cmsUInt16Number) b * 256;
-
+          color = gimp_color_config_get_out_of_gamut_color (config);
+          /* Little-CMS documentation doesn't say which space should the
+           * out-of-gamut color be. Let's just give sRGB data.
+           */
+          gegl_color_get_pixel (color, babl_format ("R'G'B' u16"), alarmCodes);
           cmsSetAlarmCodes (alarmCodes);
+
+          g_object_unref (color);
         }
 
       cache->transform =
@@ -1036,3 +1060,327 @@ gimp_widget_get_color_transform (GtkWidget               *widget,
 
   return NULL;
 }
+
+/**
+ * gimp_widget_get_render_format:
+ * @widget: (nullable): [class@Gtk.Widget] to draw the focus indicator on.
+ * @config: the color management settings.
+ * @softproof: whether the color must also be soft-proofed.
+ *
+ * Gets the Babl format to use as target format when rendering raw data on
+ * @widget. The format model with be "R'G'B'A" and the component types will be
+ * double.
+ *
+ * If @config is set, the color configuration as set by the user will be used,
+ * in particular using any custom monitor profile set in preferences (overriding
+ * system-set profile). If no such custom profile is set, it will use the
+ * profile of the monitor @widget is displayed on and will default to sRGB if
+ * @widget is %NULL.
+ *
+ * Use [func@Gimp.get_color_configuration] to retrieve the user
+ * [class@Gimp.ColorConfig].
+ *
+ * TODO: @softproof is currently unused.
+ *
+ * Since: 3.0
+ **/
+const Babl *
+gimp_widget_get_render_space (GtkWidget       *widget,
+                              GimpColorConfig *config)
+{
+  GimpColorProfile *dest_profile  = NULL;
+  const Babl       *space         = NULL;
+
+  g_return_val_if_fail (widget == NULL || GTK_IS_WIDGET (widget), NULL);
+
+  if (config)
+    _gimp_widget_get_profiles (widget, config, NULL, &dest_profile);
+  else
+    /* When no GimpColorConfig is given, we just return the monitor's color
+     * profile, disregarding any user preferences.
+     */
+    dest_profile = gimp_widget_get_color_profile (gtk_widget_get_toplevel (widget));
+
+  if (dest_profile)
+    {
+      space = gimp_color_profile_get_space (dest_profile,
+                                            GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
+                                            NULL);
+      g_object_unref (dest_profile);
+    }
+
+  return space;
+}
+
+/**
+ * gimp_widget_set_native_handle:
+ * @widget: a #GtkWindow
+ * @handle: (out): pointer to store the native handle as a #GBytes.
+ *
+ * This function is used to store the handle representing @window into
+ * @handle so that it can later be reused to set other windows as
+ * transient to this one (even in other processes, such as plug-ins).
+ *
+ * Depending on the platform, the actual content of @handle can be
+ * various types. Moreover it may be filled asynchronously in a
+ * callback, so you should not assume that @handle is set after running
+ * this function.
+ *
+ * This convenience function is safe to use even before @widget is
+ * visible as it will set the handle once it is mapped.
+ */
+void
+gimp_widget_set_native_handle (GtkWidget  *widget,
+                               GBytes    **handle)
+{
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (handle != NULL && *handle == NULL);
+
+  /* This may seem overly complicated but there is a reason: "map-event" can
+   * only be received by widgets with their own windows (see description of
+   * gtk_widget_set_events()). Instead we could just use "realize" which is
+   * received by every widgets (even by the ones whose window actually belongs
+   * to a parent widget), but this causes process locks on Wayland (see comment
+   * in gimp_widget_set_handle_on_mapped()).
+   * This is why we do this weird case-based dance.
+   */
+  if (gtk_widget_get_has_window (widget))
+    {
+      gtk_widget_add_events (widget, GDK_STRUCTURE_MASK);
+      g_signal_connect (widget, "map-event",
+                        G_CALLBACK (gimp_widget_set_handle_on_mapped),
+                        handle);
+    }
+  else
+    {
+      g_signal_connect (widget, "realize",
+                        G_CALLBACK (gimp_widget_set_handle_on_realize),
+                        handle);
+    }
+
+  if (gtk_widget_get_realized (widget))
+    gimp_widget_set_handle_on_mapped (widget, NULL, handle);
+}
+
+/**
+ * gimp_widget_free_native_handle:
+ * @widget: a #GtkWindow
+ * @window_handle: (out): same pointer previously passed to set_native_handle
+ *
+ * Disposes a widget's native window handle created asynchronously after
+ * a previous call to gimp_widget_set_native_handle.
+ * This disposes what the pointer points to, a *GBytes, if any.
+ * Call this when the widget and the window handle it owns is being disposed.
+ *
+ * This should be called at least once, paired with set_native_handle.
+ * This knows how to free @window_handle, especially that on some platforms,
+ * an asynchronous callback must be canceled else it might call back
+ * with the pointer, after the widget and its private is freed.
+ *
+ * This is safe to call when deferenced @window_handle is NULL,
+ * when the window handle was never actually set,
+ * on Wayland where actual setting is asynchronous.
+ *
+ * !!! The word "handle" has two meanings.
+ * A "window handle" is an ID of a window.
+ * A "handle" also commonly means a pointer to a pointer, in this case **GBytes.
+ * @window_handle is both kinds of handle.
+ */
+void
+gimp_widget_free_native_handle (GtkWidget  *widget,
+                                GBytes    **window_handle)
+{
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  /* window-handle as pointer to pointer must not be NULL. */
+  g_return_if_fail (window_handle != NULL);
+
+  #ifdef GDK_WINDOWING_WAYLAND
+  /* Cancel the asynch callback which has a pointer into widget's private.
+   * Cancel regardless whether callback has already come.
+   */
+  if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ()) &&
+      /* The GdkWindow is likely already destroyed. */
+      gtk_widget_get_window (widget) != NULL)
+    gdk_wayland_window_unexport_handle (gtk_widget_get_window (widget));
+  #endif
+
+  /* On some platforms, window_handle may be NULL when an asynch callback has not come yet.
+   * The dereferenced pointer is the window handle.
+   */
+  if (*window_handle != NULL)
+    {
+      /* On all platforms *window_handle is-a allocated GBytes.
+       * A GBytes is NOT a GObject and there is no way to ensure is-a GBytes.
+       *
+       * g_clear_pointer takes a pointer to pointer and unrefs at the pointer.
+       */
+      g_clear_pointer (window_handle, g_bytes_unref);
+    }
+  /* Else no GBytes was ever allocated. */
+}
+
+/**
+ * gimp_widget_animation_enabled:
+ *
+ * This function attempts to read the user's system preference for
+ * showing animation. It can be used to turn off or hide unnecessary
+ * animations such as the scrolling credits or Easter Egg animations.
+ *
+ * Returns: %TRUE if the user has animations enabled on their system
+ *
+ * Since: 3.0
+ */
+gboolean
+gimp_widget_animation_enabled (void)
+{
+  gboolean animation_enabled = TRUE;
+
+  /* Per Luca Bacci, KDE syncs their animation setting with GTK. See
+   * https://invent.kde.org/plasma/kde-gtk-config/-/blob/v6.0.90/kded/gtkconfig.cpp?ref_type=tags#L232
+   */
+  g_object_get (gtk_settings_get_default (),
+                "gtk-enable-animations", &animation_enabled,
+                NULL);
+
+#ifdef PLATFORM_OSX
+  /* The MacOS setting is TRUE if the user wants animations turned off, so
+   * we invert it to match the format of the other platforms */
+  animation_enabled =
+    ! ([[NSWorkspace sharedWorkspace] accessibilityDisplayShouldReduceMotion]);
+#endif
+#ifdef G_OS_WIN32
+  SystemParametersInfo (SPI_GETCLIENTAREAANIMATION, 0x00, &animation_enabled,
+                        0x00);
+#endif
+
+  return animation_enabled;
+}
+
+
+/* Internal functions */
+
+void
+_gimp_widget_get_profiles (GtkWidget         *widget,
+                           GimpColorConfig   *config,
+                           GimpColorProfile **proof_profile,
+                           GimpColorProfile **dest_profile)
+{
+  g_return_if_fail (dest_profile != NULL && *dest_profile == NULL);
+  g_return_if_fail (proof_profile == NULL || *proof_profile == NULL);
+
+  switch (gimp_color_config_get_mode (config))
+    {
+    case GIMP_COLOR_MANAGEMENT_OFF:
+      return;
+
+    case GIMP_COLOR_MANAGEMENT_SOFTPROOF:
+      if (proof_profile)
+        *proof_profile = gimp_color_config_get_simulation_color_profile (config, NULL);
+
+      /*  fallthru  */
+
+    case GIMP_COLOR_MANAGEMENT_DISPLAY:
+      *dest_profile = get_display_profile (widget, config);
+      break;
+    }
+}
+
+
+/* Private functions */
+
+#ifdef GDK_WINDOWING_WAYLAND
+/* Handler from asynchronous callback from Wayland.
+ * You can't know when it will be called, if ever.
+ *
+ * @phandle is the address of a pointer in some widget's private data.
+ * Before freeing that private data, you must cancel (unexport)
+ * so that this callback is not subsequently invoked.
+ */
+static void
+gimp_widget_wayland_window_exported (GdkWindow   *window,
+                                     const char  *handle,
+                                     GBytes     **phandle)
+{
+  GBytes *wayland_handle;
+
+  wayland_handle = g_bytes_new (handle, strlen (handle) + 1);
+
+  g_bytes_unref (*phandle);
+  *phandle = wayland_handle;
+}
+#endif
+
+static void
+gimp_widget_set_handle_on_realize (GtkWidget  *widget,
+                                   GBytes    **phandle)
+{
+  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+
+  gtk_widget_add_events (toplevel, GDK_STRUCTURE_MASK);
+  g_signal_connect (toplevel, "map-event",
+                    G_CALLBACK (gimp_widget_set_handle_on_mapped),
+                    phandle);
+
+  if (gtk_widget_get_mapped (toplevel))
+    gimp_widget_set_handle_on_mapped (widget, NULL, phandle);
+}
+
+static gboolean
+gimp_widget_set_handle_on_mapped (GtkWidget    *widget,
+                                  GdkEventAny  *event,
+                                  GBytes      **phandle)
+{
+  GdkWindow *surface;
+  GBytes    *handle   = NULL;
+#if defined (GDK_WINDOWING_WIN32) || defined (GDK_WINDOWING_X11)
+  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+
+  widget = toplevel;
+#endif
+
+  g_clear_pointer (phandle, g_bytes_unref);
+
+  surface = gtk_widget_get_window (GTK_WIDGET (widget));
+  g_return_val_if_fail (surface != NULL, FALSE);
+
+#ifdef GDK_WINDOWING_WIN32
+  if (GDK_IS_WIN32_WINDOW (surface))
+    {
+      HANDLE id;
+
+      id = GDK_WINDOW_HWND (gtk_widget_get_window (GTK_WIDGET (widget)));
+      handle = g_bytes_new (&id, sizeof (HANDLE));
+    }
+#endif
+
+#ifdef GDK_WINDOWING_X11
+  if (GDK_IS_X11_WINDOW (surface))
+    {
+      Window id;
+
+      id = GDK_WINDOW_XID (gtk_widget_get_window (GTK_WIDGET (widget)));
+      handle = g_bytes_new (&id, sizeof (Window));
+    }
+#endif
+
+  *phandle = handle;
+
+#ifdef GDK_WINDOWING_WAYLAND
+  if (GDK_IS_WAYLAND_WINDOW (surface))
+    {
+      /* I don't run this on "realize" event because somehow it locks
+       * the whole processus in Wayland. The "map-event" happens
+       * slightly after the window became visible and I didn't
+       * experience any lock.
+       */
+      if (! gdk_wayland_window_export_handle (surface,
+                                              (GdkWaylandWindowExported) gimp_widget_wayland_window_exported,
+                                              phandle, NULL))
+        g_printerr ("%s: gdk_wayland_window_export_handle() failed. "
+                    "It will not be possible to set windows in other processes as transient to this display shell.\n",
+                    G_STRFUNC);
+    }
+#endif
+
+  return FALSE;
+}
