@@ -17,6 +17,16 @@
 package Gimp::CodeGen::pdb;
 
 %arg_types = (
+    size        => { name            => 'SIZE',
+		     gtype           => 'G_TYPE_SIZE',
+		     type            => 'gsize ',
+		     const_type      => 'gsize ',
+		     init_value      => '0',
+		     get_value_func  => '$var = g_value_get_size ($value)',
+		     dup_value_func  => '$var = GIMP_VALUES_GET_INT ($value)',
+		     set_value_func  => 'g_value_set_size ($value, $var)',
+		     take_value_func => 'g_value_set_size ($value, $var)' },
+
     int32       => { name            => 'INT32',
 		     gtype           => 'G_TYPE_INT',
 		     type            => 'gint ',
@@ -27,17 +37,7 @@ package Gimp::CodeGen::pdb;
 		     set_value_func  => 'g_value_set_int ($value, $var)',
 		     take_value_func => 'g_value_set_int ($value, $var)' },
 
-    uchar       => { name            => 'UCHAR',
-		     gtype           => 'G_TYPE_UCHAR',
-		     type            => 'guchar ',
-		     const_type      => 'guchar ',
-		     init_value      => '0',
-		     get_value_func  => '$var = g_value_get_uchar ($value)',
-		     dup_value_func  => '$var = GIMP_VALUES_GET_UCHAR ($value)',
-		     set_value_func  => 'g_value_set_uchar ($value, $var)',
-		     take_value_func => 'g_value_set_uchar ($value, $var)' },
-
-    float       => { name            => 'FLOAT',
+    double      => { name            => 'DOUBLE',
 		     gtype           => 'G_TYPE_DOUBLE',
 		     type            => 'gdouble ',
 		     const_type      => 'gdouble ',
@@ -89,23 +89,23 @@ package Gimp::CodeGen::pdb;
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type gint32)',
 		     out_annotate    => '(element-type gint32) (transfer full)',
-		     get_value_func  => '$var = gimp_value_get_int32_array ($value)',
-		     dup_value_func  => '$var = GIMP_VALUES_DUP_INT32_ARRAY ($value)',
+		     get_value_func  => '$var = gimp_value_get_int32_array ($value, &$var_len)',
+		     dup_value_func  => '$var = GIMP_VALUES_DUP_INT32_ARRAY ($value, $var_len)',
 		     set_value_func  => 'gimp_value_set_int32_array ($value, $var, $var_len)',
 		     take_value_func => 'gimp_value_take_int32_array ($value, $var, $var_len)' },
 
-    floatarray  => { name            => 'FLOATARRAY',
-		     gtype           => 'GIMP_TYPE_FLOAT_ARRAY',
+    doublearray  => { name           => 'DOUBLEARRAY',
+		     gtype           => 'GIMP_TYPE_DOUBLE_ARRAY',
 		     type            => 'gdouble *',
 		     const_type      => 'const gdouble *',
 		     array           => 1,
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type gdouble)',
 		     out_annotate    => '(element-type gdouble) (transfer full)',
-		     get_value_func  => '$var = gimp_value_get_float_array ($value)',
-		     dup_value_func  => '$var = GIMP_VALUES_DUP_FLOAT_ARRAY ($value)',
-		     set_value_func  => 'gimp_value_set_float_array ($value, $var, $var_len)',
-		     take_value_func => 'gimp_value_take_float_array ($value, $var, $var_len)' },
+		     get_value_func  => '$var = gimp_value_get_double_array ($value, &$var_len)',
+		     dup_value_func  => '$var = GIMP_VALUES_DUP_DOUBLE_ARRAY ($value, $var_len)',
+		     set_value_func  => 'gimp_value_set_double_array ($value, $var, $var_len)',
+		     take_value_func => 'gimp_value_take_double_array ($value, $var, $var_len)' },
 
     colorarray  => { name            => 'COLOR_ARRAY',
 		     gtype           => 'GIMP_TYPE_COLOR_ARRAY',
@@ -120,159 +120,160 @@ package Gimp::CodeGen::pdb;
 		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     imagearray  => { name            => 'IMAGEARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpImage **',
 		     const_type      => 'const GimpImage **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpImage)',
-		     out_annotate    => '(element-type GimpImage) (transfer container)',
-		     get_value_func  => '$var = (const GimpImage **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_IMAGE, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_IMAGE, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpImage) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpImage) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     itemarray   => { name            => 'ITEMARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpItem **',
 		     const_type      => 'const GimpItem **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpItem)',
-		     out_annotate    => '(element-type GimpItem) (transfer container)',
-		     get_value_func  => '$var = (const GimpItem **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_ITEM, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_ITEM, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpItem) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpItem) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
+
+    drawablearray  => { name            => 'DRAWABLEARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
+		     type            => 'GimpDrawable **',
+		     const_type      => 'const GimpDrawable **',
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpDrawable) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpDrawable) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     layerarray  => { name            => 'LAYERARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpLayer **',
 		     const_type      => 'const GimpLayer **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpLayer)',
-		     out_annotate    => '(element-type GimpLayer) (transfer container)',
-		     get_value_func  => '$var = (const GimpLayer **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_LAYER, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_LAYER, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpLayer) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpLayer) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     channelarray => { name            => 'CHANNELARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpChannel **',
 		     const_type      => 'const GimpChannel **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpChannel)',
-		     out_annotate    => '(element-type GimpChannel) (transfer container)',
-		     get_value_func  => '$var = (const GimpChannel **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_CHANNEL, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_CHANNEL, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpChannel) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpChannel) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
-    vectorarray => { name            => 'VECTORSARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
-		     type            => 'GimpVectors **',
-		     const_type      => 'const GimpVectors **',
-		     array           => 1,
+    patharray    => { name            => 'PATHARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
+		     type            => 'GimpPath **',
+		     const_type      => 'const GimpPath **',
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpVectors)',
-		     out_annotate    => '(element-type GimpVectors) (transfer container)',
-		     get_value_func  => '$var = (const GimpVectors **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_VECTORS, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_VECTORS, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpPath) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpPath) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
+
+    filterarray => { name           => 'FILTERARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
+		     type            => 'GimpDrawableFilter **',
+		     const_type      => 'const GimpDrawableFilter **',
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpDrawableFilter) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpDrawableFilter) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     resourcearray => { name          => 'RESOURCEARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpResource **',
 		     const_type      => 'const GimpResource **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpResource)',
-		     out_annotate    => '(element-type GimpResource) (transfer container)',
-		     get_value_func  => '$var = (const GimpResource **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_RESOURCE, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_RESOURCE, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpResource) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpResource) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     brusharray  => { name            => 'BRUSHARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpBrush **',
 		     const_type      => 'const GimpBrush **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpBrush)',
-		     out_annotate    => '(element-type GimpBrush) (transfer container)',
-		     get_value_func  => '$var = (const GimpBrush **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_BRUSH, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_BRUSH, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpBrush) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpBrush) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     fontarray   => { name            => 'FONTARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpFont **',
 		     const_type      => 'const GimpFont **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpFont)',
-		     out_annotate    => '(element-type GimpFont) (transfer container)',
-		     get_value_func  => '$var = (const GimpFont **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_FONT, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_FONT, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpFont) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpFont) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     gradientarray => { name          => 'GRADIENTARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpGradient **',
 		     const_type      => 'const GimpGradient **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpGradient)',
-		     out_annotate    => '(element-type GimpGradient) (transfer container)',
-		     get_value_func  => '$var = (const GimpGradient **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_GRADIENT, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_GRADIENT, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpGradient) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpGradient) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     palettearray => { name           => 'PALETTEARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpPalette **',
 		     const_type      => 'const GimpPalette **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpPalette)',
-		     out_annotate    => '(element-type GimpPalette) (transfer container)',
-		     get_value_func  => '$var = (const GimpPalette **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_PALETTE, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_PALETTE, (GObject **) $var, $var_len)' },
+		     in_annotate     => '(element-type GimpPalette) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpPalette) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     patternarray => { name           => 'PATTERNARRAY',
-		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpPattern **',
 		     const_type      => 'const GimpPattern **',
-		     array           => 1,
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpPattern)',
-		     out_annotate    => '(element-type GimpPattern) (transfer container)',
-		     get_value_func  => '$var = (const GimpPattern **) gimp_value_get_object_array ($value)',
-		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
-		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_PATTERN, (GObject **) $var, $var_len)',
-		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_PATTERN, (GObject **) $var, $var_len)' },
-
-    color       => { name            => 'COLOR',
-		     gtype           => 'GIMP_TYPE_RGB',
-		     type            => 'GimpRGB ',
-		     const_type      => 'GimpRGB ',
-		     struct          => 1,
-		     init_value      => '{ 0.0, 0.0, 0.0, 1.0 }',
-		     get_value_func  => 'gimp_value_get_rgb ($value, &$var)',
-		     dup_value_func  => 'GIMP_VALUES_GET_RGB ($value, &$var)',
-		     set_value_func  => 'gimp_value_set_rgb ($value, $var)',
-		     take_value_func => 'gimp_value_set_rgb ($value, &$var)',
-		     headers         => [ qw(<cairo.h> "libgimpcolor/gimpcolor.h") ] },
+		     in_annotate     => '(element-type GimpPattern) (array zero-terminated=1)',
+		     out_annotate    => '(element-type GimpPattern) (array zero-terminated=1) (transfer container)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     geglcolor   => { name            => 'GEGLCOLOR',
 		     gtype           => 'GEGL_TYPE_COLOR',
@@ -283,7 +284,19 @@ package Gimp::CodeGen::pdb;
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = g_value_dup_object (gimp_value_array_index ($value))',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
-		     take_value_func => 'g_value_take_object ($value, $var)' },
+		     take_value_func => 'g_value_take_object ($value, $var)',
+		     headers         => [ qw(<cairo.h> "libgimpcolor/gimpcolor.h") ] },
+
+    format      => { name            => 'FORMAT',
+		     gtype           => 'GIMP_TYPE_BABL_FORMAT',
+		     type            => 'const Babl *',
+		     const_type      => 'const Babl *',
+		     init_value      => 'NULL',
+		     out_annotate    => '(transfer none)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_get_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_set_boxed ($value, $var)' },
 
     display     => { name            => 'DISPLAY',
 		     gtype           => 'GIMP_TYPE_DISPLAY',
@@ -346,6 +359,18 @@ package Gimp::CodeGen::pdb;
 		     take_value_func => 'g_value_set_object ($value, $var)',
 		     headers         => [ qw("text/gimptextlayer.h") ] },
 
+    group_layer => { name            => 'GROUP_LAYER',
+		     gtype           => 'GIMP_TYPE_GROUP_LAYER',
+		     type            => 'GimpGroupLayer *',
+		     const_type      => 'GimpGroupLayer *',
+		     init_value      => 'NULL',
+		     out_annotate    => '(transfer none)',
+		     get_value_func  => '$var = g_value_get_object ($value)',
+		     dup_value_func  => '$var = GIMP_VALUES_GET_GROUP_LAYER ($value)',
+		     set_value_func  => 'g_value_set_object ($value, $var)',
+		     take_value_func => 'g_value_set_object ($value, $var)',
+		     headers         => [ qw("core/gimpgrouplayer.h") ] },
+
     channel     => { name            => 'CHANNEL',
 		     gtype           => 'GIMP_TYPE_CHANNEL',
 		     type            => 'GimpChannel *',
@@ -394,17 +419,29 @@ package Gimp::CodeGen::pdb;
 		     take_value_func => 'g_value_set_object ($value, $var)',
 		     headers         => [ qw("core/gimplayermask.h") ] },
 
-    vectors     => { name            => 'VECTORS',
-		     gtype           => 'GIMP_TYPE_VECTORS',
-		     type            => 'GimpVectors *',
-		     const_type      => 'GimpVectors *',
+    path        => { name            => 'PATH',
+		     gtype           => 'GIMP_TYPE_PATH',
+		     type            => 'GimpPath *',
+		     const_type      => 'GimpPath *',
 		     init_value      => 'NULL',
 		     out_annotate    => '(transfer none)',
 		     get_value_func  => '$var = g_value_get_object ($value)',
-		     dup_value_func  => '$var = GIMP_VALUES_GET_VECTORS ($value)',
+		     dup_value_func  => '$var = GIMP_VALUES_GET_PATH ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
 		     take_value_func => 'g_value_set_object ($value, $var)',
-		     headers         => [ qw("vectors/gimpvectors.h") ] },
+		     headers         => [ qw("vectors/gimppath.h") ] },
+
+    filter      => { name            => 'DRAWABLE_FILTER',
+		     gtype           => 'GIMP_TYPE_DRAWABLE_FILTER',
+		     type            => 'GimpDrawableFilter *',
+		     const_type      => 'GimpDrawableFilter *',
+		     init_value      => 'NULL',
+		     out_annotate    => '(transfer none)',
+		     get_value_func  => '$var = g_value_get_object ($value)',
+		     dup_value_func  => '$var = GIMP_VALUES_GET_DRAWABLE_FILTER ($value)',
+		     set_value_func  => 'g_value_set_object ($value, $var)',
+		     take_value_func => 'g_value_set_object ($value, $var)',
+		     headers         => [ qw("core/gimpdrawablefilter.h") ] },
 
     file        => { name            => 'FILE',
 		     gtype           => 'G_TYPE_FILE',
@@ -491,16 +528,30 @@ package Gimp::CodeGen::pdb;
 		     set_value_func  => 'g_value_set_uint ($value, $var)',
 		     take_value_func => 'g_value_set_uint ($value, $var)' },
 
+    export_options => { name         => 'EXPORT_OPTIONS',
+		     gtype           => 'GIMP_TYPE_EXPORT_OPTIONS',
+		     type            => 'GimpExportOptions *',
+		     const_type      => 'GimpExportOptions *',
+		     init_value      => 'NULL',
+		     in_annotate     => '(nullable)',
+		     out_annotate    => '(transfer full)',
+		     get_value_func  => '$var = g_value_get_object ($value)',
+		     dup_value_func  => '$var = g_value_dup_object (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_object ($value, $var)',
+		     take_value_func => 'g_value_set_object ($value, $var)',
+		     headers         => [ qw("libgimpbase/gimpbase.h") ] },
+
     unit        => { name            => 'UNIT',
 		     gtype           => 'GIMP_TYPE_UNIT',
-		     type            => 'GimpUnit ',
-		     const_type      => 'GimpUnit ',
-		     init_value      => 'GIMP_UNIT_PIXEL',
+		     type            => 'GimpUnit *',
+		     const_type      => 'GimpUnit *',
+		     init_value      => 'NULL',
 		     out_annotate    => '(transfer none)',
-		     get_value_func  => '$var = g_value_get_int ($value)',
-		     dup_value_func  => '$var = GIMP_VALUES_GET_INT ($value)',
-		     set_value_func  => 'g_value_set_int ($value, $var)',
-		     take_value_func => 'g_value_set_int ($value, $var)' },
+		     get_value_func  => '$var = g_value_get_object ($value)',
+		     dup_value_func  => '$var = GIMP_VALUES_GET_UNIT ($value)',
+		     set_value_func  => 'g_value_set_object ($value, $var)',
+		     take_value_func => 'g_value_set_object ($value, $var)',
+		     headers         => [ qw("libgimpbase/gimpbase.h") ] },
 
     resource    => { name            => 'RESOURCE',
 		     gtype           => 'GIMP_TYPE_RESOURCE',
@@ -573,7 +624,17 @@ package Gimp::CodeGen::pdb;
 		     dup_value_func  => '$var = GIMP_VALUES_GET_FONT ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
 		     take_value_func => 'g_value_set_object ($value, $var)',
-		     headers         => [ qw("text/gimpfont.h") ] }
+		     headers         => [ qw("text/gimpfont.h") ] },
+
+    valuearray  => { name            => 'VALUEARRAY',
+		     gtype           => 'GIMP_TYPE_VALUE_ARRAY',
+		     type            => 'GimpValueArray *',
+		     const_type      => 'const GimpValueArray *',
+		     init_value      => 'NULL',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 );
 
 # Split out the parts of an arg constraint
