@@ -75,7 +75,7 @@ struct _OptimizeClass
 
 
 #define OPTIMIZE_TYPE  (optimize_get_type ())
-#define OPTIMIZE (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), OPTIMIZE_TYPE, Optimize))
+#define OPTIMIZE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), OPTIMIZE_TYPE, Optimize))
 
 GType                   optimize_get_type         (void) G_GNUC_CONST;
 
@@ -86,9 +86,8 @@ static GimpProcedure  * optimize_create_procedure (GimpPlugIn           *plug_in
 static GimpValueArray * optimize_run              (GimpProcedure        *procedure,
                                                    GimpRunMode           run_mode,
                                                    GimpImage            *image,
-                                                   gint                  n_drawables,
                                                    GimpDrawable        **drawables,
-                                                   const GimpValueArray *args,
+                                                   GimpProcedureConfig  *config,
                                                    gpointer              run_data);
 
 static  GimpImage     * do_optimizations          (GimpRunMode  run_mode,
@@ -123,8 +122,6 @@ static  guint             width, height;
 static  gint32            total_frames;
 static  GimpLayer       **layers;
 static  guchar            pixelstep;
-static  guchar           *palette;
-static  gint              ncolors;
 static  operatingMode     opmode;
 
 
@@ -187,7 +184,7 @@ optimize_create_procedure (GimpPlugIn  *plug_in,
                                         "final file size.  If a frame of the"
                                         "animation can use the 'combine' "
                                         "mode, this procedure attempts to "
-                                        "maximize the number of ajdacent "
+                                        "maximize the number of adjacent "
                                         "pixels having the same color, which"
                                         "improves the compression for some "
                                         "image formats such as GIF or MNG.",
@@ -295,11 +292,11 @@ optimize_create_procedure (GimpPlugIn  *plug_in,
                                       "Adam D. Moss <adam@gimp.org>",
                                       "1997-2003");
 
-      GIMP_PROC_VAL_IMAGE (procedure, "result",
-                           "Result",
-                           "Resultimg image",
-                           FALSE,
-                           G_PARAM_READWRITE);
+      gimp_procedure_add_image_return_value (procedure, "result",
+                                             "Result",
+                                             "Resulting image",
+                                             FALSE,
+                                             G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -309,9 +306,8 @@ static GimpValueArray *
 optimize_run (GimpProcedure        *procedure,
               GimpRunMode           run_mode,
               GimpImage            *image,
-              gint                  n_drawables,
               GimpDrawable        **drawables,
-              const GimpValueArray *args,
+              GimpProcedureConfig  *config,
               gpointer              run_data)
 {
   GimpValueArray *return_vals;
@@ -345,7 +341,7 @@ optimize_run (GimpProcedure        *procedure,
   image = do_optimizations (run_mode, image, diff_only);
 
   if (run_mode != GIMP_RUN_NONINTERACTIVE)
-    gimp_displays_flush();
+    gimp_displays_flush ();
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
@@ -536,7 +532,8 @@ do_optimizations (GimpRunMode  run_mode,
 
   width     = gimp_image_get_width (image);
   height    = gimp_image_get_height (image);
-  layers    = gimp_image_get_layers (image, &total_frames);
+  layers    = gimp_image_get_layers (image);
+  total_frames = gimp_core_object_array_get_length ((GObject **) layers);
   imagetype = gimp_image_get_base_type (image);
   pixelstep = (imagetype == GIMP_RGB) ? 4 : 2;
 
@@ -560,10 +557,7 @@ do_optimizations (GimpRunMode  run_mode,
   gimp_image_undo_disable (new_image);
 
   if (imagetype == GIMP_INDEXED)
-    {
-      palette = gimp_image_get_colormap (image, NULL, &ncolors);
-      gimp_image_set_colormap (new_image, palette, ncolors);
-    }
+    gimp_image_set_palette (new_image, gimp_image_get_palette (image));
 
 #if 1
   if (opmode == OPBACKGROUND ||
@@ -590,7 +584,7 @@ do_optimizations (GimpRunMode  run_mode,
 
       for (this_frame_num=0; this_frame_num<total_frames; this_frame_num++)
         {
-          these_rows[this_frame_num] = g_malloc(width * pixelstep);
+          these_rows[this_frame_num] = g_malloc (width * pixelstep);
 
           red[this_frame_num]   = g_new (guchar, width);
           green[this_frame_num] = g_new (guchar, width);
@@ -601,7 +595,7 @@ do_optimizations (GimpRunMode  run_mode,
 
       for (row = 0; row < height; row++)
         {
-          memset(num_colors, 0, width * sizeof(guint));
+          memset (num_colors, 0, width * sizeof (guint));
 
           for (this_frame_num=0; this_frame_num<total_frames; this_frame_num++)
             {
@@ -702,9 +696,9 @@ do_optimizations (GimpRunMode  run_mode,
                 (best_count == 0) ? 0 : 255;
 
               if (best_count == 0)
-                g_warning("yayyyy!");
+                g_warning ("yayyyy!");
             }
-          /*      memcpy(&back_frame[width * pixelstep * row],
+          /*      memcpy (&back_frame[width * pixelstep * row],
                   these_rows[0],
                   width * pixelstep);*/
         }
@@ -763,7 +757,7 @@ do_optimizations (GimpRunMode  run_mode,
             GIMP_DRAWABLE (layers[total_frames-(this_frame_num+1)]);
 
           /* Image has been closed/etc since we got the layer list? */
-          /* FIXME - How do we tell if a gimp_drawable_get() fails? */
+          /* FIXME - How do we tell if a gimp_drawable_get () fails? */
           if (gimp_drawable_get_width (drawable) == 0)
             {
               gimp_quit ();
@@ -1048,7 +1042,7 @@ do_optimizations (GimpRunMode  run_mode,
               /*
                * Collapse opti_frame data down such that the data
                *  which occupies the bounding box sits at the start
-               *  of the data (for convenience with ..set_rect()).
+               *  of the data (for convenience with ..set_rect ()).
                */
               destptr = opti_frame;
               /*
@@ -1095,23 +1089,23 @@ do_optimizations (GimpRunMode  run_mode,
           oldlayer_name =
             gimp_item_get_name (GIMP_ITEM (layers[total_frames-(this_frame_num+1)]));
 
-          buflen = strlen(oldlayer_name) + 40;
+          buflen = strlen (oldlayer_name) + 40;
 
-          newlayer_name = g_malloc(buflen);
+          newlayer_name = g_malloc (buflen);
 
-          remove_disposal_tag(newlayer_name, oldlayer_name);
-          g_free(oldlayer_name);
+          remove_disposal_tag (newlayer_name, oldlayer_name);
+          g_free (oldlayer_name);
 
-          oldlayer_name = g_malloc(buflen);
+          oldlayer_name = g_malloc (buflen);
 
-          remove_ms_tag(oldlayer_name, newlayer_name);
+          remove_ms_tag (oldlayer_name, newlayer_name);
 
-          g_snprintf(newlayer_name, buflen, "%s(%dms)%s",
-                     oldlayer_name, this_delay,
-                     (this_frame_num ==  0) ? "" :
-                     can_combine ? "(combine)" : "(replace)");
+          g_snprintf (newlayer_name, buflen, "%s(%dms)%s",
+                      oldlayer_name, this_delay,
+                      (this_frame_num ==  0) ? "" :
+                      can_combine ? "(combine)" : "(replace)");
 
-          g_free(oldlayer_name);
+          g_free (oldlayer_name);
 
           /* Empty frame! */
           if (bbox_right <= bbox_left ||
@@ -1142,6 +1136,7 @@ do_optimizations (GimpRunMode  run_mode,
               gimp_item_set_name (GIMP_ITEM (last_true_frame), newlayer_name);
 
               g_free (newlayer_name);
+              g_free (oldlayer_name);
             }
           else
             {
@@ -1250,7 +1245,7 @@ is_ms_tag (const gchar *str,
   gint offset;
   gint length;
 
-  length = strlen(str);
+  length = strlen (str);
 
   if (str[0] != '(')
     return FALSE;
@@ -1352,7 +1347,7 @@ parse_disposal_tag (const gchar *str)
   gint        i, dummy;
   gint        length;
 
-  length = strlen(str);
+  length = strlen (str);
 
   for (i=0; i<length; i++)
     {
@@ -1375,13 +1370,13 @@ remove_disposal_tag (gchar *dest,
   int         taglength;
   DisposeType dummy;
 
-  length = strlen(src);
+  length = strlen (src);
 
-  strcpy(dest, src);
+  strcpy (dest, src);
 
   while (offset<=length)
     {
-      if (is_disposal_tag(&src[offset], &dummy, &taglength))
+      if (is_disposal_tag (&src[offset], &dummy, &taglength))
         {
           offset += taglength;
         }
@@ -1403,13 +1398,13 @@ remove_ms_tag (gchar *dest,
   gint taglength;
   gint dummy;
 
-  length = strlen(src);
+  length = strlen (src);
 
-  strcpy(dest, src);
+  strcpy (dest, src);
 
   while (offset<=length)
     {
-      if (is_ms_tag(&src[offset], &dummy, &taglength))
+      if (is_ms_tag (&src[offset], &dummy, &taglength))
         {
           offset += taglength;
         }
