@@ -78,7 +78,7 @@ struct _GimpPlay
   GtkWidget      *step_button;
   GtkWidget      *rewind_button;
 
-  GtkBuilder     *builder;
+  GMenu          *menu;
 };
 
 
@@ -97,9 +97,8 @@ static GimpProcedure  * play_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * play_run              (GimpProcedure        *procedure,
                                                GimpRunMode           run_mode,
                                                GimpImage            *image,
-                                               gint                  n_drawables,
                                                GimpDrawable        **drawables,
-                                               const GimpValueArray *args,
+                                               GimpProcedureConfig  *config,
                                                gpointer              run_data);
 
 static void        initialize                (GimpPlay        *play);
@@ -291,7 +290,6 @@ gimp_play_class_init (GimpPlayClass *klass)
 static void
 gimp_play_init (GimpPlay *play)
 {
-  play->builder = NULL;
 }
 
 static void
@@ -301,7 +299,7 @@ gimp_play_finalize (GObject *object)
 
   G_OBJECT_CLASS (gimp_play_parent_class)->finalize (object);
 
-  g_clear_object (&play->builder);
+  g_clear_object (&play->menu);
 }
 
 static GList *
@@ -341,6 +339,11 @@ play_create_procedure (GimpPlugIn  *plug_in,
                                       "Adam D. Moss <adam@gimp.org>",
                                       "Adam D. Moss <adam@gimp.org>",
                                       "1997, 1998...");
+
+      gimp_procedure_add_bytes_aux_argument (procedure, "settings-data",
+                                             "Settings data",
+                                             "TODO: eventually we must implement proper args for every settings",
+                                             GIMP_PARAM_READWRITE);
     }
 
   return procedure;
@@ -350,13 +353,27 @@ static GimpValueArray *
 play_run (GimpProcedure        *procedure,
           GimpRunMode           run_mode,
           GimpImage            *_image,
-          gint                  n_drawables,
           GimpDrawable        **drawables,
-          const GimpValueArray *args,
+          GimpProcedureConfig  *config,
           gpointer              run_data)
 {
+  GBytes   *settings_bytes = NULL;
+  GMenu    *section;
   GimpPlay *play;
 
+  if (run_mode != GIMP_RUN_INTERACTIVE)
+    {
+      GError *error = NULL;
+
+      g_set_error (&error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Procedure '%s' only works in interactive mode."),
+                   gimp_procedure_get_name (procedure));
+
+      return gimp_procedure_new_return_values (procedure,
+                                               GIMP_PDB_CALLING_ERROR,
+                                               error);
+    }
+
   play = GIMP_PLAY (gimp_procedure_get_plug_in (procedure));
 #if GLIB_CHECK_VERSION(2,74,0)
   play->app = gtk_application_new (NULL, G_APPLICATION_DEFAULT_FLAGS);
@@ -364,64 +381,43 @@ play_run (GimpProcedure        *procedure,
   play->app = gtk_application_new (NULL, G_APPLICATION_FLAGS_NONE);
 #endif
 
-  play->builder = gtk_builder_new_from_string (
-      "<interface>"
-        "<menu id=\"anim-play-popup\">"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Start playback</attribute>"
-              "<attribute name=\"action\">win.play</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Step back to previous frame</attribute>"
-              "<attribute name=\"action\">win.step-back</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Step to next frame</attribute>"
-              "<attribute name=\"action\">win.step</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Rewind the animation</attribute>"
-              "<attribute name=\"action\">win.rewind</attribute>"
-            "</item>"
-          "</section>"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Increase the speed of the animation</attribute>"
-              "<attribute name=\"action\">win.speed-down</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Decrease the speed of the animation</attribute>"
-              "<attribute name=\"action\">win.speed-up</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Reset the speed of the animation</attribute>"
-              "<attribute name=\"action\">win.speed-reset</attribute>"
-            "</item>"
-          "</section>"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Detach the animation from the dialog window</attribute>"
-              "<attribute name=\"action\">win.detach</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Reload the image</attribute>"
-              "<attribute name=\"action\">win.refresh</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Quit</attribute>"
-              "<attribute name=\"action\">win.close</attribute>"
-            "</item>"
-          "</section>"
-        "</menu>"
-      "</interface>",
-      -1);
+  play->menu = g_menu_new ();
+
+  section = g_menu_new ();
+  g_menu_append (section, _("Start playback"), "win.play");
+  g_menu_append (section, _("Step back to previous frame"), "win.step-back");
+  g_menu_append (section, _("Step to next frame"), "win.step");
+  g_menu_append (section, _("Rewind the animation"), "win.rewind");
+  g_menu_append_section (play->menu, NULL, G_MENU_MODEL (section));
+  g_clear_object (&section);
+
+  section = g_menu_new ();
+  g_menu_append (section, _("Increase the speed of the animation"), "win.speed-down");
+  g_menu_append (section, _("Decrease the speed of the animation"), "win.speed-up");
+  g_menu_append (section, _("Reset the speed of the animation"), "win.speed-reset");
+  g_menu_append_section (play->menu, NULL, G_MENU_MODEL (section));
+  g_clear_object (&section);
+
+  section = g_menu_new ();
+  g_menu_append (section, _("Detach the animation from the dialog window"), "win.detach");
+  g_menu_append (section, _("Reload the image"), "win.refresh");
+  g_menu_append (section, _("Quit"), "win.close");
+  g_menu_append_section (play->menu, NULL, G_MENU_MODEL (section));
+  g_clear_object (&section);
 
   gegl_init (NULL, NULL);
 
   image = _image;
 
-  gimp_get_data (PLUG_IN_PROC, &settings);
+  /* Temporary code replacing legacy gimp_[gs]et_data() using an AUX argument.
+   * This doesn't actually fix the "Reset to initial values|factory defaults"
+   * features, but at least makes per-run value storage work.
+   * TODO: eventually we want proper separate arguments as a complete fix.
+   */
+  g_object_get (config, "settings-data", &settings_bytes, NULL);
+  if (settings_bytes != NULL && g_bytes_get_size (settings_bytes) == sizeof (AnimationSettings))
+    settings = *((AnimationSettings *) g_bytes_get_data (settings_bytes, NULL));
+  g_bytes_unref (settings_bytes);
 
   g_signal_connect (play->app, "activate", G_CALLBACK (on_app_activate), play);
 
@@ -429,7 +425,9 @@ play_run (GimpProcedure        *procedure,
 
   g_clear_object (&play->app);
 
-  gimp_set_data (PLUG_IN_PROC, &settings, sizeof (settings));
+  settings_bytes = g_bytes_new (&settings, sizeof (AnimationSettings));
+  g_object_set (config, "settings-data", settings_bytes, NULL);
+  g_bytes_unref (settings_bytes);
 
   if (run_mode != GIMP_RUN_NONINTERACTIVE)
     gimp_displays_flush ();
@@ -446,10 +444,8 @@ popup_menu (GtkWidget      *widget,
             GimpPlay       *play)
 {
   GtkWidget  *menu;
-  GMenuModel *model;
 
-  model = G_MENU_MODEL (gtk_builder_get_object (play->builder, "anim-play-popup"));
-  menu = gtk_menu_new_from_model (model);
+  menu = gtk_menu_new_from_model (G_MENU_MODEL (play->menu));
 
   gtk_menu_attach_to_widget (GTK_MENU (menu), window, NULL);
   gtk_menu_popup_at_pointer (GTK_MENU (menu), (GdkEvent *) event);
@@ -585,11 +581,10 @@ shape_pressed (GtkWidget      *widget,
       p->y = (gint) event->y;
 
       gtk_grab_add (widget);
-      gdk_pointer_grab (gtk_widget_get_window (widget), TRUE,
-                        GDK_BUTTON_RELEASE_MASK |
-                        GDK_BUTTON_MOTION_MASK  |
-                        GDK_POINTER_MOTION_HINT_MASK,
-                        NULL, NULL, 0);
+      gdk_seat_grab (gdk_event_get_seat ((GdkEvent *) event),
+                     gtk_widget_get_window (widget),
+                     GDK_SEAT_CAPABILITY_ALL, TRUE,
+                     NULL, (GdkEvent *) event, NULL, NULL);
       gdk_window_raise (gtk_widget_get_window (widget));
     }
 
@@ -599,9 +594,13 @@ shape_pressed (GtkWidget      *widget,
 static gboolean
 shape_released (GtkWidget *widget)
 {
+  GdkDisplay *display;
+
+  display = gtk_widget_get_display (widget);
+
   gtk_grab_remove (widget);
-  gdk_display_pointer_ungrab (gtk_widget_get_display (widget), 0);
-  gdk_display_flush (gtk_widget_get_display (widget));
+  gdk_seat_ungrab (gdk_display_get_default_seat (display));
+  gdk_display_flush (display);
 
   return FALSE;
 }
@@ -641,12 +640,12 @@ repaint_da (GtkWidget *darea,
             gpointer   data)
 {
   cairo_pattern_t *check;
-  GimpRGB          light = *(gimp_check_custom_color1 ());
-  GimpRGB          dark  = *(gimp_check_custom_color2 ());
+  GeglColor       *color1 = (GeglColor *) gimp_check_custom_color1 ();
+  GeglColor       *color2 = (GeglColor *) gimp_check_custom_color2 ();
 
-  gimp_checks_get_colors (gimp_check_type (), &light, &dark);
+  gimp_checks_get_colors (gimp_check_type (), &color1, &color2);
 
-  check = gimp_cairo_checkerboard_create (cr, 32, &light, &dark);
+  check = gimp_cairo_checkerboard_create (cr, 32, color1, color2);
 
   cairo_set_source (cr, check);
   cairo_paint (cr);
@@ -655,6 +654,9 @@ repaint_da (GtkWidget *darea,
   cairo_set_source_surface (cr, drawing_area_surface, 0, 0);
   cairo_paint (cr);
 
+  g_object_unref (color1);
+  g_object_unref (color2);
+
   return FALSE;
 }
 
@@ -721,7 +723,7 @@ refresh_dialog (gchar *imagename)
                                    MIN (expected_drawing_area_height + 90,
                                         workarea.height - 60));
 
-      gtk_widget_show (window);
+      gtk_widget_set_visible (window, TRUE);
     }
 }
 
@@ -735,6 +737,7 @@ build_dialog (GimpPlay *play,
   GtkWidget *main_vbox;
   GtkWidget *vbox;
   GtkWidget *hbox;
+  GtkWidget *abox;
   GdkVisual *rgba;
   GAction   *action;
   gint       index;
@@ -756,11 +759,11 @@ build_dialog (GimpPlay *play,
                                    ACTIONS, G_N_ELEMENTS (ACTIONS),
                                    play);
 
-  gimp_help_connect (window, gimp_standard_help_func, PLUG_IN_PROC, NULL, NULL);
+  gimp_help_connect (window, NULL, gimp_standard_help_func, PLUG_IN_PROC, NULL, NULL);
 
   main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_container_add (GTK_CONTAINER (window), main_vbox);
-  gtk_widget_show (main_vbox);
+  gtk_widget_set_visible (main_vbox, TRUE);
 
   toolbar = gtk_toolbar_new ();
   play->play_button = add_tool_button (toolbar, "win.play", "media-playback-start",
@@ -781,11 +784,11 @@ build_dialog (GimpPlay *play,
                    NULL, NULL);
 
   gtk_box_pack_start (GTK_BOX (main_vbox), toolbar, FALSE, FALSE, 0);
-  gtk_widget_show (toolbar);
+  gtk_widget_set_visible (toolbar, TRUE);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
   gtk_box_pack_start (GTK_BOX (main_vbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
+  gtk_widget_set_visible (vbox, TRUE);
 
   frame = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_set_hexpand (frame, TRUE);
@@ -793,17 +796,23 @@ build_dialog (GimpPlay *play,
   gtk_container_add (GTK_CONTAINER (vbox), frame);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (frame),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_widget_show (frame);
+  gtk_widget_set_visible (frame, TRUE);
 
   viewport = gtk_viewport_new (NULL, NULL);
   gtk_container_add (GTK_CONTAINER (frame), viewport);
-  gtk_widget_show (viewport);
+  gtk_widget_set_visible (viewport, TRUE);
+
+  abox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+  gtk_widget_set_halign (abox, GTK_ALIGN_CENTER);
+  gtk_widget_set_halign (abox, GTK_ALIGN_CENTER);
+  gtk_container_add (GTK_CONTAINER (viewport), abox);
+  gtk_widget_set_visible (abox, TRUE);
 
   /* Build a drawing area, with a default size same as the image */
   drawing_area = gtk_drawing_area_new ();
   gtk_widget_add_events (drawing_area, GDK_BUTTON_PRESS_MASK);
-  gtk_container_add (GTK_CONTAINER (viewport), drawing_area);
-  gtk_widget_show (drawing_area);
+  gtk_container_add (GTK_CONTAINER (abox), drawing_area);
+  gtk_widget_set_visible (drawing_area, TRUE);
 
   g_signal_connect (drawing_area, "size-allocate",
                     G_CALLBACK(da_size_callback),
@@ -816,18 +825,18 @@ build_dialog (GimpPlay *play,
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
+  gtk_widget_set_visible (hbox, TRUE);
 
   /* Progress bar. */
 
   progress = gtk_progress_bar_new ();
   gtk_box_pack_end (GTK_BOX (hbox), progress, TRUE, TRUE, 0);
-  gtk_widget_show (progress);
+  gtk_widget_set_visible (progress, TRUE);
 
   /* Zoom */
   zoomcombo = gtk_combo_box_text_new_with_entry ();
   gtk_box_pack_end (GTK_BOX (hbox), zoomcombo, FALSE, FALSE, 0);
-  gtk_widget_show (zoomcombo);
+  gtk_widget_set_visible (zoomcombo, TRUE);
   for (index = 0; index < 5; index++)
     {
       /* list is given in "fps" - frames per second */
@@ -851,7 +860,7 @@ build_dialog (GimpPlay *play,
   /* fps combo */
   fpscombo = gtk_combo_box_text_new ();
   gtk_box_pack_end (GTK_BOX (hbox), fpscombo, FALSE, FALSE, 0);
-  gtk_widget_show (fpscombo);
+  gtk_widget_set_visible (fpscombo, TRUE);
 
   for (index = 0; index < 9; index++)
     {
@@ -872,7 +881,7 @@ build_dialog (GimpPlay *play,
   /* Speed Combo */
   speedcombo = gtk_combo_box_text_new ();
   gtk_box_pack_end (GTK_BOX (hbox), speedcombo, FALSE, FALSE, 0);
-  gtk_widget_show (speedcombo);
+  gtk_widget_set_visible (speedcombo, TRUE);
 
   for (index = 0; index < 7; index++)
     {
@@ -916,11 +925,11 @@ build_dialog (GimpPlay *play,
                     play);
 
   gtk_box_pack_end (GTK_BOX (hbox), frame_disposal_combo, FALSE, FALSE, 0);
-  gtk_widget_show (frame_disposal_combo);
+  gtk_widget_set_visible (frame_disposal_combo, TRUE);
 
   gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
   gtk_window_set_default_size (GTK_WINDOW (window), width + 20, height + 90);
-  gtk_widget_show (window);
+  gtk_widget_set_visible (window, TRUE);
 
   /* shape_drawing_area for detached feature. */
   shape_window = gtk_window_new (GTK_WINDOW_POPUP);
@@ -1409,7 +1418,7 @@ detach_change_state (GSimpleAction *action,
       gtk_window_set_screen (GTK_WINDOW (shape_window),
                              gtk_widget_get_screen (drawing_area));
 
-      gtk_widget_show (shape_window);
+      gtk_widget_set_visible (shape_window, TRUE);
 
       if (! gtk_widget_get_realized (drawing_area))
         gtk_widget_realize (drawing_area);
@@ -1606,7 +1615,7 @@ update_scale (gdouble scale)
 
       gdk_window_get_origin (gtk_widget_get_window (shape_window), &x, &y);
       gtk_window_move (GTK_WINDOW (shape_window), x, y);
-      gtk_widget_show (shape_window);
+      gtk_widget_set_visible (shape_window, TRUE);
     }
 }
 
@@ -1784,9 +1793,9 @@ add_tool_button (GtkWidget  *toolbar,
   GtkToolItem *tool_button;
 
   tool_icon = gtk_image_new_from_icon_name (icon, GTK_ICON_SIZE_BUTTON);
-  gtk_widget_show (GTK_WIDGET (tool_icon));
+  gtk_widget_set_visible (GTK_WIDGET (tool_icon), TRUE);
   tool_button = gtk_tool_button_new (tool_icon, label);
-  gtk_widget_show (GTK_WIDGET (tool_button));
+  gtk_widget_set_visible (GTK_WIDGET (tool_button), TRUE);
   gtk_tool_item_set_tooltip_text (tool_button, tooltip);
   gtk_actionable_set_detailed_action_name (GTK_ACTIONABLE (tool_button), action);
 
@@ -1805,5 +1814,5 @@ add_tool_separator (GtkWidget *toolbar,
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
   gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (item), FALSE);
   gtk_tool_item_set_expand (item, expand);
-  gtk_widget_show (GTK_WIDGET (item));
+  gtk_widget_set_visible (GTK_WIDGET (item), TRUE);
 }
