@@ -75,7 +75,7 @@ struct _RetinexClass
 
 
 #define RETINEX_TYPE  (retinex_get_type ())
-#define RETINEX (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RETINEX_TYPE, Retinex))
+#define RETINEX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RETINEX_TYPE, Retinex))
 
 GType                   retinex_get_type         (void) G_GNUC_CONST;
 
@@ -86,9 +86,8 @@ static GimpProcedure  * retinex_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * retinex_run              (GimpProcedure        *procedure,
                                                   GimpRunMode           run_mode,
                                                   GimpImage            *image,
-                                                  gint                  n_drawables,
                                                   GimpDrawable        **drawables,
-                                                  const GimpValueArray *args,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 
 static gboolean retinex_dialog              (GimpProcedure *procedure,
@@ -195,30 +194,33 @@ retinex_create_procedure (GimpPlugIn  *plug_in,
                                       "Fabien Pelisson",
                                       "2003");
 
-      GIMP_PROC_ARG_INT (procedure, "scale",
-                         _("Scal_e"),
-                         _("Biggest scale value"),
-                         MIN_GAUSSIAN_SCALE, MAX_GAUSSIAN_SCALE, 240,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "scale",
+                                       _("Scal_e"),
+                                       _("Biggest scale value"),
+                                       MIN_GAUSSIAN_SCALE, MAX_GAUSSIAN_SCALE, 240,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "nscales",
-                         _("Scale _division"),
-                         _("Number of scales"),
-                         0, MAX_RETINEX_SCALES, 3,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "nscales",
+                                       _("Scale _division"),
+                                       _("Number of scales"),
+                                       0, MAX_RETINEX_SCALES, 3,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "scales-mode",
-                         _("Le_vel"),
-                         _("Retinex distribution through scales "
-                           "{ Uniform (0), Low (1), High (2) }"),
-                         RETINEX_UNIFORM, RETINEX_HIGH, RETINEX_UNIFORM,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "scales-mode",
+                                          _("Le_vel"),
+                                          _("Retinex distribution through scales"),
+                                          gimp_choice_new_with_values ("uniform", filter_uniform,  _("Uniform"), NULL,
+                                                                       "low",     filter_low,      _("Low"),     NULL,
+                                                                       "high",    filter_high,     _("High"),    NULL,
+                                                                       NULL),
+                                          "uniform",
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "cvar",
-                            _("Dy_namic"),
-                            _("Variance value"),
-                            0, 4, 1.2,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "cvar",
+                                          _("Dy_namic"),
+                                          _("Variance value"),
+                                          0, 4, 1.2,
+                                          G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -228,27 +230,19 @@ static GimpValueArray *
 retinex_run (GimpProcedure        *procedure,
              GimpRunMode           run_mode,
              GimpImage            *image,
-             gint                  n_drawables,
              GimpDrawable        **drawables,
-             const GimpValueArray *args,
+             GimpProcedureConfig  *config,
              gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpDrawable        *drawable;
-  gint                 x, y, width, height;
+  GimpDrawable *drawable;
+  gint          x, y, width, height;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
       g_set_error (&error, GIMP_PLUG_IN_ERROR, 0,
                    _("Procedure '%s' only works with one drawable."),
                    PLUG_IN_PROC);
@@ -265,32 +259,14 @@ retinex_run (GimpProcedure        *procedure,
   if (! gimp_drawable_mask_intersect (drawable, &x, &y, &width, &height) ||
       width  < MIN_GAUSSIAN_SCALE ||
       height < MIN_GAUSSIAN_SCALE)
-    {
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             NULL);
 
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_EXECUTION_ERROR,
-                                               NULL);
-    }
-
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-      if (! retinex_dialog (procedure, G_OBJECT (config), drawable))
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-          g_object_unref (config);
-
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
-      break;
-
-    default:
-      break;
-    }
+  if (run_mode == GIMP_RUN_INTERACTIVE && ! retinex_dialog (procedure, G_OBJECT (config), drawable))
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_CANCEL,
+                                             NULL);
 
   if (gimp_drawable_is_rgb (drawable))
     {
@@ -303,17 +279,11 @@ retinex_run (GimpProcedure        *procedure,
     }
   else
     {
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
       return gimp_procedure_new_return_values (procedure,
                                                GIMP_PDB_EXECUTION_ERROR,
                                                NULL);
     }
 
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -327,7 +297,6 @@ retinex_dialog (GimpProcedure *procedure,
   GtkWidget    *preview;
   GtkWidget    *combo;
   GtkWidget    *scale;
-  GtkListStore *store;
   gboolean      run;
 
   gimp_ui_init (PLUG_IN_BINARY);
@@ -336,26 +305,14 @@ retinex_dialog (GimpProcedure *procedure,
                                       GIMP_PROCEDURE_CONFIG (config),
                                       _("Retinex Image Enhancement"));
 
-  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                            GTK_RESPONSE_OK,
-                                            GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dialog));
-
   preview = gimp_zoom_preview_new_from_drawable (drawable);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
                       preview, TRUE, TRUE, 0);
   gtk_widget_set_margin_bottom (preview, 12);
   gtk_widget_show (preview);
 
-  store = gimp_int_store_new (_("Uniform"), filter_uniform,
-                              _("Low"),     filter_low,
-                              _("High"),    filter_high,
-                              NULL);
-  combo = gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
-                                               "scales-mode",
-                                               GIMP_INT_STORE (store));
+  combo = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                            "scales-mode", G_TYPE_NONE);
   gtk_widget_set_margin_bottom (combo, 12);
 
   scale = gimp_procedure_dialog_get_scale_entry (GIMP_PROCEDURE_DIALOG (dialog),
@@ -405,11 +362,11 @@ retinex (GObject      *config,
          GimpDrawable *drawable,
          GimpPreview  *preview)
 {
-  GeglBuffer *src_buffer;
+  GeglBuffer *src_buffer = NULL;
   GeglBuffer *dest_buffer;
-  const Babl *format;
-  guchar     *src  = NULL;
-  guchar     *psrc = NULL;
+  const Babl *format     = NULL;
+  guchar     *src        = NULL;
+  guchar     *psrc       = NULL;
   gint        x, y, width, height;
   gint        size, bytes;
 
@@ -472,7 +429,8 @@ retinex (GObject      *config,
                        format, psrc,
                        GEGL_AUTO_ROWSTRIDE);
 
-      g_object_unref (src_buffer);
+      if (src_buffer)
+        g_object_unref (src_buffer);
       g_object_unref (dest_buffer);
 
       gimp_progress_update (1.0);
@@ -677,11 +635,12 @@ MSRCR (GObject *config,
   gdouble       cvar;
 
   g_object_get (config,
-                "scales-mode", &scales_mode,
-                "scale",       &config_scale,
-                "nscales",     &nscales,
-                "cvar",        &cvar,
+                "scale",   &config_scale,
+                "nscales", &nscales,
+                "cvar",    &cvar,
                 NULL);
+  scales_mode = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                     "scales-mode");
 
   if (!preview_mode)
     {
