@@ -36,7 +36,7 @@
 
 
 #define LOAD_PROC      "file-dicom-load"
-#define SAVE_PROC      "file-dicom-save"
+#define EXPORT_PROC    "file-dicom-export"
 #define PLUG_IN_BINARY "file-dicom"
 #define PLUG_IN_ROLE   "gimp-file-dicom"
 
@@ -77,53 +77,55 @@ struct _DicomClass
 
 
 #define DICOM_TYPE  (dicom_get_type ())
-#define DICOM (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DICOM_TYPE, Dicom))
+#define DICOM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DICOM_TYPE, Dicom))
 
 GType                   dicom_get_type         (void) G_GNUC_CONST;
 
-static GList          * dicom_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * dicom_create_procedure (GimpPlugIn           *plug_in,
-                                                const gchar          *name);
+static GList          * dicom_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * dicom_create_procedure (GimpPlugIn            *plug_in,
+                                                const gchar           *name);
 
-static GimpValueArray * dicom_load             (GimpProcedure        *procedure,
-                                                GimpRunMode           run_mode,
-                                                GFile                *file,
-                                                const GimpValueArray *args,
-                                                gpointer              run_data);
-static GimpValueArray * dicom_save             (GimpProcedure        *procedure,
-                                                GimpRunMode           run_mode,
-                                                GimpImage            *image,
-                                                gint                  n_drawables,
-                                                GimpDrawable        **drawables,
-                                                GFile                *file,
-                                                const GimpValueArray *args,
-                                                gpointer              run_data);
+static GimpValueArray * dicom_load             (GimpProcedure         *procedure,
+                                                GimpRunMode            run_mode,
+                                                GFile                 *file,
+                                                GimpMetadata          *metadata,
+                                                GimpMetadataLoadFlags *flags,
+                                                GimpProcedureConfig   *config,
+                                                gpointer               run_data);
+static GimpValueArray * dicom_export           (GimpProcedure         *procedure,
+                                                GimpRunMode            run_mode,
+                                                GimpImage             *image,
+                                                GFile                 *file,
+                                                GimpExportOptions     *options,
+                                                GimpMetadata          *metadata,
+                                                GimpProcedureConfig   *config,
+                                                gpointer               run_data);
 
-static GimpImage      * load_image             (GFile                *file,
-                                                GError              **error);
-static gboolean         save_image             (GFile                *file,
-                                                GimpImage            *image,
-                                                GimpDrawable         *drawable,
-                                                GError              **error);
-static void             dicom_loader           (guint8               *pix_buf,
-                                                DicomInfo            *info,
-                                                GeglBuffer           *buffer);
-static void             guess_and_set_endian2  (guint16              *buf16,
-                                                gint                  length);
-static void             toggle_endian2         (guint16              *buf16,
-                                                gint                  length);
-static void             add_tag_pointer        (GByteArray           *group_stream,
-                                                gint                  group,
-                                                gint                  element,
-                                                const gchar          *value_rep,
-                                                const guint8         *data,
-                                                gint                  length);
-static GSList         * dicom_add_tags         (FILE                 *dicom,
-                                                GByteArray           *group_stream,
-                                                GSList               *elements);
-static gboolean         write_group_to_file    (FILE                 *dicom,
-                                                gint                  group,
-                                                GByteArray           *group_stream);
+static GimpImage      * load_image             (GFile                 *file,
+                                                GError               **error);
+static gboolean         export_image           (GFile                 *file,
+                                                GimpImage             *image,
+                                                GimpDrawable          *drawable,
+                                                GError               **error);
+static void             dicom_loader           (guint8                *pix_buf,
+                                                DicomInfo             *info,
+                                                GeglBuffer            *buffer);
+static void             guess_and_set_endian2  (guint16               *buf16,
+                                                gint                   length);
+static void             toggle_endian2         (guint16               *buf16,
+                                                gint                   length);
+static void             add_tag_pointer        (GByteArray            *group_stream,
+                                                gint                   group,
+                                                gint                   element,
+                                                const gchar           *value_rep,
+                                                const guint8          *data,
+                                                gint                   length);
+static GSList         * dicom_add_tags         (FILE                  *dicom,
+                                                GByteArray            *group_stream,
+                                                GSList                *elements);
+static gboolean         write_group_to_file    (FILE                  *dicom,
+                                                gint                   group,
+                                                GByteArray            *group_stream);
 
 
 G_DEFINE_TYPE (Dicom, dicom, GIMP_TYPE_PLUG_IN)
@@ -153,7 +155,7 @@ dicom_query_procedures (GimpPlugIn *plug_in)
   GList *list = NULL;
 
   list = g_list_append (list, g_strdup (LOAD_PROC));
-  list = g_list_append (list, g_strdup (SAVE_PROC));
+  list = g_list_append (list, g_strdup (EXPORT_PROC));
 
   return list;
 }
@@ -173,13 +175,13 @@ dicom_create_procedure (GimpPlugIn  *plug_in,
       gimp_procedure_set_menu_label (procedure, _("DICOM image"));
 
       gimp_procedure_set_documentation (procedure,
-                                        "Loads files of the dicom file format",
-                                        "Load a file in the DICOM standard "
-                                        "format. The standard is defined at "
-                                        "http://medical.nema.org/. The plug-in "
-                                        "currently only supports reading "
-                                        "images with uncompressed pixel "
-                                        "sections.",
+                                        _("Loads files of the dicom file format"),
+                                        _("Load a file in the DICOM standard "
+                                          "format. The standard is defined at "
+                                          "http://medical.nema.org/. The plug-in "
+                                          "currently only supports reading "
+                                          "images with uncompressed pixel "
+                                          "sections."),
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Dov Grobgeld",
@@ -193,11 +195,11 @@ dicom_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
                                       "128,string,DICM");
     }
-  else if (! strcmp (name, SAVE_PROC))
+  else if (! strcmp (name, EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           dicom_save, NULL, NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, dicom_export, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB, GRAY");
 
@@ -206,16 +208,16 @@ dicom_create_procedure (GimpPlugIn  *plug_in,
                                        "Medicine image"));
 
       gimp_procedure_set_documentation (procedure,
-                                        "Save file in the DICOM file format",
-                                        "Save an image in the medical "
-                                        "standard DICOM image formats. "
-                                        "The standard is defined at "
-                                        "http://medical.nema.org/. The file "
-                                        "format is defined in section 10 of "
-                                        "the standard. The files are saved "
-                                        "uncompressed and the compulsory DICOM "
-                                        "tags are filled with default dummy "
-                                        "values.",
+                                        _("Save file in the DICOM file format"),
+                                        _("Save an image in the medical "
+                                          "standard DICOM image formats. "
+                                          "The standard is defined at "
+                                          "http://medical.nema.org/. The file "
+                                          "format is defined in section 10 of "
+                                          "the standard. The files are saved "
+                                          "uncompressed and the compulsory DICOM "
+                                          "tags are filled with default dummy "
+                                          "values."),
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Dov Grobgeld",
@@ -226,17 +228,24 @@ dicom_create_procedure (GimpPlugIn  *plug_in,
                                           "image/x-dcm");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "dcm,dicom");
+
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB   |
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY,
+                                              NULL, NULL, NULL);
     }
 
   return procedure;
 }
 
 static GimpValueArray *
-dicom_load (GimpProcedure        *procedure,
-            GimpRunMode           run_mode,
-            GFile                *file,
-            const GimpValueArray *args,
-            gpointer              run_data)
+dicom_load (GimpProcedure         *procedure,
+            GimpRunMode            run_mode,
+            GFile                 *file,
+            GimpMetadata          *metadata,
+            GimpMetadataLoadFlags *flags,
+            GimpProcedureConfig   *config,
+            gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -261,65 +270,39 @@ dicom_load (GimpProcedure        *procedure,
 }
 
 static GimpValueArray *
-dicom_save (GimpProcedure        *procedure,
-            GimpRunMode           run_mode,
-            GimpImage            *image,
-            gint                  n_drawables,
-            GimpDrawable        **drawables,
-            GFile                *file,
-            const GimpValueArray *args,
-            gpointer              run_data)
+dicom_export (GimpProcedure        *procedure,
+              GimpRunMode           run_mode,
+              GimpImage            *image,
+              GFile                *file,
+              GimpExportOptions    *options,
+              GimpMetadata         *metadata,
+              GimpProcedureConfig  *config,
+              gpointer              run_data)
 {
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
-  GimpExportReturn   export = GIMP_EXPORT_CANCEL;
-  GError            *error = NULL;
+  GimpExportReturn   export = GIMP_EXPORT_IGNORE;
+  GList             *drawables;
+  GError            *error  = NULL;
 
   gegl_init (NULL, NULL);
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-    case GIMP_RUN_WITH_LAST_VALS:
-      gimp_ui_init (PLUG_IN_BINARY);
-      export = gimp_export_image (&image, &n_drawables, &drawables, "DICOM",
-                                  GIMP_EXPORT_CAN_HANDLE_RGB |
-                                  GIMP_EXPORT_CAN_HANDLE_GRAY);
+  export = gimp_export_options_get_image (options, &image);
 
-      if (export == GIMP_EXPORT_CANCEL)
-        return gimp_procedure_new_return_values (procedure,
-                                                 GIMP_PDB_CANCEL,
-                                                 NULL);
-      break;
-
-    default:
-      break;
-    }
-
-  if (n_drawables != 1)
-    {
-      g_set_error (&error, G_FILE_ERROR, 0,
-                   _("Dicom format does not support multiple layers."));
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_CALLING_ERROR,
-                                               error);
-    }
+  drawables = gimp_image_list_layers (image);
 
   if (status == GIMP_PDB_SUCCESS)
     {
-      if (! save_image (file, image, drawables[0],
-                        &error))
+      if (! export_image (file, image, drawables->data,
+                          &error))
         {
           status = GIMP_PDB_EXECUTION_ERROR;
         }
     }
 
   if (export == GIMP_EXPORT_EXPORT)
-    {
-      gimp_image_delete (image);
-      g_free (drawables);
-    }
+    gimp_image_delete (image);
 
+  g_list_free (drawables);
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
@@ -1486,15 +1469,15 @@ dicom_ensure_required_elements_present (GSList *elements,
   return elements;
 }
 
-/* save_image() saves an image in the dicom format. The DICOM format
+/* export_image() saves an image in the dicom format. The DICOM format
  * requires a lot of tags to be set. Some of them have real uses, others
  * must just be filled with dummy values.
  */
 static gboolean
-save_image (GFile        *file,
-            GimpImage    *image,
-            GimpDrawable *drawable,
-            GError      **error)
+export_image (GFile        *file,
+              GimpImage    *image,
+              GimpDrawable *drawable,
+              GError      **error)
 {
   FILE          *dicom;
   GimpImageType  drawable_type;
