@@ -48,10 +48,10 @@ struct _FileFormat
   const gchar *load_help;
   const gchar *load_op;
 
-  const gchar *save_proc;
-  const gchar *save_blurb;
-  const gchar *save_help;
-  const gchar *save_op;
+  const gchar *export_proc;
+  const gchar *export_blurb;
+  const gchar *export_help;
+  const gchar *export_op;
 };
 
 
@@ -85,12 +85,11 @@ static GimpValueArray * goat_load             (GimpProcedure         *procedure,
                                                GimpMetadataLoadFlags *flags,
                                                GimpProcedureConfig   *config,
                                                gpointer               run_data);
-static GimpValueArray * goat_save             (GimpProcedure         *procedure,
+static GimpValueArray * goat_export           (GimpProcedure         *procedure,
                                                GimpRunMode            run_mode,
                                                GimpImage             *image,
-                                               gint                   n_drawables,
-                                               GimpDrawable         **drawables,
                                                GFile                 *file,
+                                               GimpExportOptions     *options,
                                                GimpMetadata          *metadata,
                                                GimpProcedureConfig   *config,
                                                gpointer               run_data);
@@ -98,7 +97,7 @@ static GimpValueArray * goat_save             (GimpProcedure         *procedure,
 static GimpImage      * load_image            (GFile                 *file,
                                                const gchar           *gegl_op,
                                                GError               **error);
-static gboolean         save_image            (GFile                 *file,
+static gboolean         export_image          (GFile                 *file,
                                                const gchar           *gegl_op,
                                                GimpImage             *image,
                                                GimpDrawable          *drawable,
@@ -119,12 +118,12 @@ static const FileFormat file_formats[] =
     "hdr",
     "0,string,#?",
 
-    "file-load-rgbe",
+    "file-rgbe-load",
     "Load files in the RGBE file format",
     "This procedure loads images in the RGBE format, using gegl:rgbe-load",
     "gegl:rgbe-load",
 
-    "file-save-rgbe",
+    "file-rgbe-export",
     "Saves files in the RGBE file format",
     "This procedure exports images in the RGBE format, using gegl:rgbe-save",
     "gegl:rgbe-save",
@@ -138,7 +137,7 @@ static const FileFormat file_formats[] =
     /* no EXR loading (implemented in native GIMP plug-in) */
     NULL, NULL, NULL, NULL,
 
-    "file-exr-save",
+    "file-exr-export",
     "Saves files in the OpenEXR file format",
     "This procedure saves images in the OpenEXR format, using gegl:exr-save",
     "gegl:exr-save"
@@ -174,8 +173,8 @@ goat_query_procedures (GimpPlugIn *plug_in)
       if (format->load_proc)
         list = g_list_append (list, g_strdup (format->load_proc));
 
-      if (format->save_proc)
-        list = g_list_append (list, g_strdup (format->save_proc));
+      if (format->export_proc)
+        list = g_list_append (list, g_strdup (format->export_proc));
     }
 
   return list;
@@ -199,7 +198,7 @@ goat_create_procedure (GimpPlugIn  *plug_in,
                                                goat_load,
                                                (gpointer) format, NULL);
 
-          gimp_procedure_set_menu_label (procedure, format->file_type);
+          gimp_procedure_set_menu_label (procedure, _(format->file_type));
 
           gimp_procedure_set_documentation (procedure,
                                             format->load_blurb,
@@ -213,26 +212,33 @@ goat_create_procedure (GimpPlugIn  *plug_in,
           gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
                                           format->magic);
         }
-      else if (! g_strcmp0 (name, format->save_proc))
+      else if (! g_strcmp0 (name, format->export_proc))
         {
-          procedure = gimp_save_procedure_new (plug_in, name,
-                                               GIMP_PDB_PROC_TYPE_PLUGIN,
-                                               FALSE, goat_save,
-                                               (gpointer) format, NULL);
+          procedure = gimp_export_procedure_new (plug_in, name,
+                                                 GIMP_PDB_PROC_TYPE_PLUGIN,
+                                                 FALSE, goat_export,
+                                                 (gpointer) format, NULL);
 
           gimp_procedure_set_image_types (procedure, "*");
 
-          gimp_procedure_set_menu_label (procedure, format->file_type);
+          gimp_procedure_set_menu_label (procedure, _(format->file_type));
 
           gimp_procedure_set_documentation (procedure,
-                                            format->save_blurb,
-                                            format->save_help,
+                                            format->export_blurb,
+                                            format->export_help,
                                             name);
 
           gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                               format->mime_type);
           gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                               format->extensions);
+
+          gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                                  GIMP_EXPORT_CAN_HANDLE_RGB     |
+                                                  GIMP_EXPORT_CAN_HANDLE_GRAY    |
+                                                  GIMP_EXPORT_CAN_HANDLE_INDEXED |
+                                                  GIMP_EXPORT_CAN_HANDLE_ALPHA,
+                                                  NULL, NULL, NULL);
         }
     }
 
@@ -272,67 +278,36 @@ goat_load (GimpProcedure         *procedure,
 }
 
 static GimpValueArray *
-goat_save (GimpProcedure        *procedure,
-           GimpRunMode           run_mode,
-           GimpImage            *image,
-           gint                  n_drawables,
-           GimpDrawable        **drawables,
-           GFile                *file,
-           GimpMetadata         *metadata,
-           GimpProcedureConfig  *config,
-           gpointer              run_data)
+goat_export (GimpProcedure        *procedure,
+             GimpRunMode           run_mode,
+             GimpImage            *image,
+             GFile                *file,
+             GimpExportOptions    *options,
+             GimpMetadata         *metadata,
+             GimpProcedureConfig  *config,
+             gpointer              run_data)
 {
   const FileFormat  *format = run_data;
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
-  GimpExportReturn   export = GIMP_EXPORT_CANCEL;
-  GError            *error = NULL;
+  GimpExportReturn   export = GIMP_EXPORT_IGNORE;
+  GList             *drawables;
+  GError            *error  = NULL;
 
   gegl_init (NULL, NULL);
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-    case GIMP_RUN_WITH_LAST_VALS:
-      gimp_ui_init (PLUG_IN_BINARY);
+  export = gimp_export_options_get_image (options, &image);
+  drawables = gimp_image_list_layers (image);
 
-      export = gimp_export_image (&image, &n_drawables, &drawables, "GEGL",
-                                  GIMP_EXPORT_CAN_HANDLE_RGB     |
-                                  GIMP_EXPORT_CAN_HANDLE_GRAY    |
-                                  GIMP_EXPORT_CAN_HANDLE_INDEXED |
-                                  GIMP_EXPORT_CAN_HANDLE_ALPHA);
-
-      if (export == GIMP_EXPORT_CANCEL)
-        return gimp_procedure_new_return_values (procedure,
-                                                 GIMP_PDB_CANCEL,
-                                                 NULL);
-      break;
-
-    default:
-      break;
-    }
-
-  if (n_drawables != 1)
-    {
-      g_set_error (&error, G_FILE_ERROR, 0,
-                   _("GEGL export plug-in does not support multiple layers."));
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_CALLING_ERROR,
-                                               error);
-    }
-
-  if (! save_image (file, format->save_op, image, drawables[0],
+  if (! export_image (file, format->export_op, image, drawables->data,
                     &error))
     {
       status = GIMP_PDB_EXECUTION_ERROR;
     }
 
   if (export == GIMP_EXPORT_EXPORT)
-    {
-      gimp_image_delete (image);
-      g_free (drawables);
-    }
+    gimp_image_delete (image);
 
+  g_list_free (drawables);
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
@@ -490,11 +465,11 @@ load_image (GFile        *file,
 }
 
 static gboolean
-save_image (GFile         *file,
-            const gchar   *gegl_op,
-            GimpImage     *image,
-            GimpDrawable  *drawable,
-            GError       **error)
+export_image (GFile         *file,
+              const gchar   *gegl_op,
+              GimpImage     *image,
+              GimpDrawable  *drawable,
+              GError       **error)
 {
   GeglNode   *graph;
   GeglNode   *source;
