@@ -33,21 +33,23 @@
 #include <math.h>
 #include <errno.h>
 
+#include "libgimpcolor/gimpcolor-private.h"
+
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
 #include "libgimp/stdplugins-intl.h"
 
 
-#define LOAD_PROC      "file-pnm-load"
-#define PNM_SAVE_PROC  "file-pnm-save"
-#define PBM_SAVE_PROC  "file-pbm-save"
-#define PGM_SAVE_PROC  "file-pgm-save"
-#define PPM_SAVE_PROC  "file-ppm-save"
-#define PAM_SAVE_PROC  "file-pam-save"
-#define PFM_SAVE_PROC  "file-pfm-save"
-#define PLUG_IN_BINARY "file-pnm"
-#define PLUG_IN_ROLE   "gimp-file-pnm"
+#define LOAD_PROC        "file-pnm-load"
+#define PNM_EXPORT_PROC  "file-pnm-export"
+#define PBM_EXPORT_PROC  "file-pbm-export"
+#define PGM_EXPORT_PROC  "file-pgm-export"
+#define PPM_EXPORT_PROC  "file-ppm-export"
+#define PAM_EXPORT_PROC  "file-pam-export"
+#define PFM_EXPORT_PROC  "file-pfm-export"
+#define PLUG_IN_BINARY   "file-pnm"
+#define PLUG_IN_ROLE     "gimp-file-pnm"
 
 
 /* Declare local data types
@@ -67,12 +69,12 @@ typedef struct _PNMScanner PNMScanner;
 typedef struct _PNMInfo    PNMInfo;
 typedef struct _PNMRowInfo PNMRowInfo;
 
-typedef void     (* PNMLoaderFunc)  (PNMScanner    *scanner,
-                                     PNMInfo       *info,
-                                     GeglBuffer    *buffer);
-typedef gboolean (* PNMSaverowFunc) (PNMRowInfo    *info,
-                                     guchar        *data,
-                                     GError       **error);
+typedef void     (* PNMLoaderFunc)    (PNMScanner    *scanner,
+                                       PNMInfo       *info,
+                                       GeglBuffer    *buffer);
+typedef gboolean (* PNMExportrowFunc) (PNMRowInfo    *info,
+                                       guchar        *data,
+                                       GError       **error);
 
 struct _PNMScanner
 {
@@ -91,6 +93,7 @@ struct _PNMInfo
   gboolean      float_format;   /* Whether it is a floating point format */
   gint          maxval;         /* For integer format image files, the max value
                                  * which we need to normalize to */
+  gchar        *tupltype;       /* Pixel color model */
   gfloat        scale_factor;   /* PFM files have a scale factor */
   gint          np;             /* Number of image planes (0 for pbm) */
   gboolean      asciibody;      /* 1 if ascii body, 0 if raw body */
@@ -151,19 +154,18 @@ static GimpValueArray * pnm_load             (GimpProcedure          *procedure,
                                               GimpMetadataLoadFlags  *flags,
                                               GimpProcedureConfig    *config,
                                               gpointer                run_data);
-static GimpValueArray * pnm_save             (GimpProcedure          *procedure,
+static GimpValueArray * pnm_export           (GimpProcedure          *procedure,
                                               GimpRunMode             run_mode,
                                               GimpImage              *image,
-                                              gint                    n_drawables,
-                                              GimpDrawable          **drawables,
                                               GFile                  *file,
+                                              GimpExportOptions      *options,
                                               GimpMetadata           *metadata,
                                               GimpProcedureConfig    *config,
                                               gpointer                run_data);
 
 static GimpImage      * load_image           (GFile                  *file,
                                               GError                **error);
-static gint             save_image           (GFile                  *file,
+static gint             export_image         (GFile                  *file,
                                               GimpImage              *image,
                                               GimpDrawable           *drawable,
                                               FileType                file_type,
@@ -192,7 +194,7 @@ static void             pnm_load_rawpfm      (PNMScanner             *scan,
 
 static void         create_pam_header        (const gchar           **header_string,
                                               PNMRowInfo             *rowinfo,
-                                              PNMSaverowFunc         *saverow,
+                                              PNMExportrowFunc       *saverow,
                                               GimpImageType           drawable_type,
                                               GeglBuffer             *buffer,
                                               const Babl            **format,
@@ -293,12 +295,12 @@ pnm_query_procedures (GimpPlugIn *plug_in)
   GList *list = NULL;
 
   list = g_list_append (list, g_strdup (LOAD_PROC));
-  list = g_list_append (list, g_strdup (PNM_SAVE_PROC));
-  list = g_list_append (list, g_strdup (PBM_SAVE_PROC));
-  list = g_list_append (list, g_strdup (PGM_SAVE_PROC));
-  list = g_list_append (list, g_strdup (PPM_SAVE_PROC));
-  list = g_list_append (list, g_strdup (PAM_SAVE_PROC));
-  list = g_list_append (list, g_strdup (PFM_SAVE_PROC));
+  list = g_list_append (list, g_strdup (PNM_EXPORT_PROC));
+  list = g_list_append (list, g_strdup (PBM_EXPORT_PROC));
+  list = g_list_append (list, g_strdup (PGM_EXPORT_PROC));
+  list = g_list_append (list, g_strdup (PPM_EXPORT_PROC));
+  list = g_list_append (list, g_strdup (PAM_EXPORT_PROC));
+  list = g_list_append (list, g_strdup (PFM_EXPORT_PROC));
 
   return list;
 }
@@ -338,13 +340,13 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
                                       "0,string,P4,0,string,P5,0,string,P6,"
                                       "0,string,P7,0,string,PF,0,string,Pf");
     }
-  else if (! strcmp (name, PNM_SAVE_PROC))
+  else if (! strcmp (name, PNM_EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           FALSE, pnm_save,
-                                           GINT_TO_POINTER (FILE_TYPE_PNM),
-                                           NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, pnm_export,
+                                             GINT_TO_POINTER (FILE_TYPE_PNM),
+                                             NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB, GRAY, INDEXED");
 
@@ -369,19 +371,28 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "pnm");
 
-      GIMP_PROC_ARG_INT (procedure, "raw",
-                         _("Data formatting"),
-                         _("TRUE for raw output, FALSE for ascii output"),
-                         0, 1, 1,
-                         G_PARAM_READWRITE);
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB  |
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY |
+                                              GIMP_EXPORT_CAN_HANDLE_INDEXED,
+                                              NULL, NULL, NULL);
+
+      gimp_procedure_add_choice_argument (procedure, "raw",
+                                          _("_Data formatting"),
+                                          _("Whether to export ASCII or raw output"),
+                                          gimp_choice_new_with_values ("ascii-output", 0, _("ASCII"), NULL,
+                                                                       "raw-output",   1, _("Raw"),   NULL,
+                                                                       NULL),
+                                          "raw-output",
+                                          G_PARAM_READWRITE);
     }
-  else if (! strcmp (name, PBM_SAVE_PROC))
+  else if (! strcmp (name, PBM_EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           FALSE, pnm_save,
-                                           GINT_TO_POINTER (FILE_TYPE_PBM),
-                                           NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, pnm_export,
+                                             GINT_TO_POINTER (FILE_TYPE_PBM),
+                                             NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB, GRAY, INDEXED");
       gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
@@ -405,19 +416,26 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "pbm");
 
-      GIMP_PROC_ARG_INT (procedure, "raw",
-                         _("Data formatting"),
-                         _("TRUE for raw output, FALSE for ascii output"),
-                         0, 1, 1,
-                         G_PARAM_READWRITE);
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_BITMAP,
+                                              NULL, NULL, NULL);
+
+      gimp_procedure_add_choice_argument (procedure, "raw",
+                                          _("_Data formatting"),
+                                          _("Whether to export ASCII or raw output"),
+                                          gimp_choice_new_with_values ("ascii-output", 0, _("ASCII"), NULL,
+                                                                       "raw-output",   1, _("Raw"),   NULL,
+                                                                       NULL),
+                                          "raw-output",
+                                          G_PARAM_READWRITE);
     }
-  else if (! strcmp (name, PGM_SAVE_PROC))
+  else if (! strcmp (name, PGM_EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           FALSE, pnm_save,
-                                           GINT_TO_POINTER (FILE_TYPE_PGM),
-                                           NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, pnm_export,
+                                             GINT_TO_POINTER (FILE_TYPE_PGM),
+                                             NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB, GRAY, INDEXED");
 
@@ -441,19 +459,26 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "pgm");
 
-      GIMP_PROC_ARG_INT (procedure, "raw",
-                         _("Data formatting"),
-                         _("TRUE for raw output, FALSE for ascii output"),
-                         0, 1, 1,
-                         G_PARAM_READWRITE);
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY,
+                                              NULL, NULL, NULL);
+
+      gimp_procedure_add_choice_argument (procedure, "raw",
+                                          _("_Data formatting"),
+                                          _("Whether to export ASCII or raw output"),
+                                          gimp_choice_new_with_values ("ascii-output", 0, _("ASCII"), NULL,
+                                                                       "raw-output",   1, _("Raw"),   NULL,
+                                                                       NULL),
+                                          "raw-output",
+                                          G_PARAM_READWRITE);
     }
-  else if (! strcmp (name, PPM_SAVE_PROC))
+  else if (! strcmp (name, PPM_EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           FALSE, pnm_save,
-                                           GINT_TO_POINTER (FILE_TYPE_PPM),
-                                           NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, pnm_export,
+                                             GINT_TO_POINTER (FILE_TYPE_PPM),
+                                             NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB, GRAY, INDEXED");
 
@@ -477,19 +502,27 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "ppm");
 
-      GIMP_PROC_ARG_INT (procedure, "raw",
-                         "Data formatting",
-                         _("TRUE for raw output, FALSE for ascii output"),
-                         0, 1, 1,
-                         G_PARAM_READWRITE);
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB |
+                                              GIMP_EXPORT_CAN_HANDLE_INDEXED,
+                                              NULL, NULL, NULL);
+
+      gimp_procedure_add_choice_argument (procedure, "raw",
+                                          _("_Data formatting"),
+                                          _("Whether to export ASCII or raw output"),
+                                          gimp_choice_new_with_values ("ascii-output", 0, _("ASCII"), NULL,
+                                                                       "raw-output",   1, _("Raw"),   NULL,
+                                                                       NULL),
+                                          "raw-output",
+                                          G_PARAM_READWRITE);
     }
-  else if (! strcmp (name, PAM_SAVE_PROC))
+  else if (! strcmp (name, PAM_EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           FALSE, pnm_save,
-                                           GINT_TO_POINTER (FILE_TYPE_PAM),
-                                           NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, pnm_export,
+                                             GINT_TO_POINTER (FILE_TYPE_PAM),
+                                             NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB*, GRAY*, INDEXED*");
 
@@ -511,14 +544,21 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
                                           "image/x-portable-arbitrarymap");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "pam");
+
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB   |
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY  |
+                                              GIMP_EXPORT_CAN_HANDLE_ALPHA |
+                                              GIMP_EXPORT_CAN_HANDLE_INDEXED,
+                                              NULL, NULL, NULL);
     }
-  else if (! strcmp (name, PFM_SAVE_PROC))
+  else if (! strcmp (name, PFM_EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           FALSE, pnm_save,
-                                           GINT_TO_POINTER (FILE_TYPE_PFM),
-                                           NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, pnm_export,
+                                             GINT_TO_POINTER (FILE_TYPE_PFM),
+                                             NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB, GRAY, INDEXED");
 
@@ -540,6 +580,12 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
                                           "image/x-portable-floatmap");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "pfm");
+
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB |
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY,
+                                              NULL, NULL, NULL);
+
     }
 
   return procedure;
@@ -577,120 +623,49 @@ pnm_load (GimpProcedure         *procedure,
 }
 
 static GimpValueArray *
-pnm_save (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GimpImage            *image,
-          gint                  n_drawables,
-          GimpDrawable        **drawables,
-          GFile                *file,
-          GimpMetadata         *metadata,
-          GimpProcedureConfig  *config,
-          gpointer              run_data)
+pnm_export (GimpProcedure        *procedure,
+            GimpRunMode           run_mode,
+            GimpImage            *image,
+            GFile                *file,
+            GimpExportOptions    *options,
+            GimpMetadata         *metadata,
+            GimpProcedureConfig  *config,
+            gpointer              run_data)
 {
-  FileType           file_type   = GPOINTER_TO_INT (run_data);
-  GimpPDBStatusType  status      = GIMP_PDB_SUCCESS;
-  GimpExportReturn   export      = GIMP_EXPORT_CANCEL;
-  const gchar       *format_name = NULL;
-  GError            *error       = NULL;
+  FileType           file_type = GPOINTER_TO_INT (run_data);
+  GimpPDBStatusType  status    = GIMP_PDB_SUCCESS;
+  GimpExportReturn   export    = GIMP_EXPORT_IGNORE;
+  GList             *drawables;
+  GError            *error     = NULL;
 
   gegl_init (NULL, NULL);
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-    case GIMP_RUN_WITH_LAST_VALS:
-      gimp_ui_init (PLUG_IN_BINARY);
-
-      switch (file_type)
-        {
-        case FILE_TYPE_PNM:
-          format_name = "PNM";
-          export = gimp_export_image (&image, &n_drawables, &drawables, "PNM",
-                                      GIMP_EXPORT_CAN_HANDLE_RGB  |
-                                      GIMP_EXPORT_CAN_HANDLE_GRAY |
-                                      GIMP_EXPORT_CAN_HANDLE_INDEXED);
-          break;
-
-        case FILE_TYPE_PBM:
-          format_name = "PBM";
-          export = gimp_export_image (&image, &n_drawables, &drawables, "PBM",
-                                      GIMP_EXPORT_CAN_HANDLE_BITMAP);
-          break;
-
-        case FILE_TYPE_PGM:
-          format_name = "PGM";
-          export = gimp_export_image (&image, &n_drawables, &drawables, "PGM",
-                                      GIMP_EXPORT_CAN_HANDLE_GRAY);
-          break;
-
-        case FILE_TYPE_PPM:
-          format_name = "PPM";
-          export = gimp_export_image (&image, &n_drawables, &drawables, "PPM",
-                                      GIMP_EXPORT_CAN_HANDLE_RGB |
-                                      GIMP_EXPORT_CAN_HANDLE_INDEXED);
-          break;
-
-        case FILE_TYPE_PAM:
-          format_name = "PAM";
-          export = gimp_export_image (&image, &n_drawables, &drawables, "PAM",
-                                      GIMP_EXPORT_CAN_HANDLE_RGB   |
-                                      GIMP_EXPORT_CAN_HANDLE_GRAY  |
-                                      GIMP_EXPORT_CAN_HANDLE_ALPHA |
-                                      GIMP_EXPORT_CAN_HANDLE_INDEXED);
-          break;
-
-        case FILE_TYPE_PFM:
-          format_name = "PFM";
-          export = gimp_export_image (&image, &n_drawables, &drawables, "PFM",
-                                      GIMP_EXPORT_CAN_HANDLE_RGB |
-                                      GIMP_EXPORT_CAN_HANDLE_GRAY);
-          break;
-        }
-
-      if (export == GIMP_EXPORT_CANCEL)
-        return gimp_procedure_new_return_values (procedure,
-                                                 GIMP_PDB_CANCEL,
-                                                 NULL);
-      break;
-
-    default:
-      break;
-    }
-
-  if (n_drawables != 1)
-    {
-      g_set_error (&error, G_FILE_ERROR, 0,
-                   _("%s format does not support multiple layers."),
-                   format_name);
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_CALLING_ERROR,
-                                               error);
-    }
-
   if (file_type != FILE_TYPE_PFM &&
       file_type != FILE_TYPE_PAM &&
       run_mode  == GIMP_RUN_INTERACTIVE)
     {
+      gimp_ui_init (PLUG_IN_BINARY);
+
       if (! save_dialog (procedure, G_OBJECT (config), image))
         status = GIMP_PDB_CANCEL;
     }
 
+  export    = gimp_export_options_get_image (options, &image);
+  drawables = gimp_image_list_layers (image);
+
   if (status == GIMP_PDB_SUCCESS)
     {
-      if (! save_image (file, image, drawables[0], file_type, G_OBJECT (config),
-                        &error))
+      if (! export_image (file, image, drawables->data, file_type,
+                          G_OBJECT (config), &error))
         {
           status = GIMP_PDB_EXECUTION_ERROR;
         }
     }
 
   if (export == GIMP_EXPORT_EXPORT)
-    {
-      gimp_image_delete (image);
-      g_free (drawables);
-    }
+    gimp_image_delete (image);
 
+  g_list_free (drawables);
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
@@ -720,6 +695,8 @@ load_image (GFile   *file,
   /* allocate the necessary structures */
   pnminfo = g_new (PNMInfo, 1);
 
+  pnminfo->tupltype = NULL;
+
   scan = NULL;
   /* set error handling */
   if (setjmp (pnminfo->jmpbuf))
@@ -878,6 +855,8 @@ load_image (GFile   *file,
   pnmscanner_destroy (scan);
 
   g_object_unref (buffer);
+  if (pnminfo->tupltype)
+    g_free (pnminfo->tupltype);
   g_free (pnminfo);
   g_object_unref (input);
 
@@ -921,16 +900,22 @@ process_pam_header (PNMScanner *scan,
 
           /* PAM files may have custom tupltypes;
            * however, these are the only 'officially' defined
-           * ones */
+           * ones (plus CMYK since this tupltype exists in the
+           * wild. */
           if (strcmp (buf, "BLACKANDWHITE")       &&
               strcmp (buf, "BLACKANDWHITE_ALPHA") &&
               strcmp (buf, "GRAYSCALE")           &&
               strcmp (buf, "GRAYSCALE_ALPHA")     &&
               strcmp (buf, "RGB")                 &&
-              strcmp (buf, "RGB_ALPHA"))
+              strcmp (buf, "RGB_ALPHA")           &&
+              strcmp (buf, "CMYK"))
             {
               is_unsupported_tupltype = TRUE;
             }
+
+          if (! strcmp (buf, "CMYK"))
+            pnminfo->tupltype = g_strdup ("CMYK");
+
         }
       else if (! strcmp (buf, "ENDHDR"))
         {
@@ -1072,6 +1057,7 @@ pnm_load_raw (PNMScanner *scan,
               GeglBuffer *buffer)
 {
   GInputStream *input;
+  const Babl   *format = NULL;
   gint          bpc;
   guchar       *data, *d;
   gushort      *s;
@@ -1143,9 +1129,13 @@ pnm_load_raw (PNMScanner *scan,
             }
         }
 
+      if (info->tupltype != NULL && ! strcmp (info->tupltype, "CMYK"))
+        format = (bpc == 1) ? babl_format ("CMYK u8") :
+                              babl_format ("CMYK u16");
+
       gegl_buffer_set (buffer,
                        GEGL_RECTANGLE (0, y, info->xres, scanlines), 0,
-                       NULL, data, GEGL_AUTO_ROWSTRIDE);
+                       format, data, GEGL_AUTO_ROWSTRIDE);
 
       gimp_progress_update ((double) y / (double) info->yres);
     }
@@ -1306,15 +1296,15 @@ output_write (GOutputStream *output,
 }
 
 static void
-create_pam_header (const gchar   **header_string,
-                   PNMRowInfo     *rowinfo,
-                   PNMSaverowFunc *saverow,
-                   GimpImageType   drawable_type,
-                   GeglBuffer     *buffer,
-                   const Babl    **format,
-                   gint           *rowbufsize,
-                   gint           *np,
-                   gchar          *comment)
+create_pam_header (const gchar      **header_string,
+                   PNMRowInfo        *rowinfo,
+                   PNMExportrowFunc  *saverow,
+                   GimpImageType      drawable_type,
+                   GeglBuffer        *buffer,
+                   const Babl       **format,
+                   gint              *rowbufsize,
+                   gint              *np,
+                   gchar             *comment)
 {
   gint   maxval   = 255;
   gint   xres     = gegl_buffer_get_width (buffer);
@@ -1591,37 +1581,36 @@ pnmsaverow_ascii_indexed (PNMRowInfo    *ri,
 }
 
 static gboolean
-save_image (GFile         *file,
-            GimpImage     *image,
-            GimpDrawable  *drawable,
-            FileType       file_type,
-            GObject       *config,
-            GError       **error)
+export_image (GFile         *file,
+              GimpImage     *image,
+              GimpDrawable  *drawable,
+              FileType       file_type,
+              GObject       *config,
+              GError       **error)
 {
-  gboolean       status = FALSE;
-  GOutputStream *output = NULL;
-  GeglBuffer    *buffer = NULL;
-  const Babl    *format = NULL;
-  const gchar   *header_string = NULL;
-  GimpImageType  drawable_type;
-  PNMRowInfo     rowinfo;
-  PNMSaverowFunc saverow = NULL;
-  guchar         red[256];
-  guchar         grn[256];
-  guchar         blu[256];
-  guchar         alpha[256]; /* PAM only */
-  gchar          buf[BUFLEN];
-  gint           np = 0;
-  gint           xres, yres;
-  gint           ypos, yend;
-  gint           rowbufsize = 0;
-  gchar         *comment    = NULL;
-  gboolean       config_raw = TRUE;
+  gboolean         status = FALSE;
+  GOutputStream   *output = NULL;
+  GeglBuffer      *buffer = NULL;
+  const Babl      *format = NULL;
+  const gchar     *header_string = NULL;
+  GimpImageType    drawable_type;
+  PNMRowInfo       rowinfo;
+  PNMExportrowFunc saverow = NULL;
+  guchar           red[256];
+  guchar           grn[256];
+  guchar           blu[256];
+  guchar           alpha[256]; /* PAM only */
+  gchar            buf[BUFLEN];
+  gint             np = 0;
+  gint             xres, yres;
+  gint             ypos, yend;
+  gint             rowbufsize = 0;
+  gchar           *comment    = NULL;
+  gboolean         config_raw = TRUE;
 
   if (file_type != FILE_TYPE_PFM && file_type != FILE_TYPE_PAM)
-    g_object_get (config,
-                  "raw", &config_raw,
-                  NULL);
+    config_raw = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                      "raw");
 
   /*  Make sure we're not saving an image with an alpha channel
    *  unless we're exporting a PAM file  */
@@ -1819,7 +1808,7 @@ save_image (GFile         *file,
       guchar *cmap;
       gint    num_colors;
 
-      cmap = gimp_image_get_colormap (image, NULL, &num_colors);
+      cmap = gimp_palette_get_colormap (gimp_image_get_palette (image), babl_format ("R'G'B' u8"), &num_colors, NULL);
 
       if (file_type == FILE_TYPE_PBM)
         {
@@ -2017,18 +2006,14 @@ save_dialog (GimpProcedure *procedure,
 {
   GtkWidget    *dialog;
   GtkWidget    *frame;
-  GtkListStore *store;
   gboolean      run;
 
-  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
-                                           GIMP_PROCEDURE_CONFIG (config),
-                                           image);
+  dialog = gimp_export_procedure_dialog_new (GIMP_EXPORT_PROCEDURE (procedure),
+                                             GIMP_PROCEDURE_CONFIG (config),
+                                             image);
   /*  file save type  */
-  store = gimp_int_store_new (_("_ASCII"), 0,
-                              _("_Raw"),   1,
-                              NULL);
-  frame = gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dialog),
-                                               "raw", GIMP_INT_STORE (store));
+  frame = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                            "raw", GIMP_TYPE_INT_RADIO_FRAME);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 12);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog), NULL);
