@@ -275,7 +275,8 @@ read_image (FILE    *fd,
                           GIMP_INDEXED_IMAGE, 100,
                           gimp_image_get_default_new_layer_mode (image));
 
-  gimp_image_set_colormap (image, mono, 2);
+  gimp_palette_set_colormap (gimp_image_get_palette (image),
+                             babl_format ("R'G'B' u8"), (guint8 *) mono, 6);
 
   gimp_image_insert_layer (image, layer, NULL, 0);
 
