@@ -188,7 +188,7 @@ struct _JigsawClass
 
 
 #define JIGSAW_TYPE  (jigsaw_get_type ())
-#define JIGSAW (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), JIGSAW_TYPE, Jigsaw))
+#define JIGSAW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), JIGSAW_TYPE, Jigsaw))
 
 GType                   jigsaw_get_type         (void) G_GNUC_CONST;
 
@@ -199,9 +199,8 @@ static GimpProcedure  * jigsaw_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * jigsaw_run              (GimpProcedure        *procedure,
                                                  GimpRunMode           run_mode,
                                                  GimpImage            *image,
-                                                 gint                  n_drawables,
                                                  GimpDrawable        **drawables,
-                                                 const GimpValueArray *args,
+                                                 GimpProcedureConfig  *config,
                                                  gpointer              run_data);
 
 static void     jigsaw                          (GObject              *config,
@@ -409,37 +408,39 @@ jigsaw_create_procedure (GimpPlugIn  *plug_in,
                                       "Nigel Wetten",
                                       "May 2000");
 
-      GIMP_PROC_ARG_INT (procedure, "x",
-                         _("_Horizontal"),
-                         _("Number of pieces going across"),
-                         MIN_XTILES, MAX_XTILES, 5,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "x",
+                                       _("_Horizontal"),
+                                       _("Number of pieces going across"),
+                                       MIN_XTILES, MAX_XTILES, 5,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "y",
-                         _("_Vertical"),
-                         _("Number of pieces going down"),
-                         MIN_YTILES, MAX_YTILES, 5,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "y",
+                                       _("_Vertical"),
+                                       _("Number of pieces going down"),
+                                       MIN_YTILES, MAX_YTILES, 5,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "style",
-                         _("Jigsaw Style"),
-                         _("The style/shape of the jigsaw puzzle "
-                           "{ Square (0), Curved (1) }"),
-                         0, 1, BEZIER_1,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "style",
+                                          _("_Jigsaw Style"),
+                                          _("The style/shape of the jigsaw puzzle"),
+                                          gimp_choice_new_with_values ("square", BEZIER_1, _("Square"), NULL,
+                                                                       "curved", BEZIER_2, _("Curved"), NULL,
+                                                                       NULL),
+                                          "square",
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "blend-lines",
-                         _("_Blend width"),
-                         _("Degree of slope of each piece's edge"),
-                         MIN_BLEND_LINES, MAX_BLEND_LINES, 3,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "blend-lines",
+                                       _("_Blend width"),
+                                       _("Degree of slope of each piece's edge"),
+                                       MIN_BLEND_LINES, MAX_BLEND_LINES, 3,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "blend-amount",
-                            _("Hi_ghlight"),
-                            _("The amount of highlighting on the edges "
-                              "of each piece"),
-                            MIN_BLEND_AMOUNT, MAX_BLEND_AMOUNT, 0.5,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "blend-amount",
+                                          _("Hi_ghlight"),
+                                          _("The amount of highlighting on the edges "
+                                            "of each piece"),
+                                          MIN_BLEND_AMOUNT, MAX_BLEND_AMOUNT, 0.5,
+                                          G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -449,26 +450,18 @@ static GimpValueArray *
 jigsaw_run (GimpProcedure        *procedure,
             GimpRunMode           run_mode,
             GimpImage            *image,
-            gint                  n_drawables,
             GimpDrawable        **drawables,
-            const GimpValueArray *args,
+            GimpProcedureConfig  *config,
             gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpDrawable        *drawable;
+  GimpDrawable *drawable;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
       g_set_error (&error, GIMP_PLUG_IN_ERROR, 0,
                    _("Procedure '%s' only works with one drawable."),
                    PLUG_IN_PROC);
@@ -482,23 +475,10 @@ jigsaw_run (GimpProcedure        *procedure,
       drawable = drawables[0];
     }
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-      if (! jigsaw_dialog (procedure, G_OBJECT (config), drawable))
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-          g_object_unref (config);
-
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
-      break;
-
-    default:
-      break;
-    };
+  if (run_mode == GIMP_RUN_INTERACTIVE && ! jigsaw_dialog (procedure, G_OBJECT (config), drawable))
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_CANCEL,
+                                             NULL);
 
   gimp_progress_init (_("Assembling jigsaw"));
 
@@ -507,9 +487,6 @@ jigsaw_run (GimpProcedure        *procedure,
   if (run_mode != GIMP_RUN_NONINTERACTIVE)
     gimp_displays_flush ();
 
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -540,9 +517,11 @@ jigsaw (GObject      *config,
       GBytes *buffer_bytes;
 
       gimp_preview_get_size (preview, &width, &height);
+
       buffer_bytes = gimp_drawable_get_thumbnail_data (drawable,
                                                        width, height,
                                                        &width, &height, &bytes);
+
       buffer = g_bytes_unref_to_data (buffer_bytes, &buffer_size);
     }
   else
@@ -560,7 +539,7 @@ jigsaw (GObject      *config,
       bytes = babl_format_get_bytes_per_pixel (format);
 
       /* setup image buffer */
-      buffer_size = bytes * width * height;
+      buffer_size = (gsize) bytes * width * height;
       buffer = g_new (guchar, buffer_size);
 
       gegl_buffer_get (gegl_buffer, GEGL_RECTANGLE (0, 0, width, height), 1.0,
@@ -582,6 +561,9 @@ jigsaw (GObject      *config,
   if (preview)
     {
       gimp_preview_draw_buffer (preview, buffer, width * bytes);
+
+      gimp_preview_set_bounds (preview, 0, 0, width, height);
+      gimp_preview_set_size (preview, width, height);
     }
   else
     {
@@ -603,10 +585,13 @@ static void
 jigsaw_preview (GtkWidget *widget,
                 GObject   *config)
 {
-  GimpPreview  *preview  = GIMP_PREVIEW (widget);
-  GimpDrawable *drawable = g_object_get_data (config, "drawable");
+  GimpDrawablePreview  *preview;
+  GimpDrawable         *drawable;
 
-  jigsaw (config, drawable, preview);
+  preview = GIMP_DRAWABLE_PREVIEW (widget);
+  drawable = gimp_drawable_preview_get_drawable (preview);
+
+  jigsaw (config, drawable, GIMP_PREVIEW (widget));
 }
 
 static void
@@ -669,8 +654,9 @@ draw_jigsaw (GObject  *config,
                 "y",            &ytiles,
                 "blend-lines",  &blend_lines,
                 "blend-amount", &blend_amount,
-                "style",        &style,
                 NULL);
+  style = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                               "style");
 
   xlines = xtiles - 1;
   ylines = ytiles - 1;
@@ -2483,7 +2469,6 @@ jigsaw_dialog (GimpProcedure *procedure,
   GtkWidget     *preview;
   GtkWidget     *frame;
   GtkWidget     *scale;
-  GtkListStore  *store;
   gboolean       run;
 
   gimp_ui_init (PLUG_IN_BINARY);
@@ -2492,19 +2477,6 @@ jigsaw_dialog (GimpProcedure *procedure,
                                       GIMP_PROCEDURE_CONFIG (config),
                                       _("Jigsaw"));
 
-  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                            GTK_RESPONSE_OK,
-                                            GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dialog));
-
-  preview = gimp_drawable_preview_new_from_drawable (drawable);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      preview, TRUE, TRUE, 0);
-  gtk_widget_set_margin_bottom (preview, 12);
-  gtk_widget_show (preview);
-
   /* xtiles */
   scale = gimp_procedure_dialog_get_scale_entry (GIMP_PROCEDURE_DIALOG (dialog),
                                                  "x", 1.0);
@@ -2515,7 +2487,8 @@ jigsaw_dialog (GimpProcedure *procedure,
   gtk_widget_set_margin_bottom (scale, 12);
 
   gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (dialog),
-                                   "num-tiles-label", _("Number of Tiles"));
+                                   "num-tiles-label", _("Number of Tiles"),
+                                   FALSE, FALSE);
   gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dialog),
                                   "num-tiles-vbox", "x", "y", NULL);
   gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (dialog),
@@ -2532,7 +2505,8 @@ jigsaw_dialog (GimpProcedure *procedure,
   gtk_widget_set_margin_bottom (scale, 12);
 
   gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (dialog),
-                                   "bevel-label", _("Bevel Edges"));
+                                   "bevel-label", _("Bevel Edges"),
+                                   FALSE, FALSE);
   gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dialog),
                                   "bevel-vbox", "blend-lines", "blend-amount",
                                   NULL);
@@ -2541,18 +2515,17 @@ jigsaw_dialog (GimpProcedure *procedure,
                                     "bevel-vbox");
 
   /* frame for primitive radio buttons */
-  store = gimp_int_store_new (_("Square"), BEZIER_1,
-                              _("Curved"), BEZIER_2,
-                              NULL);
-  frame = gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dialog),
-                                               "style", GIMP_INT_STORE (store));
+  frame = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                            "style", GIMP_TYPE_INT_RADIO_FRAME);
   gtk_widget_set_margin_bottom (frame, 12);
 
   gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dialog),
                                   "jigsaw-vbox", "num-tiles-frame",
                                   "bevel-frame", "style", NULL);
 
-  g_object_set_data (config, "drawable", drawable);
+  preview = gimp_procedure_dialog_get_drawable_preview (GIMP_PROCEDURE_DIALOG (dialog),
+                                                        "preview", drawable);
+  gtk_widget_set_margin_bottom (preview, 12);
 
   g_signal_connect (preview, "invalidated",
                     G_CALLBACK (jigsaw_preview),
@@ -2563,7 +2536,7 @@ jigsaw_dialog (GimpProcedure *procedure,
                             preview);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
-                              "jigsaw-vbox", NULL);
+                              "preview", "jigsaw-vbox", NULL);
 
   gtk_widget_show (dialog);
 
