@@ -65,7 +65,7 @@ struct _SparkleClass
 
 
 #define SPARKLE_TYPE  (sparkle_get_type ())
-#define SPARKLE (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SPARKLE_TYPE, Sparkle))
+#define SPARKLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SPARKLE_TYPE, Sparkle))
 
 GType                   sparkle_get_type         (void) G_GNUC_CONST;
 
@@ -76,9 +76,8 @@ static GimpProcedure  * sparkle_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * sparkle_run              (GimpProcedure        *procedure,
                                                   GimpRunMode           run_mode,
                                                   GimpImage            *image,
-                                                  gint                  n_drawables,
                                                   GimpDrawable        **drawables,
-                                                  const GimpValueArray *args,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 
 static gboolean         sparkle_dialog           (GimpProcedure        *procedure,
@@ -176,7 +175,7 @@ sparkle_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("_Sparkle..."));
       gimp_procedure_add_menu_path (procedure,
-                                    "<Image>/Filters/Light and Shadow/Light");
+                                    "<Image>/Filters/Light and Shadow/[Light]");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Turn bright spots into "
@@ -192,87 +191,90 @@ sparkle_create_procedure (GimpPlugIn  *plug_in,
                                       "John Beale",
                                       "Version 1.27, September 2003");
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "lum-threshold",
-                            _("Lu_minosity threshold"),
-                            _("Adjust the luminosity threshold"),
-                            0.0, 0.1, 0.01,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "lum-threshold",
+                                          _("Lu_minosity threshold"),
+                                          _("Adjust the luminosity threshold"),
+                                          0.0, 0.1, 0.01,
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "flare-inten",
-                            _("_Flare intensity"),
-                            _("Adjust the flare intensity"),
-                            0.0, 1.0, 0.5,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "flare-inten",
+                                          _("_Flare intensity"),
+                                          _("Adjust the flare intensity"),
+                                          0.0, 1.0, 0.5,
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "spike-len",
-                         _("Spi_ke length"),
-                         _("Adjust the spike length (in pixels)"),
-                         1, 100, 20,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "spike-len",
+                                       _("Spi_ke length"),
+                                       _("Adjust the spike length (in pixels)"),
+                                       1, 100, 20,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "spike-points",
-                         _("Spike _points"),
-                         _("Adjust the number of spikes"),
-                         1, 16, 4,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "spike-points",
+                                       _("Spike _points"),
+                                       _("Adjust the number of spikes"),
+                                       1, 16, 4,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "spike-angle",
-                         _("Spike angle (-_1: random)"),
-                         _("Adjust the spike angle "
-                           "(-1 causes a random angle to be chosen)"),
-                         -1, 360, 15,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "spike-angle",
+                                       _("Spike angle (-_1: random)"),
+                                       _("Adjust the spike angle "
+                                         "(-1 causes a random angle to be chosen)"),
+                                       -1, 360, 15,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "density",
-                            _("Spike _density"),
-                            _("Adjust the spike density"),
-                            0.0, 1.0, 1.0,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "density",
+                                          _("Spike _density"),
+                                          _("Adjust the spike density"),
+                                          0.0, 1.0, 1.0,
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "transparency",
-                            _("_Transparency"),
-                            _("Adjust the opacity of the spikes"),
-                            0.0, 1.0, 0.0,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "transparency",
+                                          _("_Transparency"),
+                                          _("Adjust the opacity of the spikes"),
+                                          0.0, 1.0, 0.0,
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "random-hue",
-                            _("Random _hue"),
-                            _("Adjust how much the hue should be "
-                              "changed randomly"),
-                            0.0, 1.0, 0.0,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "random-hue",
+                                          _("Random _hue"),
+                                          _("Adjust how much the hue should be "
+                                            "changed randomly"),
+                                          0.0, 1.0, 0.0,
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_DOUBLE (procedure, "random-saturation",
-                            _("R_andom saturation"),
-                            _("Adjust how much the saturation should be "
-                              "changed randomly"),
-                            0.0, 1.0, 0.0,
-                            G_PARAM_READWRITE);
+      gimp_procedure_add_double_argument (procedure, "random-saturation",
+                                          _("R_andom saturation"),
+                                          _("Adjust how much the saturation should be "
+                                            "changed randomly"),
+                                          0.0, 1.0, 0.0,
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "preserve-luminosity",
-                             _("Preserve l_uminosity"),
-                             _("Should the luminosity be preserved?"),
-                             FALSE,
-                             G_PARAM_READWRITE);
+      gimp_procedure_add_boolean_argument (procedure, "preserve-luminosity",
+                                           _("Preserve l_uminosity"),
+                                           _("Should the luminosity be preserved?"),
+                                           FALSE,
+                                           G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "inverse",
-                             _("In_verse"),
-                             _("Should the effect be inversed?"),
-                             FALSE,
-                             G_PARAM_READWRITE);
+      gimp_procedure_add_boolean_argument (procedure, "inverse",
+                                           _("In_verse"),
+                                           _("Should the effect be inversed?"),
+                                           FALSE,
+                                           G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "border",
-                             _("Add _border"),
-                             _("Draw a border of spikes around the image"),
-                             FALSE,
-                             G_PARAM_READWRITE);
+      gimp_procedure_add_boolean_argument (procedure, "border",
+                                           _("Add _border"),
+                                           _("Draw a border of spikes around the image"),
+                                           FALSE,
+                                           G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "color-type",
-                         _("Color type"),
-                         _("Color of sparkles: { NATURAL (0), "
-                           "FOREGROUND (1), BACKGROUND (2) }"),
-                         0, 2, NATURAL,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "color-type",
+                                          _("_Color type"),
+                                          _("Color of sparkles"),
+                                          gimp_choice_new_with_values ("natural-color",    NATURAL,    _("Natural color"),    NULL,
+                                                                       "foreground-color", FOREGROUND, _("Foreground color"), NULL,
+                                                                       "background-color", BACKGROUND, _("Background color"), NULL,
+                                                                       NULL),
+                                          "natural-color",
+                                          G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -282,27 +284,19 @@ static GimpValueArray *
 sparkle_run (GimpProcedure        *procedure,
              GimpRunMode           run_mode,
              GimpImage            *image,
-             gint                  n_drawables,
              GimpDrawable        **drawables,
-             const GimpValueArray *args,
+             GimpProcedureConfig  *config,
              gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpDrawable        *drawable;
-  gint                 x, y, w, h;
+  GimpDrawable *drawable;
+  gint          x, y, w, h;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
       g_set_error (&error, GIMP_PLUG_IN_ERROR, 0,
                    _("Procedure '%s' only works with one drawable."),
                    PLUG_IN_PROC);
@@ -320,32 +314,15 @@ sparkle_run (GimpProcedure        *procedure,
     {
       g_message (_("Region selected for filter is empty"));
 
-      gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-      g_object_unref (config);
-
       return gimp_procedure_new_return_values (procedure,
                                                GIMP_PDB_SUCCESS,
                                                NULL);
     }
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-      if (! sparkle_dialog (procedure, G_OBJECT (config), drawable))
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-          g_object_unref (config);
-
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
-      break;
-
-
-    default:
-      break;
-    }
+  if (run_mode == GIMP_RUN_INTERACTIVE && ! sparkle_dialog (procedure, G_OBJECT (config), drawable))
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_CANCEL,
+                                             NULL);
 
   if (gimp_drawable_is_rgb (drawable) ||
       gimp_drawable_is_gray (drawable))
@@ -359,17 +336,11 @@ sparkle_run (GimpProcedure        *procedure,
     }
   else
     {
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
       return gimp_procedure_new_return_values (procedure,
                                                GIMP_PDB_EXECUTION_ERROR,
                                                NULL);
     }
 
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -378,12 +349,11 @@ sparkle_dialog (GimpProcedure *procedure,
                 GObject       *config,
                 GimpDrawable  *drawable)
 {
-  GtkWidget    *dialog;
-  GtkWidget    *preview;
-  GtkWidget    *hbox;
-  GtkWidget    *scale;
-  GtkListStore *store;
-  gboolean      run;
+  GtkWidget *dialog;
+  GtkWidget *preview;
+  GtkWidget *hbox;
+  GtkWidget *scale;
+  gboolean   run;
 
   gimp_ui_init (PLUG_IN_BINARY);
 
@@ -391,21 +361,10 @@ sparkle_dialog (GimpProcedure *procedure,
                                       GIMP_PROCEDURE_CONFIG (config),
                                       _("Sparkle"));
 
-  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                            GTK_RESPONSE_OK,
-                                            GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dialog));
   gtk_widget_set_size_request (dialog, 430, -1);
   gtk_container_set_border_width (
     GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 12);
 
-  preview = gimp_drawable_preview_new_from_drawable (drawable);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      preview, TRUE, TRUE, 0);
-  gtk_widget_show (preview);
-
   gimp_procedure_dialog_get_scale_entry (GIMP_PROCEDURE_DIALOG (dialog),
                                          "lum-threshold", 1.0);
 
@@ -444,18 +403,15 @@ sparkle_dialog (GimpProcedure *procedure,
 
   gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (dialog),
                                    "sparkle-bool-label",
-                                   _("Additional Options"));
+                                   _("Additional Options"),
+                                   FALSE, FALSE);
   gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (dialog),
                                     "sparkle-bool-frame", "sparkle-bool-label",
                                     FALSE, "sparkle-bool-vbox");
 
   /*  colortype  */
-  store = gimp_int_store_new (_("Natural color"),    NATURAL,
-                              _("Foreground color"), FOREGROUND,
-                              _("Background color"), BACKGROUND,
-                              NULL);
-  gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dialog),
-                                       "color-type", GIMP_INT_STORE (store));
+  gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                    "color-type", GIMP_TYPE_INT_RADIO_FRAME);
 
   hbox = gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dialog),
                                         "sparkle-row", "sparkle-bool-frame",
@@ -466,6 +422,8 @@ sparkle_dialog (GimpProcedure *procedure,
   gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
   gtk_widget_set_margin_bottom (hbox, 12);
 
+  preview = gimp_procedure_dialog_get_drawable_preview (GIMP_PROCEDURE_DIALOG (dialog),
+                                                        "preview", drawable);
   g_object_set_data (config, "drawable", drawable);
 
   g_signal_connect (preview, "invalidated",
@@ -477,6 +435,7 @@ sparkle_dialog (GimpProcedure *procedure,
                             preview);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
+                              "preview",
                               "lum-threshold", "flare-inten", "spike-len",
                               "spike-points", "spike-angle", "density",
                               "transparency", "random-hue",
@@ -1023,7 +982,7 @@ fspike (GObject      *config,
   gdouble        sfac;
   gint           i;
   gboolean       ok;
-  GimpRGB        gimp_color;
+  GeglColor     *gegl_color;
   guchar         pixel[MAX_CHANNELS];
   guchar         chosen_color[MAX_CHANNELS];
   guchar         color[MAX_CHANNELS];
@@ -1038,8 +997,9 @@ fspike (GObject      *config,
                 "random-hue",        &random_hue,
                 "random-saturation", &random_saturation,
                 "inverse",           &inverse,
-                "color-type",        &colortype,
                 NULL);
+  colortype = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                   "color-type");
 
   theta = angle;
 
@@ -1049,18 +1009,19 @@ fspike (GObject      *config,
       break;
 
     case FOREGROUND:
-      gimp_context_get_foreground (&gimp_color);
-      gimp_rgb_get_uchar (&gimp_color, &chosen_color[0], &chosen_color[1],
-                          &chosen_color[2]);
+      gegl_color = gimp_context_get_foreground ();
+      gegl_color_get_pixel (gegl_color, babl_format_with_space ("R'G'B' u8", NULL), chosen_color);
+      g_clear_object (&gegl_color);
       break;
 
     case BACKGROUND:
-      gimp_context_get_background (&gimp_color);
-      gimp_rgb_get_uchar (&gimp_color, &chosen_color[0], &chosen_color[1],
-                          &chosen_color[2]);
+      gegl_color = gimp_context_get_background ();
+      gegl_color_get_pixel (gegl_color, babl_format_with_space ("R'G'B' u8", NULL), chosen_color);
+      g_clear_object (&gegl_color);
       break;
     }
 
+  gegl_color = gegl_color_new ("black");
   /* draw the major spikes */
   for (i = 0; i < spike_pts; i++)
     {
@@ -1091,32 +1052,32 @@ fspike (GObject      *config,
 
       if (random_hue > 0.0 || random_saturation > 0.0)
         {
-          GimpRGB rgb;
-          GimpHSV hsv;
+          gfloat rgb[3];
+          gfloat hsv[3];
 
-          rgb.r = (gdouble) (255 - color[0]) / 255.0;
-          rgb.g = (gdouble) (255 - color[1]) / 255.0;
-          rgb.b = (gdouble) (255 - color[2]) / 255.0;
+          for (gint j = 0; j < 3; j++)
+            rgb[j] = (gdouble) (255 - color[j]) / 255.0;
 
-          gimp_rgb_to_hsv (&rgb, &hsv);
+          gegl_color_set_pixel (gegl_color, babl_format ("R'G'B' float"), rgb);
+          gegl_color_set_pixel (gegl_color, babl_format ("HSV float"), hsv);
 
-          hsv.h += random_hue * g_rand_double_range (gr, -0.5, 0.5);
+          hsv[0] += random_hue * g_rand_double_range (gr, -0.5, 0.5);
 
-          if (hsv.h >= 1.0)
-            hsv.h -= 1.0;
-          else if (hsv.h < 0.0)
-            hsv.h += 1.0;
+          if (hsv[0] >= 1.0)
+            hsv[0] -= 1.0;
+          else if (hsv[0] < 0.0)
+            hsv[0] += 1.0;
 
-          hsv.v += (random_saturation *
+          hsv[2] += (random_saturation *
                     g_rand_double_range (gr, -1.0, 1.0));
 
-          hsv.v = CLAMP (hsv.v, 0.0, 1.0);
+          hsv[2] = CLAMP (hsv[2], 0.0, 1.0);
 
-          gimp_hsv_to_rgb (&hsv, &rgb);
+          gegl_color_set_pixel (gegl_color, babl_format ("HSV float"), hsv);
+          gegl_color_get_pixel (gegl_color, babl_format ("R'G'B' float"), rgb);
 
-          color[0] = 255 - ROUND (rgb.r * 255.0);
-          color[1] = 255 - ROUND (rgb.g * 255.0);
-          color[2] = 255 - ROUND (rgb.b * 255.0);
+          for (gint j = 0; j < 3; j++)
+            color[j] = 255 - ROUND (rgb[j] * 255.0);
         }
 
       dx = 0.2 * cos (theta * G_PI / 180.0);
@@ -1158,4 +1119,5 @@ fspike (GObject      *config,
 
       theta += 360.0 / spike_pts;
     }
+  g_object_unref (gegl_color);
 }
