@@ -408,7 +408,7 @@ gfig_dialog (GimpGfig *gfig)
                      _("Copy"), _("Copy an object"));
   add_toggle_button (toolbar, "app.shape::delete", GFIG_ICON_DELETE_OBJECT,
                      _("Delete"), _("Delete an object"));
-  add_toggle_button (toolbar, "app.shape::select", GFIG_ICON_SELECT_OBJECT,
+  add_toggle_button (toolbar, "app.shape::select", GIMP_ICON_CURSOR,
                      _("Select"), _("Select an object"));
   add_tool_separator (toolbar, FALSE);
   add_tool_button (toolbar, "app.raise", GIMP_ICON_GO_UP,
@@ -506,7 +506,7 @@ gfig_dialog (GimpGfig *gfig)
   /* brush selector in Stroke frame */
   gfig_context->brush_select
     = gimp_brush_chooser_new (_("Brush"), _("Brush"),
-                              GIMP_RESOURCE (gfig_context->default_style.brush));
+                              gfig_context->default_style.brush);
   g_signal_connect (gfig_context->brush_select, "resource-set",
                     G_CALLBACK (gfig_brush_changed_callback), NULL);
   gtk_box_pack_start (GTK_BOX (vbox), gfig_context->brush_select,
@@ -570,7 +570,7 @@ gfig_dialog (GimpGfig *gfig)
   /* A page for the pattern selector */
   gfig_context->pattern_select
     = gimp_pattern_chooser_new (_("Pattern"), _("Pattern"),
-                                GIMP_RESOURCE (gfig_context->default_style.pattern));
+                                gfig_context->default_style.pattern);
   g_signal_connect (gfig_context->pattern_select, "resource-set",
                     G_CALLBACK (gfig_pattern_changed_callback), NULL);
   gtk_widget_show (gfig_context->pattern_select);
@@ -580,7 +580,7 @@ gfig_dialog (GimpGfig *gfig)
   /* A page for the gradient selector */
   gfig_context->gradient_select
     = gimp_gradient_chooser_new (_("Gradient"), _("Gradient"),
-                                 GIMP_RESOURCE (gfig_context->default_style.gradient));
+                                 gfig_context->default_style.gradient);
   g_signal_connect (gfig_context->gradient_select, "resource-set",
                     G_CALLBACK (gfig_gradient_changed_callback), NULL);
   gtk_widget_show (gfig_context->gradient_select);
@@ -630,7 +630,8 @@ gfig_dialog (GimpGfig *gfig)
   gfig_list_load_all (gfig_path);
 
   /* Setup initial brush settings */
-  set_context_bdesc (gimp_context_get_brush ());
+  if (gimp_context_get_brush ())
+    set_context_bdesc (gimp_context_get_brush ());
 
   gtk_widget_show (main_hbox);
 
