@@ -18,7 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,7 +31,8 @@ void d_star_object_class_init (void);
 
 void d_star_start             (GdkPoint  *pnt,
                                gboolean   shift_down);
-void d_star_end               (GdkPoint  *pnt,
+void d_star_end               (GimpGfig  *gfig,
+                               GdkPoint  *pnt,
                                gboolean   shift_down);
 
 
