@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -25,9 +24,6 @@
 
 #include <libgimpmath/gimpmath.h>
 
-#include <gtk/gtklist.h>
-#include <gtk/gtkpreview.h>
-
 #include "gimpressionist.h"
 #include "ppmtool.h"
 #include "brush.h"
@@ -42,11 +38,11 @@ static void  update_brush_preview (const char *fn);
 static GtkWidget    *brush_preview    = NULL;
 static GtkListStore *brush_list_store = NULL;
 
-static GtkWidget *brush_list          = NULL;
-static GtkObject *brush_relief_adjust = NULL;
-static GtkObject *brush_aspect_adjust = NULL;
-static GtkObject *brush_gamma_adjust  = NULL;
-static gboolean   brush_dont_update   = FALSE;
+static GtkWidget     *brush_list          = NULL;
+static GtkWidget     *brush_relief_scale  = NULL;
+static GtkWidget     *brush_aspect_scale  = NULL;
+static GtkAdjustment *brush_gamma_adjust  = NULL;
+static gboolean       brush_dont_update   = FALSE;
 
 static gchar *last_selected_brush = NULL;
 
@@ -58,15 +54,15 @@ void
 brush_restore (void)
 {
   reselect (brush_list, pcvals.selected_brush);
-  gtk_adjustment_set_value (GTK_ADJUSTMENT (brush_gamma_adjust), pcvals.brushgamma);
-  gtk_adjustment_set_value (GTK_ADJUSTMENT (brush_relief_adjust), pcvals.brush_relief);
-  gtk_adjustment_set_value (GTK_ADJUSTMENT (brush_aspect_adjust), pcvals.brush_aspect);
+  gtk_adjustment_set_value (brush_gamma_adjust, pcvals.brushgamma);
+  gimp_label_spin_set_value (GIMP_LABEL_SPIN (brush_relief_scale), pcvals.brush_relief);
+  gimp_label_spin_set_value (GIMP_LABEL_SPIN (brush_aspect_scale), pcvals.brush_aspect);
 }
 
 void
 brush_store (void)
 {
-  pcvals.brushgamma = GTK_ADJUSTMENT (brush_gamma_adjust)->value;
+  pcvals.brushgamma = gtk_adjustment_get_value (brush_gamma_adjust);
 }
 
 void
@@ -96,26 +92,47 @@ set_colorbrushes (const gchar *fn)
   pcvals.color_brushes = file_is_color (fn);
 }
 
+static const Babl *
+get_u8_format (GimpDrawable *drawable)
+{
+  if (gimp_drawable_is_rgb (drawable))
+    {
+      if (gimp_drawable_has_alpha (drawable))
+        return babl_format ("R'G'B'A u8");
+      else
+        return babl_format ("R'G'B' u8");
+    }
+  else
+    {
+      if (gimp_drawable_has_alpha (drawable))
+        return babl_format ("Y'A u8");
+      else
+        return babl_format ("Y' u8");
+    }
+}
+
 static void
 brushdmenuselect (GtkWidget *widget,
                   gpointer   data)
 {
-  GimpPixelRgn  src_rgn;
+  GeglBuffer   *src_buffer;
+  const Babl   *format;
   guchar       *src_row;
   guchar       *src;
-  gint          id;
-  gint          alpha, bpp;
-  gboolean      has_alpha;
+  gint          bpp;
   gint          x, y;
   ppm_t        *p;
-  gint          x1, y1, x2, y2;
+  gint          x1, y1, w, h;
   gint          row;
+  gint32        drawable_id = -1;
   GimpDrawable *drawable;
   gint          rowstride;
 
-  gimp_int_combo_box_get_active (GIMP_INT_COMBO_BOX (widget), &id);
+  gimp_int_combo_box_get_active (GIMP_INT_COMBO_BOX (widget), &drawable_id);
 
-  if (id == -1)
+  drawable = gimp_drawable_get_by_id (drawable_id);
+
+  if (! drawable)
     return;
 
   if (brush_from_file == 2)
@@ -129,53 +146,58 @@ brushdmenuselect (GtkWidget *widget,
       preset_save_button_set_sensitive (FALSE);
     }
 
-  gtk_adjustment_set_value (GTK_ADJUSTMENT (brush_gamma_adjust), 1.0);
-  gtk_adjustment_set_value (GTK_ADJUSTMENT (brush_aspect_adjust), 0.0);
+  gtk_adjustment_set_value (brush_gamma_adjust, 1.0);
+  gimp_label_spin_set_value (GIMP_LABEL_SPIN (brush_aspect_scale), 0.0);
 
-  drawable = gimp_drawable_get (id);
+  if (! gimp_drawable_mask_intersect (drawable, &x1, &y1, &w, &h))
+    return;
 
-  gimp_drawable_mask_bounds (drawable->drawable_id, &x1, &y1, &x2, &y2);
-
-  bpp = gimp_drawable_bpp (drawable->drawable_id);
-  has_alpha = gimp_drawable_has_alpha (drawable->drawable_id);
-  alpha = (has_alpha) ? bpp - 1 : bpp;
+  format = get_u8_format (drawable);
+  bpp    = babl_format_get_bytes_per_pixel (format);
 
   ppm_kill (&brushppm);
-  ppm_new (&brushppm, x2 - x1, y2 - y1);
+  ppm_new (&brushppm, w, h);
   p = &brushppm;
 
   rowstride = p->width * 3;
 
-  src_row = g_new (guchar, (x2 - x1) * bpp);
+  src_row = g_new (guchar, w * bpp);
 
-  gimp_pixel_rgn_init (&src_rgn, drawable,
-                       0, 0, x2 - x1, y2 - y1,
-                       FALSE, FALSE);
+  src_buffer = gimp_drawable_get_buffer (drawable);
 
   if (bpp == 3)
     { /* RGB */
-      int bpr = (x2 - x1) * 3;
+      gint bpr = w * 3;
+      gint y2 = y1 + h;
 
       for (row = 0, y = y1; y < y2; row++, y++)
         {
-          gimp_pixel_rgn_get_row (&src_rgn, src_row, x1, y, (x2 - x1));
+          gegl_buffer_get (src_buffer, GEGL_RECTANGLE (x1, y, w, 1), 1.0,
+                           format, src_row,
+                           GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
+
           memcpy (p->col + row*rowstride, src_row, bpr);
         }
     }
   else
     { /* RGBA (bpp > 3) GrayA (bpp == 2) or Gray */
       gboolean is_gray = ((bpp > 3) ? TRUE : FALSE);
+      gint y2 = y1 + h;
 
       for (row = 0, y = y1; y < y2; row++, y++)
         {
-          guchar *tmprow = p->col + row * rowstride;
           guchar *tmprow_ptr;
+          guchar *tmprow = p->col + row * rowstride;
+          gint    x2     = x1 + w;
+
+          gegl_buffer_get (src_buffer, GEGL_RECTANGLE (x1, y, w, 1), 1.0,
+                           format, src_row,
+                           GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
-          gimp_pixel_rgn_get_row (&src_rgn, src_row, x1, y, (x2 - x1));
           src = src_row;
           tmprow_ptr = tmprow;
           /* Possible micro-optimization here:
-           * src_end = src + src_rgn.bpp * (x2-x1);
+           * src_end = src + src_rgn.bpp * w);
            * for ( ; src < src_end ; src += src_rgn.bpp)
            */
           for (x = x1; x < x2; x++)
@@ -183,10 +205,13 @@ brushdmenuselect (GtkWidget *widget,
               *(tmprow_ptr++) = src[0];
               *(tmprow_ptr++) = src[is_gray ? 1 : 0];
               *(tmprow_ptr++) = src[is_gray ? 2 : 0];
-              src += src_rgn.bpp;
+              src += bpp;
             }
         }
     }
+
+  g_object_unref (src_buffer);
+
   g_free (src_row);
 
   if (bpp >= 3)
@@ -198,17 +223,6 @@ brushdmenuselect (GtkWidget *widget,
   update_brush_preview (NULL);
 }
 
-#if 0
-void
-dummybrushdmenuselect (GtkWidget *w, gpointer data)
-{
-  ppm_kill (&brushppm);
-  ppm_new (&brushppm, 10,10);
-  brush_from_file = 0;
-  update_brush_preview (NULL);
-}
-#endif
-
 static void
 brushlistrefresh (void)
 {
@@ -253,16 +267,16 @@ savebrush (GtkWidget *wg,
                                  GTK_WINDOW (gtk_widget_get_toplevel (wg)),
                                  GTK_FILE_CHOOSER_ACTION_SAVE,
 
-                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                 GTK_STOCK_SAVE,   GTK_RESPONSE_OK,
+                                 _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                 _("_Save"),   GTK_RESPONSE_OK,
 
                                  NULL);
 
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                            GTK_RESPONSE_OK,
                                            GTK_RESPONSE_CANCEL,
                                            -1);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog),
                                                   TRUE);
@@ -284,12 +298,12 @@ savebrush (GtkWidget *wg,
 }
 
 static gboolean
-validdrawable (gint32    imageid,
-               gint32    drawableid,
-               gpointer  data)
+validdrawable (GimpImage *image,
+               GimpItem  *item,
+               gpointer   data)
 {
-  return (gimp_drawable_is_rgb (drawableid) ||
-          gimp_drawable_is_gray (drawableid));
+  return (gimp_drawable_is_rgb  (GIMP_DRAWABLE (item)) ||
+          gimp_drawable_is_gray (GIMP_DRAWABLE (item)));
 }
 
 /*
@@ -364,7 +378,7 @@ update_brush_preview (const gchar *fn)
 
       set_colorbrushes (fn);
 
-      sc = GTK_ADJUSTMENT (brush_gamma_adjust)->value;
+      sc = gtk_adjustment_get_value (brush_gamma_adjust);
       if (sc != 1.0)
         for (i = 0; i < 256; i++)
           gammatable[i] = pow (i / 255.0, sc) * 255;
@@ -373,7 +387,7 @@ update_brush_preview (const gchar *fn)
           gammatable[i] = i;
 
       newheight = p.height *
-                    pow (10, GTK_ADJUSTMENT (brush_aspect_adjust)->value);
+        pow (10, gimp_label_spin_get_value (GIMP_LABEL_SPIN (brush_aspect_scale)));
 
       sc = p.width > newheight ? p.width : newheight;
       sc = 100.0 / sc;
@@ -448,8 +462,8 @@ brush_select (GtkTreeSelection *selection, gboolean force)
         }
 
       brush_dont_update = TRUE;
-      gtk_adjustment_set_value (GTK_ADJUSTMENT (brush_gamma_adjust), 1.0);
-      gtk_adjustment_set_value (GTK_ADJUSTMENT (brush_aspect_adjust), 0.0);
+      gtk_adjustment_set_value (brush_gamma_adjust, 1.0);
+      gimp_label_spin_set_value (GIMP_LABEL_SPIN (brush_aspect_scale), 0.0);
       brush_dont_update = FALSE;
 
       if (brush)
@@ -486,9 +500,10 @@ brush_preview_size_allocate (GtkWidget *preview)
 }
 
 static void
-brush_asepct_adjust_cb (GtkWidget *w, gpointer data)
+brush_aspect_adjust_cb (GimpLabelSpin *scale,
+                        gdouble       *value)
 {
-  gimp_double_adjustment_update (GTK_ADJUSTMENT (w), data);
+  gimpressionist_scale_entry_update_double (scale, value);
   update_brush_preview (pcvals.selected_brush);
 }
 
@@ -497,7 +512,7 @@ create_brushpage (GtkNotebook *notebook)
 {
   GtkWidget        *box1, *box2, *box3, *thispage;
   GtkWidget        *view;
-  GtkWidget        *tmpw, *table;
+  GtkWidget        *tmpw, *grid;
   GtkWidget        *frame;
   GtkWidget        *combo;
   GtkWidget        *label;
@@ -506,11 +521,11 @@ create_brushpage (GtkNotebook *notebook)
 
   label = gtk_label_new_with_mnemonic (_("_Brush"));
 
-  thispage = gtk_vbox_new (FALSE, 12);
+  thispage = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (thispage), 12);
   gtk_widget_show (thispage);
 
-  box1 = gtk_hbox_new (FALSE, 12);
+  box1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_start (GTK_BOX (thispage), box1, TRUE,TRUE,0);
   gtk_widget_show (box1);
 
@@ -521,7 +536,7 @@ create_brushpage (GtkNotebook *notebook)
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 
-  box2 = gtk_vbox_new (FALSE, 12);
+  box2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
   gtk_widget_show (box2);
 
@@ -537,18 +552,18 @@ create_brushpage (GtkNotebook *notebook)
   g_signal_connect (brush_preview, "size-allocate",
                     G_CALLBACK (brush_preview_size_allocate), NULL);
 
-  box3 = gtk_vbox_new (FALSE, 2);
+  box3 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
   gtk_box_pack_end (GTK_BOX (box2), box3, FALSE, FALSE,0);
   gtk_widget_show (box3);
 
   tmpw = gtk_label_new (_("Gamma:"));
-  gtk_misc_set_alignment (GTK_MISC (tmpw), 0.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (tmpw), 0.0);
   gtk_box_pack_start (GTK_BOX (box3), tmpw, FALSE, FALSE,0);
   gtk_widget_show (tmpw);
 
   brush_gamma_adjust = gtk_adjustment_new (pcvals.brushgamma,
                                            0.5, 3.0, 0.1, 0.1, 1.0);
-  tmpw = gtk_hscale_new (GTK_ADJUSTMENT (brush_gamma_adjust));
+  tmpw = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, brush_gamma_adjust);
   gtk_widget_set_size_request (GTK_WIDGET (tmpw), 100, 30);
   gtk_scale_set_draw_value (GTK_SCALE (tmpw), FALSE);
   gtk_scale_set_digits (GTK_SCALE (tmpw), 2);
@@ -561,65 +576,64 @@ create_brushpage (GtkNotebook *notebook)
   gimp_help_set_help_data
     (tmpw, _("Changes the gamma (brightness) of the selected brush"), NULL);
 
-  box3 = gtk_hbox_new (FALSE, 6);
+  box3 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_box_pack_start (GTK_BOX (thispage), box3, FALSE, FALSE,0);
   gtk_widget_show (box3);
 
   group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
   tmpw = gtk_label_new (_("Select:"));
-  gtk_misc_set_alignment (GTK_MISC (tmpw), 0.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (tmpw), 0.0);
   gtk_box_pack_start (GTK_BOX (box3), tmpw, FALSE, FALSE, 0);
   gtk_widget_show (tmpw);
 
   gtk_size_group_add_widget (group, tmpw);
   g_object_unref (group);
 
-  combo = gimp_drawable_combo_box_new (validdrawable, NULL);
+  combo = gimp_drawable_combo_box_new (validdrawable, NULL, NULL);
   gimp_int_combo_box_connect (GIMP_INT_COMBO_BOX (combo), -1,
                               G_CALLBACK (brushdmenuselect),
-                              NULL);
+                              NULL, NULL);
 
   gtk_box_pack_start (GTK_BOX (box3), combo, TRUE, TRUE, 0);
   gtk_widget_show (combo);
 
-  tmpw = gtk_button_new_from_stock (GTK_STOCK_SAVE_AS);
+  tmpw = gtk_button_new_with_mnemonic (_("Save _as"));
   gtk_box_pack_start (GTK_BOX (box3),tmpw, FALSE, FALSE, 0);
   g_signal_connect (tmpw, "clicked", G_CALLBACK (savebrush), NULL);
   gtk_widget_show (tmpw);
 
-  table = gtk_table_new (2, 3, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_box_pack_start (GTK_BOX (thispage), table, FALSE, FALSE, 0);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_box_pack_start (GTK_BOX (thispage), grid, FALSE, FALSE, 0);
+  gtk_widget_show (grid);
 
-  brush_aspect_adjust =
-    gimp_scale_entry_new (GTK_TABLE (table), 0, 0,
-                          _("Aspect ratio:"),
-                          150, -1, pcvals.brush_aspect,
-                          -1.0, 1.0, 0.1, 0.1, 2,
-                          TRUE, 0, 0,
-                          _("Specifies the aspect ratio of the brush"),
-                          NULL);
+  brush_aspect_scale =
+    gimp_scale_entry_new (_("Aspect ratio:"), pcvals.brush_aspect, -1.0, 1.0, 2);
+  gimp_help_set_help_data (brush_aspect_scale,
+                           _("Specifies the aspect ratio of the brush"),
+                           NULL);
   gtk_size_group_add_widget (group,
-                             GIMP_SCALE_ENTRY_LABEL (brush_aspect_adjust));
-  g_signal_connect (brush_aspect_adjust, "value-changed",
-                    G_CALLBACK (brush_asepct_adjust_cb), &pcvals.brush_aspect);
+                             gimp_labeled_get_label (GIMP_LABELED (brush_aspect_scale)));
+  g_signal_connect (brush_aspect_scale, "value-changed",
+                    G_CALLBACK (brush_aspect_adjust_cb),
+                    &pcvals.brush_aspect);
+  gtk_grid_attach (GTK_GRID (grid), brush_aspect_scale, 0, 0, 3, 1);
+  gtk_widget_show (brush_aspect_scale);
 
-  brush_relief_adjust =
-    gimp_scale_entry_new (GTK_TABLE (table), 0, 1,
-                          _("Relief:"),
-                          150, -1, pcvals.brush_relief,
-                          0.0, 100.0, 1.0, 10.0, 1,
-                          TRUE, 0, 0,
-                          _("Specifies the amount of embossing to apply to the image (in percent)"),
-                          NULL);
+  brush_relief_scale =
+    gimp_scale_entry_new (_("Relief:"), pcvals.brush_relief, 0.0, 100.0, 1);
+  gimp_help_set_help_data (brush_relief_scale,
+                           _("Specifies the amount of embossing to apply to the image (in percent)"),
+                           NULL);
   gtk_size_group_add_widget (group,
-                             GIMP_SCALE_ENTRY_LABEL (brush_relief_adjust));
-  g_signal_connect (brush_relief_adjust, "value-changed",
-                    G_CALLBACK (gimp_double_adjustment_update),
+                             gimp_labeled_get_label (GIMP_LABELED (brush_relief_scale)));
+  g_signal_connect (brush_relief_scale, "value-changed",
+                    G_CALLBACK (gimpressionist_scale_entry_update_double),
                     &pcvals.brush_relief);
+  gtk_grid_attach (GTK_GRID (grid), brush_relief_scale, 0, 1, 3, 1);
+  gtk_widget_show (brush_relief_scale);
 
   brush_select (selection, FALSE);
   readdirintolist ("Brushes", view, pcvals.selected_brush);
