@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -46,7 +46,8 @@ typedef struct {
   CommandFactory_t cmd_edit;
 } Selection_t;
 
-Selection_t *make_selection(ObjectList_t *list);
+Selection_t *make_selection (ObjectList_t *list,
+                             GimpImap     *imap);
 void selection_toggle_visibility(Selection_t *selection);
 void selection_freeze(Selection_t *selection);
 void selection_thaw(Selection_t *selection);
