@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -39,7 +39,7 @@ static void       print_settings_add_to_key_file             (const gchar
 
 static GKeyFile * print_settings_key_file_from_resource_file (void);
 
-static GKeyFile * print_settings_key_file_from_parasite      (gint32             image_ID);
+static GKeyFile * print_settings_key_file_from_parasite      (GimpImage         *image);
 
 static gboolean   print_settings_load_from_key_file          (PrintData         *data,
                                                               GKeyFile          *key_file);
@@ -54,7 +54,7 @@ static gboolean   print_settings_check_version               (GKeyFile
 gboolean
 print_settings_load (PrintData *data)
 {
-  GKeyFile *key_file = print_settings_key_file_from_parasite (data->image_id);
+  GKeyFile *key_file = print_settings_key_file_from_parasite (data->image);
 
   if (! key_file)
     key_file = print_settings_key_file_from_resource_file ();
@@ -79,14 +79,29 @@ print_settings_save (PrintData *data)
   GKeyFile *key_file = print_settings_key_file_from_settings (data);
 
   /* image setup */
-  if (gimp_image_is_valid (data->image_id))
+  if (gimp_image_is_valid (data->image))
     {
+      gdouble xres;
+      gdouble yres;
+
+      gimp_image_get_resolution (data->image, &xres, &yres);
+
       g_key_file_set_integer (key_file, "image-setup",
-                              "unit", data->unit);
-      g_key_file_set_double  (key_file, "image-setup",
-                              "x-resolution", data->xres);
-      g_key_file_set_double  (key_file, "image-setup",
-                              "y-resolution", data->yres);
+                              "unit", gimp_unit_get_id (data->unit));
+      /* Do not save the print resolution when it is the expected image
+       * resolution so that changing it (i.e. in "print size" dialog)
+       * is not overridden by any previous prints.
+       */
+      if ((data->min_xres <= xres && ABS (xres - data->xres) > 0.1)          ||
+          (data->min_yres <= yres && ABS (yres - data->yres) > 0.1)          ||
+          (data->min_xres > xres && ABS (data->min_xres - data->xres) > 0.1) ||
+          (data->min_yres > yres && ABS (data->min_yres - data->yres) > 0.1))
+        {
+          g_key_file_set_double  (key_file, "image-setup",
+                                  "x-resolution", data->xres);
+          g_key_file_set_double  (key_file, "image-setup",
+                                  "y-resolution", data->yres);
+        }
       g_key_file_set_double  (key_file, "image-setup",
                               "x-offset", data->offset_x);
       g_key_file_set_double  (key_file, "image-setup",
@@ -99,7 +114,7 @@ print_settings_save (PrintData *data)
                               "crop-marks", data->draw_crop_marks);
 
       print_utils_key_file_save_as_parasite (key_file,
-                                             data->image_id,
+                                             data->image,
                                              PRINT_SETTINGS_NAME);
     }
 
@@ -182,11 +197,11 @@ print_settings_key_file_from_resource_file (void)
  * NULL otherwise
  */
 static GKeyFile *
-print_settings_key_file_from_parasite (gint32 image_ID)
+print_settings_key_file_from_parasite (GimpImage *image)
 {
   GKeyFile *key_file;
 
-  key_file = print_utils_key_file_load_from_parasite (image_ID,
+  key_file = print_utils_key_file_load_from_parasite (image,
                                                       PRINT_SETTINGS_NAME);
 
   if (key_file && ! print_settings_check_version (key_file))
@@ -235,8 +250,8 @@ print_settings_load_from_key_file (PrintData *data,
 
   if (g_key_file_has_key (key_file, "image-setup", "unit", NULL))
     {
-      data->unit = g_key_file_get_integer (key_file, "image-setup",
-                                           "unit", NULL);
+      data->unit = gimp_unit_get_by_id (g_key_file_get_integer (key_file, "image-setup",
+                                                               "unit", NULL));
     }
 
   if (g_key_file_has_key (key_file, "image-setup", "x-resolution", NULL) &&
