@@ -142,7 +142,7 @@ struct _TwainClass
 
 
 #define TWAIN_TYPE  (twain_get_type ())
-#define TWAIN (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TWAIN_TYPE, Twain))
+#define TWAIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TWAIN_TYPE, Twain))
 
 GType                   twain_get_type         (void) G_GNUC_CONST;
 
@@ -153,9 +153,8 @@ static GimpProcedure  * twain_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * twain_run              (GimpProcedure        *procedure,
                                                 GimpRunMode           run_mode,
                                                 GimpImage            *image,
-                                                gint                  n_drawables,
                                                 GimpDrawable        **drawables,
-                                                const GimpValueArray *args,
+                                                GimpProcedureConfig  *config,
                                                 gpointer              run_data);
 
 G_DEFINE_TYPE (Twain, twain, GIMP_TYPE_PLUG_IN)
@@ -216,22 +215,17 @@ twain_create_procedure (GimpPlugIn  *plug_in,
                                       PLUG_IN_COPYRIGHT,
                                       PLUG_IN_VERSION);
 
-      GIMP_PROC_VAL_INT (procedure, "image-count",
-                         "Number of acquired images",
-                         "Number of acquired images",
-                         0, G_MAXINT, 0,
-                         G_PARAM_READWRITE);
-
-      GIMP_PROC_VAL_OBJECT_ARRAY (procedure, "images",
-                                  "Array of acquired images",
-                                  "Array of acquired images",
-                                  GIMP_TYPE_IMAGE,
-                                  G_PARAM_READWRITE);
+      gimp_procedure_add_core_object_array_return_value (procedure, "images",
+                                                         "Array of acquired images",
+                                                         "Array of acquired images",
+                                                         GIMP_TYPE_IMAGE,
+                                                         G_PARAM_READWRITE);
      }
 
   return procedure;
 }
 
+#if 0
 /* Data structure holding data between runs */
 /* Currently unused... Eventually may be used
  * to track dialog data.
@@ -257,6 +251,7 @@ static TwainValues twainvals =
   0, 0,
   TWPT_RGB
 };
+#endif
 
 /* The standard callback functions */
 TXFR_CB_FUNCS standardCbFuncs =
@@ -389,9 +384,8 @@ static GimpValueArray *
 twain_run (GimpProcedure        *procedure,
            GimpRunMode           run_mode,
            GimpImage            *image,
-           gint                  n_drawables,
            GimpDrawable        **drawables,
-           const GimpValueArray *args,
+           GimpProcedureConfig  *config,
            gpointer              run_data)
 {
   /* Initialize the return values
@@ -412,37 +406,19 @@ twain_run (GimpProcedure        *procedure,
    */
   if (! twainIsAvailable ())
     {
+      GError *error = NULL;
+
+      g_set_error (&error, GIMP_PLUG_IN_ERROR, 0,
+                   _("TWAIN driver not found, scanning not available"));
       return gimp_procedure_new_return_values (procedure, GIMP_PDB_EXECUTION_ERROR,
-                                               NULL);
+                                               error);
     }
 
-  /* How are we running today? */
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-      /* Retrieve values from the last run...
-       * Currently ignored
-       */
-      gimp_get_data (PLUG_IN_NAME, &twainvals);
-      break;
-
-    case GIMP_RUN_NONINTERACTIVE:
-      /* Currently, we don't do non-interactive calls.
-       * Bail if someone tries to call us non-interactively
-       */
-      return gimp_procedure_new_return_values (procedure, GIMP_PDB_CALLING_ERROR,
-                                               NULL);
-
-    case GIMP_RUN_WITH_LAST_VALS:
-      /* Retrieve values from the last run...
-       * Currently ignored
-       */
-      gimp_get_data (PLUG_IN_NAME, &twainvals);
-      break;
-
-    default:
-      break;
-    }
+  if (run_mode == GIMP_RUN_NONINTERACTIVE)
+    /* Currently, we don't do non-interactive calls.
+     * Bail if someone tries to call us non-interactively
+     */
+    return gimp_procedure_new_return_values (procedure, GIMP_PDB_CALLING_ERROR, NULL);
 
   /* Have we succeeded so far? */
   if (status == GIMP_PDB_SUCCESS)
@@ -457,29 +433,22 @@ twain_run (GimpProcedure        *procedure,
        * datasource.  Do final Interactive
        * steps.
        */
-      if (run_mode == GIMP_RUN_INTERACTIVE)
-        {
-          /* Store variable states for next run */
-          gimp_set_data (PLUG_IN_NAME, &twainvals, sizeof (TwainValues));
-        }
 
       num_images = g_list_length (image_list);
-      images     = g_new (GimpImage *, num_images);
+      images     = g_new0 (GimpImage *, num_images + 1);
 
       for (list = image_list, i = 0;
            list;
            list = g_list_next (list), i++)
         {
-          images[i] = g_object_ref (list->data);
+          images[i] = list->data;
         }
 
       g_list_free (image_list);
 
       /* Set return values */
-      return_vals = gimp_procedure_new_return_values (procedure, status,
-                                                      NULL);
-      GIMP_VALUES_SET_INT           (return_vals, 1, num_images);
-      GIMP_VALUES_TAKE_OBJECT_ARRAY (return_vals, 2, GIMP_TYPE_IMAGE, images, num_images);
+      return_vals = gimp_procedure_new_return_values (procedure, status, NULL);
+      GIMP_VALUES_TAKE_CORE_OBJECT_ARRAY (return_vals, 1, images);
 
       return return_vals;
     }
@@ -654,7 +623,7 @@ beginTransferCallback (pTW_IMAGEINFO  imageInfo,
   gimp_image_set_resolution (theClientData->image,
                              FIX32ToFloat (imageInfo->XResolution),
                              FIX32ToFloat (imageInfo->YResolution));
-  gimp_image_set_unit (theClientData->image, GIMP_UNIT_INCH);
+  gimp_image_set_unit (theClientData->image, gimp_unit_inch ());
 
   /* Create a layer */
   theClientData->layer = gimp_layer_new (theClientData->image,
